@@ -1,48 +1,92 @@
 /*   asn2ff2.c
-* ===========================================================================
-*
-*                            PUBLIC DOMAIN NOTICE
-*            National Center for Biotechnology Information (NCBI)
-*
-*  This software/database is a "United States Government Work" under the
-*  terms of the United States Copyright Act.  It was written as part of
-*  the author's official duties as a United States Government employee and
-*  thus cannot be copyrighted.  This software/database is freely available
-*  to the public for use. The National Library of Medicine and the U.S.
-*  Government do not place any restriction on its use or reproduction.
-*  We would, however, appreciate having the NCBI and the author cited in
-*  any work or product based on this material
-*
-*  Although all reasonable efforts have been taken to ensure the accuracy
-*  and reliability of the software and data, the NLM and the U.S.
-*  Government do not and cannot warrant the performance or results that
-*  may be obtained by using this software or data. The NLM and the U.S.
-*  Government disclaim all warranties, express or implied, including
-*  warranties of performance, merchantability or fitness for any particular
-*  purpose.
-*
-* ===========================================================================
-*
-* File Name:  asn2ff2.c
-*
-* Author:  Karl Sirotkin, Tom Madden, Tatiana Tatusov
-*
-* Version Creation Date:   7/15/95
-*
-* $Revision: 6.5 $
-*
-* File Description: 
-*
-* Modifications:  
-* --------------------------------------------------------------------------
-* Date     Name        Description of modification
-* -------  ----------  -----------------------------------------------------
-*
-*
-* ==========================================================================
-*/
-
-/*************************************
+ * ===========================================================================
+ *
+ *                            PUBLIC DOMAIN NOTICE
+ *            National Center for Biotechnology Information (NCBI)
+ *
+ *  This software/database is a "United States Government Work" under the
+ *  terms of the United States Copyright Act.  It was written as part of
+ *  the author's official duties as a United States Government employee and
+ *  thus cannot be copyrighted.  This software/database is freely available
+ *  to the public for use. The National Library of Medicine and the U.S.
+ *  Government do not place any restriction on its use or reproduction.
+ *  We would, however, appreciate having the NCBI and the author cited in
+ *  any work or product based on this material
+ *
+ *  Although all reasonable efforts have been taken to ensure the accuracy
+ *  and reliability of the software and data, the NLM and the U.S.
+ *  Government do not and cannot warrant the performance or results that
+ *  may be obtained by using this software or data. The NLM and the U.S.
+ *  Government disclaim all warranties, express or implied, including
+ *  warranties of performance, merchantability or fitness for any particular
+ *  purpose.
+ *
+ * ===========================================================================
+ *
+ * File Name:  asn2ff2.c
+ *
+ * Author:  Karl Sirotkin, Tom Madden, Tatiana Tatusov
+ *
+ * Version Creation Date:   7/15/95
+ *
+ * $Revision: 6.35 $
+ *
+ * File Description: 
+ *
+ * Modifications:  
+ * --------------------------------------------------------------------------
+ * Date     Name        Description of modification
+ * -------  ----------  -----------------------------------------------------
+ *
+ * $Log: asn2ff2.c,v $
+ * Revision 6.35  2001/11/09 20:20:41  kans
+ * new GENOME ANNOTATION REFSEQ text
+ *
+ * Revision 6.34  2001/11/09 15:31:47  kans
+ * bail on evidence user object missing name or method
+ *
+ * Revision 6.33  2001/11/09 15:29:19  kans
+ * display curator for reviewed refseq
+ *
+ * Revision 6.32  2001/11/09 14:45:49  kans
+ * added VALIDATED REFSEQ message
+ *
+ * Revision 6.31  2001/11/09 13:38:58  kans
+ * changed from TranscriptModelGeneration to ModelEvidence user object
+ *
+ * Revision 6.30  2001/10/02 19:16:32  yaschenk
+ * Removing memory leaks
+ *
+ * Revision 6.29  2001/08/07 15:51:08  kans
+ * use NUM_SEQID, added third party annotation seqids
+ *
+ * Revision 6.28  2000/12/27 19:43:20  kans
+ * empty comment could lead to original dsp->vnp being freed twice, corrupting memory on Mac
+ *
+ * Revision 6.27  2000/12/04 20:16:25  tatiana
+ * mrna evidence comments added
+ *
+ * Revision 6.26  2000/08/10 13:54:32  tatiana
+ * predicted refseq comment added
+ *
+ * Revision 6.25  2000/08/07 14:02:55  tatiana
+ * printf format fix
+ *
+ * Revision 6.24  2000/07/11 17:01:43  kans
+ * fixed bug of insufficient allocated memory block
+ *
+ * Revision 6.23  2000/07/06 22:42:00  tatiana
+ * REGSEQ text revised
+ *
+ * Revision 6.22  2000/06/19 16:36:27  kans
+ * NULL out ajp->Spop->ptr before calling StdFormatPrint, since string is freed elsewhere leaving ajp->Spop->ptr dangling
+ *
+ * Revision 6.21  2000/06/05 17:52:05  tatiana
+ * increase size of feature arrays to Int4
+ *
+ * Revision 6.20  2000/02/29 16:54:35  bazhin
+ * Removed memory leaks.
+ *
  * Revision 5.11  1997/06/19 18:36:58  vakatov
  * [WIN32,MSVC++]  Adopted for the "NCBIOBJ.LIB" DLL'ization
  *
@@ -167,15 +211,15 @@
 NLM_EXTERN Int2 GetGenDate PROTO ((Asn2ffJobPtr ajp, GBEntryPtr gbp, CharPtr buffer));
 NLM_EXTERN void PrintComment PROTO ((CharPtr string, Boolean identifier, Uint1 format));
 NLM_EXTERN CharPtr DateToGB PROTO ((CharPtr buf, NCBI_DatePtr ndp));
+NLM_EXTERN CharPtr mRNAEvidenceComment PROTO ((UserObjectPtr obj, Boolean add));
 static ValNodePtr AddToUniqueList PROTO ((SeqIdPtr sid, ValNodePtr list));
 static ValNodePtr GetDBSourceForNuclDB PROTO ((Asn2ffJobPtr ajp, GBEntryPtr gbp));
 
 static Boolean has_comment;
 
-#define NUM_ORDER 16
 /*---------- order for other id FASTA_LONG (copied from SeqIdWrite) ------- */
  
-static Uint1 fasta_order[NUM_ORDER] = {
+static Uint1 fasta_order[NUM_SEQID] = {
 33, /* 0 = not set */
 20, /* 1 = local Object-id */
 15,  /* 2 = gibbsq */
@@ -186,12 +230,15 @@ static Uint1 fasta_order[NUM_ORDER] = {
 10, /* 7 = pir */
 10, /* 8 = swissprot */
 15,  /* 9 = patent */
-20, /* 10 = other TextSeqId */
+18, /* 10 = other TextSeqId */
 20, /* 11 = general Dbtag */
 31,  /* 12 = gi */
 10, /* 13 = ddbj */
 10, /* 14 = prf */
-12  /* 15 = pdb */
+12, /* 15 = pdb */
+10,  /* 16 = tpg */
+10,  /* 17 = tpe */
+10   /* 18 = tpd */
 };
 
 static ComStructPtr ComStructNew(void)
@@ -209,9 +256,20 @@ static CharPtr SeqIdWriteProc (Asn2ffJobPtr ajp, SeqIdPtr sip)
 {
 	StdPrintOptionsPtr Spop=ajp->Spop;
 
-	if (! StdFormatPrint((Pointer)sip, 
-		(AsnWriteFunc)SeqIdAsnWrite, "StdSeqId", Spop)) {
-		    ErrPostStr (SEV_WARNING, 0, 0, "StdFormatPrint failed");
+	if (Spop != NULL) {
+		Spop->ptr = NULL; /* reused ajp->Spop->ptr left dangling by previous call */
+	}
+
+	if (ajp->show_version) {
+		if (! StdFormatPrint((Pointer)sip, 
+			(AsnWriteFunc)SeqIdAsnWrite, "VersionSeqId", Spop)) {
+		    	ErrPostStr (SEV_WARNING, 0, 0, "VersionSeqId_StdFormatPrint failed");
+		}
+	} else {
+		if (! StdFormatPrint((Pointer)sip, 
+			(AsnWriteFunc)SeqIdAsnWrite, "StdSeqId", Spop)) {
+		    	ErrPostStr (SEV_WARNING, 0, 0, "StdSeqId_StdFormatPrint failed");
+		}
 	}
 	if (Spop->ptr != NULL && *((CharPtr) (Spop->ptr)) != '\0') 
 		return Spop->ptr;
@@ -224,13 +282,17 @@ static CharPtr BioseqIdPrintProc (Asn2ffJobPtr ajp, GBEntryPtr gbp)
 	BioseqPtr bsp=gbp->bsp;
 	StdPrintOptionsPtr Spop=ajp->Spop;
 
+	if (Spop != NULL) {
+		Spop->ptr = NULL; /* reused ajp->Spop->ptr left dangling by previous call */
+	}
+
 	if (gbp == NULL) 
 		return NULL;
 	if ((bsp=gbp->bsp) == NULL)
 		return NULL;
 	if (! StdFormatPrint((Pointer)bsp, 
 		(AsnWriteFunc)BioseqAsnWrite, "StdBioseqId", Spop))
-		    ErrPostStr (SEV_WARNING, 0, 0, "StdFormatPrint failed");
+		    ErrPostStr (SEV_WARNING, 0, 0, "StdBioseqId_StdFormatPrint failed");
 
 	if (Spop->ptr != NULL && *((CharPtr) (Spop->ptr)) != '\0') 
 		return Spop->ptr;
@@ -244,11 +306,15 @@ static CharPtr TableIdPrintProc (Asn2ffJobPtr ajp, GBEntryPtr gbp)
 	BioseqPtr bsp=ajp->asn2ffwep->seg;
 	StdPrintOptionsPtr Spop=ajp->Spop;
 
+	if (Spop != NULL) {
+		Spop->ptr = NULL; /* reused ajp->Spop->ptr left dangling by previous call */
+	}
+
 	if (bsp == NULL)
 		return NULL;
 
 	if (! StdFormatPrint((Pointer)bsp, 
-		(AsnWriteFunc)BioseqAsnWrite, "StdBioseqId", Spop)) {
+		(AsnWriteFunc)BioseqAsnWrite, "TableIdPrintProcStdBioseqId", Spop)) {
                     ErrPostStr (SEV_WARNING, 0, 0, "StdFormatPrint failed");
 	}
 	if (Spop->ptr != NULL && *((CharPtr) (Spop->ptr)) != '\0') 
@@ -261,9 +327,13 @@ static CharPtr SPBlockPrintProc (Asn2ffJobPtr ajp, ValNodePtr vnp)
 {
 	StdPrintOptionsPtr Spop=ajp->Spop;
 
+	if (Spop != NULL) {
+		Spop->ptr = NULL; /* reused ajp->Spop->ptr left dangling by previous call */
+	}
+
 	if (! StdFormatPrint((Pointer)vnp->data.ptrvalue, 
 		(AsnWriteFunc)SPBlockAsnWrite, "StdSPBlock", Spop)) {
-                    ErrPostStr (SEV_WARNING, 0, 0, "StdFormatPrint failed");
+                    ErrPostStr (SEV_WARNING, 0, 0, "StdSPBlockStdFormatPrint failed");
 	}
 	if (Spop->ptr != NULL && *((CharPtr) (Spop->ptr)) != '\0') 
 		return Spop->ptr;
@@ -275,6 +345,10 @@ static CharPtr PIRBlockPrintProc (Asn2ffJobPtr ajp, ValNodePtr vnp)
 {
 	StdPrintOptionsPtr Spop=ajp->Spop;
 
+	if (Spop != NULL) {
+		Spop->ptr = NULL; /* reused ajp->Spop->ptr left dangling by previous call */
+	}
+
 	if (! StdFormatPrint((Pointer)vnp->data.ptrvalue, 
 		(AsnWriteFunc)PirBlockAsnWrite, "StdPirBlock", Spop)) {
                     ErrPostStr (SEV_WARNING, 0, 0, "StdFormatPrint failed");
@@ -289,6 +363,10 @@ static CharPtr PDBBlockPrintProc (Asn2ffJobPtr ajp, ValNodePtr vnp)
 {
 	StdPrintOptionsPtr Spop=ajp->Spop;
 
+	if (Spop != NULL) {
+		Spop->ptr = NULL; /* reused ajp->Spop->ptr left dangling by previous call */
+	}
+
 	if (! StdFormatPrint((Pointer)vnp->data.ptrvalue, 
 		(AsnWriteFunc)PdbBlockAsnWrite, "StdPDBBlock", Spop))
                     ErrPostStr (SEV_WARNING, 0, 0, "StdFormatPrint failed");
@@ -303,6 +381,10 @@ static CharPtr PRFBlockPrintProc (Asn2ffJobPtr ajp, ValNodePtr vnp)
 {
 	StdPrintOptionsPtr Spop=ajp->Spop;
 
+	if (Spop != NULL) {
+		Spop->ptr = NULL; /* reused ajp->Spop->ptr left dangling by previous call */
+	}
+
 	if (! StdFormatPrint((Pointer)vnp->data.ptrvalue, 
 		(AsnWriteFunc)PrfBlockAsnWrite, "StdPRFBlock", Spop))
                     ErrPostStr (SEV_WARNING, 0, 0, "StdFormatPrint failed");
@@ -341,14 +423,14 @@ static DescrStructPtr tie_next_dsp(DescrStructPtr head, DescrStructPtr next)
 
 static CharPtr GetSfpComment (Asn2ffJobPtr ajp, GBEntryPtr gbp)
 {
-	Int2 index;
+	Int4 index;
 	SeqFeatPtr sfp;
 	SortStructPtr p;
 
 	if (gbp == NULL || gbp->feat == NULL) {
 		return  NULL;
 	}
-	index = (Int2) ajp->pap_index;
+	index = ajp->pap_index;
 	p = gbp->feat->Commlist + index;
 	if ((sfp = p->sfp) == NULL) {
 		GatherItemWithLock(p->entityID, p->itemID, 
@@ -364,14 +446,14 @@ NLM_EXTERN void PrintCommentByNumber(Asn2ffJobPtr ajp, GBEntryPtr gbp)
 {
 	CharPtr newstring;
 	ComStructPtr s;
-	Int2 i, index = (Int2) ajp->pap_index;
+	Int4 i, index = ajp->pap_index;
 	
 	if (gbp == NULL) {
 		return;
 	}
 	for (s = gbp->comm, i=0; s && i < index; s = s->next, i++);
 	newstring = CheckEndPunctuation(s->string, '.');
-	PrintComment(newstring, FALSE, (Uint1)ajp->format);
+	www_PrintComment(newstring, FALSE, (Uint1)ajp->format);
 	newstring = MemFree(newstring);
 	return;
 }
@@ -379,7 +461,7 @@ NLM_EXTERN void PrintFirstComment(Asn2ffJobPtr ajp, GBEntryPtr gbp)
 {
 	CharPtr newstring;
 	ComStructPtr s;
-	Int2 i, index = (Int2) ajp->pap_index;
+	Int4 i, index = ajp->pap_index;
 	
 	if (gbp == NULL) {
 		return;
@@ -390,7 +472,7 @@ NLM_EXTERN void PrintFirstComment(Asn2ffJobPtr ajp, GBEntryPtr gbp)
 		} else {
 			newstring = CheckEndPunctuation(s->string, '.');
 		}
-		PrintComment(newstring, TRUE, (Uint1)ajp->format);
+		www_PrintComment(newstring, TRUE, (Uint1)ajp->format);
 		newstring = MemFree(newstring);
 	}
 	return;
@@ -413,6 +495,140 @@ static ComStructPtr tie_next_comm(ComStructPtr head, ComStructPtr next)
 	return head;
 }
 
+static CharPtr GetStrForBankit(UserObjectPtr uop)
+{
+    ObjectIdPtr		oip;
+	UserFieldPtr	ufp, tmp, u;
+	CharPtr			ptr=NULL, ptr1 = NULL, str;
+	Int2			i=0, acclen, ptrlen = 0;
+	CharPtr			p;
+	
+	if ((oip = uop->type) == NULL) return NULL;
+	if (StringCmp(oip->str, "Submission") != 0) return NULL;
+	for (ufp=uop->data; ufp; ufp=ufp->next) {
+		oip = ufp->label;
+		if (StringCmp(oip->str, "UniVecComment") == 0) {
+			if (ufp->data.ptrvalue) {
+				ptrlen = (StringLen("Vector Explanation: ")
+							+ StringLen(ufp->data.ptrvalue) + 1);
+				ptr = (CharPtr) MemNew(ptrlen);
+				sprintf(ptr, "Vector Explanation: %s", 
+										(CharPtr) ufp->data.ptrvalue);
+			}
+		}
+	}
+	for (ufp=uop->data; ufp; ufp=ufp->next) {
+		oip = ufp->label;
+		if (StringCmp(oip->str, "AdditionalComment") == 0) {
+			if (ufp->data.ptrvalue) {
+				ptr1 = (CharPtr) MemNew(StringLen("Bankit Comment: ")
+							+ StringLen(ufp->data.ptrvalue) + 2 + ptrlen);
+				if (ptr) {
+					sprintf(ptr1, "%s~Bankit Comment: %s", 
+									ptr, (CharPtr) ufp->data.ptrvalue);
+				} else {
+					sprintf(ptr1, "Bankit Comment: %s", 
+										(CharPtr) ufp->data.ptrvalue);
+				}
+				return ptr1;
+			}
+		}
+	}
+	
+	return ptr;
+}
+
+static CharPtr GetStrForUserObject(UserObjectPtr uop)
+{
+    ObjectIdPtr		oip;
+	UserFieldPtr	ufp, tmp, u, urf;
+	CharPtr			curator = NULL, ptr=NULL, st;
+	Int2			i=0, acclen;
+	CharPtr			p = NULL, pp;
+	Int2			review = 0, len;
+	
+	if ((oip = uop->type) == NULL) return NULL;
+	if (StringCmp(oip->str, "RefGeneTracking") != 0) return NULL;
+	len = StringLen("The reference sequence was derived from ");
+	for (ufp=uop->data; ufp; ufp=ufp->next) {
+		oip = ufp->label;
+		if (StringCmp(oip->str, "Assembly") == 0) {
+			urf = ufp;
+		}
+		if (StringCmp(oip->str, "Status") == 0) {
+			st = (CharPtr) ufp->data.ptrvalue;
+			if (StringCmp(st, "Provisional") == 0) {
+				review = 1;
+			} else if (StringCmp(st, "Reviewed") == 0) {
+				review = 2;
+			} else if (StringCmp(st, "Predicted") == 0) {
+				review = 3;
+			} else if (StringCmp(st, "Validated") == 0) {
+				review = 4;
+			}
+		} else if (StringCmp (oip->str, "Collaborator") == 0) {
+			st = (CharPtr) ufp->data.ptrvalue;
+			if (! StringHasNoText (st)) {
+				curator = st;
+			}
+		}
+	}
+	if (urf && urf->choice == 11) {
+		for (tmp=urf->data.ptrvalue; tmp; tmp=tmp->next, i++) ;
+		if (review == 2) {
+			if (curator == NULL) {
+				curator = "NCBI staff";
+			}
+			ptr = (CharPtr) MemNew(
+			StringLen("REVIEWED REFSEQ: This record has been curated by . ") + StringLen (curator) + len + 18*i + 1);
+			sprintf(ptr, "REVIEWED REFSEQ: This record has been curated by %s. ", curator);
+		} else if (review == 1) {
+			ptr = (CharPtr) MemNew(
+			StringLen("PROVISIONAL REFSEQ: This record has not yet been subject to final NCBI review. ") + len + 18*i + 1);
+			sprintf(ptr, "PROVISIONAL REFSEQ: This record has not yet been subject to final NCBI review. ");
+		} else if (review == 3) {
+			ptr = (CharPtr) MemNew(
+			StringLen("PREDICTED REFSEQ: The mRNA record is supported by experimental evidence; however, the coding sequence is predicted. ") + len + 18*i + 1);
+			sprintf(ptr, "PREDICTED REFSEQ: The mRNA record is supported by experimental evidence; however, the coding sequence is predicted. ");
+		} else if (review == 4) {
+			ptr = (CharPtr) MemNew(
+			StringLen("VALIDATED REFSEQ: This record has undergone preliminary review of the sequence, but has not yet been subject to final NCBI review. ") + len + 18*i + 1);
+			sprintf(ptr, "VALIDATED REFSEQ: This record has undergone preliminary review of the sequence, but has not yet been subject to final NCBI review. ");
+		} else {
+			ptr = (CharPtr) MemNew( StringLen("REFSEQ: ") + len + 18*i + 1);
+			sprintf(ptr, "REFSEQ: ");
+		}
+		if (i > 0) {
+			pp = (CharPtr) MemNew(len + 1);
+			sprintf(pp, "The reference sequence was derived from ");
+			StringCat(ptr, pp);
+			MemFree(pp);
+		}
+		for (tmp=urf->data.ptrvalue; tmp; tmp=tmp->next) {
+			for (u = tmp->data.ptrvalue; u; u=u->next) {
+				oip = u->label;
+				if (StringCmp(oip->str, "accession") == 0) {
+					break;
+				}
+			}
+			if (u) {
+				if ((acclen = StringLen(u->data.ptrvalue)) > 15) {
+					continue;
+				}
+				p = (CharPtr) MemNew(acclen + 4);
+				if (tmp->next) {
+					sprintf(p, "%s, ", (CharPtr) u->data.ptrvalue);
+				} else {
+					sprintf(p, "%s.~~", (CharPtr) u->data.ptrvalue);
+				}
+			}
+			StringCat(ptr, p);
+			MemFree(p);
+		}
+	}
+	return ptr;
+}
+
 static CharPtr GetStrForMap(DbtagPtr dbtag)
 {
 	Int2 total;
@@ -447,6 +663,88 @@ static CharPtr GetStrForMap(DbtagPtr dbtag)
 	return NULL;
 }
 
+static CharPtr GetEvidence(Asn2ffJobPtr ajp, GBEntryPtr gbp)
+{
+	CharPtr		retval=NULL;
+	SeqDescrPtr		descr;
+	UserObjectPtr	uop=NULL;
+    ObjectIdPtr		oip;
+	ValNodePtr tvnp, ds_vnp, vnp;
+	DescrStructPtr dsp;
+
+	if (gbp == NULL || gbp->bsp == NULL) {
+		return NULL;
+	}
+	tvnp = GatherDescrListByChoice(ajp, gbp, Seq_descr_user); 
+	for (ds_vnp= tvnp; ds_vnp;) {
+		dsp = (DescrStructPtr) ds_vnp->data.ptrvalue;
+		if(vnp = dsp->vnp){
+			if(!retval){
+				uop = (UserObjectPtr) vnp->data.ptrvalue;
+				retval=mRNAEvidenceComment(uop, FALSE);
+			}
+			MemFree(vnp);
+		}
+		MemFree(dsp);
+		vnp=ds_vnp;
+		ds_vnp=ds_vnp->next;
+		MemFree(vnp);
+	}
+	return retval;
+}
+
+static CharPtr genanreftext1 = "GENOME ANNOTATION REFSEQ:  This model reference sequence was predicted from NCBI contig";
+static CharPtr genanreftext2 = "by automated computational analysis";
+static CharPtr genanreftext3 = "using gene prediction method:";
+static CharPtr genanreftext4 = "~Also see:~    Documentation of NCBI's Annotation Process~    Evidence Viewer - alignments supporting this model";
+
+static CharPtr GetAnnotationComment(Asn2ffJobPtr ajp, GBEntryPtr gbp)
+{
+	SeqDescrPtr		descr;
+	UserObjectPtr	uop=NULL;
+    ObjectIdPtr		oip;
+	UserFieldPtr	ufp, tmp, u, urf;
+	CharPtr 		retval = NULL, name = NULL, method = NULL;
+	
+	ValNodePtr tvnp, ds_vnp, vnp;
+	DescrStructPtr dsp;
+
+	if (gbp == NULL || gbp->bsp == NULL) {
+		return NULL;
+	}
+	tvnp = GatherDescrListByChoice(ajp, gbp, Seq_descr_user); 
+	for (ds_vnp= tvnp;
+				ds_vnp; ds_vnp=ds_vnp->next) {
+		dsp = (DescrStructPtr) ds_vnp->data.ptrvalue;
+		if ((vnp = dsp->vnp) == NULL)
+			continue;
+		uop = (UserObjectPtr) vnp->data.ptrvalue;
+	}
+	if (uop == NULL) return retval;
+	if ((oip = uop->type) == NULL) return retval;
+	if (StringCmp(oip->str, "ModelEvidence") != 0) return retval;
+	for (ufp=uop->data; ufp; ufp=ufp->next) {
+		oip = ufp->label;
+		if (StringCmp(oip->str, "Contig Name") == 0) {
+			name = (CharPtr) ufp->data.ptrvalue;
+		}
+		if (StringCmp(oip->str, "Method") == 0) {
+			if (ufp->data.ptrvalue) {
+				method = StringSave((CharPtr) ufp->data.ptrvalue);
+			}
+		}
+	}
+	if (name == NULL) return NULL;
+	retval = MemNew(StringLen(genanreftext1) + StringLen(genanreftext2) + StringLen(genanreftext3) + StringLen(genanreftext4) + StringLen(name) + StringLen(method) + 15);
+	if (method != NULL) {
+		sprintf(retval, "%s %s %s %s %s. %s", genanreftext1, name, genanreftext2, genanreftext3, method, genanreftext4);
+	} else {
+		sprintf(retval, "%s %s %s. %s", genanreftext1, name, genanreftext2, genanreftext4);
+	}
+	return retval;
+	
+}
+
 static void GetCommentByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp, Uint1 choice)
 {
 	ValNodePtr ds_vnp, tvnp, vnp = NULL;
@@ -458,9 +756,14 @@ static void GetCommentByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp, Uint1 choice)
 	ComStructPtr csp;
 	MolInfoPtr mfp;
 	CharPtr buf = NULL, ptr = NULL, string=NULL, new_string=NULL, str=NULL;
-	Int2 buflen = 0;
+	Int4 buflen = 0;
 	Int4 num_s = 0, num_g = 0;
+	Boolean is_other;
+	CharPtr p;
 
+	if (bsp && bsp->id) {
+		is_other = (bsp->id->choice == SEQID_OTHER) ? TRUE : FALSE; 
+	}
 	tvnp = GatherDescrListByChoice(ajp, gbp, choice); 
 	for (ds_vnp = tvnp;	ds_vnp; ds_vnp=ds_vnp->next) {
 		dsp = (DescrStructPtr) ds_vnp->data.ptrvalue;
@@ -468,6 +771,30 @@ static void GetCommentByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp, Uint1 choice)
 		if (vnp == NULL) {
 			continue;
 		}
+		if (choice == Seq_descr_user) {
+			if (ajp->bankit && 
+				(new_string = GetStrForBankit(vnp->data.ptrvalue))) {
+				csp = ComStructNew();
+				csp->string = new_string;
+				csp->gsdb_id = FALSE;
+				csp->entityID = dsp->entityID;
+				csp->itemID = dsp->itemID;
+				csp->itemtype = dsp->itemtype;
+				gbp->comm = tie_next_comm(gbp->comm, csp);
+				gbp->comm_num++;
+			}
+			if ((new_string = GetStrForUserObject(vnp->data.ptrvalue))) {
+				csp = ComStructNew();
+				csp->gsdb_id = FALSE;
+				csp->string = StringSave(new_string);
+				new_string = MemFree(new_string);
+				csp->entityID = dsp->entityID;
+				csp->itemID = dsp->itemID;
+				csp->itemtype = dsp->itemtype;
+				gbp->comm = tie_next_comm(gbp->comm, csp);
+				gbp->comm_num++;
+			}
+		}
 		csp = ComStructNew();
 		csp->string = NULL;
 		csp->gsdb_id = FALSE;
@@ -484,46 +811,91 @@ static void GetCommentByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp, Uint1 choice)
 			str = CheckEndPunctuation(new_string, '.');
 			new_string = MemFree(new_string);
 			csp->string = StringSave(str);
+			MemFree(str);
 			break;
 		case Seq_descr_molinfo:
-			mfp = vnp->data.ptrvalue;
-			if (mfp && mfp->tech > 1) {
-				for (d = (DeltaSeqPtr)(bsp->seq_ext); d != NULL; d = d->next) {
-					buflen +=80;
-				}
-				if (buflen > 0) {
-					buf = MemNew(buflen+1);
-					CountGapsInDeltaSeq(bsp, &num_s, 
-						&num_g, NULL, NULL, buf, buflen);
-				}
-				if (mfp->tech == MI_TECH_htgs_1) {
-					tmp = StringSave(
-					"***                                                 ***~*** WARNING: Phase 1 High Throughput Genome Sequence ***~***                                                 ***~* This sequence is unfinished.");
-					if (num_s > 0) {
-						sprintf(buffer, 
-						"It consists of %ld contigs for~* which the order is not known; their order in this record is~* arbitrary. In some cases, the exact lengths of the gaps~* between the contigs are also unknown; these gaps are  presented~* as runs of N as a convenience only.", (long) (num_s-num_g));
-						tmp = Cat2Strings(tmp, buffer, " ", 0);
+			if ((mfp = vnp->data.ptrvalue) != NULL) {
+				if (is_other == TRUE  && mfp->completeness) {
+					switch (mfp->completeness) {
+						case 1:
+				csp->string = StringSave("COMPLETENESS: full length");
+						break;
+						case 2:
+				csp->string = StringSave("COMPLETENESS: not full length");
+						break;
+						case 3:
+			csp->string = StringSave("COMPLETENESS: incomplete on the 5' end");
+						break;
+						case 4:
+			csp->string = StringSave("COMPLETENESS: incomplete on the 3' end");
+						break;
+						case 5:
+			csp->string = StringSave("COMPLETENESS: incomplete on both ends");
+						break;
+						case 6:
+			csp->string = StringSave("COMPLETENESS: complete on the 5' end");
+						break;
+						case 7:
+			csp->string = StringSave("COMPLETENESS: complete on the 3' end");
+						break;
+						default:
+			csp->string = StringSave("COMPLETENESS: unknown");
+						break;
 					}
-					tmp3 = StringSave("When sequencing is complete,~* the sequence data presented in this record will be replaced~*by a single finished sequence with the same accession number.");
-					tmp = Cat2Strings(tmp, tmp3, " ", 0);
-					csp->string = Cat2Strings(tmp, buf, "~", 0);
-				} else if (mfp->tech == MI_TECH_htgs_2) {
-					tmp = StringSave(
-					"***                                                 ***~*** WARNING: Phase 2 High Throughput Genome Sequence ***~***                                                 ***~* This sequence is unfinished."); 
-					if (num_s > 0) {
-						sprintf(buffer, "It consists of %ld contigs for~* which the order is known. The lengths of the gaps have been~* estimated by the submitter but are not known exactly.", (long) (num_s-num_g));
-						tmp = Cat2Strings(tmp, buffer, " ", 0);
+				} else if(mfp->tech > 1) {
+					for (d=(DeltaSeqPtr)(bsp->seq_ext); d != NULL; d=d->next) {
+						buflen +=80;
 					}
-					tmp3 = StringSave("When~* sequencing is complete, the sequence data presented in this~* record will be replaced by a single finished sequence~* with the same accession number.");
-					tmp = Cat2Strings(tmp, tmp3, " ", 0);
-					csp->string = Cat2Strings(tmp, buf, "~", 0);
-				} else if ((str = StringForSeqTech(mfp->tech)) != NULL) {
-                                        ptr = MemNew(StringLen(str) + 10);
-					sprintf(ptr, "Method: %s.", str); 
-					csp->string = StringSave(ptr);
-                                        MemFree(ptr);
+					if (buflen > 0) {
+						buf = MemNew(buflen+1);
+						CountGapsInDeltaSeq(bsp, &num_s, 
+							&num_g, NULL, NULL, buf, buflen);
+					}
+					if (mfp->tech == MI_TECH_htgs_0) {
+						if (num_s > 0) {
+							sprintf(buffer, 
+							"* NOTE: This record contains %ld individual~* sequencing reads that have not been assembled into~* contigs. Runs of N are used to separate the reads~* and the order in which they appear is completely~* arbitrary. Low-pass sequence sampling is useful for~* identifying clones that may be gene-rich and allows~* overlap relationships among clones to be deduced.~* However, it should not be assumed that this clone~* will be sequenced to completion. In the event that~* the record is updated, the accession number will~* be preserved.", (long) (num_s-num_g));
+						}
+						csp->string = Cat2Strings(buffer, buf, "~", 0);
+					} else if (mfp->tech == MI_TECH_htgs_1) {
+						tmp = StringSave(
+						"* NOTE: This is a \"working draft\" sequence.");
+						if (num_s > 0) {
+							sprintf(buffer, 
+							" It currently~* consists of %ld contigs. The true order of the pieces~* is not known and their order in this sequence record is~* arbitrary. Gaps between the contigs are represented as~* runs of N, but the exact sizes of the gaps are unknown.", (long) (num_s-num_g));
+							p = Cat2Strings(tmp, buffer, "", 0);
+							MemFree(tmp);
+							tmp = p;
+						}
+						tmp3 = StringSave("* This record will be updated with the finished sequence~* as soon as it is available and the accession number will~* be preserved.");
+						p = Cat2Strings(tmp, tmp3, "~", 0);
+						MemFree(tmp);
+						MemFree(tmp3);
+						csp->string = Cat2Strings(p, buf, "~", 0);
+						MemFree(p);
+					} else if (mfp->tech == MI_TECH_htgs_2) {
+						tmp = StringSave(
+						"* NOTE: This is a \"working draft\" sequence."); 
+						if (num_s > 0) {
+							sprintf(buffer, " It currently~* consists of %ld contigs. Gaps between the contigs~* are represented as runs of N. The order of the pieces~* is believed to be correct as given, however the sizes~* of the gaps between them are based on estimates that have~* provided by the submittor.", (long) (num_s-num_g));
+							p = Cat2Strings(tmp, buffer, "", 0);
+							MemFree(tmp);
+							tmp = p;
+						}
+						tmp3 = StringSave("* This sequence will be replaced~* by the finished sequence as soon as it is available and~* the accession number will be preserved.");
+						p = Cat2Strings(tmp, tmp3, "~", 0);
+						MemFree(tmp);
+						MemFree(tmp3);
+						csp->string = Cat2Strings(p, buf, "~", 0);
+						MemFree(p);
+					} else if ((str = StringForSeqTech(mfp->tech)) != NULL) {
+						ptr = MemNew(StringLen(str) + 10);
+						sprintf(ptr, "Method: %s.", str); 
+						csp->string = StringSave(ptr);
+						MemFree(ptr);
+					}
+					MemFree(buf);
 				}
-				MemFree(buf);
 			}
 			break;
 		case Seq_descr_method:
@@ -538,7 +910,7 @@ static void GetCommentByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp, Uint1 choice)
 		default:
 			break;
 		}
-		if (csp->string != NULL) {
+		if (choice != Seq_descr_user && csp->string) {
 			csp->entityID = dsp->entityID;
 			csp->itemID = dsp->itemID;
 			csp->itemtype = dsp->itemtype;
@@ -547,6 +919,7 @@ static void GetCommentByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp, Uint1 choice)
 		} else {
 			MemFree(csp);
 		}
+		MemFree(vnp);
 		MemFree(dsp);
 	}
 	ValNodeFree(tvnp);
@@ -557,10 +930,49 @@ NLM_EXTERN Int2 GB_GetSeqDescrComms(Asn2ffJobPtr ajp, GBEntryPtr gbp)
 {
 	ComStructPtr csp;
     DbtagPtr	DBtag = NULL;
-	static Char buf[128];
+	static Char buf[1001], tmp[28];
 	SeqIdPtr sid = NULL;
+	SeqHistPtr hist;
+	Int2 num_gi;
+	DatePtr dat;
+	CharPtr strd = NULL;
+	TextSeqIdPtr tsip = NULL;
+	Boolean is_contig = FALSE;
 	
 	gbp->comm_num = 0;
+/*  Get  NT_ contig comment */
+	for (sid = gbp->bsp->id; sid; sid=sid->next) {
+		if (sid->choice == SEQID_OTHER) {
+			tsip = (TextSeqIdPtr) sid->data.ptrvalue;
+			break;
+		}
+	}
+	if (tsip != NULL) {
+		if (StringNCmp(tsip->accession, "NT", 2) == 0) {
+			is_contig = TRUE;			
+			csp = ComStructNew();
+			csp->string = StringSave("GENOME ANNOTATION REFSEQ:  NCBI contigs are derived from assembled genomic sequence data. They may include both draft and finished sequence.");
+		} else if (StringNCmp(tsip->accession, "XP_", 3) == 0 || StringNCmp(tsip->accession, "XM_", 3) == 0) {
+			is_contig = TRUE;			
+			csp = ComStructNew();
+			csp->string = GetAnnotationComment(ajp, gbp);
+		}
+		if (is_contig) {			
+			gbp->comm = tie_next_comm(gbp->comm, csp);
+			gbp->comm_num++;
+		}
+		/*
+		if ((strd = GetEvidence(ajp, gbp)) != NULL) {
+			csp = ComStructNew();
+			csp->string =StringSave( strd);
+			gbp->comm = tie_next_comm(gbp->comm, csp);
+			gbp->comm_num++;
+		}
+		*/
+	}
+
+	GetCommentByChoice(ajp, gbp, Seq_descr_user);
+
 /*  Get GSDB Id */
 	for (sid = gbp->bsp->id; sid; sid=sid->next) {
 		if (sid->choice != SEQID_GENERAL)
@@ -580,6 +992,70 @@ NLM_EXTERN Int2 GB_GetSeqDescrComms(Asn2ffJobPtr ajp, GBEntryPtr gbp)
 			gbp->comm_num++;
 		}
 	}
+	if ((hist = gbp->bsp->hist) != NULL) {
+		if (hist->replaced_by_ids != NULL && hist->replaced_by_date != NULL) {
+			num_gi = 0;
+			for (sid=hist->replaced_by_ids; sid; sid=sid->next) {
+				if (sid->choice == SEQID_GI) {
+					num_gi++;
+				}
+			}
+			if ((dat = hist->replaced_by_date) != NULL) {
+				strd = PrintDate(dat);
+				if (strd[StringLen(strd)-1] == '\n') {
+					if (strd[StringLen(strd)-2] == '.') {
+						strd[StringLen(strd)-2] = '\0';
+					} else {
+						strd[StringLen(strd)-1] = '\0';
+					}
+				}
+			}
+			csp = ComStructNew();
+			csp->string = MemNew((128 + 16*num_gi)*sizeof(Char));
+			sprintf(csp->string, "[WARNING] On %s this sequence was replaced by a newer version", strd);
+			if(strd != NULL)
+				strd = MemFree(strd);
+			for (sid=hist->replaced_by_ids; sid; sid=sid->next) {
+				if (sid->choice == SEQID_GI) {
+					sprintf(tmp, " gi:%ld", (long) sid->data.intvalue);
+					StringCat(csp->string, tmp);
+				}
+			}
+			gbp->comm = tie_next_comm(gbp->comm, csp);
+			gbp->comm_num++;
+		}
+		if (hist->replace_ids != NULL && hist->replace_date != NULL) {
+			num_gi = 0;
+			for (sid=hist->replace_ids; sid; sid=sid->next) {
+				if (sid->choice == SEQID_GI) {
+					num_gi++;
+				}
+			}
+			if ((dat = hist->replace_date) != NULL) {
+				strd = PrintDate(dat);
+				if (strd[StringLen(strd)-1] == '\n') {
+					if (strd[StringLen(strd)-2] == '.') {
+						strd[StringLen(strd)-2] = '\0';
+					} else {
+						strd[StringLen(strd)-1] = '\0';
+					}
+				}
+			}
+			csp = ComStructNew();
+			csp->string = MemNew((128 + 16*num_gi)*sizeof(Char));
+			sprintf(csp->string, "On %s this sequence version replaced", strd);
+			if(strd != NULL)
+				strd = MemFree(strd);
+			for (sid=hist->replace_ids; sid; sid=sid->next) {
+				if (sid->choice == SEQID_GI) {
+					sprintf(tmp, " gi:%ld", (long) sid->data.intvalue);
+					StringCat(csp->string, tmp);
+				}
+			}
+			gbp->comm = tie_next_comm(gbp->comm, csp);
+			gbp->comm_num++;
+		}
+	}
 	GetCommentByChoice(ajp, gbp, Seq_descr_comment);
 	GetCommentByChoice(ajp, gbp, Seq_descr_maploc);
 	GetCommentByChoice(ajp, gbp, Seq_descr_region);
@@ -903,7 +1379,7 @@ NLM_EXTERN void PrintDBSourceLine(Asn2ffJobPtr ajp, GBEntryPtr gbp)
 	ff_StartPrint(0, 12, ASN2FF_GB_MAX, NULL);
 	ff_AddString("DBSOURCE");
 	TabToColumn(13);
-	sid = SeqIdSelect(gbp->bsp->id, fasta_order, NUM_ORDER);
+	sid = SeqIdSelect(gbp->bsp->id, fasta_order, NUM_SEQID);
 	if (sid && ((sid->choice == SEQID_PIR) ||
                 (sid->choice == SEQID_SWISSPROT) ||
                 (sid->choice == SEQID_PDB) ||
@@ -923,6 +1399,10 @@ than the individual parts of a segmented set, at least in Swiss-prot. */
 			(sid->choice == SEQID_DDBJ) || 
 			(sid->choice == SEQID_GIBBSQ) || 
 			(sid->choice == SEQID_GIBBMT) || 
+            (sid->choice == SEQID_OTHER) ||
+			(sid->choice == SEQID_TPG) || 
+			(sid->choice == SEQID_TPE) || 
+			(sid->choice == SEQID_TPD) || 
 			(sid->choice == SEQID_GI) || 
 			(sid->choice == SEQID_GIIM || (ChoicePID(sid) == TRUE))))  {
 		vnp = GetDBSourceForNuclDB(ajp, gbp);
@@ -1023,7 +1503,7 @@ itself. */
 	if (list == NULL) {
 		if ((bsp = ajp->asn2ffwep->seg) != NULL)
 		{
-			sid = SeqIdSelect(bsp->id, fasta_order, NUM_ORDER);
+			sid = SeqIdSelect(bsp->id, fasta_order, NUM_SEQID);
 			if (sid->choice == SEQID_GIBBMT) 
 				ValNodeAddPointer(&list, 0, bsp->id);	
 		}
@@ -1037,7 +1517,7 @@ bsp->id.  If no bsp, keep the id that was found above by SeqLocId. */
 		id = vnp->data.ptrvalue;
 		bsp = BioseqFindCore(id);
 		if (bsp) {
-			sid = SeqIdSelect(bsp->id, fasta_order, NUM_ORDER);
+			sid = SeqIdSelect(bsp->id, fasta_order, NUM_SEQID);
 			vnp->data.ptrvalue = sid;
 		} else if (id && id->choice == SEQID_GI) {
 			if ((sid = GetSeqIdForGI(id->data.intvalue)) != NULL)
@@ -1382,7 +1862,7 @@ NLM_EXTERN ValNodePtr GatherDescrByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp, Uint
   	MemSet ((Pointer) (&gsc), 0, sizeof (GatherScope));
 	MemSet ((Pointer) (gsc.ignore), (int)(TRUE), (size_t) (OBJ_MAX * sizeof(Boolean)));
 	gsc.ignore[OBJ_SEQDESC] = FALSE;
-	if (gbp) {
+	if (gbp && gbp->bsp != NULL) {
 		bsp = gbp->bsp;
 		bspID = ObjMgrGetEntityIDForPointer(bsp);
 		slp = ValNodeNew(NULL);
@@ -1392,8 +1872,10 @@ NLM_EXTERN ValNodePtr GatherDescrByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp, Uint
 	} else {
 		gsc.target = NULL;
 	}
-	if (ajp->only_one || ajp->genome_view) {
-		gsc.target = NULL;
+	if (ajp->sep && ajp->sep->choice == 1) {
+		if (ajp->only_one || ajp->genome_view) {
+			gsc.target = NULL;
+		}
 	}
 	GatherEntity(ajp->entityID, &dsp, get_descr_vnp, &gsc); 
 	if (slp) {
@@ -1434,7 +1916,7 @@ NLM_EXTERN ValNodePtr GatherDescrListByChoice(Asn2ffJobPtr ajp, GBEntryPtr gbp,
   	gsc.get_feats_location = TRUE;
 	MemSet ((Pointer) (gsc.ignore), (int)(TRUE), (size_t) (OBJ_MAX * sizeof(Boolean)));
 	gsc.ignore[OBJ_SEQDESC] = FALSE;
-	if (gbp) {
+	if (gbp && gbp->bsp != NULL) {
 		bsp = gbp->bsp; 
 		slp = ValNodeNew(NULL);
 		slp->choice = SEQLOC_WHOLE;
@@ -2046,10 +2528,10 @@ NLM_EXTERN CharPtr DateToGB (CharPtr buf, NCBI_DatePtr ndp)
 
 		if (day < 10) {
 			sprintf(buf, "%ld%ld-%s-%ld", (long) tens_place, (long) day, 
-							(long) month_names[month-1], (long) year);
+							month_names[month-1], (long) year);
 		} else {
 			sprintf(buf, "%ld-%s-%ld", 
-				(long) day, (long) month_names[month-1], (long) year);
+				(long) day, month_names[month-1], (long) year);
 		}
 	} else {
 		if (ASN2FF_SHOW_ERROR_MSG == TRUE && ASN2FF_DATE_ERROR_MSG == TRUE) {
