@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 2/4/94
 *
-* $Revision: 6.3 $
+* $Revision: 6.14 $
 *
 * File Description:  Sequence editing utilities
 *
@@ -39,6 +39,39 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: edutil.c,v $
+* Revision 6.14  2001/06/01 18:07:20  kans
+* changes to SeqLocAdd to allow one plus and one unknown strand to be accepted
+*
+* Revision 6.13  2001/02/23 21:30:09  shkeda
+* Fixed SeqLocAdd: Int-fuzz pointers should be set to NULL after IntFuzzFree
+*
+* Revision 6.12  2001/02/23 01:26:07  ostell
+* Added support to BioseqDelete() for delta seqs
+*
+* Revision 6.11  2000/10/31 17:11:06  kans
+* SeqLocReplaceID was handling SEQLOC_PACKED_PNT incorrectly
+*
+* Revision 6.10  1999/12/20 20:47:12  kans
+* oldscope test was wrong everywhere
+*
+* Revision 6.9  1999/12/15 20:52:16  kans
+* added IndexedSeqFeatsCopy if SeqMgrFeaturesAreIndexed
+*
+* Revision 6.8  1999/12/07 20:32:13  kans
+* for most editing functions, if BioseqFind failed, temporarily clear scope/try again/reset scope
+*
+* Revision 6.7  1999/11/19 19:54:19  kans
+* SeqLocAdd checks for NULL slp before dereferencing
+*
+* Revision 6.6  1998/09/03 20:43:52  kans
+* added delta bioseq support to BioseqCopy
+*
+* Revision 6.5  1998/06/22 20:00:46  kans
+* DelFeat was a bit too agressive when there were multiple feature tables
+*
+* Revision 6.4  1998/06/17 21:50:11  kans
+* fixed unix compiler warnings, including 64-bit SGI
+*
 * Revision 6.3  1997/11/10 19:40:48  bazhin
 * Fixed incorrect comment for ISAGappedSeqLoc() function.
 *
@@ -125,6 +158,7 @@
 */
 
 #include <edutil.h>
+#include <explore.h>
 
 
 
@@ -198,6 +232,8 @@ NLM_EXTERN SeqLocPtr LIBCALL SeqLocAdd (SeqLocPtr PNTR head, SeqLocPtr slp, Bool
 	SeqLocPtr tmp, last, retval = NULL;
 	Boolean merged = FALSE;   /* intervals were merged */
 
+	if (slp == NULL) return NULL;
+
 	last = NULL;
 	if (* head != NULL)
 	{
@@ -226,17 +262,32 @@ NLM_EXTERN SeqLocPtr LIBCALL SeqLocAdd (SeqLocPtr PNTR head, SeqLocPtr slp, Bool
 		if ((last->choice == SEQLOC_INT) && (slp->choice == SEQLOC_INT))
 		{
 			SeqIntPtr sip1, sip2;
+			Boolean samestrand;
+			Uint1 strand = Seq_strand_unknown;
 
 			sip1 = (SeqIntPtr)(last->data.ptrvalue);
 			sip2 = (SeqIntPtr)(slp->data.ptrvalue);
-			if ((sip1->strand == sip2->strand) && (SeqIdForSameBioseq(sip1->id, sip2->id)))
+			samestrand = FALSE;
+			if ((sip1->strand == sip2->strand) ||
+				(sip1->strand == Seq_strand_unknown && sip2->strand != Seq_strand_minus) ||
+          		(sip1->strand == Seq_strand_unknown && sip2->strand != Seq_strand_minus)) {
+				samestrand = TRUE;
+				if (sip1->strand == Seq_strand_minus || sip1->strand == Seq_strand_minus) {
+					strand = Seq_strand_minus;
+				} else if (sip1->strand == Seq_strand_plus || sip1->strand == Seq_strand_plus) {
+					strand = Seq_strand_plus;
+				} else {
+					strand = Seq_strand_unknown;
+				}
+          	}
+			if (samestrand && (SeqIdForSameBioseq(sip1->id, sip2->id)))
 			{
-				if (sip1->strand == Seq_strand_minus)
+				if (strand == Seq_strand_minus)
 				{
 					if (sip1->from == (sip2->to + 1))  /* they are adjacent */
 					{
 						sip1->from = sip2->from;
-						IntFuzzFree(sip1->if_from);
+						sip1->if_from = IntFuzzFree(sip1->if_from);
 						if (sip2->if_from != NULL)   /* copy the fuzz */
 						{
 							if (do_copy)
@@ -247,6 +298,7 @@ NLM_EXTERN SeqLocPtr LIBCALL SeqLocAdd (SeqLocPtr PNTR head, SeqLocPtr slp, Bool
 								sip1->if_from = sip2->if_from;
 								sip2->if_from = NULL;
 							}
+							sip1->strand = strand;
 						}
 						merged = TRUE;
 					}
@@ -256,7 +308,7 @@ NLM_EXTERN SeqLocPtr LIBCALL SeqLocAdd (SeqLocPtr PNTR head, SeqLocPtr slp, Bool
 					if (sip1->to == (sip2->from - 1))  /* they are adjacent */
 					{
 						sip1->to = sip2->to;
-						IntFuzzFree(sip1->if_to);
+						sip1->if_to = IntFuzzFree(sip1->if_to);
 						if (sip2->if_to != NULL)   /* copy the fuzz */
 						{
 							if (do_copy)
@@ -267,6 +319,7 @@ NLM_EXTERN SeqLocPtr LIBCALL SeqLocAdd (SeqLocPtr PNTR head, SeqLocPtr slp, Bool
 								sip1->if_to = sip2->if_to;
 								sip2->if_to = NULL;
 							}
+							sip1->strand = strand;
 						}
 						merged = TRUE;
 					}
@@ -280,7 +333,7 @@ NLM_EXTERN SeqLocPtr LIBCALL SeqLocAdd (SeqLocPtr PNTR head, SeqLocPtr slp, Bool
 			sip2 = (SeqPntPtr)(slp->data.ptrvalue);
 			if ((sip1->strand == sip2->strand) && sip1->point == sip2->point && (SeqIdForSameBioseq(sip1->id, sip2->id)))
 			{
-				IntFuzzFree(sip1->fuzz);
+				sip1->fuzz = IntFuzzFree(sip1->fuzz);
 				if (sip2->fuzz != NULL)   /* copy the fuzz */
 				{
 					if (do_copy)
@@ -416,17 +469,18 @@ static CharPtr seqlitdbtag = "SeqLit";
 NLM_EXTERN Int4 LIBCALL ISADeltaSeqsToSeqLoc (SeqLocPtr slp)
 {
 	SeqIdPtr sip;
+	Int4 retval = 0;
 
-	if (slp == NULL) return 0;
+	if (slp == NULL) return retval;
 	sip = SeqLocId(slp);
-	if (sip == NULL) return 0;
+	if (sip == NULL) return retval;
 
-	if (sip->choice != SEQID_GENERAL) return 0;
+	if (sip->choice != SEQID_GENERAL) return retval;
 
 	if (! StringCmp(seqlitdbtag, ((DbtagPtr)(sip->data.ptrvalue))->db))
-		return ((DbtagPtr)(sip->data.ptrvalue))->tag->id;
+		retval = (((DbtagPtr)(sip->data.ptrvalue))->tag->id);
 
-	return 0;
+	return retval;
 }
 
 /*****************************************************************************
@@ -480,7 +534,7 @@ NLM_EXTERN SeqLocPtr LIBCALL DeltaSeqsToSeqLocs (DeltaSeqPtr dsp)
 }
 
 /*****************************************************************************
-*
+* GOHERE
 *   SeqLocsToDeltaSeqs(dsp, slp)
 *   	converts a chain of seqlocs	generated by DeltaSeqToSeqLocs() back into
 *         delta seqs. dsp is the original chain of DeltaSeqs, which is required
@@ -498,21 +552,29 @@ NLM_EXTERN DeltaSeqPtr LIBCALL SeqLocsToDeltaSeqs (DeltaSeqPtr dsp, SeqLocPtr sl
 	SeqPortPtr spps;
 	ByteStorePtr bsp;
 	Int2 residue;
+	ValNode vn;
 
 	if ((dsp == NULL) || (slp == NULL))
 		return dhead;
 
+	vn.choice = SEQLOC_MIX;
+	vn.next = NULL;
+	vn.data.ptrvalue = slp;
 	scurr = NULL;
-	while ((scurr = SeqLocFindNext(slp, scurr)) != NULL)
+	while ((scurr = SeqLocFindNext(&vn, scurr)) != NULL)
 	{
 		dcurr = ValNodeNew(dhead);
 		if (dhead == NULL)
 			dhead = dcurr;
 
-		if ((index = ISADeltaSeqsToSeqLoc(scurr)) != 0)   /* just a SeqLoc */
+		index = ISADeltaSeqsToSeqLoc(scurr);
+
+		if (index == 0)   /* just a SeqLoc */
 		{
 			dcurr->choice = 1;
+			dcurr->data.ptrvalue = NULL;
 			dcurr->data.ptrvalue = AsnIoMemCopy((Pointer)scurr, (AsnReadFunc)SeqLocAsnRead, (AsnWriteFunc)SeqLocAsnWrite);
+
 		}
 		else                                 /* convert to a delta seq */
 		{
@@ -615,8 +677,17 @@ NLM_EXTERN Boolean LIBCALL BioseqDelete (SeqIdPtr target, Int4 from, Int4 to, Bo
 	SeqLocPtr PNTR newheadptr;
 	SeqFeatPtr sfpcurr, sfpnext, sfpprev;
 	Int2 dropped;
+	SeqEntryPtr oldscope;
+	DeltaSeqPtr tdsp;
 
 	bsp = BioseqFind(target);
+	if (bsp == NULL) {
+		oldscope = SeqEntrySetScope (NULL);
+		if (oldscope != NULL) {
+			bsp = BioseqFind(target);
+			SeqEntrySetScope (oldscope);
+		}
+	}
 	if (bsp == NULL) return retval;
 
 	if ((from < 0) || (from >= bsp->length) || (to < 0) ||
@@ -650,11 +721,17 @@ NLM_EXTERN Boolean LIBCALL BioseqDelete (SeqIdPtr target, Int4 from, Int4 to, Bo
 	}
 
 			   /* update segmented sequence */
-	if (bsp->repr == Seq_repr_seg)
+	if ((bsp->repr == Seq_repr_seg) || (bsp->repr == Seq_repr_delta))
 	{
 		head = ValNodeNew(NULL);  /* allocate to facilitate SeqLocFree */
 		head->choice = SEQLOC_MIX;   /* make a SeqLoc out of the extension */
-		head->data.ptrvalue = bsp->seq_ext;
+		if (bsp->repr == Seq_repr_seg)
+			head->data.ptrvalue = bsp->seq_ext;
+		else
+		{
+			tdsp = (DeltaSeqPtr)(bsp->seq_ext);
+			head->data.ptrvalue = DeltaSeqsToSeqLocs(tdsp);
+		}
 		
 		newhead = NULL;
 		newheadptr = &newhead;
@@ -738,7 +815,14 @@ NLM_EXTERN Boolean LIBCALL BioseqDelete (SeqIdPtr target, Int4 from, Int4 to, Bo
 			prev = tmp;
 		}
 
-		bsp->seq_ext = newhead;
+		if (bsp->repr == Seq_repr_seg)
+			bsp->seq_ext = newhead;
+		else
+		{
+			bsp->seq_ext = SeqLocsToDeltaSeqs(tdsp, newhead);
+			DeltaSeqSetFree(tdsp);
+			SeqLocSetFree(newhead);
+		}
 		SeqLocFree(head);
 		retval = TRUE;
 	}
@@ -749,7 +833,7 @@ NLM_EXTERN Boolean LIBCALL BioseqDelete (SeqIdPtr target, Int4 from, Int4 to, Bo
 		sfpnext = NULL;
 		sfpcurr = (SeqFeatPtr)(bsp->seq_ext);
 		bsp->seq_ext = NULL;
-		for (sfpcurr; sfpcurr != NULL; sfpcurr = sfpnext)
+		for (; sfpcurr != NULL; sfpcurr = sfpnext)
 		{
 			sfpnext = sfpcurr->next;
 			dropped = SeqFeatDelete(sfpcurr, target, from, to, TRUE);
@@ -792,9 +876,17 @@ NLM_EXTERN Boolean LIBCALL BioseqOverwrite (SeqIdPtr target, Int4 pos, Uint1 res
 {
 	BioseqPtr bsp;
 	Boolean retval = FALSE;
+	SeqEntryPtr oldscope;
 
 
 	bsp = BioseqFind(target);
+	if (bsp == NULL) {
+		oldscope = SeqEntrySetScope (NULL);
+		if (oldscope != NULL) {
+			bsp = BioseqFind(target);
+			SeqEntrySetScope (oldscope);
+		}
+	}
 	if (bsp == NULL) return retval;
 
 	if ((pos < 0) || (pos >= bsp->length)) return retval;
@@ -839,7 +931,7 @@ NLM_EXTERN Boolean LIBCALL SeqDeleteByLoc (SeqLocPtr slp, Boolean do_feat, Boole
 {
 	SeqLocPtr tmp;
 	Boolean retval = FALSE;
-	Int2 numloc, i, ctr, pick, totloc;
+	Int2 numloc, i = 0, ctr, pick, totloc;
 	SeqLocPtr PNTR locs, PNTR tlocs, PNTR theorder;
 	SeqIdPtr the_id = NULL;
 	BioseqPtr bsp;
@@ -1435,56 +1527,57 @@ NLM_EXTERN void DelFeat (SeqEntryPtr sep, Pointer data, Int4 index, Int2 indent)
 	DelStructPtr dsp;
 	BioseqPtr bsp;
 	BioseqSetPtr bssp;
-	SeqAnnotPtr sap, prevsap, PNTR sapp, sapnext;
-	SeqFeatPtr sfp, prev, next;
+	SeqAnnotPtr sap, nextsap;
+	SeqFeatPtr sfp, nextsfp;
+	Pointer PNTR prevsap, PNTR prevsfp;
 
 	dsp = (DelStructPtr)data;
 	if (IS_Bioseq(sep))
 	{
 		bsp = (BioseqPtr)(sep->data.ptrvalue);
 		sap = bsp->annot;
-		sapp = &(bsp->annot);
+		prevsap = (Pointer PNTR) &(bsp->annot);
 	}
 	else
 	{
 		bssp = (BioseqSetPtr)(sep->data.ptrvalue);
 		sap = bssp->annot;
-		sapp = &(bssp->annot);
+		prevsap = (Pointer PNTR) &(bssp->annot);
 	}
 
-	prevsap = NULL;
 	while (sap != NULL)
 	{
-		sapnext = sap->next;
+		nextsap = sap->next;
 		if (sap->type == 1)   /* feature table */
 		{
-			prev = NULL;
-			for (sfp = (SeqFeatPtr)(sap->data); sfp != NULL; sfp = next)
+			sfp = (SeqFeatPtr) sap->data;
+			prevsfp = (Pointer PNTR) &(sap->data);
+			while (sfp != NULL)
 			{
-				next = sfp->next;
+				nextsfp = sfp->next;
 				if (SeqFeatDelete(sfp, dsp->sip, dsp->from, dsp->to, dsp->merge) == 2)
 				{
-				   SeqFeatFree(sfp);        /* location completely gone */
-				   if (prev != NULL)
-					prev->next = next;
-				   else
-					sap->data = (Pointer)next;
+					/* location completely gone */
+					*(prevsfp) = sfp->next;
+					sfp->next = NULL;
+					SeqFeatFree(sfp);
+				} else {
+					prevsfp = (Pointer PNTR) &(sfp->next);
 				}
-				else
-					prev = sfp;
+				sfp = nextsfp;
 			}
 		}
 
 		if (sap->data == NULL)  /* all features deleted */
 		{
-			if (prevsap != NULL)
-				prevsap->next = sapnext;
-			else
-				*sapp = sapnext;
-			SeqAnnotFree(sap);
+			*(prevsap) = sap->next;
+			sap->next = NULL;
+			SeqAnnotFree (sap);
+		} else {
+			prevsap = (Pointer PNTR) &(sap->next);
 		}
 
-		sap = sapnext;
+		sap = nextsap;
 	}
 
 	return;
@@ -1565,11 +1658,20 @@ NLM_EXTERN BioseqPtr LIBCALL BioseqCopy (SeqIdPtr newid, SeqIdPtr sourceid, Int4
 	SeqLocPtr the_segs, head, curr;
 	Boolean handled = FALSE, split;
 	SeqFeatPtr sfp, newsfp, lastsfp;
+	DeltaSeqPtr dsp;
+	SeqEntryPtr oldscope;
 
 
 	if ((sourceid == NULL) || (from < 0)) return FALSE;
 
 	oldbsp = BioseqFind(sourceid);
+	if (oldbsp == NULL) {
+		oldscope = SeqEntrySetScope (NULL);
+		if (oldscope != NULL) {
+			oldbsp = BioseqFind(sourceid);
+			SeqEntrySetScope (oldscope);
+		}
+	}
 	if (oldbsp == NULL) return NULL;
 
 	len = to - from + 1;
@@ -1586,6 +1688,13 @@ NLM_EXTERN BioseqPtr LIBCALL BioseqCopy (SeqIdPtr newid, SeqIdPtr sourceid, Int4
 		tmp->data.ptrvalue = (Pointer)oid;
 		oid->str = StringSave("Clipboard");
 		tmpbsp = BioseqFind(tmp);   /* old clipboard present? */
+		if (tmpbsp == NULL) {
+			oldscope = SeqEntrySetScope (NULL);
+			if (oldscope != NULL) {
+				tmpbsp = BioseqFind(tmp);
+				SeqEntrySetScope (oldscope);
+			}
+		}
 		if (tmpbsp != NULL)
 			BioseqFree(tmpbsp);
 		newbsp->id = tmp;
@@ -1630,7 +1739,8 @@ NLM_EXTERN BioseqPtr LIBCALL BioseqCopy (SeqIdPtr newid, SeqIdPtr sourceid, Int4
 	}
 
 	if ((newbsp->repr == Seq_repr_seg) ||
-		(newbsp->repr == Seq_repr_ref))
+		(newbsp->repr == Seq_repr_ref) ||
+		(newbsp->repr == Seq_repr_delta))
 	{
         if (newbsp->repr == Seq_repr_seg)  /* segmented */
 		{
@@ -1640,9 +1750,18 @@ NLM_EXTERN BioseqPtr LIBCALL BioseqCopy (SeqIdPtr newid, SeqIdPtr sourceid, Int4
 			the_segs = (SeqLocPtr)&fake;
 			head = SeqLocCopyPart (the_segs, from, to, strand, FALSE, NULL, NULL);
 		}
-		else                      /* reference: is a Seq-loc */
+		else if (newbsp->repr == Seq_repr_ref)  /* reference: is a Seq-loc */
+		{
 			head = SeqLocCopyPart ((SeqLocPtr)(oldbsp->seq_ext), from, to,
 			                         strand, TRUE, NULL, NULL);
+		}
+		else if (newbsp->repr == Seq_repr_delta)
+		{
+			dsp = (DeltaSeqPtr)(oldbsp->seq_ext);  /* real data is here */
+			the_segs = DeltaSeqsToSeqLocs(dsp);
+			head = SeqLocCopyPart (the_segs, from, to, strand, FALSE, NULL, NULL);
+			SeqLocFree (the_segs);
+		}
 
         newbsp->seq_ext = (Pointer)head;
 		handled = TRUE;
@@ -1864,10 +1983,10 @@ NLM_EXTERN SeqLocPtr LIBCALL SeqLocCopyPart (SeqLocPtr the_segs, Int4 from, Int4
 *   SeqFeatCopy(new, old, from, to, strand)
 *
 *****************************************************************************/
-NLM_EXTERN Int2 LIBCALL SeqFeatsCopy (BioseqPtr newbsp, BioseqPtr oldbsp, Int4 from, Int4 to, Uint1 strand)
+static Int2 LIBCALL IndexedSeqFeatsCopy (BioseqPtr newbsp, BioseqPtr oldbsp, Int4 from, Int4 to, Uint1 strand)
+
 {
 	Int2 ctr=0;
-	BioseqContextPtr bcp;
 	SeqFeatPtr head=NULL, sfp, last=NULL, newsfp;
 	SeqInt si;
 	ValNode vn;
@@ -1879,6 +1998,119 @@ NLM_EXTERN Int2 LIBCALL SeqFeatsCopy (BioseqPtr newbsp, BioseqPtr oldbsp, Int4 f
 	CodeBreakPtr cbp, prevcbp, nextcbp;
 	RnaRefPtr rrp;
 	tRNAPtr trp;
+	SeqMgrFeatContext fcontext;
+
+	region = &vn;
+	vn.choice = SEQLOC_INT;
+	vn.data.ptrvalue = (Pointer)(&si);
+	si.from = from;
+	si.to = to;
+	si.id = oldbsp->id;
+	si.if_from = NULL;
+	si.if_to = NULL;
+
+	sfp = NULL;
+	while ((sfp = SeqMgrGetNextFeature (oldbsp, sfp, 0, 0, &fcontext)) != NULL)
+	{
+		/* can exit once past rightmost limit */
+		if (fcontext.left > to) return ctr;
+
+		if (fcontext.right >= from && fcontext.left <= to) {
+
+			split = FALSE;
+			newloc = SeqLocCopyRegion(newbsp->id, sfp->location, oldbsp, from, to, strand, &split);
+			if (newloc != NULL)   /* got one */
+			{
+				newsfp = (SeqFeatPtr)AsnIoMemCopy((Pointer)sfp, (AsnReadFunc)SeqFeatAsnRead, (AsnWriteFunc)SeqFeatAsnWrite);
+				SeqLocFree(newsfp->location);
+				newsfp->location = newloc;
+				if (split)
+					newsfp->partial = TRUE;
+				if (last == NULL)  /* first one */
+				{
+					sap = SeqAnnotNew();
+					if (newbsp->annot == NULL)
+						newbsp->annot = sap;
+					else
+					{
+						for (saptmp = newbsp->annot; saptmp->next != NULL; saptmp = saptmp->next)
+							continue;
+						saptmp->next = sap;
+					}
+					sap->type = 1;   /* feature table */
+					sap->data = (Pointer)newsfp;
+				}
+				else
+					last->next = newsfp;
+				last = newsfp;
+
+				switch (newsfp->data.choice)
+				{
+					case SEQFEAT_CDREGION:   /* cdregion */
+						crp = (CdRegionPtr)(newsfp->data.value.ptrvalue);
+						prevcbp = NULL;
+						for (cbp = crp->code_break; cbp != NULL; cbp = nextcbp)
+						{
+							nextcbp = cbp->next;
+							cbp->loc = SeqLocCopyRegion(newbsp->id, cbp->loc, oldbsp, from, to, strand, &split);
+							if (cbp->loc == NULL)
+							{
+								if (prevcbp != NULL)
+									prevcbp->next = nextcbp;
+								else
+									crp->code_break = nextcbp;
+								cbp->next = NULL;
+								CodeBreakFree(cbp);
+							}
+							else
+								prevcbp = cbp;
+						}
+						break;
+					case SEQFEAT_RNA:
+						rrp = (RnaRefPtr)(newsfp->data.value.ptrvalue);
+						if (rrp->ext.choice == 2)   /* tRNA */
+						{
+							trp = (tRNAPtr)(rrp->ext.value.ptrvalue);
+							if (trp->anticodon != NULL)
+							{
+								trp->anticodon = SeqLocCopyRegion(newbsp->id, trp->anticodon, oldbsp, from, to, strand, &split);
+							}
+						}
+						break;
+					default:
+						break;
+				}
+			}
+		}
+		
+	}
+	return ctr;
+}
+
+NLM_EXTERN Int2 LIBCALL SeqFeatsCopy (BioseqPtr newbsp, BioseqPtr oldbsp, Int4 from, Int4 to, Uint1 strand)
+{
+	Int2 ctr=0;
+	BioseqContextPtr bcp = NULL;
+	SeqFeatPtr head=NULL, sfp, last=NULL, newsfp;
+	SeqInt si;
+	ValNode vn;
+	ValNodePtr region;
+	SeqLocPtr newloc;
+	Boolean split = FALSE;
+	SeqAnnotPtr sap = NULL, saptmp;
+	CdRegionPtr crp;
+	CodeBreakPtr cbp, prevcbp, nextcbp;
+	RnaRefPtr rrp;
+	tRNAPtr trp;
+	Uint2 entityID;
+
+	if (oldbsp == NULL) return ctr;
+
+	entityID = ObjMgrGetEntityIDForPointer (oldbsp);
+	if (entityID > 0 && SeqMgrFeaturesAreIndexed (entityID)) {
+		/* indexed version should be much faster */
+		return IndexedSeqFeatsCopy (newbsp, oldbsp, from, to, strand);
+	}
 
 	bcp = BioseqContextNew(oldbsp);
 	if (bcp == NULL) return ctr;
@@ -1961,6 +2193,7 @@ NLM_EXTERN Int2 LIBCALL SeqFeatsCopy (BioseqPtr newbsp, BioseqPtr oldbsp, Int4 f
 		}
 		
 	}
+	BioseqContextFree (bcp);
 	return ctr;
 }
 
@@ -2474,11 +2707,19 @@ NLM_EXTERN Boolean LIBCALL BioseqInsert (SeqIdPtr from_id, Int4 from, Int4 to, U
 	CodeBreakPtr cbp, prevcbp, nextcbp;
 	RnaRefPtr rrp;
 	tRNAPtr trp;
+	SeqEntryPtr oldscope;
 
 
 	if ((from_id == NULL) || (to_id == NULL)) return FALSE;
 
 	tobsp = BioseqFind(to_id);
+	if (tobsp == NULL) {
+		oldscope = SeqEntrySetScope (NULL);
+		if (oldscope != NULL) {
+			tobsp = BioseqFind(to_id);
+			SeqEntrySetScope (oldscope);
+		}
+	}
 	if (tobsp == NULL) return FALSE;
 
 	len = BioseqGetLen(tobsp);
@@ -2491,6 +2732,13 @@ NLM_EXTERN Boolean LIBCALL BioseqInsert (SeqIdPtr from_id, Int4 from, Int4 to, U
 	if ((pos < 0) || (pos > len)) return FALSE;
 
 	frombsp = BioseqFind(from_id);
+	if (frombsp == NULL) {
+		oldscope = SeqEntrySetScope (NULL);
+		if (oldscope != NULL) {
+			frombsp = BioseqFind(from_id);
+			SeqEntrySetScope (oldscope);
+		}
+	}
 	if (frombsp == NULL) return FALSE;
 	
 	from_type = ISA_na(frombsp->mol);
@@ -3128,12 +3376,12 @@ NLM_EXTERN SeqLocPtr LIBCALL SeqLocSubtract (SeqLocPtr head, SeqLocPtr piece)
 NLM_EXTERN SeqLocPtr SeqLocReplaceID (SeqLocPtr slp, SeqIdPtr new_sip)
 {
   SeqLocPtr        curr;
+  PackSeqPntPtr    pspp;
   SeqIntPtr        target_sit;
   SeqPntPtr        spp;
 
   switch (slp->choice) {
      case SEQLOC_PACKED_INT :
-     case SEQLOC_PACKED_PNT :
      case SEQLOC_MIX :
      case SEQLOC_EQUIV :
         curr = NULL;
@@ -3141,6 +3389,13 @@ NLM_EXTERN SeqLocPtr SeqLocReplaceID (SeqLocPtr slp, SeqIdPtr new_sip)
            curr = SeqLocReplaceID (curr, new_sip);
         }
         break;
+     case SEQLOC_PACKED_PNT :
+        pspp = (PackSeqPntPtr) slp->data.ptrvalue;
+        if (pspp != NULL) {
+          SeqIdFree (pspp->id);
+          pspp->id = SeqIdDup (new_sip);
+        }
+        break;
      case SEQLOC_EMPTY :
      case SEQLOC_WHOLE :
         SeqIdFree ((SeqIdPtr) slp->data.ptrvalue);
