@@ -3,9 +3,24 @@
 *   -- all routines for checking genbank feature table
 *   -- all extern variables are in gbftglob.c
 *                                                                  10-11-93
-$Revision: 6.3 $
+$Revision: 6.8 $
 *
 * $Log: gbfeat.c,v $
+* Revision 6.8  2001/12/06 17:00:41  kans
+* TextSave takes size_t, not Int2, otherwise titin protein tries to allocate negative number
+*
+* Revision 6.7  2001/06/08 20:09:53  bazhin
+* From now on "absent" is a legal value for /cons_splice qualifier.
+*
+* Revision 6.6  2000/02/02 22:10:19  kans
+* use TextSave instead of TextSaveEx, which is not available
+*
+* Revision 6.5  2000/02/02 21:03:09  tatiana
+* CkNumberType() added
+*
+* Revision 6.4  1998/06/15 15:00:17  tatiana
+* UNIX compiler warnings fixed
+*
 * Revision 6.3  1998/04/30 21:44:05  tatiana
 * *** empty log message ***
 *
@@ -302,7 +317,7 @@ NLM_EXTERN int GBFeatKeyQualValid(ValNodePtr cit, Int2 keyindx, GBQualPtr PNTR g
 NLM_EXTERN int SplitMultiValQual(GBQualPtr PNTR gbqp, 
      Boolean error_msgs, Boolean perform_corrections)
 {
-   Int2        val, len;
+   Int2        val/*, len -- UNUSED */;
    GBQualPtr   next_q, curq, preq = NULL, first_q, tmp;
    int retval = GB_FEAT_ERR_NONE;
    CharPtr	bptr, ptr, buf;
@@ -311,7 +326,7 @@ NLM_EXTERN int SplitMultiValQual(GBQualPtr PNTR gbqp,
         next_q = curq -> next;  /* in case deleted */
 
        val = GBQualSplit(curq->qual);
-       len = StringLen(curq->qual);
+/*       len = StringLen(curq->qual); -- NO EFFECT */
 
        if (val == -1) {
        		preq = curq;
@@ -341,13 +356,13 @@ NLM_EXTERN int SplitMultiValQual(GBQualPtr PNTR gbqp,
 		   "Splited qualifier %s", curq->qual); 
 		buf = bptr;
 		bptr++;
-		curq->val = TextSave(bptr, (Int2)(ptr-bptr));
+		curq->val = TextSave(bptr, ptr-bptr);
 		bptr = ptr + 1;
 		curq->next = NULL;
 		while ((ptr = StringChr(bptr, ',')) != NULL) {
 			tmp = GBQualNew();
 			tmp->qual = StringSave(curq->qual);
-			tmp->val = TextSave(bptr, (Int2)(ptr-bptr));
+			tmp->val = TextSave(bptr, ptr-bptr);
 			curq = tie_qual(curq, tmp);
 			bptr = ptr + 1;
 		} 
@@ -511,6 +526,13 @@ NLM_EXTERN int GBQualSemanticValid(GBQualPtr PNTR gbqp,
                         retval = ret;
                      }
                      break;
+                case Class_number:
+                     ret = CkQualTokenType(gbqp, curq, preq, error_msgs, 
+                           perform_corrections, ParFlat_Number_type);
+                     if (ret > retval){
+                        retval = ret;
+                     }
+                     break;
                 case Class_rpt:
                      ret = CkQualMatchToken(gbqp, curq, preq, error_msgs, 
                                perform_corrections,
@@ -678,7 +700,7 @@ NLM_EXTERN int CkQualPosaa(GBQualPtr PNTR head_gbqp, GBQualPtr gbqp,
                    ++str;
 
                if ((eptr = StringChr(str, ')')) != NULL) {
-                  aa = TextSave(str, (Int2)(eptr-str));
+                  aa = TextSave(str, eptr-str);
                   
                   
                  retval = CkQualPosSeqaa(head_gbqp,  gbqp, preq,
@@ -793,7 +815,7 @@ NLM_EXTERN void ConvertEmbedQual(CharPtr value)
                                                    && *bptr != '\0'; bptr++)
                  continue;
 
-             qualname = TextSave(ptr, (Int2)(bptr-ptr));
+             qualname = TextSave(ptr, bptr-ptr);
 
              val = GBQualNameValid(qualname);
              if (val >= 0)
@@ -830,7 +852,7 @@ NLM_EXTERN CharPtr ScanEmbedQual(CharPtr value)
                                                    && *bptr != '\0'; bptr++)
                  continue;
 
-             qual = TextSave(ptr, (Int2)(bptr-ptr));
+             qual = TextSave(ptr, bptr-ptr);
 
              val = GBQualNameValid(qual);
 
@@ -860,7 +882,7 @@ NLM_EXTERN int CkQualText (GBQualPtr PNTR head_gbqp,  GBQualPtr gbqp,
    Boolean PNTR has_embedded, Boolean from_note, Boolean error_msgs, 
    Boolean perform_corrections)
 {
-   CharPtr  value=NULL, bptr, eptr, str, s;
+   CharPtr  value=NULL, bptr, eptr, str;
    int retval = GB_FEAT_ERR_NONE;
 
    if (has_embedded != NULL){
@@ -908,7 +930,7 @@ NLM_EXTERN int CkQualText (GBQualPtr PNTR head_gbqp,  GBQualPtr gbqp,
 /* ERROR  here  sets retval*/
    }
 
-      value = TextSave(bptr, (Int2)(eptr-bptr));
+      value = TextSave(bptr, eptr-bptr);
 /* Some check must be done for illegal characters in gbpq->val
       for (s = value; *s != '\0'; s++) {
       	if (!IS_WHITESP(*s) && !IS_ALPHANUM(*s) && *s != '\"') {
@@ -990,7 +1012,7 @@ NLM_EXTERN int CkQualSeqaa (GBQualPtr PNTR head_gbqp, GBQualPtr gbqp,
                 ++str;
 
             if ((eptr = StringChr(str, ')')) != NULL) {
-               aa = TextSave(str, (Int2)(eptr-str));
+               aa = TextSave(str, eptr-str);
                
                  retval = CkQualPosSeqaa(head_gbqp,  gbqp, preq,
                     error_msgs, perform_corrections,  aa, eptr);
@@ -1071,7 +1093,7 @@ NLM_EXTERN int CkQualMatchToken
        str++;
 
    if (*str == '\0') {
-      msg = TextSave(bptr, (Int2)(eptr-bptr));
+      msg = TextSave(bptr, eptr-bptr);
 
       if (MatchArrayStringIcase(array_string, totalstr, msg) == -1) {
         if (error_msgs){ 
@@ -1157,19 +1179,22 @@ NLM_EXTERN int CkQualSite ( GBQualPtr PNTR head_gbqp, GBQualPtr gbqp,
    int retval = GB_FEAT_ERR_NONE;
    CharPtr  bptr, str;
    Boolean ok=FALSE;
-   CharPtr yes_or_no = "not \'YES\' or \'NO\'";
+   CharPtr yes_or_no = "not \'YES\', \'NO\' or \'ABSENT\'";
 
    str = gbqp->val;
    if (StringNICmp(str, "(5'site:", 8) == 0) {
       bptr = str;
       str += 8;
 
-      if (StringNICmp(str, "YES", 3) == 0 || StringNICmp(str, "NO", 2) == 0) {
+      if (StringNICmp(str, "YES", 3) == 0 || StringNICmp(str, "NO", 2) == 0 ||
+          StringNICmp(str, "ABSENT", 6) == 0) {
 
          if (StringNICmp(str, "YES", 3) == 0)
             str += 3;
-         else
+         else if (StringNICmp(str, "NO", 2) == 0)
             str += 2;
+         else
+            str += 6;
 
          for (; *str == ' '; str++);
          for (; *str == ','; str++);
@@ -1179,12 +1204,15 @@ NLM_EXTERN int CkQualSite ( GBQualPtr PNTR head_gbqp, GBQualPtr gbqp,
          if (StringNICmp(str, "3'site:", 7) == 0) {
             str += 7;
 
-            if (StringNICmp(str, "YES", 3) == 0 
-                                        || StringNICmp(str, "NO", 2) == 0) {
+            if (StringNICmp(str, "YES", 3) == 0 ||
+                StringNICmp(str, "NO", 2) == 0 ||
+                StringNICmp(str, "ABSENT", 6) == 0) {
                if (StringNICmp(str, "YES", 3) == 0)
                   str += 3;
-               else
+               else if (StringNICmp(str, "NO", 2) == 0)
                   str += 2;
+               else
+                  str += 6;
 
                if (*str == ')') {
    
@@ -1232,8 +1260,8 @@ NLM_EXTERN int CkQualSite ( GBQualPtr PNTR head_gbqp, GBQualPtr gbqp,
 *  -- format   single token
 *  -- example  ParFlat_Stoken_type        /label=Albl_exonl  /mod_base=m5c
 *              ParFlat_BracketInt_type    /citation=[3] or /citation= ([1],[3])
-*              ParFlat_Integer_type       /number=4       /transl_table=4
-*
+*              ParFlat_Integer_type           /transl_table=4
+*				ParFlat_Number_type			/number=4b
 *  -- not implemented yet, treat as ParFlat_Stoken_type:
 *     -- feature_label or base_range              
 *                 /rpt_unit=Alu_rpt1   /rpt_unit=202..245
@@ -1276,7 +1304,7 @@ NLM_EXTERN int CkQualTokenType (GBQualPtr PNTR head_gbqp,  GBQualPtr gbqp,
 
    if (*str == '\0') {
 /*------single token found ----*/
-      token = TextSave(bptr, (Int2)(eptr-bptr));
+      token = TextSave(bptr, eptr-bptr);
 
       bptr = token;
 
@@ -1295,6 +1323,9 @@ NLM_EXTERN int CkQualTokenType (GBQualPtr PNTR head_gbqp,  GBQualPtr gbqp,
          case ParFlat_Stoken_type:
          		str = CkLabelType(token);
                break;
+         case ParFlat_Number_type:
+         		str = CkNumberType(token);
+               break;
         default:
               str = NULL;
               break;
@@ -1436,6 +1467,23 @@ NLM_EXTERN CharPtr CkBracketType(CharPtr str)
 	}
 }
 
+/*------------------------ CkNumberType() --------------------*/
+/*****************************************************************************
+*   CkNumberType:
+*	checks /number=single_token  - numbers and letters
+*	/number=4 or /number=6b
+*                                                          -Tatiana 2/1/00
+******************************************************************************/
+NLM_EXTERN CharPtr CkNumberType(CharPtr str)
+{
+		for (;  *str != '\0' && !IS_ALPHANUM(*str); str++)
+			continue;
+		if (*str != '\0') {
+			return NULL;  
+		}
+		return str;
+} 
+
 /*------------------------ CkLabelType() --------------------*/
 /*****************************************************************************
 *   CkLabelType:
