@@ -165,7 +165,7 @@ NLM_EXTERN SeqIdPtr find_sip(SeqIdPtr sip)
   Uint1  order [NUM_ORDER];
 
   SeqIdBestRank (order, NUM_ORDER);
-  order [SEQID_LOCAL] = 1;
+  order [SEQID_LOCAL] = 2;
   order [SEQID_GENBANK] = 5;
   order [SEQID_EMBL] = 5;
   order [SEQID_PIR] = 5;
@@ -174,7 +174,7 @@ NLM_EXTERN SeqIdPtr find_sip(SeqIdPtr sip)
   order [SEQID_PRF] = 5;
   order [SEQID_PDB] = 5;
   order [SEQID_PATENT] = 10;
-  order [SEQID_OTHER] = 10;
+  order [SEQID_OTHER] = 1;
   order [SEQID_GENERAL] = 10;
   order [SEQID_GIBBSQ] = 15;
   order [SEQID_GIBBMT] = 15;
@@ -516,7 +516,7 @@ static Boolean get_cit_edit(CharPtr fmark, Int2 order, ValNodePtr head)
 	       vnp = head;
 	       while(vnp){
 		 if(vnp->choice == 4){
-		   sprintf(fmark, "muid %ld", (Int4)(vnp->data.intvalue));
+		   sprintf(fmark, "muid %ld", (long)(vnp->data.intvalue));
 		   return TRUE;
 		  }
 		  vnp = vnp->next;
@@ -550,7 +550,7 @@ static Boolean get_cit_edit(CharPtr fmark, Int2 order, ValNodePtr head)
 			   if(alp->names){
 			      tmp= alp->names;
 			      if(tmp->choice ==3){
-			      sprintf(fmark, "%s", tmp->data.ptrvalue);
+			      sprintf(fmark, "%s", (CharPtr) tmp->data.ptrvalue);
 			      return TRUE;
 			      }
 			   }
@@ -669,7 +669,7 @@ NLM_EXTERN Uint1 label_feature(SeqFeatPtr sfp, CharPtr fmark, Int2 order, Boolea
 	     vnp = prp->name;
 	     dbase = prp->db;
 	     if(vnp && dbase){
-	       sprintf(temp, "%s %s", dbase->data.ptrvalue, vnp->data.ptrvalue);
+	       sprintf(temp, "%s %s", (CharPtr) dbase->data.ptrvalue, (CharPtr) vnp->data.ptrvalue);
 	       myStringNCpy(fmark, temp, 20);
 	       add_name_partial(fmark, trunc5, trunc3);
 	       return TRUE;
@@ -725,6 +725,11 @@ NLM_EXTERN Uint1 label_feature(SeqFeatPtr sfp, CharPtr fmark, Int2 order, Boolea
 	       add_name_partial(fmark, trunc5, trunc3);
 	       return TRUE;
 	     }
+	     if(rrp->type == 7){
+	       StringCpy(fmark, "snoRNA");
+	       add_name_partial(fmark, trunc5, trunc3);
+	       return TRUE;
+	     }
 	     if(rrp->type == 255){
 	       StringCpy(fmark, "Other");
 	       add_name_partial(fmark, trunc5, trunc3);
@@ -1330,42 +1335,45 @@ NLM_EXTERN SeqAlignPtr get_sep_align(SeqEntryPtr sep)
 ****/
 NLM_EXTERN SeqIdPtr get_align_id(SeqAlignPtr sap_p, Int2 order)
 {
-  DenseSegPtr dsp;
-  StdSegPtr   ssp;
-  DenseDiagPtr ddp;
-
-  Int2 i;
-  SeqIdPtr sip;
-  SeqLocPtr loc;
-
-	i =0;
-	switch(sap_p->segtype)
-	{
-		case 1:		/*Dense Diag*/
-			ddp = sap_p->segs;
-			sip = ddp->id;
-			for(; sip!=NULL; sip = sip->next, ++i)
-				if(order == i)
-					return sip;
-			return NULL;
-		case 2:		/*Dense Seg*/
-			dsp = sap_p->segs;
-			sip = dsp->ids;
-			for(; sip!=NULL; sip = sip->next, ++i)
-				if(order == i)
-					return sip;
-			return NULL;
-		case 3:
-			ssp = sap_p->segs;
-			loc = ssp->loc;
-			for(; loc != NULL; loc = loc->next, ++i)
-				if(order == i)
-					return SeqLocId(loc);
-			return NULL;
-		default:
-			return NULL;
-	}
+    DenseSegPtr  dsp;
+    StdSegPtr    ssp;
+    DenseDiagPtr ddp;
+    SeqAlignPtr  sap;
+    Int2         i;
+    SeqIdPtr     sip;
+    SeqLocPtr    loc;
+    
+    i =0;
+    switch(sap_p->segtype) {
+    case 1:		/*Dense Diag*/
+        ddp = sap_p->segs;
+        sip = ddp->id;
+        for(; sip!=NULL; sip = sip->next, ++i)
+            if(order == i)
+                return sip;
+        return NULL;
+    case 2:		/*Dense Seg*/
+        dsp = sap_p->segs;
+        sip = dsp->ids;
+        for(; sip!=NULL; sip = sip->next, ++i)
+            if(order == i)
+                return sip;
+        return NULL;
+    case 3:
+        ssp = sap_p->segs;
+        loc = ssp->loc;
+        for(; loc != NULL; loc = loc->next, ++i)
+            if(order == i)
+                return SeqLocId(loc);
+        return NULL;
+    case 5: /* Discontinuous alignment */
+        sap = (SeqAlignPtr) sap_p->segs;
+        sip = get_align_id(sap, order);
+        return  sip;
 
+    default:
+        return NULL;
+    }
 }
 			
 
@@ -1377,37 +1385,41 @@ NLM_EXTERN SeqIdPtr get_align_id(SeqAlignPtr sap_p, Int2 order)
 ***/
 NLM_EXTERN Uint1 get_align_strand(SeqAlignPtr sap_p, Int2 order)
 {
-  DenseSegPtr dsp;
-  StdSegPtr   ssp;
-  DenseDiagPtr ddp;
-  SeqLocPtr loc;
-  Int2 i;
+    DenseSegPtr dsp;
+    StdSegPtr   ssp;
+    DenseDiagPtr ddp;
+    SeqLocPtr loc;
+    Int2 i;
+    SeqAlignPtr sap;
 
-	switch(sap_p->segtype)
-	{
-		case 1:		/*Dense Diag*/
-			ddp = sap_p->segs;
-			order = order%(ddp->dim);
-			if(ddp->strands)
-				return ddp->strands[order];
-			return 0;
-		case 2:		/*Dense Seg*/
-			dsp = sap_p->segs;
-			order = order%(dsp->dim);
-			if(dsp->strands)
-				return dsp->strands[order];
-			return 0;
-		case 3:
-			ssp = sap_p->segs;
-			loc = ssp->loc;
-			for(i = 0; loc != NULL; loc = loc->next, ++i)
-				if(order == i)
-					return SeqLocStrand(loc);
-			return 0;
-		default:
-			return 0;
-	}
+    switch(sap_p->segtype) {
+    case 1:		/*Dense Diag*/
+        ddp = sap_p->segs;
+        order = order%(ddp->dim);
+        if(ddp->strands)
+            return ddp->strands[order];
+        return 0;
+    case 2:		/*Dense Seg*/
+        dsp = sap_p->segs;
+        order = order%(dsp->dim);
+        if(dsp->strands)
+            return dsp->strands[order];
+        return 0;
+    case 3:
+        ssp = sap_p->segs;
+        loc = ssp->loc;
+        for(i = 0; loc != NULL; loc = loc->next, ++i)
+            if(order == i)
+                return SeqLocStrand(loc);
+        return 0;
+    case 5: /* Discontinuous alignment */
+        
+        sap = (SeqAlignPtr) sap_p->segs;
+        return get_align_strand(sap, order);
 
+    default:
+        return 0;
+    }
 }
 			
 /************************************************************************
@@ -1418,42 +1430,46 @@ NLM_EXTERN Uint1 get_align_strand(SeqAlignPtr sap_p, Int2 order)
 ****/
 NLM_EXTERN Int2 m_id_match(SeqAlignPtr sap_p, SeqIdPtr match_id)
 {
-  DenseSegPtr dsp;
-  StdSegPtr   ssp;
-  DenseDiagPtr ddp;
+    DenseSegPtr dsp;
+    StdSegPtr   ssp;
+    DenseDiagPtr ddp;
+    
+    Int2 i;
+    SeqIdPtr sip;
+    SeqLocPtr loc;
+    SeqAlignPtr sap;
 
-  Int2 i;
-  SeqIdPtr sip;
-  SeqLocPtr loc;
-
-	i =0;
-	switch(sap_p->segtype)
-	{
-		case 1:		/*Dense Diag*/
-			ddp = sap_p->segs;
-			sip = ddp->id;
-			for(; sip!=NULL; sip = sip->next, ++i)
-				if(SeqIdForSameBioseq(sip, match_id))
-					return i;
-			return -1;
-		case 2:		/*Dense Seg*/
-			dsp = sap_p->segs;
-			sip = dsp->ids;
-			for(; sip!=NULL; sip = sip->next, ++i)
-				if(SeqIdForSameBioseq(sip, match_id))
-					return i;
-			return -1;
-		case 3:
-			ssp = sap_p->segs;
-			loc = ssp->loc;
-			for(; loc != NULL; loc = loc->next, ++i)
-				if(SeqIdForSameBioseq(SeqLocId(loc), match_id))
-					return i;
-			return -1;
-		default:
-			return -1;
-	}
+    i =0;
+    switch(sap_p->segtype) {
+    case 1:		/*Dense Diag*/
+        ddp = sap_p->segs;
+        sip = ddp->id;
+        for(; sip!=NULL; sip = sip->next, ++i)
+            if(SeqIdForSameBioseq(sip, match_id))
+                return i;
+        return -1;
+    case 2:		/*Dense Seg*/
+        dsp = sap_p->segs;
+        sip = dsp->ids;
+        for(; sip!=NULL; sip = sip->next, ++i)
+            if(SeqIdForSameBioseq(sip, match_id))
+                return i;
+        return -1;
+    case 3:
+        ssp = sap_p->segs;
+        loc = ssp->loc;
+        for(; loc != NULL; loc = loc->next, ++i)
+            if(SeqIdForSameBioseq(SeqLocId(loc), match_id))
+                return i;
+        return -1;
+    case 5: /* Discontinuous alignment */
 
+        sap = (SeqAlignPtr) sap_p->segs;
+        return m_id_match(sap, match_id);
+        
+    default:
+        return -1;
+    }   
 }
 
 
@@ -1717,7 +1733,7 @@ NLM_EXTERN Boolean get_gene_syn(SeqFeatPtr sfp, CharPtr name, BoolPtr has_name,
 		{
 			if(has_prev)
 			{
-				sprintf(tmp, " %s", vnp->data.ptrvalue);
+				sprintf(tmp, " %s", (CharPtr) vnp->data.ptrvalue);
 				StringCat(syn, tmp);
 			}
 			else
@@ -2083,7 +2099,7 @@ NLM_EXTERN Int2 get_vnp_num(ValNodePtr vnp)
 ###################################################################*/
 typedef struct seqid_count{
 	SeqIdPtr sip;
-	Uint2 count;
+	Uint4 count;
 }SeqIdCount, PNTR SeqIdCountPtr;
 
 static ValNodePtr find_repeat(ValNodePtr head, SeqIdPtr sip)
@@ -2110,7 +2126,7 @@ static ValNodePtr find_repeat(ValNodePtr head, SeqIdPtr sip)
 
 static SeqIdPtr find_max(ValNodePtr vnp)
 {
-	Uint2 max;
+	Uint4 max;
 	SeqIdPtr sip= NULL;
 	SeqIdCountPtr sicp;
 
@@ -2143,51 +2159,54 @@ static SeqIdPtr find_max(ValNodePtr vnp)
 *******************************************************************/
 NLM_EXTERN SeqIdPtr make_master( SeqAlignPtr sap_p)
 {
-	ValNodePtr vnp= NULL;   /*record the frequency of Seq-ids in alignment**/
-	SeqIdPtr sip;
-	DenseSegPtr dsp;
-	DenseDiagPtr ddp;
-	StdSegPtr ssp;
-	SeqLocPtr slp;
-	
+    ValNodePtr vnp= NULL;   /*record the frequency of Seq-ids in alignment**/
+    SeqIdPtr sip;
+    DenseSegPtr dsp;
+    DenseDiagPtr ddp;
+    StdSegPtr ssp;
+    SeqLocPtr slp;
+    SeqAlignPtr sap;
     
-	while(sap_p !=NULL)
-	{
-	   switch(sap_p->segtype)
-	   {
-		case 1:
-			ddp = (DenseDiagPtr)(sap_p->segs);
-			while(ddp)
-			{
-				for(sip = ddp->id; sip!=NULL; sip = sip->next)
-					vnp = find_repeat(vnp, sip);
-				ddp = ddp->next;
-			}
-			break;
-
-		case 2:
-			dsp = (DenseSegPtr)(sap_p->segs);
-			for(sip = dsp->ids; sip!=NULL; sip = sip->next)
-				vnp = find_repeat(vnp, sip);
-			break;
-		
-		case 3:
-			ssp = (StdSegPtr)(sap_p->segs);
-			for(slp = ssp->loc; slp != NULL; slp = slp->next)
-			{
-				sip = SeqLocId(slp);
-				vnp = find_repeat(vnp, sip);
-			}
-			break;
-		default:
-			break;
-	   }
-	   sap_p = sap_p->next;
-	}
-
-	sip = find_max(vnp);
-	ValNodeFreeData(vnp);
-	return sip;
+    while(sap_p !=NULL) {
+        switch(sap_p->segtype) {
+        case 1:
+            ddp = (DenseDiagPtr)(sap_p->segs);
+            while(ddp) {
+                for(sip = ddp->id; sip!=NULL; sip = sip->next)
+                    vnp = find_repeat(vnp, sip);
+                ddp = ddp->next;
+            }
+            break;
+            
+        case 2:
+            dsp = (DenseSegPtr)(sap_p->segs);
+            for(sip = dsp->ids; sip!=NULL; sip = sip->next)
+                vnp = find_repeat(vnp, sip);
+            break;
+            
+        case 3:
+            ssp = (StdSegPtr)(sap_p->segs);
+            for(slp = ssp->loc; slp != NULL; slp = slp->next) {
+                sip = SeqLocId(slp);
+                vnp = find_repeat(vnp, sip);
+            }
+            break;
+        case 5:
+            for(sap = sap_p->segs; sap != NULL; sap = sap->next) {
+                sip = make_master(sap);
+                vnp = find_repeat(vnp, sip);
+            }
+            break;
+            
+        default:
+            break;
+        }
+        sap_p = sap_p->next;
+    }
+    
+    sip = find_max(vnp);
+    ValNodeFreeData(vnp);
+    return sip;
 }
 
 NLM_EXTERN Boolean use_multiple_dimension(SeqAlignPtr align)
@@ -2301,3 +2320,132 @@ NLM_EXTERN void AddAlignInfoToSeqAnnot(SeqAnnotPtr annot, Uint1 align_type)
 	AddAlignInfoToSeqAnnotEx(annot, align_type, NULL);
 }
 
+NLM_EXTERN ValNodePtr SortValNode (ValNodePtr list, int (LIBCALLBACK *compar )PROTO ((Nlm_VoidPtr, Nlm_VoidPtr )))
+{
+	ValNodePtr tmp, PNTR head;
+	Int4 count, i;
+
+	if(list == NULL)
+		return NULL;
+	
+	count = get_node_num(list);
+	head = MemNew(((size_t)count+1) * sizeof (ValNodePtr));
+	for(tmp = list, i=0; tmp !=NULL && i<count; ++i)
+	{
+		head[i] = tmp;
+		tmp = tmp->next;
+	}
+
+	HeapSort(head, (size_t)count, sizeof(ValNodePtr), compar);
+	for(i =0; i<count; ++i)
+	{
+		tmp = head[i];
+		tmp->next = head[i+1];
+	}
+	list = head[0];
+	MemFree(head);
+
+	return list;
+}
+
+static Boolean get_align_score(SeqAlignPtr sap, FloatHiPtr score)
+{
+
+        ScorePtr scp;
+
+        if(sap->score == NULL || score == NULL)
+                return FALSE;
+	*score = 0;
+        scp = sap->score;
+	if(scp->choice == 2)
+	{
+        	*score = scp->value.realvalue;
+        	return TRUE;
+	}
+	else
+		return FALSE;
+}
+
+typedef struct align_score {
+	SeqAlignPtr align;
+	FloatHi score;
+}AlignScore, PNTR AlignScorePtr;
+
+static int LIBCALLBACK AlignScoreCompProc(VoidPtr ptr1, VoidPtr ptr2)
+{
+  ValNodePtr   vnp1;
+  ValNodePtr   vnp2;
+  AlignScorePtr asp1, asp2;
+
+  if (ptr1 != NULL && ptr2 != NULL) {
+    vnp1 = *((ValNodePtr PNTR) ptr1);
+    vnp2 = *((ValNodePtr PNTR) ptr2);
+    if (vnp1 != NULL && vnp2 != NULL) {
+      asp1 = (AlignScorePtr) vnp1->data.ptrvalue;
+      asp2 = (AlignScorePtr) vnp2->data.ptrvalue;
+      if (asp1 != NULL && asp2 != NULL) {
+	if(asp1->score > asp2->score)
+		return -1;
+	else if(asp1->score < asp2->score)
+		return 1;
+	else
+		return 0;
+   }
+  }
+  }
+  return 0;
+}
+
+
+NLM_EXTERN SeqAlignPtr sort_align_by_score(SeqAlignPtr PNTR halign)
+{
+	ValNodePtr list, curr;
+	AlignScorePtr asp;
+	Int4 score;
+	FloatHi bit_score, evalue;
+	Int4 number;
+	Boolean comp = FALSE;
+	SeqAlignPtr align, prev;
+
+	list = NULL;
+	align = *halign;
+	while(align)
+	{
+		asp = MemNew(sizeof(AlignScore));
+		asp->align = align;
+		if(GetScoreAndEvalue(align, &score, &bit_score, &evalue, &number))
+			asp->score = (FloatHi)score;
+		else
+			get_align_score(align, &(asp->score));
+		if(asp->score > 0.0)
+			comp = TRUE;
+		ValNodeAddPointer(&list, 0, asp);
+		align = align->next;
+	}
+	if(list == NULL)
+		return NULL;
+	if(list->next == NULL || !comp)
+	{
+		ValNodeFreeData(list);
+		return (*halign);
+	}
+
+	list = SortValNode(list, AlignScoreCompProc);
+	prev = NULL;
+	for(curr = list; curr != NULL; curr = curr->next)
+	{
+		asp = curr->data.ptrvalue;
+		asp->align->next = NULL;
+		if(prev == NULL)
+		{
+			*halign = asp->align;
+		}
+		else
+			prev->next = asp->align;
+		prev = asp->align;
+	}
+	ValNodeFreeData(list);
+	return (*halign);
+}
+
+
