@@ -1,4 +1,4 @@
-/* $Id: maputil.h,v 6.6 1997/11/14 22:13:41 vakatov Exp $
+/* $Id: maputil.h,v 6.8 2001/06/26 16:42:59 vakatov Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -29,12 +29,18 @@
 *
 * Initial Version Creation Date: 03/21/97
 *
-* $Revision: 6.6 $
+* $Revision: 6.8 $
 *
 * File Description:
 *         External include file for graphical alignments
 *
 * $Log: maputil.h,v $
+* Revision 6.8  2001/06/26 16:42:59  vakatov
+* POINT --> BAND_POINT  (to avoid conflicts with MS-Win standard headers)
+*
+* Revision 6.7  1998/07/22 00:02:37  zjing
+* move SortValNode from maputil to jzmisc
+*
 * Revision 6.6  1997/11/14 22:13:41  vakatov
 * [WIN32,DLL]  Added NLM_EXTERN's
 *
@@ -53,12 +59,18 @@
 * Revision 5.9  1997/07/23 13:45:29  zjing
 * look up for the sequence index map
 *
-* $Revision: 6.6 $
+* $Revision: 6.8 $
 *
 * File Description:
 *         External include file for graphical alignments
 *
 * $Log: maputil.h,v $
+* Revision 6.8  2001/06/26 16:42:59  vakatov
+* POINT --> BAND_POINT  (to avoid conflicts with MS-Win standard headers)
+*
+* Revision 6.7  1998/07/22 00:02:37  zjing
+* move SortValNode from maputil to jzmisc
+*
 * Revision 6.6  1997/11/14 22:13:41  vakatov
 * [WIN32,DLL]  Added NLM_EXTERN's
 *
@@ -390,7 +402,7 @@ NLM_EXTERN Uint1 ck_cyto_type PROTO((SeqFeatPtr sfp));
 #define CEN 4 		/*centromere*/
 
 /*band used for Human chromosome*/
-#define POINT 5		/*inclue the centromere and telemere*/	
+#define BAND_POINT 5		/*inclue the centromere and telemere*/	
 #define GIEMSA_POS 6	/*Giemsa positive*/
 #define GIEMSA_NEG 7	/*Giemsa negative*/
 #define ACRO_CENTRIC 8	/*Acrocentric*/
@@ -610,7 +622,10 @@ NLM_EXTERN Uint1 FindContigDB PROTO((SeqIdPtr sip));
 NLM_EXTERN ValNodePtr FindContigList PROTO((SeqLocPtr chr_slp));
 
 
-NLM_EXTERN ValNodePtr SortValNode PROTO((ValNodePtr list, int (LIBCALLBACK *compar )PROTO ((Nlm_VoidPtr, Nlm_VoidPtr ))));
+NLM_EXTERN Boolean is_lod_score_annot PROTO((SeqAnnotPtr annot));
+NLM_EXTERN Int2 GetLODScoreNumber PROTO((BioseqPtr bsp));
+NLM_EXTERN CharPtr GetAnnotTitle PROTO((SeqAnnotPtr annot));
+NLM_EXTERN Uint1 GetLODScoreBitValue PROTO((SeqFeatPtr sfp));
 
 
 #ifdef __cplusplus
