@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 9/94
 *
-* $Revision: 6.3 $
+* $Revision: 6.31 $
 *
 * File Description:  Manager for Bioseqs and BioseqSets
 *
@@ -40,6 +40,90 @@
 *
 *
 * $Log: objmgr.h,v $
+* Revision 6.31  2001/11/19 15:26:19  kans
+* added ObjMgrDeleteAllInRecord, still need to bail in ObjMgrDelete if bulkIndexFree, then call from BioseqFree and BioseqSetFree
+*
+* Revision 6.30  2001/11/15 18:15:48  kans
+* set bsp->omdp at creation, SeqMgrDeleteIndexesInRecord sets omdp->bulkIndexFree
+*
+* Revision 6.29  2001/05/31 22:58:25  kans
+* added ObjMgrReapOne, DEFAULT_MAXOBJ, autoclean reaps and frees one entity at a time, as needed
+*
+* Revision 6.28  2001/05/31 22:33:03  kans
+* added autoclean and maxobj to ObjMgr structure, ObjMgrAddFunc optionally calls ObjMgrReap and ObjMgrFreeCache to completely clear out least recently accessed objects if currobj >= maxobj
+*
+* Revision 6.27  2001/02/16 21:34:49  ostell
+* changed GetSecs() to ObjMgrTouchCnt() to reduce system calls
+*
+* Revision 6.26  2000/11/28 22:59:02  kans
+* omdp->lockcnt now a Uint2 to avoid overflow when locking accessions repeatedly referenced by genomic contig
+*
+* Revision 6.25  2000/11/28 21:43:55  kans
+* added ObjMgrReportFunc for debugging
+*
+* Revision 6.24  2000/10/30 21:26:09  shavirin
+* Changes and fixes for some MT-safety related problems.
+*
+* Revision 6.23  2000/04/07 15:01:24  durand
+* added UPDATE_TYPE_NEWSEQ define
+*
+* Revision 6.22  2000/03/02 21:11:05  lewisg
+* use bandalign for import sequence, make standalone ddv use viewmgr, make dialogs modal, send color update
+*
+* Revision 6.21  2000/02/10 17:05:52  kans
+* added UPDATE_TYPE_CN3D
+*
+* Revision 6.20  2000/02/10 16:53:55  kans
+* added UPDATE_TYPE_VIEWMGR define (presumably in the right place)
+*
+* Revision 6.19  1999/12/06 22:38:52  durand
+* add UPDATE_TYPE_CARETPOS to be used by DDV
+*
+* Revision 6.18  1999/12/03 23:17:23  lewisg
+* Patrick's new global update msg, argument passing when launching ddv, experimental editing
+*
+* Revision 6.17  1999/11/18 00:21:42  lewisg
+* draw speedups and selection on mouseup
+*
+* Revision 6.16  1999/10/29 18:53:08  durand
+* add DDVUpdateMSG typedef
+*
+* Revision 6.15  1999/10/15 20:56:40  lewisg
+* append DDV_ColorGlobal as userdata.  free memory when cn3d terminates.
+*
+* Revision 6.14  1999/09/27 20:03:15  kans
+* added rowID to OMMsgStruct, new ObjMgrSendRowMsg function
+*
+* Revision 6.13  1999/08/11 15:17:54  kans
+* added ObjMgrFreeByEntityID
+*
+* Revision 6.12  1999/07/06 17:17:28  ywang
+* add message OM_MSG_FLUSH
+*
+ * Revision 6.11  1998/09/28  19:54:12  kans
+ * made ObjMgrDump debugging function public, no longer conditionally compiled
+ *
+* Revision 6.10  1998/07/01 19:11:51  kans
+* added fromProcID, toProcID, OM_MSG_PROCESS, ObjMgrSendProcMsg, time of indexing, ObjMgrGetProcID, moved protFeat and cdsOrRnaFeat to seqmgr structure
+*
+* Revision 6.9  1998/06/30 21:40:52  kans
+* added seqmgr feature indexed time stamp to omdp
+*
+* Revision 6.8  1998/06/29 20:35:53  kans
+* added reap and reload extra functions
+*
+* Revision 6.7  1998/06/29 00:23:54  kans
+* several changes to new indexing functions
+*
+* Revision 6.6  1998/06/27 22:23:44  kans
+* improvements and further implementation of new indexing, exploration functions
+*
+* Revision 6.5  1998/06/26 22:36:19  kans
+* initial work on tracking sorted features, and cds and prot links, for rapid collection
+*
+* Revision 6.4  1998/05/29 20:34:11  chappey
+* New objmgr messages OM_MSG_HIDE, OM_MSG_SHOW
+*
 * Revision 6.3  1998/03/24 23:55:33  kans
 * added OBJ_PROJECT define
 *
@@ -112,6 +196,10 @@
 #include <asn.h>
 #endif
 
+#ifndef _NCBI_Seqfeat_
+#include <objfeat.h>
+#endif
+
 #undef NLM_EXTERN
 #ifdef NLM_IMPORT
 #define NLM_EXTERN NLM_IMPORT
@@ -248,7 +336,11 @@ typedef struct ommsgstruct {
 	Int2 message;    /* the message code, defined below */
 	Uint2 entityID,
 		itemID,
-		itemtype;
+		itemtype,
+		rowID;
+	Uint2 fromProcID,
+		toProcID;         /* allows communication to and from particular procs */
+	Pointer procmsgdata;  /* procedure specific data to be sent, assumes toProcID set */
 	Pointer omuserdata;	  /* this is really an OMUserDataPtr */
 	Pointer region;       /* this restricts the message to a region. See below */
 	Uint1 regiontype;     /*  see define below */
@@ -284,6 +376,30 @@ typedef Int2 (LIBCALLBACK *OMMessageFunc) PROTO((OMMsgStructPtr message));
 #define OM_MSG_CONNECT      9      /* registered entity being incorporated into
 		                               another. */
 #define OM_MSG_SETCOLOR    10     /* set color of region */
+#define OM_MSG_HIDE        11     /* hide sequence */
+#define OM_MSG_SHOW        12     /* show sequence */
+#define OM_MSG_PROCESS     13     /* process-specific message */
+#define OM_MSG_FLUSH       14     /* close all existed windows */
+#define OM_MSG_MOUSEUP     15     /* mouse up on selection has occurred */
+
+/*****************************************************************************
+*
+*  data type used to send additional information for an update msg
+*
+*****************************************************************************/
+#define UPDATE_TYPE_LAYOUT       (1)
+#define UPDATE_TYPE_COLOR        (2)
+#define UPDATE_TYPE_EDIT_DELBSP  (3)
+#define UPDATE_TYPE_CARETPOS     (4)
+#define UPDATE_TYPE_VIEWMGR      (5)
+#define UPDATE_TYPE_RECOLOR      (6)
+#define UPDATE_TYPE_NEWSEQ       (7)
+
+typedef struct ddvupdatemsg{
+	Uint1   type;
+	Pointer data;
+	}DDVUpdateMSG, PNTR DDVUpdateMSGPtr;
+
 
 /*****************************************************************************
 *
@@ -291,9 +407,18 @@ typedef Int2 (LIBCALLBACK *OMMessageFunc) PROTO((OMMsgStructPtr message));
 *       Directly invokes the objmgr messaging system
 *       should be used cautiously as most objmgr calls do their own messaging
 *
+*
+*   ObjMgrSendProcMsg(msg, entityID, itemID, itemtype, fromProcID, toProcID)
+*       Allows messages to be sent to particular procs
+*
 *****************************************************************************/
 NLM_EXTERN Boolean LIBCALL ObjMgrSendMsg PROTO((Uint2 msg, Uint2 entityID, Uint2 itemID, Uint2 itemtype));
 
+NLM_EXTERN Boolean LIBCALL ObjMgrSendProcMsg PROTO((Uint2 msg, Uint2 entityID, Uint2 itemID, Uint2 itemtype,
+                                                    Uint2 fromProcID, Uint2 toProcID, Pointer procmsgdata));
+
+NLM_EXTERN Boolean LIBCALL ObjMgrSendRowMsg PROTO((Uint2 msg, Uint2 entityID, Uint2 itemID, Uint2 itemtype, Uint2 rowID));
+
 	                              /* user supplied data on entity */
 typedef struct omuserdata {
 	Uint2 procid,                 /* proc that put userdata here */
@@ -312,16 +437,23 @@ typedef struct objmgrdata {
 	Pointer parentptr;
 	Uint2 choicetype;
 	ValNodePtr choice;           /* CHOICE wrapper (if datatype=Bioseq or Set) */
-	Uint1 lockcnt;             /* count of locks */
+	Uint2 lockcnt;             /* count of locks */
 	Uint1 tempload;
-	time_t touch;                /* last time this was unlocked */
+	Uint4 touch;                /* last time this was unlocked */
 	Uint2 EntityID;             /* arbitrary ID assigned to complete entity */
 	Boolean clipboard;         /* is this in the clipboard? */
 	Boolean dirty;             /* updated without a save? */
 	Boolean being_freed;       /* in-process of being freed, set by ObjMgrDel */
 	Boolean free;
+	Boolean bulkIndexFree;     /* used to suppress individual SeqMgrDeleteFromBioseqIndex */
 	OMUserDataPtr userdata;    /* for user supplied data */
 	Uint2 options;             /* options set with bit flags, defined below */
+	Uint2 lastDescrItemID;     /* for new seqmgr extra bioseq exploration functions */
+	time_t indexed;            /* time that entity was seqmgr indexed */
+	Pointer extradata;         /* used (initially) for extra bioseq feature indices */
+	OMFreeFunc reapextra;      /* function to call when caching out, passed extradata */
+	OMFreeFunc reloadextra;    /* function to call when reloading, passed extradata */
+	OMFreeFunc freeextra;      /* function to call when deleting, passed extradata */
 } ObjMgrData, PNTR ObjMgrDataPtr;
 
 /** values for tempload (above) **/
@@ -440,8 +572,9 @@ typedef struct omproccontrol {
 #define OMPROC_REPLACE	11
 #define OMPROC_FILTER	12
 #define OMPROC_FETCH	13
+#define OMPROC_COLORMGR 14
 
-#define OMPROC_MAX      13
+#define OMPROC_MAX      14
 
 
 #define NUM_OMD 50
@@ -506,9 +639,13 @@ typedef struct objmng {        /* functions for data management */
 
 	OMUserDataPtr userdata;      /* for global messageing */
 
+	Boolean autoclean;           /* if TRUE, reap and free cache if currobj > maxobj */
+	Uint2 maxobj;                /* number of objects that tiggers autocleaning */
+
 } ObjMgr, PNTR ObjMgrPtr;
 
 #define DEFAULT_MAXTEMP 5   /* default number of temp entries to keep */
+#define DEFAULT_MAXOBJ 50000   /* default number of objects to keep for autocleaning */
 
 /*****************************************************************************
 *
@@ -651,7 +788,7 @@ NLM_EXTERN Boolean LIBCALL ObjMgrTestOptions PROTO((Uint2 option, Uint2 entityID
 *
 *****************************************************************************/
 
-NLM_EXTERN Int2 LIBCALL ObjMgrTypeLookup PROTO((ObjMgrPtr omp, Uint2 type));
+NLM_EXTERN Int4 LIBCALL ObjMgrTypeLookup PROTO((ObjMgrPtr omp, Uint2 type));
 
 /*****************************************************************************
 *
@@ -695,7 +832,7 @@ NLM_EXTERN Boolean LIBCALL ObjMgrTypeSetLabelFunc PROTO((Uint2 type, OMLabelFunc
 *
 *****************************************************************************/
 
-NLM_EXTERN Int2 LIBCALL ObjMgrProcLookup PROTO((ObjMgrPtr omp, Uint2 procID));
+NLM_EXTERN Int4 LIBCALL ObjMgrProcLookup PROTO((ObjMgrPtr omp, Uint2 procID));
 
 /*****************************************************************************
 *
@@ -761,6 +898,14 @@ NLM_EXTERN Uint2 ObjMgrProcLoadEx PROTO((Uint2 proctype, CharPtr procname, CharP
 NLM_EXTERN ObjMgrProcPtr LIBCALL ObjMgrProcFind PROTO((ObjMgrPtr omp, Uint2 procid,
                                                        CharPtr procname, Uint2 proctype));
 
+/*****************************************************************************
+*
+*   ObjMgrGetProcID(omp, procname, proctype)
+*   	returns procid given procname and proctype
+*
+*****************************************************************************/
+NLM_EXTERN Uint2 LIBCALL ObjMgrGetProcID PROTO((ObjMgrPtr omp, CharPtr procname, Uint2 proctype));
+
 /*****************************************************************************
 *
 *   ObjMgrProcFindNext(omp, proctype, inputtype, outputtype, last)
@@ -799,9 +944,10 @@ NLM_EXTERN Uint2 LIBCALL OMGetNextUserKey PROTO((void));
 *****************************************************************************/
 
 NLM_EXTERN ObjMgrDataPtr LIBCALL ObjMgrFindByData PROTO((ObjMgrPtr omp, Pointer ptr));
-NLM_EXTERN Int2 LIBCALL ObjMgrLookup PROTO((ObjMgrPtr omp, Pointer data));
+NLM_EXTERN Int4 LIBCALL ObjMgrLookup PROTO((ObjMgrPtr omp, Pointer data));
 NLM_EXTERN Boolean LIBCALL ObjMgrAdd PROTO((Uint2 type, Pointer data));
 NLM_EXTERN Boolean LIBCALL ObjMgrDelete PROTO((Uint2 type, Pointer data));
+NLM_EXTERN Boolean LIBCALL ObjMgrDeleteAllInRecord (void);
 NLM_EXTERN Boolean LIBCALL ObjMgrConnect PROTO((Uint2 type, Pointer data, Uint2 parenttype, Pointer parentdata));
 NLM_EXTERN Boolean LIBCALL ObjMgrConnectFunc PROTO((ObjMgrPtr omp, Uint2 type, Pointer data, Uint2 parenttype, Pointer parentdata));
 
@@ -835,9 +981,10 @@ NLM_EXTERN Boolean LIBCALL ObjMgrSetDirtyFlag PROTO((Uint2 entityID, Boolean the
 NLM_EXTERN Boolean LIBCALL ObjMgrGetDirtyFlag PROTO((Uint2 entityID));
 
 NLM_EXTERN Boolean LIBCALL ObjMgrSetChoice PROTO((Uint2 type, ValNodePtr choice, Pointer data));
-NLM_EXTERN Int2 LIBCALL ObjMgrLock PROTO((Uint2 type, Pointer data, Boolean lockit));
+NLM_EXTERN Int4 LIBCALL ObjMgrLock PROTO((Uint2 type, Pointer data, Boolean lockit));
 NLM_EXTERN Boolean LIBCALL ObjMgrSetTempLoad PROTO((ObjMgrPtr omp, Pointer ptr));
 NLM_EXTERN Boolean LIBCALL ObjMgrReap PROTO((ObjMgrPtr omp));
+NLM_EXTERN Boolean LIBCALL ObjMgrReapOne PROTO((ObjMgrPtr omp));
 NLM_EXTERN ObjMgrDataPtr LIBCALL ObjMgrFindTop PROTO((ObjMgrPtr omp, ObjMgrDataPtr smdp));
 NLM_EXTERN ObjMgrDataPtr LIBCALL ObjMgrGetDataStruct PROTO((ObjMgrPtr omp,
 														   Uint2 entityID));
@@ -986,6 +1133,14 @@ NLM_EXTERN Boolean LIBCALL ObjMgrIsTemp PROTO((Pointer data));
 *****************************************************************************/
 NLM_EXTERN Boolean LIBCALL ObjMgrIsChild PROTO((Pointer parent, Pointer child));
 
+/*****************************************************************************
+*
+*   ObjMgrDump(fp)
+*       prints status of internal object manager data for debugging
+*
+*****************************************************************************/
+
+NLM_EXTERN void LIBCALL ObjMgrDump PROTO((FILE * fp, CharPtr title));
 
 /*****************************************************************************
 *
@@ -1135,6 +1290,22 @@ NLM_EXTERN Pointer LIBCALL ObjMgrMemCopy PROTO((Uint2 type, Pointer ptr));
 ******************************************************************************/
 NLM_EXTERN Pointer LIBCALL ObjMgrFree PROTO((Uint2 type, Pointer ptr));
 
+/******************************************************************************
+*
+*  ObjMgrFreeByEntityID(entityID)
+*    Obtains type and ptr from ObjMgrDataPtr, then calls ObjMgrFree
+*
+******************************************************************************/
+NLM_EXTERN Pointer LIBCALL ObjMgrFreeByEntityID PROTO((Uint2 entityID));
+
+NLM_EXTERN void LIBCALL ObjMgrResetAll PROTO((void));
+
+/* debugging function */
+
+NLM_EXTERN void LIBCALL ObjMgrReportFunc (CharPtr filename);
+
+
+
 #ifdef __cplusplus
 }
 #endif
