@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 7/13/91
 *
-* $Revision: 6.2 $
+* $Revision: 6.30 $
 *
 * File Description:  Ports onto Bioseqs
 *
@@ -40,6 +40,90 @@
 *
 *
 * $Log: seqport.h,v $
+* Revision 6.30  2001/02/18 20:58:52  kans
+* added GetSequenceByBsp
+*
+* Revision 6.29  2000/12/18 18:03:26  kans
+* added GetScoresbySeqId
+*
+* Revision 6.28  2000/09/24 23:31:18  kans
+* added GetSequenceByFeature
+*
+* Revision 6.27  2000/09/24 22:52:47  kans
+* added GetSequenceByIdOrAccnDotVer
+*
+* Revision 6.26  2000/09/05 21:33:50  kans
+* productInterval_to_locationIntervals replaces aaInterval_to_dnaIntervals, also works for mRNA feature (JO)
+*
+* Revision 6.25  2000/08/31 18:12:54  shavirin
+* Added new function TransTableFreeAll().
+*
+* Revision 6.24  2000/08/11 18:09:49  kans
+* GetScoresbyAccessionDotVersion passes length back through new parameter
+*
+* Revision 6.23  2000/08/11 18:03:25  kans
+* added GetScoresbyAccessionDotVersion - prototyped in seqport.h but implemented in sqnutil2.c
+*
+* Revision 6.22  2000/08/10 17:22:38  kans
+* added GetDNAbyAccessionDotVersion for genome processing effort
+*
+* Revision 6.21  2000/08/04 15:45:22  kans
+* added ContigRevComp - still need to implement for delta bioseqs
+*
+* Revision 6.20  2000/08/03 19:02:54  kans
+* added PersistentTransTableByGenCode and PersistentTransTableByCdRegion
+*
+* Revision 6.19  2000/08/01 20:02:58  kans
+* separate macros for IsOrfStart, IsAmbigStart, IsAnyStart
+*
+* Revision 6.18  2000/07/22 22:45:37  kans
+* more work on trans table translation functions
+*
+* Revision 6.17  2000/07/21 15:28:36  kans
+* first pass at TransTableTranslate functions - more work remains
+*
+* Revision 6.16  2000/07/05 17:02:12  kans
+* added spp->gapIsZero, SeqPortSet_do_virtualEx, using ncbi4na with gap of 0 to distinguish quality scores under N versus quality scores under gap
+*
+* Revision 6.15  2000/05/23 20:41:17  ostell
+* added MolWtForLoc()
+*
+* Revision 6.14  1999/11/17 00:56:33  kans
+* improved seqsearch fsa, removed protein part, still need to allow single mismatch
+*
+* Revision 6.13  1999/11/12 21:00:50  kans
+* added TransTableProcessBioseq for 6-frame translation, SeqSearchAddNucleotidePattern and SeqSearchAddProteinPattern for SeqSearch
+*
+* Revision 6.12  1999/11/11 00:58:28  kans
+* added SeqSearch sequence search finite state machine - still need more functions to add protein patterns, read from rsite file
+*
+* Revision 6.11  1999/10/06 22:09:02  kans
+* ComposeCodonsRecognizedString to handle degenerate codons
+*
+* Revision 6.10  1999/08/06 20:22:19  kans
+* TransTable simplified to eliminate single and double letter states
+*
+* Revision 6.9  1999/08/06 02:20:16  kans
+* finite state machine for 6-frame translation and orf search enhanced to handle nucleotide ambiguity characters
+*
+* Revision 6.8  1999/02/12 20:48:24  kans
+* made fast byte expansion functions public
+*
+* Revision 6.7  1998/12/14 20:56:24  kans
+* dnaLoc_to_aaLoc takes allowTerminator parameter to handle stop codons created by polyA tail
+*
+* Revision 6.6  1998/11/16 21:10:08  kans
+* added IsATGStart and IsAltStart macros
+*
+* Revision 6.5  1998/11/16 17:20:31  kans
+* nextBase in codon fsa is Uint1, cast state array index to int in macros
+*
+* Revision 6.4  1998/11/14 00:30:21  kans
+* added TransTableInit and macros for 6-frame translation and orf-finding finite state machine
+*
+* Revision 6.3  1998/09/16 21:40:42  kans
+* added SPCacheQ for rapid 2na/4na to iupacna conversion
+*
 * Revision 6.2  1998/02/24 15:09:17  kans
 * made AAForCodon prototype public
 *
@@ -162,6 +246,11 @@ typedef struct spcache {
 	Uint1 buf[100];
 } SPCache, PNTR SPCachePtr;
 
+typedef struct spcacheq {
+	Int2 ctr, total;
+	Char buf[400];
+} SPCacheQ, PNTR SPCacheQPtr;
+
 typedef struct seqport {
     BioseqPtr bsp;             /* 1 seqentry per port */
 	Boolean locked;              /* TRUE if Lock function used */
@@ -175,6 +264,7 @@ typedef struct seqport {
     Boolean is_circle ,          /* go around the end of a circle? */
             is_seg ,             /* return EOS at the end of segments? */
             do_virtual,          /* deliver 'N''X' over virtual seqs */
+            gapIsZero,           /* deliver 0 for ncbi4na over virtual seqs - also needs do_virtual */
             eos,                 /* set when comp strand tries to back off */
 			isa_null,            /* TRUE if seqport represents a NULL location */
 			isa_virtual,         /* represents a virtual interval or Bioseq */
@@ -193,6 +283,7 @@ typedef struct seqport {
         PNTR segs,                    /* segments if seg or ref */
         PNTR next;                    /* if part of a segment chain */
 	SPCachePtr cache;
+	SPCacheQPtr cacheq;          /* used instead of cache for ncbi2na or ncbi4na to iupacna fasta lookup */
 	ByteStorePtr bp;             /* used by both raw and delta seq pieces */
 } SeqPort, PNTR SeqPortPtr;
 
@@ -217,12 +308,22 @@ NLM_EXTERN SeqPortPtr SeqPortFree PROTO((SeqPortPtr spp));
 NLM_EXTERN Int4 SeqPortTell PROTO((SeqPortPtr spp));
 NLM_EXTERN Int2 SeqPortSeek PROTO((SeqPortPtr spp, Int4 offset, Int2 origin));
 NLM_EXTERN Int4 SeqPortLen PROTO((SeqPortPtr spp));
-NLM_EXTERN Uint1 SeqPortGetResidue PROTO((SeqPortPtr spp));
-NLM_EXTERN Int2 SeqPortRead PROTO((SeqPortPtr spp, BytePtr buf, Int2 len));
+NLM_EXTERN Uint1 LIBCALL SeqPortGetResidue PROTO((SeqPortPtr spp));
+NLM_EXTERN Int2 LIBCALL SeqPortRead PROTO((SeqPortPtr spp, BytePtr buf, Int2 len));
 NLM_EXTERN Uint1 GetGapCode PROTO((Uint1 seqcode));
 NLM_EXTERN Boolean LIBCALL SeqPortSetUpFields PROTO((SeqPortPtr spp, Int4 start, Int4 stop, Uint1 strand, Uint1 newcode));
 NLM_EXTERN Boolean LIBCALL SeqPortSetUpAlphabet PROTO((SeqPortPtr spp, Uint1 curr_code, Uint1 newcode));
 
+/*
+the following functions are for quick alphabet expansion, and require buffers
+allocated with 4-byte or 2-byte alignment, because they cast 2 or 4 bytes into
+Uint2 or Uint4 for fast integer copying.
+*/
+
+NLM_EXTERN Uint4Ptr LIBCALL MapNa2ByteToIUPACString PROTO((Uint1Ptr bytep, Uint4Ptr buf, Int4 total));
+NLM_EXTERN Uint2Ptr LIBCALL MapNa4ByteToIUPACString PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
+NLM_EXTERN Uint2Ptr LIBCALL MapNa2ByteToNa4String PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
+
 /*****************************************************************************
 *
 *   SeqPortSetValues(spp)
@@ -271,6 +372,7 @@ NLM_EXTERN Boolean LIBCALL SeqPortSet_is_seg (SeqPortPtr spp, Boolean value);
 *
 *****************************************************************************/
 NLM_EXTERN Boolean LIBCALL SeqPortSet_do_virtual (SeqPortPtr spp, Boolean value);
+NLM_EXTERN Boolean LIBCALL SeqPortSet_do_virtualEx (SeqPortPtr spp, Boolean value, Boolean gapIsZero);
 
 /*****************************************************************************
 *
@@ -340,16 +442,30 @@ NLM_EXTERN Int2 GetFrameFromLoc PROTO((SeqLocPtr slp));
 
 /******************************************************************
 *
-*	dnaLoc_to_aaLoc(sfp, dna_loc, merge, frame)
+*	dnaLoc_to_aaLoc(sfp, dna_loc, merge, frame, allowTerminator)
 *	map a SeqLoc on the DNA sequence
 *       to a Seq-loc in the	protein sequence
 *       through a CdRegion feature
 *   if (merge) adjacent intervals on the amino acid sequence
 *      are merged into one. This should be the usual case.
 *   We try to report the frame if the caller provides a suitable pointer
+*   If allowTerminator, can map the termination codon as a legal location
 *
 ******************************************************************/
-NLM_EXTERN SeqLocPtr LIBCALL dnaLoc_to_aaLoc(SeqFeatPtr sfp, SeqLocPtr dna_loc, Boolean merge, Int4Ptr frame);
+NLM_EXTERN SeqLocPtr LIBCALL dnaLoc_to_aaLoc(SeqFeatPtr sfp, SeqLocPtr dna_loc, Boolean merge, Int4Ptr frame, Boolean allowTerminator);
+
+/******************************************************************
+*
+*	productLoc_to_locationLoc(sfp, product_loc)
+*	map a SeqLoc on the product sequence
+*       to a Seq-loc in the location sequence
+*       through a feature
+*
+*       this more general function is now called by
+*          aaLoc_to_dnaLoc()
+*
+******************************************************************/
+NLM_EXTERN SeqLocPtr LIBCALL productLoc_to_locationLoc(SeqFeatPtr sfp, SeqLocPtr product_loc);
 
 /******************************************************************
 *
@@ -378,12 +494,12 @@ NLM_EXTERN SeqLocPtr LIBCALL aaFeatLoc_to_dnaFeatLoc(SeqFeatPtr sfp, SeqLocPtr a
 
 /******************************************************************
 *
-*	aaInterval_to_dnaIntervals(sfp, aa_start, aa_stop)
+*	productInterval_to_locationIntervals(sfp, aa_start, aa_stop)
 *	map the amino acid sequence to a chain of Seq-locs in the 
 *	DNA sequence through a CdRegion feature
 *
 ******************************************************************/
-NLM_EXTERN SeqLocPtr LIBCALL aaInterval_to_dnaIntervals (SeqFeatPtr sfp, Int4 aa_start, Int4 aa_stop);
+NLM_EXTERN SeqLocPtr LIBCALL productInterval_to_locationIntervals (SeqFeatPtr sfp, Int4 aa_start, Int4 aa_stop);
 
 /*-------------- BioseqRevComp () ---------------------------*/
 /***********************************************************************
@@ -412,6 +528,12 @@ NLM_EXTERN Boolean LIBCALL BioseqComplement (BioseqPtr bsp);
 NLM_EXTERN Boolean LIBCALL BioseqReverse (BioseqPtr bsp);
 
 
+/*-------------- ContigRevComp () ---------------------------*/
+/***********************************************************************
+*   ContigRevComp:   Reverse complement segmented or delta bioseq
+************************************************************************/
+NLM_EXTERN Boolean LIBCALL ContigRevComp (BioseqPtr bsp);
+
 /*****************************************************************************
 *
 *  SPCompressNew(void); - allocated memory for SPCompress structure
@@ -453,6 +575,198 @@ NLM_EXTERN SPCompressPtr SPCompressDNA(SeqPortPtr spp);
 *****************************************************************************/
 NLM_EXTERN Boolean SPRebuildDNA(SPCompressPtr spc);
 
+/*****************************************************************************
+*
+*   ComposeCodonsRecognizedString (trna, buf, buflen);
+*       Copies codon recognized string to buf, returns number of codons
+*
+*****************************************************************************/
+
+NLM_EXTERN Int2 ComposeCodonsRecognizedString (tRNAPtr trna, CharPtr buf, size_t buflen);
+
+/*****************************************************************************
+*
+*   TransTableNew (Int2 genCode);
+*       Initializes TransTable finite state machine for 6-frame translation
+*       and open reading frame search, allowing nucleotide ambiguity characters
+*
+*****************************************************************************/
+
+typedef struct fsatranstable {
+  Int2     genCode;
+  Char     ncbieaa [65];
+  Char     sncbieaa [65];
+  Uint2    nextBase [3376];
+  Char     aminoAcid [3376] [2];
+  Char     orfStart [3376] [2];
+  Uint1    basesToIdx [256];
+} TransTable, PNTR TransTablePtr;
+
+/* allocate 6-frame finite state translation table and initialize with indicated genetic code */
+NLM_EXTERN TransTablePtr TransTableNew (Int2 genCode);
+NLM_EXTERN TransTablePtr TransTableFree (TransTablePtr tbl);
+NLM_EXTERN void TransTableFreeAll (void);
+
+#define TTBL_TOP_STRAND  0
+#define TTBL_BOT_STRAND  1
+
+#define TTBL_ATG_STATE  48
+#define TTBL_CAT_STATE 229
+
+/* macros for using finite state machine for 6-frame translation */
+#define NextCodonState(tbl,cur,ch) (tbl->nextBase [(int) (Uint2) cur] + tbl->basesToIdx [(int) (Uint1) ch])
+#define GetCodonResidue(tbl,cur,stnd) (tbl->aminoAcid [(int) (Uint2) cur] [stnd])
+#define GetStartResidue(tbl,cur,stnd) (tbl->orfStart [(int) (Uint2) cur] [stnd])
+#define IsOrfStart(tbl,cur,stnd) ((Boolean) (GetStartResidue(tbl,cur,stnd) == 'M'))
+#define IsAmbigStart(tbl,cur,stnd) ((Boolean) (GetStartResidue(tbl,cur,stnd) == 'X'))
+#define IsAnyStart(tbl,cur,stnd) ((Boolean) (GetStartResidue(tbl,cur,stnd) != '-'))
+#define IsOrfStop(tbl,cur,stnd) ((Boolean) (GetCodonResidue(tbl,cur,stnd) == '*'))
+#define IsATGStart(tbl,cur,stnd) ((Boolean) (IsOrfStart(tbl,cur,stnd) && (stnd ? (cur == TTBL_CAT_STATE) : (cur == TTBL_ATG_STATE))))
+#define IsAltStart(tbl,cur,stnd) ((Boolean) (IsOrfStart(tbl,cur,stnd) && (stnd ? (cur != TTBL_CAT_STATE) : (cur != TTBL_ATG_STATE))))
+
+typedef void (LIBCALLBACK *TransTableMatchProc) (Int4 position, Char residue, Boolean atgStart, Boolean altStart, Boolean orfStop, Int2 frame, Uint1 strand, Pointer userdata);
+
+/* convenience function calls user callback for each strand of entire bioseq */
+
+NLM_EXTERN void TransTableProcessBioseq (
+  TransTablePtr tbl,
+  TransTableMatchProc matchProc,
+  Pointer userdata,
+  BioseqPtr bsp
+);
+
+/* trans table translation functions can be passed cds feature or individual parameters */
+
+NLM_EXTERN ByteStorePtr TransTableTranslateCdRegion (
+  TransTablePtr  PNTR tblptr,
+  SeqFeatPtr cds,
+  Boolean include_stop,
+  Boolean remove_trailingX
+);
+
+NLM_EXTERN ByteStorePtr TransTableTranslateSeqLoc (
+  TransTablePtr  PNTR tblptr,
+  SeqLocPtr location,
+  Int2 genCode,
+  Uint1 frame,
+  Boolean include_stop,
+  Boolean remove_trailingX
+);
+
+/* allow reuse of translation tables by saving as AppProperty, avoids unnecessary initializations */
+
+NLM_EXTERN TransTablePtr PersistentTransTableByGenCode (
+  Int2 genCode
+);
+
+NLM_EXTERN TransTablePtr PersistentTransTableByCdRegion (
+  SeqFeatPtr cds
+);
+
+/*****************************************************************************
+*
+*   SeqSearch
+*       Initializes SeqSearch finite state machine for sequence searching
+*       Based on Practical Algorithms for Programmers by Binstock and Rex
+*
+*****************************************************************************/
+
+struct SeqSearch;
+typedef struct SeqSearch* SeqSearchPtr;
+
+typedef void (LIBCALLBACK *SeqSearchMatchProc) (Int4 position, CharPtr name, CharPtr pattern, Int2 cutSite, Uint1 strand, Pointer userdata);
+
+/* create empty nucleotide sequence search finite state machine */
+
+NLM_EXTERN SeqSearchPtr SeqSearchNew (
+  SeqSearchMatchProc matchproc,
+  Pointer userdata,
+  Boolean allowOneMismatch
+);
+
+/*
+   add nucleotide pattern or restriction site to sequence search finite state
+   machine, uses ambiguity codes, e.g., R = A and G, H = A, C and T
+*/
+
+NLM_EXTERN void SeqSearchAddNucleotidePattern (
+  SeqSearchPtr tbl,
+  CharPtr name,
+  CharPtr pattern,
+  Int2 cutSite
+);
+
+/* program passes each character in turn to finite state machine */
+
+NLM_EXTERN void SeqSearchProcessCharacter (
+  SeqSearchPtr tbl,
+  Char ch
+);
+
+/* convenience function calls SeqSearchProcessCharacter for entire bioseq */
+
+NLM_EXTERN void SeqSearchProcessBioseq (
+  SeqSearchPtr tbl,
+  BioseqPtr bsp
+);
+
+/* reset state and position to allow another run with same search patterns */
+
+NLM_EXTERN void SeqSearchReset (
+  SeqSearchPtr tbl
+);
+
+/* clean up sequence search finite state machine allocated memory */
+
+NLM_EXTERN SeqSearchPtr SeqSearchFree (
+  SeqSearchPtr tbl
+);
+
+
+/*
+   Convenience functions for genome processing use BioseqLockById to get sequence
+   record (perhaps with phrap quality score graphs) so fetching from some network
+   or local server must be enabled, or sequences must already be in memory.
+*/
+
+NLM_EXTERN CharPtr GetSequenceByBsp (
+  BioseqPtr bsp
+);
+
+NLM_EXTERN CharPtr GetSequenceByIdOrAccnDotVer (
+  SeqIdPtr sip,
+  CharPtr accession,
+  Boolean is_na
+);
+
+NLM_EXTERN CharPtr GetSequenceByFeature (
+  SeqFeatPtr sfp
+);
+
+NLM_EXTERN CharPtr GetDNAbyAccessionDotVersion (
+  CharPtr accession
+);
+
+NLM_EXTERN BytePtr GetScoresbyAccessionDotVersion (
+  CharPtr accession,
+  Int4Ptr bsplength
+);
+
+NLM_EXTERN BytePtr GetScoresbySeqId (
+  SeqIdPtr sip,
+  Int4Ptr bsplength
+);
+
+/**************************************************************
+*
+*  Returns a protein molecular weight for a SeqLoc
+*    If it cannot calculate the value it returns -1.0
+*    If sequence contains X,B,U,*,orZ it fails
+*
+***************************************************************/
+NLM_EXTERN FloatHi MolWtForLoc (SeqLocPtr slp);
+
+
 #ifdef __cplusplus
 }
 #endif
