@@ -31,7 +31,7 @@
 *   
 * Version Creation Date: 11/3/93
 *
-* $Revision: 6.5 $
+* $Revision: 6.39 $
 *
 * File Description: Utilities for creating ASN.1 submissions
 *
@@ -42,6 +42,108 @@
 *
 *
 * $Log: subutil.h,v $
+* Revision 6.39  2001/12/12 16:01:47  kans
+* added MOLECULE_TYPE_TRANSCRIBED_RNA, SUBSRC_transgenic, SUBSRC_environmental_sample, and SUBSRC_isolation_source
+*
+* Revision 6.38  2001/12/11 16:50:41  kans
+* added TpaAssembly user object creation functions
+*
+* Revision 6.37  2001/12/10 13:47:44  kans
+* added MOLECULE_TYPE defines for CRNA and SNORNA
+*
+* Revision 6.36  2001/11/09 15:12:21  kans
+* moved collaborator to separate function per user object
+*
+* Revision 6.35  2001/11/09 13:21:08  kans
+* added collaborator field to reftrack user object
+*
+* Revision 6.34  2001/11/09 12:54:47  kans
+* added model evidence user object functions
+*
+* Revision 6.33  2001/07/29 16:23:58  kans
+* added AddOrganismToEntryEx to allow entry of taxonID
+*
+* Revision 6.32  2001/01/25 19:55:14  kans
+* added AddGenBankSetToSubmission for patents and other bulk submissions of unrelated sequences
+*
+* Revision 6.31  2000/11/15 23:18:00  kans
+* gene ontology user object functions
+*
+* Revision 6.30  2000/10/10 15:06:01  kans
+* added SUBSRC_endogenous_virus_name
+*
+* Revision 6.29  2000/09/28 11:41:33  ostell
+* added GENOME_endogenous_virus
+*
+* Revision 6.28  2000/09/20 12:32:54  kans
+* removed AddMethodToRefGeneTrackUserObject and AddSupportToRefGeneTrackUserObject
+*
+* Revision 6.27  2000/08/29 22:27:02  kans
+* reftrack support now has organism field
+*
+* Revision 6.26  2000/07/21 10:54:14  kans
+* added functions for adding Method and Support to RefGeneTrack user object
+*
+* Revision 6.25  2000/07/12 22:45:15  kans
+* added ORGMOD_old_lineage
+*
+* Revision 6.24  2000/06/20 17:17:28  kans
+* added orgmod.subtype authority through breed
+*
+* Revision 6.23  2000/06/14 16:41:20  kans
+* added SUBSRC_segment
+*
+* Revision 6.22  2000/03/10 18:35:46  kans
+* added CreateContigCloneUserObject for genome contig RefSeq records
+*
+* Revision 6.21  2000/02/26 23:55:34  kans
+* added AddPhrapGraphToSeqLit, using internal extended SeqLit to track, readjust graphs to current position of seqlit in delta seq - may need to move fields to public SeqLit if problems arise
+*
+* Revision 6.20  2000/02/24 23:15:00  kans
+* added AddSecondaryAccnToEntry
+*
+* Revision 6.19  2000/02/23 19:25:54  kans
+* AddFakeGapToDeltaSeq returns SeqLitPtr so program can set fuzz after SpreadGapsInDeltaSeq is no longer being called
+*
+* Revision 6.18  2000/02/23 18:28:39  kans
+* added AddFakeGapToDeltaSeq to set fuzz even if nonzero length is passed in
+*
+* Revision 6.17  2000/02/07 16:48:08  ostell
+* added AddDeltaSeqToNucProtEntry()
+*
+* Revision 6.16  1999/12/20 19:57:45  kans
+* added AddStatusToRefGeneTrackUserObject
+*
+* Revision 6.15  1999/12/14 17:32:45  ostell
+* changed protoplast to proplastid
+*
+* Revision 6.14  1999/10/05 20:36:01  kans
+* added more GENOME_ types (JO)
+*
+* Revision 6.13  1999/08/05 20:43:44  kans
+* added CreateSubmissionUserObject (JP)
+*
+* Revision 6.12  1999/03/17 22:49:38  kans
+* changed SUBSOURCE_ to SUBSRC_
+*
+* Revision 6.11  1999/03/17 20:04:40  kans
+* added SUBSOURCE_ and ORGMOD_ defines
+*
+* Revision 6.10  1998/10/02 17:48:12  kans
+* new parameters to AddAccessionToRefGeneTrackUserObject, and added CreateMrnaProteinLinkUserObject
+*
+* Revision 6.9  1998/09/29 15:34:20  kans
+* added comment for new choices in molinfo.completeness
+*
+* Revision 6.8  1998/09/23 16:41:49  kans
+* removed from and to from AddAccessionToRefGeneTrackUserObject
+*
+* Revision 6.7  1998/08/20 20:37:00  kans
+* add comment parameter to reftrack user object function
+*
+* Revision 6.6  1998/08/11 22:27:26  kans
+* added CreateRefGeneTrackUserObject and AddAccessionToRefGeneTrackUserObject
+*
 * Revision 6.5  1998/02/05 19:19:30  kans
 * added AddPhrapGraph
 *
@@ -432,6 +534,9 @@ NLM_EXTERN SeqEntryPtr AddPhySetToSubmission (
 	NCBISubPtr submission );
 
 NLM_EXTERN SeqEntryPtr AddMutSetToSubmission (
+	NCBISubPtr submission );
+
+NLM_EXTERN SeqEntryPtr AddGenBankSetToSubmission (
 	NCBISubPtr submission );
 
 			/*** Entry contains nucleotide and translated proteins ***/
@@ -454,6 +559,19 @@ NLM_EXTERN SeqEntryPtr AddSeqToNucProtEntry (   /** add unsegmented nuc or prot
 								  /** add segmented nuc or prot bioseq set */
 
 NLM_EXTERN SeqEntryPtr AddSegmentedSeqToNucProtEntry (
+	NCBISubPtr submission,
+	SeqEntryPtr nuc_prot_entry ,
+	CharPtr local_name ,
+	CharPtr genbank_locus ,
+	CharPtr genbank_accession ,
+	Int4 gi_number ,
+	Int2 molecule_class,
+	Int2 molecule_type ,
+	Int4 length ,
+	Int2 topology ,
+	Int2 strandedness );
+
+NLM_EXTERN SeqEntryPtr AddDeltaSeqToNucProtEntry (
 	NCBISubPtr submission,
 	SeqEntryPtr nuc_prot_entry ,
 	CharPtr local_name ,
@@ -485,6 +603,11 @@ NLM_EXTERN Boolean AddGapToDeltaSeq (
 	SeqEntryPtr delta_seq_entry,
 	Int4 length_of_gap );    /** 0 if not known */
 
+NLM_EXTERN SeqLitPtr AddFakeGapToDeltaSeq (
+	NCBISubPtr submission,
+	SeqEntryPtr delta_seq_entry,
+	Int4 length_of_gap );    /** returns slp so program can set lim - unk fuzz after empty gaps are spread */
+
 NLM_EXTERN SeqLitPtr AddLiteralToDeltaSeq (
 	NCBISubPtr submission,
 	SeqEntryPtr delta_seq_entry,
@@ -506,6 +629,9 @@ NLM_EXTERN SeqLitPtr AddLiteralToDeltaSeq (
 #define MOLECULE_TYPE_PEPTIDE 8
 #define MOLECULE_TYPE_OTHER_GENETIC_MATERIAL 9
 #define MOLECULE_TYPE_GENOMIC_MRNA_MIX 10
+#define MOLECULE_TYPE_CRNA 11
+#define MOLECULE_TYPE_SNORNA 12
+#define MOLECULE_TYPE_TRANSCRIBED_RNA 13
 
 #define TOPOLOGY_LINEAR 1
 #define TOPOLOGY_CIRCULAR 2
@@ -567,6 +693,11 @@ NLM_EXTERN Boolean AddTitleToEntry (
 	SeqEntryPtr entry ,
 	CharPtr title );
 
+NLM_EXTERN Boolean AddSecondaryAccnToEntry (
+NCBISubPtr submission,
+	SeqEntryPtr entry ,
+	CharPtr accn );
+
 /*****************************************************************
 *
 *   rules for long comments
@@ -606,6 +737,21 @@ NLM_EXTERN Boolean AddOrganismToEntryNew (
            ** AddOrganismToEntryNew() before calling
            ** SetGeneticCodeForEntry() **/
 
+NLM_EXTERN Boolean AddOrganismToEntryEx (
+	NCBISubPtr submission,
+	SeqEntryPtr entry ,
+	CharPtr scientific_name ,
+	CharPtr common_name ,
+	CharPtr virus_name ,
+	CharPtr strain ,
+	CharPtr synonym1,
+	CharPtr synonym2,
+	CharPtr synonym3,
+	CharPtr taxonomy,
+	Int4 taxid );
+
+           /** AddOrganismToEntryEx() allows taxonID to be entered **/
+
 NLM_EXTERN Boolean SetGeneticCodeForEntry (
 	NCBISubPtr submission,
         SeqEntryPtr entry,
@@ -653,6 +799,11 @@ NLM_EXTERN Boolean AddGenBankBlockToEntry (
 #define GENOME_cyanelle 12
 #define GENOME_proviral 13
 #define GENOME_virion 14
+#define GENOME_nucleomorph 15
+#define GENOME_apicoplast 16
+#define GENOME_leucoplast 17
+#define GENOME_proplastid 18
+#define GENOME_endogenous_virus 19
 
 /********************************************
 *  Genome describes the type of genome from which the DNA or gene for
@@ -673,6 +824,7 @@ NLM_EXTERN Boolean AddGenBankBlockToEntry (
         cyanelle (12) ,
         proviral (13) ,
         virion (14) } DEFAULT unknown ,
+         more types added, see GENOME_.. above
 **********************************************/
  
 NLM_EXTERN Boolean AddGenomeToEntry (
@@ -680,6 +832,36 @@ NLM_EXTERN Boolean AddGenomeToEntry (
 	SeqEntryPtr entry ,
 	Int2	 type );
 
+#define SUBSRC_chromosome 1
+#define SUBSRC_map 2
+#define SUBSRC_clone 3
+#define SUBSRC_subclone 4
+#define SUBSRC_haplotype 5
+#define SUBSRC_genotype 6
+#define SUBSRC_sex 7
+#define SUBSRC_cell_line 8
+#define SUBSRC_cell_type 9
+#define SUBSRC_tissue_type 10
+#define SUBSRC_clone_lib 11
+#define SUBSRC_dev_stage 12
+#define SUBSRC_frequency 13
+#define SUBSRC_germline 14
+#define SUBSRC_rearranged 15
+#define SUBSRC_lab_host 16
+#define SUBSRC_pop_variant 17
+#define SUBSRC_tissue_lib 18
+#define SUBSRC_plasmid_name 19
+#define SUBSRC_transposon_name 20
+#define SUBSRC_insertion_seq_name 21
+#define SUBSRC_plastid_name 22
+#define SUBSRC_country 23
+#define SUBSRC_segment 24
+#define SUBSRC_endogenous_virus_name 25
+#define SUBSRC_transgenic 26
+#define SUBSRC_environmental_sample 27
+#define SUBSRC_isolation_source 28
+#define SUBSRC_other 255
+
 /*********************************************
 *  SubSource defines subclasses of source material
 *    (also see OrgMod below for subclasses of organism names)
@@ -718,6 +900,40 @@ NLM_EXTERN Boolean AddSubSourceToEntry (
 	Int2	 type ,
 	CharPtr value);
 
+#define ORGMOD_strain 2
+#define ORGMOD_substrain 3
+#define ORGMOD_type 4
+#define ORGMOD_subtype 5
+#define ORGMOD_variety 6
+#define ORGMOD_serotype 7
+#define ORGMOD_serogroup 8
+#define ORGMOD_serovar 9
+#define ORGMOD_cultivar 10
+#define ORGMOD_pathovar 11
+#define ORGMOD_chemovar 12
+#define ORGMOD_biovar 13
+#define ORGMOD_biotype 14
+#define ORGMOD_group 15
+#define ORGMOD_subgroup 16
+#define ORGMOD_isolate 17
+#define ORGMOD_common 18
+#define ORGMOD_acronym 19
+#define ORGMOD_dosage 20
+#define ORGMOD_nat_host 21
+#define ORGMOD_sub_species 22
+#define ORGMOD_specimen_voucher 23
+#define ORGMOD_authority 24
+#define ORGMOD_forma 25
+#define ORGMOD_forma_specialis 26
+#define ORGMOD_ecotype 27
+#define ORGMOD_synonym 28
+#define ORGMOD_anamorph 29
+#define ORGMOD_teleomorph 30
+#define ORGMOD_breed 31
+#define ORGMOD_old_lineage 253
+#define ORGMOD_old_name 254
+#define ORGMOD_other 255 
+
 /*********************************************
 *  OrgMod defines subclasses of organism names
 *    (also see SubSource above for subclasses of source material)
@@ -829,9 +1045,11 @@ NLM_EXTERN Boolean AddTechToEntry (
       unknown (0) ,
       complete (1) ,                   -- complete biological entity
       partial (2) ,                    -- partial but no details given
-      no-left (3),                     -- missing 5' or NH3 end
-      no-right (4) ,                   -- missing 3' or COOH end
-      no-ends (5) ,                    -- missing both ends
+      no-left (3),                     -- KNOWN missing 5' or NH3 end
+      no-right (4) ,                   -- KNOWN missing 3' or COOH end
+      no-ends (5) ,                    -- KNOWN missing both ends
+      has-left (6) ,                   -- KNOWN has complete 5' or NH3 end
+      has-right (7) ,                  -- KNOWN has complete 3' or COOH end
       other (255) } DEFAULT unknown }
 
 *******************************************/
@@ -1308,6 +1526,74 @@ NLM_EXTERN Boolean AddPhrapGraph (
 	CharPtr local_name ,
 	BytePtr phrap_values );
 
+NLM_EXTERN Boolean AddPhrapGraphToSeqLit (
+	NCBISubPtr submission,
+	SeqLitPtr slp ,
+	BytePtr phrap_values );
+
+/* internal functions for reference gene project */
+NLM_EXTERN UserObjectPtr CreateRefGeneTrackUserObject (void);
+NLM_EXTERN void AddStatusToRefGeneTrackUserObject (UserObjectPtr uop, CharPtr status);
+NLM_EXTERN void AddCuratorToRefGeneTrackUserObject (UserObjectPtr uop, CharPtr collaborator);
+NLM_EXTERN void AddAccessionToRefGeneTrackUserObject (UserObjectPtr uop, CharPtr field,
+                                                      CharPtr accn, Int4 gi,
+                                                      Boolean sequenceChange,
+                                                      Boolean annotationChange,
+                                                      CharPtr comment);
+
+/* experimental function to associate mRNA with protein product in cases of alt splicing */
+NLM_EXTERN UserObjectPtr CreateMrnaProteinLinkUserObject (BioseqPtr protbsp);
+
+/* vector screen, validator count, general submission comment user object (JP) */
+NLM_EXTERN UserObjectPtr CreateSubmissionUserObject (CharPtr univecComment,
+                                                     CharPtr additionalComment,
+                                                     Int4 validatorErrorCount,
+                                                     Int4 validatorHashCode,
+                                                     Boolean isCloningVector);
+
+/* clone name and ID for genomic contig RefSeq records */
+NLM_EXTERN UserObjectPtr CreateContigCloneUserObject (CharPtr name, Int4 ID);
+
+/* gene ontology process, component, and function user object */
+NLM_EXTERN UserObjectPtr CreateGeneOntologyUserObject (
+  void
+);
+NLM_EXTERN void AddToGeneOntologyUserObject (
+  UserObjectPtr uop,
+  CharPtr type,
+  CharPtr text,
+  Int4 goid,
+  Int4 pmid,
+  CharPtr evidence
+);
+
+/* model evidence user object */
+NLM_EXTERN UserObjectPtr CreateModelEvidenceUserObject (
+  CharPtr method,
+  CharPtr contigParent
+);
+NLM_EXTERN void AddMrnaOrESTtoModelEvidence (
+  UserObjectPtr uop,
+  CharPtr type,
+  CharPtr accn,
+  Int4 length,
+  Int4 gaplen
+);
+NLM_EXTERN UserFieldPtr FindModelEvidenceField (
+  UserObjectPtr uop,
+  CharPtr type
+);
+
+/* third party accession list user object manipulation */
+NLM_EXTERN UserObjectPtr CreateTpaAssemblyUserObject (
+  void
+);
+NLM_EXTERN void AddAccessionToTpaAssemblyUserObject (
+  UserObjectPtr uop,
+  CharPtr accn
+);
+
+
 #ifdef __cplusplus
 }
 #endif
