@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 7/12/91
 *
-* $Revision: 6.4 $
+* $Revision: 6.16 $
 *
 * File Description:  various sequence objects to fasta output
 *
@@ -39,6 +39,42 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: tofasta.h,v $
+* Revision 6.16  2001/08/31 15:29:54  kans
+* added CreateDefLineEx to ignore existing title descriptor, force generation of protein title computationally
+*
+* Revision 6.15  2001/08/17 13:27:26  kans
+* ClearProteinTitlesInNucProts, do not do it for outside protein databases
+*
+* Revision 6.14  2001/06/25 23:47:48  kans
+* added InstantiateProteinTitles and ClearProteinTitles
+*
+* Revision 6.13  2001/02/27 21:52:37  madden
+* Added BioseqToFastaDump and FastaDumpFileFunc for dumping BLAST db in FASTA
+*
+* Revision 6.12  2000/10/12 16:03:58  kans
+* added SeqEntryToFastaEx, printid_general field to MyFsa structure, to support processing software that requires gnl ID in FASTA defline even in the presence of higher-priority ref ID
+*
+* Revision 6.11  2000/07/20 17:48:28  dondosha
+* Added function FastaToSeqBuffForDb analogous to FastaToSeqEntryForDbtofasta.c
+*
+* Revision 6.10  2000/05/30 19:44:45  ostell
+* added FastaSeqLineEx() with another parameter, do_virtual
+*
+* Revision 6.9  2000/04/03 22:09:26  kans
+* added ClearGenBankKeywords for RefSeq processing
+*
+* Revision 6.8  2000/03/29 22:00:23  kans
+* added NC_Cleanup function used internally for genome RefSeq processing
+*
+* Revision 6.7  2000/02/17 17:20:58  sicotte
+* Add Reading of Lowercase Characters as SeqLoc for inputting masking informatioin. Use FastaToSeqEntryForDb or FastaToSeqEntryInternalEx
+*
+* Revision 6.6  2000/02/04 16:38:29  kans
+* added FastaToSeqEntryForDb and FastaToSeqEntryInternalEx, giving control over generation of unique SeqID
+*
+* Revision 6.5  1999/09/20 18:37:55  shavirin
+* Added definition of the function Int4 GetOrderBySeqId().
+*
 * Revision 6.4  1998/02/23 16:51:27  egorov
 * Changes to make the tofasta.c independent on readdb.h
 *
@@ -146,6 +182,7 @@ typedef struct myfsa {   /* struct used for fasta searches */
     Boolean no_sequence;  /* used to disable sequence printing */
     Uint1	code;	/* coding of sequence */
     Boolean	formatdb; /* TRUE, if is used in formatdb */
+    Boolean printid_general; /* show gi and gnl for SeqId */
 } MyFsa, PNTR MyFsaPtr;
 
 typedef struct tofasta {
@@ -161,7 +198,11 @@ typedef struct tofasta {
 NLM_EXTERN Boolean BioseqRawToFastaExtra PROTO((BioseqPtr bsp, FILE *fp, Int2 line_length));
 NLM_EXTERN Boolean BioseqRawToFasta PROTO((BioseqPtr bsp, FILE * fp, Boolean is_na));
 NLM_EXTERN Boolean SeqEntryToFasta PROTO((SeqEntryPtr sep, FILE * fp, Boolean is_na));
+NLM_EXTERN Boolean SeqEntryToFastaEx PROTO((SeqEntryPtr sep, FILE * fp, Boolean is_na, Boolean printid_general));
 NLM_EXTERN Boolean BioseqToFasta PROTO((BioseqPtr bsp, FILE *fp, Boolean is_na));
+NLM_EXTERN Boolean BioseqToFastaDump PROTO((BioseqPtr bsp, FILE *fp, Boolean is_na));
+
+
 
 NLM_EXTERN void	SeqEntryFasta PROTO ((SeqEntryPtr sep, Pointer data,
                                      Int4 index, Int2 indent));
@@ -195,6 +236,18 @@ NLM_EXTERN Boolean FastaFileFunc PROTO((BioseqPtr bsp, Int2 key,
                                         CharPtr buf, Uint4 buflen, Pointer data));
 
 
+/*****************************************************************************
+*
+*   FastaDumpFileFunc(key, buf, data)
+*       standard "write to file" callback
+*
+*       Used for BLAST (FASTA) databases.  If the defline is
+*       longer than buflen, then check that an ID is not
+*       truncated in the middle.
+*
+*****************************************************************************/
+NLM_EXTERN Boolean FastaDumpFileFunc PROTO((BioseqPtr bsp, Int2 key, CharPtr buf, Uint4 buflen, Pointer data));
+
 /*****************************************************************************
 *
 *   Reads a Fasta File into a SeqEntry structure
@@ -229,9 +282,30 @@ NLM_EXTERN SeqEntryPtr FastaToSeqBuff PROTO((CharPtr buffer,
 NLM_EXTERN SeqEntryPtr FastaToSeqBuffEx(CharPtr buffer, CharPtr PNTR last_char, 
                              Boolean is_na, CharPtr PNTR errormsg,
                              Boolean parseSeqId);
+NLM_EXTERN SeqEntryPtr FastaToSeqBuffForDb(CharPtr buffer, CharPtr PNTR last_char, 
+                             Boolean is_na, CharPtr PNTR errormsg,
+                             Boolean parseSeqId, CharPtr prefix, Int2Ptr ctrptr,
+			     SeqLocPtr PNTR mask_ptr);
 NLM_EXTERN SeqEntryPtr FastaToSeqEntryEx (FILE *fp, Boolean is_na, 
                                CharPtr PNTR errormsg,
                                Boolean parseSeqId); 
+NLM_EXTERN SeqEntryPtr FastaToSeqEntryForDb ( FILE *fp, Boolean is_na,
+                               CharPtr PNTR errormsg,
+                               Boolean parseSeqId,
+                               CharPtr prefix, Int2Ptr ctrptr, 
+                               SeqLocPtr PNTR mask_ptr);
+
+/********* DEFINES for input type *********/
+
+#define FASTA_MEM_IO  1   /* type of reading from buffer in memory */
+#define FASTA_FILE_IO 2   /* type of reading from file */
+
+NLM_EXTERN SeqEntryPtr FastaToSeqEntryInternalEx ( VoidPtr input,
+                               Int4 type, CharPtr PNTR next_char,
+                               Boolean is_na, CharPtr PNTR errormsg,
+                               Boolean parseSeqId, CharPtr special_symbol,
+                               CharPtr prefix, Int2Ptr ctrptr,
+                               SeqLocPtr PNTR mask_ptr);
 
 /*****************************************************************************
 *
@@ -294,6 +368,7 @@ NLM_EXTERN Boolean FastaId PROTO((BioseqPtr bsp, CharPtr buf, Int2 buflen));
 NLM_EXTERN Boolean FastaDefLine PROTO((BioseqPtr bsp, CharPtr buf, Int2 buflen, CharPtr accession, CharPtr organism, Uint1 tech));
 
 NLM_EXTERN Boolean CreateDefLine PROTO((ItemInfoPtr dip, BioseqPtr bsp, CharPtr buf, Int2 buflen, Uint1 tech, CharPtr accession, CharPtr organism));
+NLM_EXTERN Boolean CreateDefLineEx (ItemInfoPtr iip, BioseqPtr bsp, CharPtr buf, Int2 buflen, Uint1 tech, CharPtr accession, CharPtr organism, Boolean ignoreTitle);
 /*****************************************************************************
 *
 *   FastaSeqPort(bsp, is_na, do_virtual)
@@ -310,10 +385,46 @@ NLM_EXTERN SeqPortPtr FastaSeqPort PROTO((BioseqPtr bsp, Boolean is_na,
 *     fills buf with linelen bases
 *     assumes buf[linelen] = '\0'
 *     returns FALSE when no more residues to print
+*     here for backward compatibility. Is the same as calling
+*      FastaSeqLineEx() with do_virtual = FALSE;
 *
 *****************************************************************************/
 NLM_EXTERN Boolean FastaSeqLine PROTO((SeqPortPtr spp, CharPtr buf, Int2 linelen, Boolean is_na));
 
+/****************************************************************************
+*
+*  FastaSeqLineEx(spp, buf, linelen, is_na, do_virtual)
+*
+*    adds a parameter to indicate if virtual sequences should be shown
+*     this is necessary in order to avoid showing SEQPORT_VIRT return
+*     (gap of 0 length) as '-', when do_virtual is TRUE. 
+*
+*****************************************************************************/
+NLM_EXTERN Boolean FastaSeqLineEx(SeqPortPtr spp, CharPtr buf, Int2 linelen, Boolean is_na, Boolean
+do_virtual);
+
+Int4 GetOrderBySeqId(Int4 choice, Boolean is_prot);
+
+/*****************************************************************************
+*
+*   NC_Cleanup (entityID, ptr) and ClearGenBankKeywords (entityID, ptr)
+*     internal functions for genome RefSeq processing
+*
+*****************************************************************************/
+NLM_EXTERN void NC_Cleanup (Uint2 entityID, Pointer ptr);
+NLM_EXTERN void ClearGenBankKeywords (Uint2 entityID, Pointer ptr);
+
+/*****************************************************************************
+*
+*   InstantiateProteinTitles (entityID, ptr) and ClearProteinTitles (entityID, ptr)
+*     allows proteins titles to be kept as Seq_descr_title rather than always
+*     being generated on the fly
+*
+*****************************************************************************/
+NLM_EXTERN void InstantiateProteinTitles (Uint2 entityID, Pointer ptr);
+NLM_EXTERN void ClearProteinTitlesInNucProts (Uint2 entityID, Pointer ptr);
+
+
 #ifdef __cplusplus
 }
 #endif
