@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 10/15/91
 *
-* $Revision: 6.1 $
+* $Revision: 6.8 $
 *
 * File Description:  conversion to medlars format
 *
@@ -40,6 +40,27 @@
 *
 *
 * $Log: tomedlin.c,v $
+* Revision 6.8  2001/10/29 20:37:06  kans
+* MakeAuthorString for structured authors
+*
+* Revision 6.7  2001/10/22 13:37:12  kans
+* break up huge author list
+*
+* Revision 6.6  2001/10/15 12:24:05  kans
+* MedlineEntryToDataFile breaks up long mesh, substance, and xref lists
+*
+* Revision 6.5  1999/10/26 20:17:04  kans
+* allocate separate string for abstract since some may be very long
+*
+* Revision 6.4  1999/10/01 17:20:08  kans
+* fixed stack overflow found by DV
+*
+* Revision 6.3  1999/03/11 23:32:08  kans
+* sprintf casts
+*
+* Revision 6.2  1998/06/12 20:05:50  kans
+* fixed unix compiler warnings
+*
 * Revision 6.1  1997/12/02 17:42:29  kans
 * added cast in sprintf
 *
@@ -140,7 +161,6 @@ static CharPtr  pos;
 
 static void ClearString PROTO((void));
 static void AddString PROTO((CharPtr string));
-static void AddChar PROTO((Char ch));
 static CharPtr CleanAuthorString PROTO((CharPtr auth));
 static CharPtr CDECL StrngAppend VPROTO((CharPtr first, ...));
 
@@ -160,17 +180,6 @@ static void AddString (CharPtr string)
   *pos = '\0';
 }
 
-static void AddChar (Char ch)
-
-{
-  Char  str [3];
-
-  str [0] = ch;
-  str [1] = '\0';
-  pos = StringMove (pos, str);
-  *pos = '\0';
-}
-
 static ColData table [2] = {{0, 6, 0, 'l', TRUE, TRUE, FALSE},
                             {0, 72, 0, 'l', TRUE, TRUE, TRUE}};
 
@@ -183,25 +192,29 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
 {
   CharPtr         abstract;
   AffilPtr        affil;
-  AuthListPtr     authors;
+  AuthListPtr     authors = NULL;
   CitArtPtr       cit;
   CitJourPtr      citjour;
-  DatePtr         date;
+  Int2            count;
+  DatePtr         date = NULL;
   ValNodePtr      gene;
   Int2            i;
-  ImprintPtr      imp;
-  CharPtr         issue;
+  ImprintPtr      imp = NULL;
+  CharPtr         issue = NULL;
+  size_t          len;
   MedlineMeshPtr  mesh;
   ValNodePtr      names;
   CharPtr         p;
-  CharPtr         pages;
+  CharPtr         pages = NULL;
   ParData         para;
+  CharPtr         ptr;
   ValNodePtr      qual;
   Boolean         rsult;
   Char            str [32];
   MedlineRnPtr    substance;
   ValNodePtr      title;
-  CharPtr         volume;
+  CharPtr         tmp;
+  CharPtr         volume = NULL;
   ValNodePtr      xref;
   CharPtr         lpTemp;
   CharPtr         lpMedAbbr = NULL;
@@ -214,7 +227,7 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
     para.openSpace = FALSE;
     ClearString ();
     AddString ("UI  -\t");
-    sprintf (str, "%ld", mep->uid);
+    sprintf (str, "%ld", (long) mep->uid);
     AddString (str);
     AddString ("\n");
     rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
@@ -223,7 +236,7 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
     if ( mep->pmid > 0 ) {
       ClearString ();
       AddString ("PM  -\t");
-      sprintf (str, "%ld", mep->pmid);
+      sprintf (str, "%ld", (long) mep->pmid);
       AddString (str);
       AddString ("\n");
       rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
@@ -233,8 +246,7 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
     if ( mep->pub_type != NULL ) {
       ClearString ();
       AddString ("PT  -\t");
-      sprintf (str, "%s", (CharPtr) mep->pub_type->data.ptrvalue);
-      AddString (str);
+      AddString ((CharPtr) mep->pub_type->data.ptrvalue);
       AddString ("\n");
       rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
       ClearString ();
@@ -245,28 +257,50 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
       authors = cit->authors;
       if (authors != NULL && (authors->choice == 2 || authors->choice == 3)) {
         names = authors->names;
+        count = 0;
         while (names != NULL) {
+          if (count >= 20) {
+            rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
+            ClearString ();
+            count = 0;
+          }
           AddString ("AU  -\t");
           AddString (names->data.ptrvalue);
           AddString ("\n");
           names = names->next;
+          count++;
         }
       }
+      rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
+      ClearString ();
       title = cit->title;
+      count = 0;
       while (title != NULL) {
+        if (count >= 20) {
+          rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
+          ClearString ();
+          count = 0;
+        }
         if (title->choice == 1) {
           AddString ("TI  -\t");
           AddString (title->data.ptrvalue);
           AddString ("\n");
         }
         title = title->next;
+        count++;
       }
+      rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
+      ClearString ();
     }
 
-    rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
-    ClearString ();
     mesh = mep->mesh;
+    count = 0;
     while (mesh != NULL) {
+      if (count >= 20) {
+        rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
+        ClearString ();
+        count = 0;
+      }
       AddString ("MH  -\t");
       if (mesh->mp) {
         AddString ("*");
@@ -288,11 +322,18 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
       }
       AddString ("\n");
       mesh = mesh->next;
+      count++;
     }
     rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
     ClearString ();
     substance = mep->substance;
+    count = 0;
     while (substance != NULL) {
+      if (count >= 20) {
+        rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
+        ClearString ();
+        count = 0;
+      }
       AddString ("RN  -\t");
       switch (substance->type) {
         case 0:
@@ -318,6 +359,7 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
       }
       AddString ("\n");
       substance = substance->next;
+      count++;
     }
     rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
     ClearString ();
@@ -329,7 +371,13 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
       gene = gene->next;
     }
     xref = mep->xref;
+    count = 0;
     while (xref != NULL) {
+      if (count >= 20) {
+        rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
+        ClearString ();
+        count = 0;
+      }
       AddString ("SI  -\t");
       switch (xref->choice) {
         case 1:
@@ -378,6 +426,7 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
       AddString (xref->data.ptrvalue);
       AddString ("\n");
       xref = xref->next;
+      count++;
     }
     rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
     ClearString ();
@@ -459,11 +508,14 @@ NLM_EXTERN Boolean MedlineEntryToDataFile (MedlineEntryPtr mep, FILE *fp)
     ClearString ();
     abstract = mep->abstract;
     if (abstract != NULL) {
-      AddString ("AB  -\t");
-      AddString (abstract);
-      AddString ("\n");
+      len = StringLen (abstract);
+      tmp = (CharPtr) MemNew (sizeof (Char) * (len + 10));
+      ptr = StringMove (tmp, "AB  -\t");
+      ptr = StringMove (ptr, abstract);
+      ptr = StringMove (ptr, "\n");
+      rsult = (Boolean) (SendTextToFile (fp, tmp, &para, table) && rsult);
+      MemFree (tmp);
     }
-    rsult = (Boolean) (SendTextToFile (fp, buffer, &para, table) && rsult);
     ClearString ();
     if (cit != NULL && authors != NULL) {
       affil = authors->affil;
@@ -679,25 +731,79 @@ static CharPtr CleanAuthorString (CharPtr auth)
   return str;
 }
 
+static CharPtr MakeAuthorString (
+  CharPtr name,
+  CharPtr initials,
+  CharPtr suffix
+)
+
+{
+  Char     ch;
+  size_t   len;
+  CharPtr  ptr;
+  CharPtr  str;
+  CharPtr  tmp;
+
+  if (name == NULL) return NULL;
+
+  len = StringLen (name) + StringLen (initials) * 3 + StringLen (suffix);
+  str = MemNew (sizeof (Char) * (len + 4));
+  if (str == NULL) return NULL;
+
+  tmp = str;
+
+  ptr = initials;
+  if (! StringHasNoText (initials)) {
+    ch = *ptr;
+    while (ch != '\0') {
+      if (ch == '-') {
+        *tmp = '-';
+        tmp++;
+      } else if (ch != '.') {
+        *tmp = ch;
+        tmp++;
+        *tmp = '.';
+        tmp++;
+        *tmp = ' ';
+        tmp++;
+      }
+      ptr++;
+      ch = *ptr;
+    }
+    *tmp = '\0';
+  }
+
+  tmp = StringMove (tmp, name);
+  if (! StringHasNoText (suffix)) {
+    tmp = StringMove (tmp, " ");
+    tmp = StringMove (tmp, suffix);
+  }
+
+  return str;
+}
+
 NLM_EXTERN MedlinePtr ParseMedline (MedlineEntryPtr mep)
 
 {
   AffilPtr         affil;
+  AuthorPtr        ap;
   AuthListPtr      authors;
   CharPtr          chptr;
   CitArtPtr        cit;
   CitJourPtr       citjour;
+  CharPtr          curr;
   DatePtr          date;
   ValNodePtr       gene;
   ImprintPtr       imp;
   CharPtr          last;
   MedlineMeshPtr   mesh;
-  MedlinePtr       mPtr;
+  MedlinePtr       mPtr = NULL;
   ValNodePtr       names;
+  NameStdPtr       nsp;
+  PersonIdPtr      pid;
   ValNodePtr       qual;
   Char             str [32];
   MedlineRnPtr     substance;
-  CharPtr          this;
   ValNodePtr       title;
 
   buffer = MemNew (BUFSIZE);
@@ -705,7 +811,7 @@ NLM_EXTERN MedlinePtr ParseMedline (MedlineEntryPtr mep)
     ClearString ();
     mPtr = MemNew (sizeof (MedlineData));
     if (mPtr != NULL && mep != NULL) {
-      sprintf (str, "%1ld", mep->uid);
+      sprintf (str, "%1ld", (long) mep->uid);
       mPtr->uid = StringSave (str);
       cit = mep->cit;
       if (cit != NULL) {
@@ -765,23 +871,45 @@ NLM_EXTERN MedlinePtr ParseMedline (MedlineEntryPtr mep)
         }
         authors = cit->authors;
         if (authors != NULL) {
-          if (authors->choice == 2 || authors->choice == 3) {
+          if (authors->choice == 1 || authors->choice == 2 || authors->choice == 3) {
             names = authors->names;
             while (names != NULL) {
               last = mPtr->authors;
+              curr = NULL;
+              if (authors->choice == 1) {
+                ap = (AuthorPtr) names->data.ptrvalue;
+                if (ap != NULL) {
+                  pid = ap->name;
+                  if (pid != NULL) {
+                    if (pid->choice == 2) {
+                      nsp = (NameStdPtr) pid->data;
+                      if (nsp != NULL) {
+                        if (! StringHasNoText (nsp->names [0])) {
+                          curr = MakeAuthorString (nsp->names [0], nsp->names [4], nsp->names [5]);
+                        } else if (! StringHasNoText (nsp->names [3])) {
+                          curr = MakeAuthorString (nsp->names [3], NULL, NULL);
+                        }
+                      }
+                    } else if (pid->choice == 3 || pid->choice == 4) {
+                      curr = MakeAuthorString ((CharPtr) pid->data, NULL, NULL);
+                    }
+                  }
+                }
+              } else if (authors->choice == 2 || authors->choice == 3) {
+                curr = CleanAuthorString ((CharPtr) names->data.ptrvalue);
+              }
               if (last != NULL) {
-                this = CleanAuthorString ((CharPtr) names->data.ptrvalue);
                 if (names->next != NULL) {
                   mPtr->authors = StrngAppend (last, (CharPtr) ", ",
-                                                (CharPtr) this, (CharPtr) NULL);
+                                                (CharPtr) curr, (CharPtr) NULL);
                 } else {
                   mPtr->authors = StrngAppend (last, (CharPtr) " & ",
-                                                (CharPtr) this, (CharPtr) NULL);
+                                                (CharPtr) curr, (CharPtr) NULL);
                 }
                 last = MemFree (last);
-                this = MemFree (this);
+                curr = MemFree (curr);
               } else {
-                mPtr->authors = CleanAuthorString ((CharPtr) names->data.ptrvalue);
+                mPtr->authors = curr;
               }
               names = names->next;
             }
@@ -911,9 +1039,12 @@ static ColData  mshFmt [1] = {{0, 80, 0, 'l', FALSE, FALSE, TRUE}};
 static Boolean MedlineEntryToDocOrAbsFile (MedlineEntryPtr mep, FILE *fp, Boolean showMesh)
 
 {
+  size_t      len;
   MedlinePtr  mPtr;
   ParData     para;
+  CharPtr     ptr;
   Boolean     rsult;
+  CharPtr     tmp;
 
   rsult = TRUE;
   if (fp != NULL && mep != NULL) {
@@ -959,10 +1090,12 @@ static Boolean MedlineEntryToDocOrAbsFile (MedlineEntryPtr mep, FILE *fp, Boolea
           ClearString ();
         }
         if (mPtr->abstract != NULL) {
-          AddString (mPtr->abstract);
-          AddString ("\n");
-          rsult = (Boolean) (SendTextToFile (fp, buffer, NULL, NULL) && rsult);
-          ClearString ();
+          len = StringLen (mPtr->abstract);
+          tmp = (CharPtr) MemNew (sizeof (Char) * (len + 10));
+          ptr = StringMove (tmp, mPtr->abstract);
+          ptr = StringMove (ptr, "\n");
+          rsult = (Boolean) (SendTextToFile (fp, tmp, NULL, NULL) && rsult);
+          MemFree (tmp);
         }
         if (showMesh) {
           if (mPtr->mesh != NULL) {
@@ -1185,7 +1318,6 @@ static void
         cInputType = cInputType;
       }
 
-    /* Qualifier List. A list of two-letter subheading (qualifier)
     /* Qualifier List. A list of two-letter subheading (qualifier)
        abbreviations, separated by slashes if there is more than
        one value. Each subheading abbreviation may have an *
