@@ -9,8 +9,8 @@
 #include <asn.h>
 #endif
 
-static char * asnfilename = "asnseq.h60";
-static AsnValxNode avnx[136] = {
+static char * asnfilename = "asnseq.h64";
+static AsnValxNode avnx[143] = {
     {20,"unknown" ,0,0.0,&avnx[1] } ,
     {20,"genomic" ,1,0.0,&avnx[2] } ,
     {20,"pre-mRNA" ,2,0.0,&avnx[3] } ,
@@ -66,100 +66,107 @@ static AsnValxNode avnx[136] = {
     {20,"aligns" ,2,0.0,NULL } ,
     {20,"seq" ,0,0.0,&avnx[54] } ,
     {20,"sites" ,1,0.0,&avnx[55] } ,
-    {20,"feats" ,2,0.0,NULL } ,
+    {20,"feats" ,2,0.0,&avnx[56] } ,
+    {20,"no-target" ,3,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[58] } ,
-    {20,"genomic" ,1,0.0,&avnx[59] } ,
-    {20,"pre-RNA" ,2,0.0,&avnx[60] } ,
-    {20,"mRNA" ,3,0.0,&avnx[61] } ,
-    {20,"rRNA" ,4,0.0,&avnx[62] } ,
-    {20,"tRNA" ,5,0.0,&avnx[63] } ,
-    {20,"snRNA" ,6,0.0,&avnx[64] } ,
-    {20,"scRNA" ,7,0.0,&avnx[65] } ,
-    {20,"peptide" ,8,0.0,&avnx[66] } ,
-    {20,"other-genetic" ,9,0.0,&avnx[67] } ,
-    {20,"genomic-mRNA" ,10,0.0,&avnx[68] } ,
+    {20,"unknown" ,0,0.0,&avnx[59] } ,
+    {20,"genomic" ,1,0.0,&avnx[60] } ,
+    {20,"pre-RNA" ,2,0.0,&avnx[61] } ,
+    {20,"mRNA" ,3,0.0,&avnx[62] } ,
+    {20,"rRNA" ,4,0.0,&avnx[63] } ,
+    {20,"tRNA" ,5,0.0,&avnx[64] } ,
+    {20,"snRNA" ,6,0.0,&avnx[65] } ,
+    {20,"scRNA" ,7,0.0,&avnx[66] } ,
+    {20,"peptide" ,8,0.0,&avnx[67] } ,
+    {20,"other-genetic" ,9,0.0,&avnx[68] } ,
+    {20,"genomic-mRNA" ,10,0.0,&avnx[69] } ,
+    {20,"cRNA" ,11,0.0,&avnx[70] } ,
+    {20,"snoRNA" ,12,0.0,&avnx[71] } ,
+    {20,"transcribed-RNA" ,13,0.0,&avnx[72] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[71] } ,
-    {20,"standard" ,1,0.0,&avnx[72] } ,
-    {20,"est" ,2,0.0,&avnx[73] } ,
-    {20,"sts" ,3,0.0,&avnx[74] } ,
-    {20,"survey" ,4,0.0,&avnx[75] } ,
-    {20,"genemap" ,5,0.0,&avnx[76] } ,
-    {20,"physmap" ,6,0.0,&avnx[77] } ,
-    {20,"derived" ,7,0.0,&avnx[78] } ,
-    {20,"concept-trans" ,8,0.0,&avnx[79] } ,
-    {20,"seq-pept" ,9,0.0,&avnx[80] } ,
-    {20,"both" ,10,0.0,&avnx[81] } ,
-    {20,"seq-pept-overlap" ,11,0.0,&avnx[82] } ,
-    {20,"seq-pept-homol" ,12,0.0,&avnx[83] } ,
-    {20,"concept-trans-a" ,13,0.0,&avnx[84] } ,
-    {20,"htgs-1" ,14,0.0,&avnx[85] } ,
-    {20,"htgs-2" ,15,0.0,&avnx[86] } ,
-    {20,"htgs-3" ,16,0.0,&avnx[87] } ,
+    {20,"unknown" ,0,0.0,&avnx[75] } ,
+    {20,"standard" ,1,0.0,&avnx[76] } ,
+    {20,"est" ,2,0.0,&avnx[77] } ,
+    {20,"sts" ,3,0.0,&avnx[78] } ,
+    {20,"survey" ,4,0.0,&avnx[79] } ,
+    {20,"genemap" ,5,0.0,&avnx[80] } ,
+    {20,"physmap" ,6,0.0,&avnx[81] } ,
+    {20,"derived" ,7,0.0,&avnx[82] } ,
+    {20,"concept-trans" ,8,0.0,&avnx[83] } ,
+    {20,"seq-pept" ,9,0.0,&avnx[84] } ,
+    {20,"both" ,10,0.0,&avnx[85] } ,
+    {20,"seq-pept-overlap" ,11,0.0,&avnx[86] } ,
+    {20,"seq-pept-homol" ,12,0.0,&avnx[87] } ,
+    {20,"concept-trans-a" ,13,0.0,&avnx[88] } ,
+    {20,"htgs-1" ,14,0.0,&avnx[89] } ,
+    {20,"htgs-2" ,15,0.0,&avnx[90] } ,
+    {20,"htgs-3" ,16,0.0,&avnx[91] } ,
+    {20,"fli-cdna" ,17,0.0,&avnx[92] } ,
+    {20,"htgs-0" ,18,0.0,&avnx[93] } ,
+    {20,"htc" ,19,0.0,&avnx[94] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[90] } ,
-    {20,"complete" ,1,0.0,&avnx[91] } ,
-    {20,"partial" ,2,0.0,&avnx[92] } ,
-    {20,"no-left" ,3,0.0,&avnx[93] } ,
-    {20,"no-right" ,4,0.0,&avnx[94] } ,
-    {20,"no-ends" ,5,0.0,&avnx[95] } ,
+    {20,"unknown" ,0,0.0,&avnx[97] } ,
+    {20,"complete" ,1,0.0,&avnx[98] } ,
+    {20,"partial" ,2,0.0,&avnx[99] } ,
+    {20,"no-left" ,3,0.0,&avnx[100] } ,
+    {20,"no-right" ,4,0.0,&avnx[101] } ,
+    {20,"no-ends" ,5,0.0,&avnx[102] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"not-set" ,0,0.0,&avnx[98] } ,
-    {20,"virtual" ,1,0.0,&avnx[99] } ,
-    {20,"raw" ,2,0.0,&avnx[100] } ,
-    {20,"seg" ,3,0.0,&avnx[101] } ,
-    {20,"const" ,4,0.0,&avnx[102] } ,
-    {20,"ref" ,5,0.0,&avnx[103] } ,
-    {20,"consen" ,6,0.0,&avnx[104] } ,
-    {20,"map" ,7,0.0,&avnx[105] } ,
-    {20,"delta" ,8,0.0,&avnx[106] } ,
+    {20,"not-set" ,0,0.0,&avnx[105] } ,
+    {20,"virtual" ,1,0.0,&avnx[106] } ,
+    {20,"raw" ,2,0.0,&avnx[107] } ,
+    {20,"seg" ,3,0.0,&avnx[108] } ,
+    {20,"const" ,4,0.0,&avnx[109] } ,
+    {20,"ref" ,5,0.0,&avnx[110] } ,
+    {20,"consen" ,6,0.0,&avnx[111] } ,
+    {20,"map" ,7,0.0,&avnx[112] } ,
+    {20,"delta" ,8,0.0,&avnx[113] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"not-set" ,0,0.0,&avnx[108] } ,
-    {20,"dna" ,1,0.0,&avnx[109] } ,
-    {20,"rna" ,2,0.0,&avnx[110] } ,
-    {20,"aa" ,3,0.0,&avnx[111] } ,
-    {20,"na" ,4,0.0,&avnx[112] } ,
+    {20,"not-set" ,0,0.0,&avnx[115] } ,
+    {20,"dna" ,1,0.0,&avnx[116] } ,
+    {20,"rna" ,2,0.0,&avnx[117] } ,
+    {20,"aa" ,3,0.0,&avnx[118] } ,
+    {20,"na" ,4,0.0,&avnx[119] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"not-set" ,0,0.0,&avnx[114] } ,
-    {20,"linear" ,1,0.0,&avnx[115] } ,
-    {20,"circular" ,2,0.0,&avnx[116] } ,
-    {20,"tandem" ,3,0.0,&avnx[117] } ,
+    {20,"not-set" ,0,0.0,&avnx[121] } ,
+    {20,"linear" ,1,0.0,&avnx[122] } ,
+    {20,"circular" ,2,0.0,&avnx[123] } ,
+    {20,"tandem" ,3,0.0,&avnx[124] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,1,0.0,NULL } ,
-    {20,"not-set" ,0,0.0,&avnx[120] } ,
-    {20,"ss" ,1,0.0,&avnx[121] } ,
-    {20,"ds" ,2,0.0,&avnx[122] } ,
-    {20,"mixed" ,3,0.0,&avnx[123] } ,
+    {20,"not-set" ,0,0.0,&avnx[127] } ,
+    {20,"ss" ,1,0.0,&avnx[128] } ,
+    {20,"ds" ,2,0.0,&avnx[129] } ,
+    {20,"mixed" ,3,0.0,&avnx[130] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"genbank" ,1,0.0,&avnx[125] } ,
-    {20,"embl" ,2,0.0,&avnx[126] } ,
-    {20,"ddbj" ,3,0.0,&avnx[127] } ,
-    {20,"pir" ,4,0.0,&avnx[128] } ,
-    {20,"sp" ,5,0.0,&avnx[129] } ,
-    {20,"bbone" ,6,0.0,&avnx[130] } ,
-    {20,"pdb" ,7,0.0,&avnx[131] } ,
+    {20,"genbank" ,1,0.0,&avnx[132] } ,
+    {20,"embl" ,2,0.0,&avnx[133] } ,
+    {20,"ddbj" ,3,0.0,&avnx[134] } ,
+    {20,"pir" ,4,0.0,&avnx[135] } ,
+    {20,"sp" ,5,0.0,&avnx[136] } ,
+    {20,"bbone" ,6,0.0,&avnx[137] } ,
+    {20,"pdb" ,7,0.0,&avnx[138] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"ref" ,1,0.0,&avnx[133] } ,
-    {20,"alt" ,2,0.0,&avnx[134] } ,
-    {20,"blocks" ,3,0.0,&avnx[135] } ,
+    {20,"ref" ,1,0.0,&avnx[140] } ,
+    {20,"alt" ,2,0.0,&avnx[141] } ,
+    {20,"blocks" ,3,0.0,&avnx[142] } ,
     {20,"other" ,255,0.0,NULL } };
 
 static AsnType atx[206] = {
   {401, "Bioseq" ,1,0,0,0,0,1,0,0,NULL,&atx[32],&atx[1],0,&atx[166]} ,
   {0, "id" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&atx[2],0,&atx[5]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[3],NULL,0,NULL} ,
-  {420, "Seq-id" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[134]} ,
+  {421, "Seq-id" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[134]} ,
   {314, "SET OF" ,0,17,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "descr" ,128,1,0,1,0,0,0,0,NULL,&atx[6],NULL,0,&atx[97]} ,
   {404, "Seq-descr" ,1,0,0,0,0,1,0,0,NULL,&atx[4],&atx[7],0,&atx[8]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[8],NULL,0,NULL} ,
   {405, "Seqdesc" ,1,0,0,0,0,1,0,0,NULL,&atx[50],&atx[9],0,&atx[24]} ,
   {0, "mol-type" ,128,0,0,0,0,0,0,0,NULL,&atx[10],NULL,0,&atx[12]} ,
-  {429, "GIBB-mol" ,1,0,0,0,0,0,0,0,NULL,&atx[11],&avnx[0],0,&atx[14]} ,
+  {409, "GIBB-mol" ,1,0,0,0,0,1,0,0,NULL,&atx[11],&avnx[0],0,&atx[81]} ,
   {310, "ENUMERATED" ,0,10,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "modif" ,128,1,0,0,0,0,0,0,NULL,&atx[4],&atx[13],0,&atx[15]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[14],NULL,0,NULL} ,
@@ -170,7 +177,7 @@ static AsnType atx[206] = {
   {323, "VisibleString" ,0,26,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "title" ,128,4,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[20]} ,
   {0, "org" ,128,5,0,0,0,0,0,0,NULL,&atx[21],NULL,0,&atx[22]} ,
-  {418, "Org-ref" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[90]} ,
+  {419, "Org-ref" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[90]} ,
   {0, "comment" ,128,6,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[23]} ,
   {0, "num" ,128,7,0,0,0,0,0,0,NULL,&atx[24],NULL,0,&atx[51]} ,
   {406, "Numbering" ,1,0,0,0,0,1,0,0,NULL,&atx[50],&atx[25],0,&atx[88]} ,
@@ -192,7 +199,7 @@ static AsnType atx[206] = {
   {435, "Num-ref" ,1,0,0,0,0,0,0,0,NULL,&atx[32],&atx[41],0,&atx[45]} ,
   {0, "type" ,128,0,0,0,0,0,0,0,NULL,&atx[11],&avnx[50],0,&atx[42]} ,
   {0, "aligns" ,128,1,0,1,0,0,0,0,NULL,&atx[43],NULL,0,NULL} ,
-  {414, "Seq-align" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[140]} ,
+  {415, "Seq-align" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[140]} ,
   {0, "real" ,128,3,0,0,0,0,0,0,NULL,&atx[45],NULL,0,NULL} ,
   {436, "Num-real" ,1,0,0,0,0,0,0,0,NULL,&atx[32],&atx[46],0,&atx[107]} ,
   {0, "a" ,128,0,0,0,0,0,0,0,NULL,&atx[47],NULL,0,&atx[48]} ,
@@ -201,15 +208,15 @@ static AsnType atx[206] = {
   {0, "units" ,128,2,0,1,0,0,0,0,NULL,&atx[18],NULL,0,NULL} ,
   {315, "CHOICE" ,0,-1,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "maploc" ,128,8,0,0,0,0,0,0,NULL,&atx[52],NULL,0,&atx[53]} ,
-  {411, "Dbtag" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[171]} ,
+  {412, "Dbtag" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[171]} ,
   {0, "pir" ,128,9,0,0,0,0,0,0,NULL,&atx[54],NULL,0,&atx[55]} ,
-  {423, "PIR-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[79]} ,
+  {424, "PIR-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[79]} ,
   {0, "genbank" ,128,10,0,0,0,0,0,0,NULL,&atx[56],NULL,0,&atx[57]} ,
-  {422, "GB-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[54]} ,
+  {423, "GB-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[54]} ,
   {0, "pub" ,128,11,0,0,0,0,0,0,NULL,&atx[58],NULL,0,&atx[72]} ,
   {403, "Pubdesc" ,1,0,0,0,0,1,0,0,NULL,&atx[32],&atx[59],0,&atx[6]} ,
   {0, "pub" ,128,0,0,0,0,0,0,0,NULL,&atx[60],NULL,0,&atx[61]} ,
-  {417, "Pub-equiv" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[21]} ,
+  {418, "Pub-equiv" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[21]} ,
   {0, "name" ,128,1,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[62]} ,
   {0, "fig" ,128,2,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[63]} ,
   {0, "num" ,128,3,0,1,0,0,0,0,NULL,&atx[24],NULL,0,&atx[64]} ,
@@ -220,41 +227,41 @@ static AsnType atx[206] = {
   {351, "StringStore" ,64,1,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "align-group" ,128,8,0,1,0,0,0,0,NULL,&atx[28],NULL,0,&atx[70]} ,
   {0, "comment" ,128,9,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[71]} ,
-  {0, "reftype" ,128,10,0,0,1,0,0,0,&avnx[56],&atx[28],&avnx[53],0,NULL} ,
+  {0, "reftype" ,128,10,0,0,1,0,0,0,&avnx[57],&atx[28],&avnx[53],0,NULL} ,
   {0, "region" ,128,12,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[73]} ,
   {0, "user" ,128,13,0,0,0,0,0,0,NULL,&atx[74],NULL,0,&atx[75]} ,
-  {413, "User-object" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[43]} ,
+  {414, "User-object" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[43]} ,
   {0, "sp" ,128,14,0,0,0,0,0,0,NULL,&atx[76],NULL,0,&atx[77]} ,
-  {425, "SP-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[84]} ,
+  {426, "SP-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[84]} ,
   {0, "dbxref" ,128,15,0,0,0,0,0,0,NULL,&atx[52],NULL,0,&atx[78]} ,
   {0, "embl" ,128,16,0,0,0,0,0,0,NULL,&atx[79],NULL,0,&atx[80]} ,
-  {424, "EMBL-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[76]} ,
+  {425, "EMBL-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[76]} ,
   {0, "create-date" ,128,17,0,0,0,0,0,0,NULL,&atx[81],NULL,0,&atx[82]} ,
-  {409, "Date" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[103]} ,
+  {410, "Date" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[103]} ,
   {0, "update-date" ,128,18,0,0,0,0,0,0,NULL,&atx[81],NULL,0,&atx[83]} ,
   {0, "prf" ,128,19,0,0,0,0,0,0,NULL,&atx[84],NULL,0,&atx[85]} ,
-  {426, "PRF-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[86]} ,
+  {427, "PRF-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[86]} ,
   {0, "pdb" ,128,20,0,0,0,0,0,0,NULL,&atx[86],NULL,0,&atx[87]} ,
-  {427, "PDB-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[98]} ,
+  {428, "PDB-block" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[98]} ,
   {0, "het" ,128,21,0,0,0,0,0,0,NULL,&atx[88],NULL,0,&atx[89]} ,
   {407, "Heterogen" ,1,0,0,0,0,1,0,0,NULL,&atx[18],NULL,0,&atx[152]} ,
   {0, "source" ,128,22,0,0,0,0,0,0,NULL,&atx[90],NULL,0,&atx[91]} ,
-  {419, "BioSource" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[3]} ,
+  {420, "BioSource" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[3]} ,
   {0, "molinfo" ,128,23,0,0,0,0,0,0,NULL,&atx[92],NULL,0,NULL} ,
   {432, "MolInfo" ,1,0,0,0,0,0,0,0,NULL,&atx[32],&atx[93],0,&atx[26]} ,
-  {0, "biomol" ,128,0,0,0,1,0,0,0,&avnx[69],&atx[28],&avnx[57],0,&atx[94]} ,
-  {0, "tech" ,128,1,0,0,1,0,0,0,&avnx[88],&atx[28],&avnx[70],0,&atx[95]} ,
+  {0, "biomol" ,128,0,0,0,1,0,0,0,&avnx[73],&atx[28],&avnx[58],0,&atx[94]} ,
+  {0, "tech" ,128,1,0,0,1,0,0,0,&avnx[95],&atx[28],&avnx[74],0,&atx[95]} ,
   {0, "techexp" ,128,2,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[96]} ,
-  {0, "completeness" ,128,3,0,0,1,0,0,0,&avnx[96],&atx[28],&avnx[89],0,NULL} ,
+  {0, "completeness" ,128,3,0,0,1,0,0,0,&avnx[103],&atx[28],&avnx[96],0,NULL} ,
   {0, "inst" ,128,2,0,0,0,0,0,0,NULL,&atx[98],NULL,0,&atx[164]} ,
-  {428, "Seq-inst" ,1,0,0,0,0,0,0,0,NULL,&atx[32],&atx[99],0,&atx[10]} ,
-  {0, "repr" ,128,0,0,0,0,0,0,0,NULL,&atx[11],&avnx[97],0,&atx[100]} ,
-  {0, "mol" ,128,1,0,0,0,0,0,0,NULL,&atx[11],&avnx[107],0,&atx[101]} ,
+  {429, "Seq-inst" ,1,0,0,0,0,0,0,0,NULL,&atx[32],&atx[99],0,&atx[14]} ,
+  {0, "repr" ,128,0,0,0,0,0,0,0,NULL,&atx[11],&avnx[104],0,&atx[100]} ,
+  {0, "mol" ,128,1,0,0,0,0,0,0,NULL,&atx[11],&avnx[114],0,&atx[101]} ,
   {0, "length" ,128,2,0,1,0,0,0,0,NULL,&atx[28],NULL,0,&atx[102]} ,
   {0, "fuzz" ,128,3,0,1,0,0,0,0,NULL,&atx[103],NULL,0,&atx[104]} ,
-  {410, "Int-fuzz" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[52]} ,
-  {0, "topology" ,128,4,0,0,1,0,0,0,&avnx[118],&atx[11],&avnx[113],0,&atx[105]} ,
-  {0, "strand" ,128,5,0,1,0,0,0,0,NULL,&atx[11],&avnx[119],0,&atx[106]} ,
+  {411, "Int-fuzz" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[52]} ,
+  {0, "topology" ,128,4,0,0,1,0,0,0,&avnx[125],&atx[11],&avnx[120],0,&atx[105]} ,
+  {0, "strand" ,128,5,0,1,0,0,0,0,NULL,&atx[11],&avnx[126],0,&atx[106]} ,
   {0, "seq-data" ,128,6,0,1,0,0,0,0,NULL,&atx[107],NULL,0,&atx[129]} ,
   {437, "Seq-data" ,1,0,0,0,0,0,0,0,NULL,&atx[50],&atx[108],0,&atx[130]} ,
   {0, "iupacna" ,128,0,0,0,0,0,0,0,NULL,&atx[109],NULL,0,&atx[110]} ,
@@ -283,13 +290,13 @@ static AsnType atx[206] = {
   {0, "seg" ,128,0,0,0,0,0,0,0,NULL,&atx[132],NULL,0,&atx[135]} ,
   {439, "Seg-ext" ,1,0,0,0,0,0,0,0,NULL,&atx[38],&atx[133],0,&atx[136]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[134],NULL,0,NULL} ,
-  {421, "Seq-loc" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[56]} ,
+  {422, "Seq-loc" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[56]} ,
   {0, "ref" ,128,1,0,0,0,0,0,0,NULL,&atx[136],NULL,0,&atx[137]} ,
   {440, "Ref-ext" ,1,0,0,0,0,0,0,0,NULL,&atx[134],NULL,0,&atx[138]} ,
   {0, "map" ,128,2,0,0,0,0,0,0,NULL,&atx[138],NULL,0,&atx[141]} ,
   {441, "Map-ext" ,1,0,0,0,0,0,0,0,NULL,&atx[38],&atx[139],0,&atx[142]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[140],NULL,0,NULL} ,
-  {415, "Seq-feat" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[201]} ,
+  {416, "Seq-feat" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[201]} ,
   {0, "delta" ,128,3,0,0,0,0,0,0,NULL,&atx[142],NULL,0,NULL} ,
   {442, "Delta-ext" ,1,0,0,0,0,0,0,0,NULL,&atx[38],&atx[143],0,&atx[144]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[144],NULL,0,NULL} ,
@@ -301,7 +308,7 @@ static AsnType atx[206] = {
   {0, "fuzz" ,128,1,0,1,0,0,0,0,NULL,&atx[103],NULL,0,&atx[150]} ,
   {0, "seq-data" ,128,2,0,1,0,0,0,0,NULL,&atx[107],NULL,0,NULL} ,
   {0, "hist" ,128,8,0,1,0,0,0,0,NULL,&atx[152],NULL,0,NULL} ,
-  {408, "Seq-hist" ,1,0,0,0,0,1,0,0,NULL,&atx[32],&atx[153],0,&atx[81]} ,
+  {408, "Seq-hist" ,1,0,0,0,0,1,0,0,NULL,&atx[32],&atx[153],0,&atx[10]} ,
   {0, "assembly" ,128,0,0,1,0,0,0,0,NULL,&atx[4],&atx[154],0,&atx[155]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[43],NULL,0,NULL} ,
   {0, "replaces" ,128,1,0,1,0,0,0,0,NULL,&atx[156],NULL,0,&atx[160]} ,
@@ -320,10 +327,10 @@ static AsnType atx[206] = {
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[169],NULL,0,NULL} ,
   {456, "Annot-id" ,1,0,0,0,0,0,0,0,NULL,&atx[50],&atx[170],0,&atx[177]} ,
   {0, "local" ,128,0,0,0,0,0,0,0,NULL,&atx[171],NULL,0,&atx[172]} ,
-  {412, "Object-id" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[74]} ,
+  {413, "Object-id" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[74]} ,
   {0, "ncbi" ,128,1,0,0,0,0,0,0,NULL,&atx[28],NULL,0,&atx[173]} ,
   {0, "general" ,128,2,0,0,0,0,0,0,NULL,&atx[52],NULL,0,NULL} ,
-  {0, "db" ,128,1,0,1,0,0,0,0,NULL,&atx[28],&avnx[124],0,&atx[175]} ,
+  {0, "db" ,128,1,0,1,0,0,0,0,NULL,&atx[28],&avnx[131],0,&atx[175]} ,
   {0, "name" ,128,2,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[176]} ,
   {0, "desc" ,128,3,0,1,0,0,0,0,NULL,&atx[177],NULL,0,&atx[194]} ,
   {457, "Annot-descr" ,1,0,0,0,0,0,0,0,NULL,&atx[4],&atx[178],0,&atx[179]} ,
@@ -339,7 +346,7 @@ static AsnType atx[206] = {
   {0, "src" ,128,7,0,0,0,0,0,0,NULL,&atx[3],NULL,0,&atx[188]} ,
   {0, "align" ,128,8,0,0,0,0,0,0,NULL,&atx[189],NULL,0,&atx[193]} ,
   {459, "Align-def" ,1,0,0,0,0,0,0,0,NULL,&atx[32],&atx[190],0,NULL} ,
-  {0, "align-type" ,128,0,0,0,0,0,0,0,NULL,&atx[28],&avnx[132],0,&atx[191]} ,
+  {0, "align-type" ,128,0,0,0,0,0,0,0,NULL,&atx[28],&avnx[139],0,&atx[191]} ,
   {0, "ids" ,128,1,0,1,0,0,0,0,NULL,&atx[4],&atx[192],0,NULL} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[3],NULL,0,NULL} ,
   {0, "region" ,128,9,0,0,0,0,0,0,NULL,&atx[134],NULL,0,NULL} ,
@@ -350,14 +357,14 @@ static AsnType atx[206] = {
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[43],NULL,0,NULL} ,
   {0, "graph" ,128,2,0,0,0,0,0,0,NULL,&atx[4],&atx[200],0,&atx[202]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[201],NULL,0,NULL} ,
-  {416, "Seq-graph" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[60]} ,
+  {417, "Seq-graph" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[60]} ,
   {0, "ids" ,128,3,0,0,0,0,0,0,NULL,&atx[4],&atx[203],0,&atx[204]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[3],NULL,0,NULL} ,
   {0, "locs" ,128,4,0,0,0,0,0,0,NULL,&atx[4],&atx[205],0,NULL} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[134],NULL,0,NULL} };
 
 static AsnModule ampx[1] = {
-  { "NCBI-Sequence" , "asnseq.h60",&atx[0],NULL,NULL,0,0} };
+  { "NCBI-Sequence" , "asnseq.h64",&atx[0],NULL,NULL,0,0} };
 
 static AsnValxNodePtr avn = avnx;
 static AsnTypePtr at = atx;
@@ -457,6 +464,8 @@ static AsnModulePtr amp = ampx;
 #define SEQ_HIST_deleted_bool &at[162]
 #define SEQ_HIST_deleted_date &at[163]
 
+#define GIBB_MOL &at[10]
+
 #define SEQ_INST &at[98]
 #define SEQ_INST_repr &at[99]
 #define SEQ_INST_mol &at[100]
@@ -468,8 +477,6 @@ static AsnModulePtr amp = ampx;
 #define SEQ_INST_ext &at[129]
 #define SEQ_INST_hist &at[151]
 
-#define GIBB_MOL &at[10]
-
 #define GIBB_MOD &at[14]
 
 #define GIBB_METHOD &at[16]
