@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   10/18/95
 *
-* $Revision: 6.2 $
+* $Revision: 6.7 $
 *
 * File Description: Code for Modelstruc -> Biostruc conversion
 *                   And User-feature handling.
@@ -39,6 +39,21 @@
 * Date     Name        Description of modification
 * -------  ----------  -----------------------------------------------------
 * $Log: mmdbapi4.c,v $
+* Revision 6.7  2000/12/05 22:13:07  ywang
+* fix bugs for MakePDBSeqId2
+*
+* Revision 6.6  1999/03/12 18:37:08  kans
+* fixed ErrPostEx problem
+*
+* Revision 6.5  1999/02/25 23:11:53  ywang
+* synchronize with cn3d to save out mime data
+*
+* Revision 6.4  1998/09/02 16:25:27  kans
+* removed comments around debug ifdef to suppress printf statement
+*
+* Revision 6.3  1998/08/26 18:02:43  kans
+* fixed -v -fd warnings
+*
 * Revision 6.2  1998/03/10 04:31:08  lewisg
 * change name of MakePDBSeqId to MakePDBSeqId2 to avoid name conflict
 *
@@ -127,8 +142,8 @@ Boolean LIBCALL  RefreshSurface(SurfaceCoordinatesPtr  pscThis, PMOD pmodThis)
     Int4 iCount = 0;
     Int4 iCoords = 0;
     Int4 iVal = 0;
-    
-    
+    Boolean rsult = FALSE;
+
     switch (pscThis->Surface_surface->choice)
       {
 	   case Surface_surface_cylinder:
@@ -295,9 +310,9 @@ Boolean LIBCALL  RefreshSurface(SurfaceCoordinatesPtr  pscThis, PMOD pmodThis)
 	     ptriThis->number_of_triangles = iCoords;
 	     return TRUE;
 	   default:
-	     return FALSE;
+	     rsult = FALSE;
        }
-    return FALSE;   
+    return rsult;   
     
     objallocerr:
    
@@ -1124,7 +1139,7 @@ Boolean LIBCALL RebuildChemGraphAsn(PDNMS pdnmsThis)
       /* clear all ASN data out - must start save process from scratch */
     FreeRedundantAsn(pdnmsThis);
     ErrClear(); 
-    ErrPostEx(SEV_ERROR,0,0, "Out of memory (at %d) while trying to save.\n Try again after closing other windows/programs.");
+    ErrPostEx(SEV_ERROR,0,0, "Out of memory while trying to save.\n Try again after closing other windows/programs.");
     ErrShow();
     return FALSE;
   
@@ -1184,9 +1199,9 @@ Boolean LIBCALL WriteASNChemGraphOnly(PDNMS pdnmsThis, CharPtr pcSave,  Byte bSa
 	bWriteOK = FALSE;
 	goto seeyalater;
      }
-/* #ifdef _DEBUG_5 */
+#ifdef _DEBUG_5
 printf("Chem Graph rebuilt \n");
-/* #endif    */   
+#endif  
     
     if (bSave & (Byte) SAVE_BINARY) pcSaveMode = SAVE_BINARY_STRING; 
     else pcSaveMode = SAVE_ASCII_STRING;  
@@ -1461,7 +1476,7 @@ printf("Re-linking pbsmModels\n");
 
 
 Boolean LIBCALL WriteAsnModelList(PDNMS pdnmsThis,   Int2 iNumModels,  Int2Ptr i2Vec,  
-				    CharPtr pcSave,  Byte bSave)
+				    CharPtr pcSave,  Byte bSave, Boolean iCn3d)
 {
     /* The master ASN.1 writer routine */
     /* MODELS not specified for writing are ERASED for congruency */
@@ -1596,7 +1611,14 @@ printf("Refreshing Model failed %d\n ", (int) i2Vec[iIndex]);
 	  }
       }
      
+     if(iCn3d){
+        if(pmsdThis->pbsBS) bRet = TRUE;
+        return bRet;
+               /* leave cn3d handle FreeRedundantAsn - yanli */
+     }
+
      bRet = WriteOutBiostruc(pmsdThis->pbsBS, pcSave, bSave);
+
 #ifdef _DEBUG_5
 if (!bRet) printf("WriteOutBiostruc Failed\n");
 #endif    
@@ -1621,7 +1643,7 @@ Boolean LIBCALL WriteAsnOneModel(PDNMS pdnmsThis,  Int2 iModel,  CharPtr pcSave,
     if (!(pmsdThis->bMe == (Byte) AM_MSD)) return FALSE;
    
     bRet = WriteAsnModelList(pdnmsThis,  1, (Int2Ptr) &iModel, 
-				pcSave, bSave);
+				pcSave, bSave, 0);
     return bRet;
 }
 
@@ -1664,7 +1686,7 @@ Boolean LIBCALL WriteAsnAllModel(PDNMS pdnmsThis,  CharPtr pcSave,  Byte bSave)
 	   }
 	  pdnmlThis = pdnmlThis->next;
 	}
-    bRet = WriteAsnModelList(pdnmsThis, iCount, i2Vec, pcSave, bSave);
+    bRet = WriteAsnModelList(pdnmsThis, iCount, i2Vec, pcSave, bSave, 0);
     if (i2Vec) I2VectorFree(i2Vec, 0);  /* free vector */
     return bRet;	  
 }
@@ -1672,16 +1694,14 @@ Boolean LIBCALL WriteAsnAllModel(PDNMS pdnmsThis,  CharPtr pcSave,  Byte bSave)
 
 
 
-static SeqIdPtr MakePDBSeqId2 (CharPtr pcPDB, Char cChain, int iDomain, Boolean getgi)
+SeqIdPtr MakePDBSeqId2 (CharPtr pcPDB, Char cChain, int iDomain, Boolean getgi)
 {
   SeqIdPtr        sip;
   PDBSeqIdPtr     psip;
   Int4 gi;
 
   psip = PDBSeqIdNew ();
-  if (cChain == '-' || cChain == '_') {
-    cChain = ' ';
-  }
+
   psip->chain = cChain;
   psip->rel = NULL;
   psip->mol = StringSave (pcPDB);
