@@ -29,7 +29,7 @@
 *
 * Version Creation Date:  3/4/91
 *
-* $Revision: 6.0 $
+* $Revision: 6.5 $
 *
 * File Description:
 *   ByteStore functions
@@ -56,6 +56,21 @@
 * 04-15-93 Schuler     Changed _cdecl to LIBCALL
 *
 * $Log: ncbibs.c,v $
+* Revision 6.5  2000/05/26 23:34:58  kans
+* added BSDupAndSwapUint4 for copying and swapping of UID lists passed over network to BIG_ENDIAN server
+*
+* Revision 6.4  1999/11/05 19:59:09  beloslyu
+* Fixed the error with size_t vs Int4 comparisons
+*
+* Revision 6.3  1999/08/25 17:43:04  madden
+* Removed extra comment starts
+*
+* Revision 6.2  1999/01/21 20:08:37  ostell
+* added SwitchUint2 and 4, added integer bytestores
+*
+* Revision 6.1  1998/06/11 18:59:58  shavirin
+* Fixed some compiler warnings.
+*
 * Revision 6.0  1997/08/25 18:15:13  madden
 * Revision changed to 6.0
 *
@@ -108,20 +123,22 @@ NLM_EXTERN Nlm_VoidPtr LIBCALL  Nlm_BSMerge (Nlm_ByteStorePtr bsp, Nlm_VoidPtr d
 {
 	Nlm_BytePtr tmp, from;
 	Nlm_BSUnitPtr bsup;
-    Nlm_Int4 size;
+    size_t size;
 
 	if (bsp == NULL)
 		return NULL;
 
     size = Nlm_BSLen(bsp) + 1;
     if (size > SIZE_MAX) {
+		ErrPostEx(SEV_ERROR, 0, 0, "size[%u] is bigger then SIZE_MAX[%u]",
+			size, SIZE_MAX);
         /* should post an error here */
         return NULL;
     }
 
 	if (dest == NULL)         /* allocate storage with room for null at end */
 	{
-		dest = Nlm_MemNew((size_t) size);
+		dest = Nlm_MemNew(size);
 		if (dest == NULL) return dest;
 	}
 
@@ -437,7 +454,7 @@ NLM_EXTERN Nlm_Int4 LIBCALL  Nlm_BSTell (Nlm_ByteStorePtr bsp)
 NLM_EXTERN Nlm_Int2 LIBCALL  Nlm_BSSeek (Nlm_ByteStorePtr bsp, Nlm_Int4 offset, Nlm_Int2 origin)
 
 {
-	Nlm_Int4 sp, start;
+	Nlm_Int4 sp, start = 0;
 	Nlm_BSUnitPtr bsup;
 	Nlm_Boolean done;
 
@@ -819,6 +836,12 @@ NLM_EXTERN Nlm_ByteStorePtr LIBCALL  Nlm_BSFree (Nlm_ByteStorePtr bsp)
 	return (Nlm_ByteStorePtr) Nlm_MemFree(bsp);
 }
 
+/*****************************************************************************
+*
+*   ByteStorePtr Nlm_BSDup(bsp)
+*
+*****************************************************************************/
+
 #define BUFSIZE 256
 
 NLM_EXTERN Nlm_ByteStorePtr LIBCALL Nlm_BSDup (Nlm_ByteStorePtr source)
@@ -854,3 +877,149 @@ NLM_EXTERN Nlm_ByteStorePtr LIBCALL Nlm_BSDup (Nlm_ByteStorePtr source)
   }
   return dest;
 }
+
+/* for copying and swapping of UID lists passed over network to BIG_ENDIAN server */
+
+NLM_EXTERN Nlm_ByteStorePtr LIBCALL Nlm_BSDupAndSwapUint4 (Nlm_ByteStorePtr source)
+
+{
+  Nlm_Byte          buf [BUFSIZE + 4];
+  Nlm_Int4          count;
+  Nlm_ByteStorePtr  dest;
+  Nlm_Int4          sourceLen;
+  Nlm_Int4          sourceLoc;
+  Nlm_Int4          swapLen;
+
+  dest = NULL;
+  if (source != NULL) {
+    sourceLen = Nlm_BSLen (source);
+    dest = Nlm_BSNew (sourceLen);
+    /* read the original location */
+    sourceLoc = Nlm_BSTell(source);
+    Nlm_BSSeek(source, 0L, SEEK_SET);
+    if (dest != NULL) {
+      count = MIN (sourceLen, (Nlm_Int4) BUFSIZE);
+      while (count > 0 && sourceLen > 0) {
+        swapLen = Nlm_BSRead (source, buf, count);
+        swapLen /= 4;
+        /* swap Uint4 if IS_LITTLE_ENDIAN */
+        Nlm_SwapUint4Buff (buf, swapLen);
+        Nlm_BSWrite (dest, buf, count);
+        sourceLen -= count;
+        count = MIN (sourceLen, (Nlm_Int4) BUFSIZE);
+      }
+      /* for neatness, make the duplicate point to the
+	     same location as the old one */
+      Nlm_BSSeek(dest, sourceLoc, SEEK_SET);
+    }
+    /* restore original location */
+    Nlm_BSSeek(source, sourceLoc, SEEK_SET);
+  }
+  return dest;
+}
+
+/****************************************************************************
+*
+*   Integer storage utilities
+*      These assume integers are store in BIG_ENDIAN order in the ByteStore
+*      They read and write Uint2 or Uint4 in the NATIVE endian order
+*      All work with UNSIGNED 2 or 4 byte integers
+*         (you should cast to make them signed)
+*      These are just helper functions. They do no internal consistency
+*         checking.
+*      They are primarily to facilitate encoding SEQUENCE OF INTEGER as
+*         OCTET STRING for ASN.1
+*
+****************************************************************************/
+
+NLM_EXTERN Nlm_Uint2 LIBCALL Nlm_BSGetUint2 (Nlm_ByteStorePtr bsp)
+{
+	Nlm_Uint2 retval = 0;
+
+	Nlm_BSRead(bsp, (Nlm_VoidPtr)(&retval), (Nlm_Int4)2);
+	Nlm_SwapUint2(retval);
+
+	return retval;
+}
+
+NLM_EXTERN Nlm_Uint4 LIBCALL Nlm_BSGetUint4 (Nlm_ByteStorePtr bsp)
+{
+	Nlm_Uint4 retval = 0;
+
+	Nlm_BSRead(bsp, (Nlm_VoidPtr)(&retval), (Nlm_Int4)4);
+	Nlm_SwapUint4(retval);
+
+	return retval;
+}
+
+NLM_EXTERN Nlm_Int2 LIBCALL Nlm_BSPutUint2 (Nlm_ByteStorePtr bsp, Nlm_Uint2 value)
+{
+#ifdef IS_LITTLE_ENDIAN
+	Nlm_SwitchUint2(value);
+#endif
+	if (Nlm_BSWrite(bsp, (Nlm_VoidPtr)(&value), (Nlm_Int4)2) == (Nlm_Int2)2)
+		return (Nlm_Int2)1;
+	else
+		return (Nlm_Int2)0;
+}
+
+NLM_EXTERN Nlm_Int2 LIBCALL Nlm_BSPutUint4 (Nlm_ByteStorePtr bsp, Nlm_Uint4 value)
+{
+#ifdef IS_LITTLE_ENDIAN
+	Nlm_SwitchUint4(value);
+#endif
+	if (Nlm_BSWrite(bsp, (Nlm_VoidPtr)(&value), (Nlm_Int4)4) == (Nlm_Int2)4)
+		return (Nlm_Int2)1;
+	else
+		return (Nlm_Int2)0;
+}
+
+       /* In functions below, size is 2 or 4 */
+       /* Integers are converted from ByteStore to native endian (BSUintXRead) */
+	   /*   or from native endian to ByteStore (BSUintXWrite) */
+	   /* len is number of INTEGERS, not number of bytes */
+	   /* returns count of integers put in ptr */
+	   /* WARNING: On LITTLE_ENDIAN machines the data in ptr is changed to BIG_ENDIAN in the */
+	   /*   XXWrite functions and LEFT THAT WAY */
+NLM_EXTERN Nlm_Int4 LIBCALL Nlm_BSUint4Read (Nlm_ByteStorePtr bsp, Nlm_Uint4Ptr ptr, Nlm_Int4 len)
+{
+	Nlm_Int4 len2;
+
+	len2 = Nlm_BSRead(bsp, (Nlm_VoidPtr)ptr, (len * 4));
+	len2 /= 4;
+	Nlm_SwapUint4Buff(ptr, len2);
+	return len2;
+}
+
+NLM_EXTERN Nlm_Int4 LIBCALL Nlm_BSUint4Write (Nlm_ByteStorePtr bsp, Nlm_Uint4Ptr ptr, Nlm_Int4 len)
+{
+	Nlm_Int4 len2;
+
+#ifdef IS_LITTLE_ENDIAN
+	Nlm_SwitchUint4Buff(ptr, len);
+#endif
+	len2 = Nlm_BSWrite(bsp, (Nlm_VoidPtr)ptr, (len * 4));
+	return (len2 / 4);
+}
+
+NLM_EXTERN Nlm_Int4 LIBCALL Nlm_BSUint2Read (Nlm_ByteStorePtr bsp, Nlm_Uint2Ptr ptr, Nlm_Int4 len)
+{
+	Nlm_Int4 len2;
+
+	len2 = Nlm_BSRead(bsp, (Nlm_VoidPtr)ptr, (len * 2));
+	len2 /= 2;
+	Nlm_SwapUint2Buff(ptr, len2);
+	return len2;
+}
+
+NLM_EXTERN Nlm_Int4 LIBCALL Nlm_BSUint2Write (Nlm_ByteStorePtr bsp, Nlm_Uint2Ptr ptr, Nlm_Int4 len)
+{
+	Nlm_Int4 len2;
+
+#ifdef IS_LITTLE_ENDIAN
+	Nlm_SwitchUint2Buff(ptr, len);
+#endif
+	len2 = Nlm_BSWrite(bsp, (Nlm_VoidPtr)ptr, (len * 2));
+	return (len2 / 2);
+}
+
