@@ -23,9 +23,9 @@
 *
 * ===========================================================================
 *
-* $Id: ncbierr.c,v 6.8 1998/05/04 23:25:46 vakatov Exp $
+* $Id: ncbierr.c,v 6.17 2001/02/08 22:27:04 vakatov Exp $
 *
-* $Revision: 6.8 $
+* $Revision: 6.17 $
 *
 * Authors:  Schuler, Sirotkin (UserErr stuff)
 *
@@ -71,6 +71,34 @@
 * 03-06-95 Schuler     Fixed problem with ErrMsgRoot_fopen
 *
 * $Log: ncbierr.c,v $
+* Revision 6.17  2001/02/08 22:27:04  vakatov
+* + Nlm_CallErrHandlerOnly() -- to allow processing of posted
+* err.messages by the hook only.
+*
+* Revision 6.16  2000/06/14 22:27:52  vakatov
+* Nlm_AbnormalExit() -- print message only if called for the first time
+*
+* Revision 6.15  1999/10/01 14:41:39  kans
+* added SEV_REJECT between SEV_ERROR and SEV_FATAL
+*
+* Revision 6.14  1999/08/23 19:14:34  vakatov
+* Let SEV_MAX not be an exception among the "SEV_*"
+*
+* Revision 6.13  1998/12/29 06:16:21  vakatov
+* Nlm_ErrPostStr() -- Check for the severity code validity
+*
+* Revision 6.12  1998/10/22 16:52:40  shavirin
+* Added function *Nlm_GetErrLongText able to retrieve long error messages
+*
+* Revision 6.11  1998/09/29 16:20:53  vakatov
+* Nlm_AssertionFailed():  printout the module name and file/line in Message
+*
+* Revision 6.10  1998/08/24 17:42:00  kans
+* fixed old style function definition warnings
+*
+* Revision 6.9  1998/06/11 21:02:23  shavirin
+* Fixed few warnings
+*
 * Revision 6.8  1998/05/04 23:25:46  vakatov
 * Raised the message posting level to SEV_ERROR(for non-_DEBUG only)
 *
@@ -209,8 +237,13 @@ typedef struct AppErrInfo *AppErrInfoPtr;
 
 
 static char * _szPropKey = "_AppErrInfo";
-static char * _szSevKey [] = { "", "SEV_INFO", "SEV_WARNING", "SEV_ERROR", "SEV_FATAL" };
-static char * _szSevDef [] = { "", "NOTE:", "WARNING:", "ERROR:", "FATAL ERROR:" };
+
+static char * _szSevKey [SEV_MAX + 1] = {
+  "", "SEV_INFO", "SEV_WARNING", "SEV_ERROR", "SEV_REJECT", "SEV_FATAL", "SEV_MAX"};
+
+static char * _szSevDef [SEV_MAX + 1] = {
+  "", "NOTE:", "WARNING:", "ERROR:", "REJECT:", "FATAL ERROR:", "FATAL ERROR:" };
+
 static char * _szSeverity[SEV_MAX + 1];
 
 static AppErrInfoPtr GetAppErrInfo PROTO((void));
@@ -219,6 +252,9 @@ static AppErrInfoPtr GetAppErrInfo PROTO((void));
 #define TEST_BITS(inf,x)  (FUSE_BITS(inf) & (x))
 
 
+static Nlm_Boolean s_HookOnly = FALSE;
+
+
 
 /***************************************************************************\
 |                           POSTING AN ERROR                                |
@@ -352,7 +388,11 @@ NLM_EXTERN int LIBCALL Nlm_ErrPostStr (ErrSev sev, int lev1, int lev2, const cha
   ErrMsgRoot *root  = NULL;
   ErrMsgNode *node1 = NULL;
   ErrMsgNode *node2 = NULL;
-  int severity = sev;
+
+  int severity;
+  if (sev < 0  ||  SEV_MAX < sev)
+    sev = SEV_FATAL;
+  severity = sev;
 
   if (info->busy & AEI_BUSY_2)
     {
@@ -412,7 +452,7 @@ NLM_EXTERN int LIBCALL Nlm_ErrPostStr (ErrSev sev, int lev1, int lev2, const cha
 
 
   /* ----- Log the error according to the current options ----- */
-  if (severity >= info->opts.log_level)
+  if (!s_HookOnly  &&  severity >= info->opts.log_level)
     {
       NlmMutexLockEx( &corelibMutex );
 
@@ -467,13 +507,19 @@ NLM_EXTERN int LIBCALL Nlm_ErrPostStr (ErrSev sev, int lev1, int lev2, const cha
   if (info->hook != NULL)
     {
       int retval;
-      if ((retval = (*info->hook)(&(info->desc))) != 0)
+      if ((retval = (*info->hook)(&info->desc)) != 0)
         {
           ErrClear();
           return retval;
         }
     }
 
+  if ( s_HookOnly ) {
+      die_if_necessary(sev, info);
+      ErrClear();
+      return ANS_NONE;
+  }
+
   /* ----- Workaround for obsolete ERR_IGNORE option ----- */
   if (info->opts.actopt == ERR_IGNORE)
     {
@@ -651,7 +697,7 @@ NLM_EXTERN int LIBCALL  Nlm_ErrCopy (ErrDesc FAR *err)
  * MODIFICATIONS
  * 12-13-93 Schuler   Just clear flags instead of zeroing the whole struct.
  */
-NLM_EXTERN void LIBCALL Nlm_ErrClear ()
+NLM_EXTERN void LIBCALL Nlm_ErrClear (void)
 {
 	AppErrInfoPtr info = GetAppErrInfo();
 	info->any_error = 0;
@@ -680,7 +726,7 @@ NLM_EXTERN void LIBCALL Nlm_ErrClear ()
 * TO DO
 * - allocate buffer for the message
 */
-NLM_EXTERN int LIBCALL Nlm_ErrShow ()
+NLM_EXTERN int LIBCALL Nlm_ErrShow (void)
 {
   AppErrInfoPtr info = GetAppErrInfo();
   if (info->any_error)
@@ -898,7 +944,7 @@ static ErrSev atosev (const char *sevstr)
 	int i;
 	if (sevstr)
 	{
-		for (i=SEV_MIN; i<SEV_MAX; ++i)
+		for (i=SEV_MIN;  i <= SEV_MAX;  ++i)
 		{
 			if (strcmp(sevstr,_szSevKey[i]) == 0)
 				return (ErrSev)i;
@@ -927,7 +973,7 @@ static int ErrMsgRoot_Read (ErrMsgRoot *idx)
   {{
     char line[80], *p;
     int  linenum = 0;
-    long tofs, cofs, tmpofs;
+    long tofs = 0, cofs = 0, tmpofs;
     ErrMsgNode *lev1;
     ErrMsgNode *lev2;
     int any_text;
@@ -1312,6 +1358,17 @@ NLM_EXTERN ErrHookProc LIBCALL Nlm_ErrSetHandler (ErrHookProc hookNew)
 }
 
 
+NLM_EXTERN Nlm_Boolean LIBCALL Nlm_CallErrHandlerOnly(Nlm_Boolean hook_only)
+{
+    Nlm_Boolean prev;
+    NlmMutexLockEx(&corelibMutex);
+    prev       = s_HookOnly;
+    s_HookOnly = hook_only;
+    NlmMutexUnlock(corelibMutex);
+    return prev;
+}
+
+
 /*-------------------------------------------------------------------------
  * ErrUserInstall   [Sirotkin]
  *
@@ -1448,7 +1505,7 @@ NLM_EXTERN ErrSev LIBCALL ErrSetLogLevel (ErrSev level)
 	return prev;
 }
 
-NLM_EXTERN ErrSev LIBCALL ErrGetLogLevel ()
+NLM_EXTERN ErrSev LIBCALL ErrGetLogLevel (void)
 {
 	return (ErrSev)GetAppErrInfo()->opts.log_level;
 }
@@ -1467,7 +1524,7 @@ NLM_EXTERN ErrSev LIBCALL ErrSetMessageLevel (ErrSev level)
 	return prev;
 }
 
-NLM_EXTERN ErrSev LIBCALL ErrGetMessageLevel ()
+NLM_EXTERN ErrSev LIBCALL ErrGetMessageLevel (void)
 {
 	return (ErrSev) GetAppErrInfo()->opts.msg_level;
 }
@@ -1486,7 +1543,7 @@ NLM_EXTERN ErrSev LIBCALL ErrSetFatalLevel (ErrSev level)
 	return prev;
 }
 
-NLM_EXTERN ErrSev LIBCALL ErrGetFatalLevel ()
+NLM_EXTERN ErrSev LIBCALL ErrGetFatalLevel (void)
 {
 	return (ErrSev) GetAppErrInfo()->opts.die_level;
 }
@@ -1775,16 +1832,23 @@ NLM_EXTERN void LIBCALL Nlm_TraceStr (const char *str)
 * 12-15-93 Schuler   Added call to Message() so user knows what happened!
 * 01-21-94 Schuler   Changed MSG_OK to MSG_POST
 */
-NLM_EXTERN void LIBCALL Nlm_AssertionFailed (const char *expression, const char *module,
-				const char *fname, int linenum)
+NLM_EXTERN void LIBCALL Nlm_AssertionFailed
+(const char *expression, const char *module,
+ const char *fname, int linenum)
 {
 	ErrLogPrintf("\nAssertion Failed:  %s\n", expression);
-	if (module != NULL)
-		ErrLogPrintf("  %s",module);
-	if (fname != NULL)
-		ErrLogPrintf("  %s, line %d",fname,linenum);
+	if (module  &&  *module)
+     ErrLogPrintf("  %s",module);
+	if ( fname )
+     ErrLogPrintf("  %s, line %d", fname, linenum);
 	ErrLogPrintStr("\n");
-	Message(MSG_POST,"Assertion Failed:\n%s",expression);
+
+	Message(MSG_POST, "Assertion Failed:\n%s", expression);
+	if (module  &&  *module)
+     Message(MSG_POST, "Module \"%s\"", module);
+	if ( fname )
+	  Message(MSG_POST, "File \"%s\", line %d", fname, linenum);
+
 	AbnormalExit(1);
 }
 
@@ -1818,7 +1882,11 @@ NLM_EXTERN void LIBCALL Nlm_AbnormalExitPure(int code)
 
 NLM_EXTERN void LIBCALL  Nlm_AbnormalExit (int code)
 {
-  ErrLogPrintStr( "\n***** ABNORMAL PROGRAM EXIT *****\n" );
+  static Nlm_Boolean s_Exiting = FALSE;
+  if ( !s_Exiting ) {
+    s_Exiting = TRUE;
+    ErrLogPrintStr( "\n***** ABNORMAL PROGRAM EXIT *****\n" );
+  }
   Nlm_AbnormalExitPure( code );
 }
 
@@ -1909,7 +1977,7 @@ extern void ReleaseAppErrInfo( void )
 }
 
 
-static struct AppErrInfo * GetAppErrInfo()
+static struct AppErrInfo * GetAppErrInfo(void)
 {
   AppErrInfoPtr info = (AppErrInfoPtr) GetAppProperty(_szPropKey);
 
@@ -1953,7 +2021,7 @@ static struct AppErrInfo * GetAppErrInfo()
 
       if ( severity_init )
         {
-          for (i = SEV_MIN;  i < SEV_MAX;  i++)
+          for (i = SEV_MIN;  i <= SEV_MAX;  i++)
             {
               GetAppParam(_file, _section, _szSevKey[i], _szSevDef[i],
                           buffer,sizeof buffer);
@@ -2004,3 +2072,31 @@ NLM_EXTERN void LIBCALL Nlm_ErrPathReset (void)
       FileBuildPath(info->msgpath, NULL, NULL);
 }
 
+NLM_EXTERN char *Nlm_GetErrLongText (char *module, 
+                                     int errcode, int subcode)
+{
+    ErrMsgRoot *root  = NULL;
+    ErrMsgNode *node1 = NULL;
+    ErrMsgNode *node2 = NULL;
+    ErrMsgNode *the_node = NULL;
+    char *text = NULL;
+    
+    if (module != NULL) {
+        root = (ErrMsgRoot*) ErrGetMsgRoot(module);
+        for (node1=root->list; node1; node1=node1->next) {
+            if (node1->code == errcode) {
+                for (node2=node1->list; node2; node2=node2->next)
+                    if (node2->code == subcode)
+                        break;
+                break;
+            }
+        }
+    }
+    
+    the_node = node2 ? node2 : node1;
+    
+    text = (char *) ErrGetExplanation(root, the_node);
+    
+    return text;
+}
+
