@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   3/4/91
 *
-* $Revision: 6.9 $
+* $Revision: 6.23 $
 *
 * File Description: 
 *     portable file routines
@@ -43,6 +43,59 @@
 * 11-27-94 Ostell      moved includes to ncbiwin.h to avoid conflict MSC
 *
 * $Log: ncbifile.c,v $
+* Revision 6.23  2001/08/29 17:33:15  juran
+* Cleanup.
+*
+* Revision 6.22  2001/04/26 16:47:40  juran
+* Refactored CreateDir to quash warnings.
+*
+* Revision 6.21  2001/04/06 20:09:00  juran
+* Disabled #include of FullPath.h.
+#if'ed out MacFSSpec2FullPathname(), which uses FullPath,
+but which we're not using.
+Disabled includes of other MoreFiles headers from MoreFilesExtras.c.
+We should be entirely free of MoreFiles now.
+Removed disabled CD routines.
+*
+* Revision 6.20  2001/04/05 22:30:27  juran
+* Inserted five functions from MoreFileExtras.c.
+*
+* Revision 6.19  2001/04/05 21:59:13  juran
+* Added back #include of MoreFilesExtras.h.
+*
+* Revision 6.18  2001/04/05 21:36:08  juran
+* Mac changes:
+* Added support for converting between an FSSpec an an arbitrarily long C 
+* string containing a pathname.
+* Added a wrapper for DirCreate() that takes a pathname and uses said 
+* support.  Changed Nlm_CreateDir() to use it.
+* Disabled CD routines.
+*
+* Revision 6.17  2000/10/16 17:54:46  kans
+* null out param block for FileLengthEx on Mac
+*
+* Revision 6.16  2000/03/08 17:55:47  vakatov
+* Use Int8 for the file size.
+* Also, get rid of the WIN16 code, do other cleanup.
+*
+* Revision 6.15  1999/12/30 16:36:38  kans
+* additional cleanup (Churchill)
+*
+* Revision 6.14  1999/12/27 22:21:01  shavirin
+* Fixed FileWrite() function for size == 0
+*
+* Revision 6.13  1999/12/21 17:52:39  kans
+* removed MPW/THINKC conditional code, starting upgrade to Carbon compatibility - Churchill
+*
+* Revision 6.12  1998/06/26 20:39:41  vakatov
+* Added FilePathFind() -- complimentary to FileNameFind()
+*
+* Revision 6.11  1998/06/26 14:11:56  madden
+* tempnam called with NULLs rather than empty strings
+*
+* Revision 6.10  1998/06/25 19:39:17  vakatov
+* Added "FileLengthEx()" -- it returns -1(not 0!) if the file does not exist
+*
 * Revision 6.9  1998/05/28 15:59:55  vakatov
 * [WIN16-BORLAND]  Nlm_DirCatalog:  "_find_t" -> "find_t"(Borland-specific)
 *
@@ -109,6 +162,10 @@
 #define THIS_FILE  _this_file
 
 #include "corepriv.h"
+#ifdef OS_MAC
+#include <Navigation.h>
+#include <Script.h>
+#endif
 
 #ifdef OS_UNIX_SUN
 #define DEFAULT_CDROM "/dev/sr0"
@@ -145,6 +202,191 @@
 extern char *g_corelib;
 static char * _this_file = __FILE__;
 
+#ifdef OS_MAC
+#define INLINE_MOREFILES
+//#include "FullPath.h"
+//#include "MoreFilesExtra.h"
+#ifdef INLINE_MOREFILES
+
+// MoreFilesExtras.c
+// -----------------
+
+/*
+**	Apple Macintosh Developer Technical Support
+**
+**	A collection of useful high-level File Manager routines.
+**
+**	by Jim Luther, Apple Developer Technical Support Emeritus
+**
+**	File:		MoreFilesExtras.c
+**
+**	Copyright  1992-1999 Apple Computer, Inc.
+**	All rights reserved.
+**
+**	You may incorporate this sample code into your applications without
+**	restriction, though the sample code has been provided "AS IS" and the
+**	responsibility for its operation is 100% yours.  However, what you are
+**	not permitted to do is to redistribute the source as "DSC Sample Code"
+**	after having made changes. If you're going to re-distribute the source,
+**	we require that you make it clear in the source that the code was
+**	descended from Apple Sample Code, but that you've made changes.
+*/
+
+#include <Types.h>
+#include <Traps.h>
+#include <OSUtils.h>
+#include <Errors.h>
+#include <Files.h>
+#include <Devices.h>
+#include <Finder.h>
+#include <Folders.h>
+#include <FSM.h>
+#include <Disks.h>
+#include <Gestalt.h>
+#include <TextUtils.h>
+#include <Script.h>
+#include <Math64.h>
+#include <CodeFragments.h>
+#include <stddef.h>
+
+#define	__COMPILINGMOREFILES
+
+#if 0
+#include "MoreFiles.h"
+#include "MoreFilesExtras.h"
+#include "MoreDesktopMgr.h"
+#include "FSpCompat.h"
+#endif
+
+/*
+**	GetVolumeInfoNoName uses pathname and vRefNum to call PBHGetVInfoSync
+**	in cases where the returned volume name is not needed by the caller.
+**	The pathname and vRefNum parameters are not touched, and the pb
+**	parameter is initialized by PBHGetVInfoSync except that ioNamePtr in
+**	the parameter block is always returned as NULL (since it might point
+**	to the local tempPathname).
+**
+**	I noticed using this code in several places, so here it is once.
+**	This reduces the code size of MoreFiles.
+*/
+static pascal	OSErr	GetVolumeInfoNoName(ConstStr255Param pathname,
+									short vRefNum,
+									HParmBlkPtr pb)
+{
+	Str255 tempPathname;
+	OSErr error;
+	
+	/* Make sure pb parameter is not NULL */ 
+	if ( pb != NULL )
+	{
+		pb->volumeParam.ioVRefNum = vRefNum;
+		if ( pathname == NULL )
+		{
+			pb->volumeParam.ioNamePtr = NULL;
+			pb->volumeParam.ioVolIndex = 0;		/* use ioVRefNum only */
+		}
+		else
+		{
+			BlockMoveData(pathname, tempPathname, pathname[0] + 1);	/* make a copy of the string and */
+			pb->volumeParam.ioNamePtr = (StringPtr)tempPathname;	/* use the copy so original isn't trashed */
+			pb->volumeParam.ioVolIndex = -1;	/* use ioNamePtr/ioVRefNum combination */
+		}
+		error = PBHGetVInfoSync(pb);
+		pb->volumeParam.ioNamePtr = NULL;	/* ioNamePtr may point to local	tempPathname, so don't return it */
+	}
+	else
+	{
+		error = paramErr;
+	}
+	return ( error );
+}
+
+/*****************************************************************************/
+
+static pascal	OSErr	DetermineVRefNum(ConstStr255Param pathname,
+								 short vRefNum,
+								 short *realVRefNum)
+{
+	HParamBlockRec pb;
+	OSErr error;
+
+	error = GetVolumeInfoNoName(pathname,vRefNum, &pb);
+	if ( error == noErr )
+	{
+		*realVRefNum = pb.volumeParam.ioVRefNum;
+	}
+	return ( error );
+}
+
+/*****************************************************************************/
+
+static pascal	OSErr GetCatInfoNoName(short vRefNum,
+							   long dirID,
+							   ConstStr255Param name,
+							   CInfoPBPtr pb)
+{
+	Str31 tempName;
+	OSErr error;
+	
+	/* Protection against File Sharing problem */
+	if ( (name == NULL) || (name[0] == 0) )
+	{
+		tempName[0] = 0;
+		pb->dirInfo.ioNamePtr = tempName;
+		pb->dirInfo.ioFDirIndex = -1;	/* use ioDirID */
+	}
+	else
+	{
+		pb->dirInfo.ioNamePtr = (StringPtr)name;
+		pb->dirInfo.ioFDirIndex = 0;	/* use ioNamePtr and ioDirID */
+	}
+	pb->dirInfo.ioVRefNum = vRefNum;
+	pb->dirInfo.ioDrDirID = dirID;
+	error = PBGetCatInfoSync(pb);
+	pb->dirInfo.ioNamePtr = NULL;
+	return ( error );
+}
+
+/*****************************************************************************/
+
+static pascal	OSErr	GetDirectoryID(short vRefNum,
+							   long dirID,
+							   ConstStr255Param name,
+							   long *theDirID,
+							   Boolean *isDirectory)
+{
+	CInfoPBRec pb;
+	OSErr error;
+
+	error = GetCatInfoNoName(vRefNum, dirID, name, &pb);
+	if ( error == noErr )
+	{
+		*isDirectory = (pb.hFileInfo.ioFlAttrib & kioFlAttribDirMask) != 0;
+		if ( *isDirectory )
+		{
+			*theDirID = pb.dirInfo.ioDrDirID;
+		}
+		else
+		{
+			*theDirID = pb.hFileInfo.ioFlParID;
+		}
+	}
+	
+	return ( error );
+}
+
+/*****************************************************************************/
+
+static pascal	OSErr	FSpGetDirectoryID(const FSSpec *spec,
+								  long *theDirID,
+								  Boolean *isDirectory)
+{
+	return ( GetDirectoryID(spec->vRefNum, spec->parID, spec->name,
+			 theDirID, isDirectory) );
+}
+
+#endif  /* INLINE_MOREFILES */
+#endif  /* OS_MAC */
 
 /*****************************************************************************
 *
@@ -153,99 +395,129 @@ static char * _this_file = __FILE__;
 *****************************************************************************/
 
 #ifdef OS_MAC
-static short Nlm_MacGetVRefNum (Nlm_CharPtr pathname, OSErr *errptr)
 
+static OSErr MacPathname2FSSpec(const char *inPathname, FSSpec *outFSS)
 {
-  OSErr           err;
-  Nlm_Char        filename [FILENAME_MAX];
-  Nlm_Char        path [256];
-  HParamBlockRec  pbh;
-  Nlm_CharPtr     ptr;
-
-  memset (&pbh, 0, sizeof (HParamBlockRec));
-  Nlm_StringNCpy_0(path, pathname, sizeof(path));
-  ptr = Nlm_StringRChr (path, (int) DIRDELIMCHR);
-  if (ptr != NULL) {
-    ptr++;
-    Nlm_StringNCpy_0(filename, ptr, sizeof(filename));
-    *ptr = '\0';
-    Nlm_CtoPstr ((Nlm_CharPtr) path);
-    pbh.volumeParam.ioNamePtr = (StringPtr) path;
-    pbh.volumeParam.ioVolIndex = -1;
-    err = PBHGetVInfo (&pbh, FALSE);
-    if (errptr != NULL) {
-      *errptr = err;
-    }
-    return pbh.volumeParam.ioVRefNum;
-  } else {
-    if (errptr != NULL) {
-      *errptr = noErr;
-    }
-    return 0;
-  }
+	OSErr err;
+	size_t len;
+	char *p;
+	short vRefNum;
+	long dirID;
+	FSSpec fss;
+	
+	if (inPathname == NULL || outFSS == NULL) {
+		return paramErr;
+	}
+	
+	err = HGetVol(NULL, &vRefNum, &dirID);  // default volume and directory
+	if (err != noErr) return err;
+	
+	len = strlen(inPathname);
+	
+	p = strchr(inPathname, ':');
+	if (p == NULL) {
+		// Partial pathname -- filename only
+		Str31 filename;
+		assert(len <= 31);
+		c2pstrcpy(filename, inPathname);
+		err = FSMakeFSSpec(vRefNum, dirID, filename, outFSS);
+	} else {
+		Str31 name;
+		int nameLen;
+		if (inPathname[0] == ':') {
+			// Relative pathname including directory path
+			
+		} else {
+			// Absolute pathname
+			//Str31 volName;  // We would use Str28 if it was defined -- 27, plus 1 for ':'.
+			nameLen = p - inPathname;
+			assert(nameLen <= 27);
+			name[0] = nameLen + 1;
+			memcpy(name + 1, inPathname, nameLen + 1);  // Copy the volume name and the colon.
+			err = DetermineVRefNum(name, 0, &vRefNum);
+			if (err != noErr) return err;
+			dirID = 2;
+		}
+		// vRefNum and dirID now specify the directory in which we should descend
+		// the path pointed to by p (pointing to the first colon).
+		p++;
+		while (p != NULL && *p != '\0') {
+			char *q = strchr(p, ':');
+			if (q != NULL) {
+				Boolean isDir;
+				nameLen = q - p;
+				assert(nameLen <= 31);
+				name[0] = nameLen;
+				memcpy(name + 1, p, nameLen);
+				err = FSMakeFSSpec(vRefNum, dirID, name, &fss);
+				if (err != noErr) return err;
+				if (q[1] == '\0') {
+					p = NULL;
+					*outFSS = fss;
+				} else {
+					err = FSpGetDirectoryID(&fss, &dirID, &isDir);
+					assert(isDir == true);
+					if (err != noErr) return err;
+					p = q + 1;
+				}
+			} else {
+				q = strchr(p, '\0');  // go to end of string
+				nameLen = q - p;
+				assert(nameLen > 0);
+				assert(nameLen <= 31);
+				c2pstrcpy(name, p);
+				p = NULL;
+				err = FSMakeFSSpec(vRefNum, dirID, name, outFSS);
+			}
+		}
+	}
+	return err;
 }
 
-static long Nlm_MacGetDirID (Nlm_CharPtr pathname, short newVRefNum, OSErr *errptr)
-
+#if 0
+static OSErr MacFSSpec2FullPathname(const FSSpec *inFSS, char **outPathname)
 {
-  OSErr           err;
-  Nlm_Char        path [256];
-  CInfoPBRec      pbc;
-  Nlm_CharPtr     ptr;
+	OSErr err;
+	Handle h;
+	short fullPathLength;
+	static char *fullPath = NULL;
+	
+	if (fullPath != NULL) {
+		Nlm_Free(fullPath);
+		fullPath = NULL;
+	}
+	err = FSpGetFullPath(inFSS, &fullPathLength, &h);
+	if (err != noErr) return err;
+	
+	assert(fullPathLength >= 2);  // An absolute pathname must be at least two chars long
+	fullPath = (char *)Nlm_Malloc(fullPathLength + 1);
+	if (fullPath == NULL) {
+		err = memFullErr;
+	} else {
+		strncpy(fullPath, *h, fullPathLength);
+	}
+	
+	DisposeHandle(h);
+	
+	*outPathname = fullPath;
+	return err;
+}
+#endif
 
-  memset (&pbc, 0, sizeof (CInfoPBRec));
-  Nlm_StringNCpy_0(path, pathname, sizeof(path));
-  ptr = Nlm_StringRChr (path, (int) DIRDELIMCHR);
-  if (ptr != NULL) {
-    ptr++;
-    Nlm_StringNCpy_0(path, pathname, sizeof(path));
-    *ptr = '\0';
-    Nlm_CtoPstr ((Nlm_CharPtr) path);
-    pbc.dirInfo.ioNamePtr = (StringPtr) path;
-    pbc.dirInfo.ioVRefNum = newVRefNum;
-    err = PBGetCatInfo (&pbc, FALSE);
-    if (errptr != NULL) {
-      *errptr = err;
-    }
-    return pbc.dirInfo.ioDrDirID;
-  } else {
-    if (errptr != NULL) {
-      *errptr = noErr;
-    }
-    return 0;
-  }
+static OSErr MacCreateDirectory(const char *inPathname)
+{
+	OSErr err;
+	FSSpec fss;
+	ScriptCode scriptTag = 0;
+	long createdDirID;
+	
+	err = MacPathname2FSSpec(inPathname, &fss);
+	if (err != noErr) return err;
+	
+	err = FSpDirCreate(&fss, scriptTag, &createdDirID);
+	return err;
 }
 
-static OSErr Nlm_SetDefault (short newVRefNum, long newDirID, short *oldVRefNum, long *oldDirID)
-
-{
-  OSErr  error;
-
-  error = HGetVol (NULL, oldVRefNum, oldDirID);
-  if (error == noErr) {
-    error = HSetVol (NULL, newVRefNum, newDirID);
-  }
-  return (error);
-}
-
-static OSErr Nlm_RestoreDefault (short oldVRefNum, long oldDirID)
-
-{
-  OSErr  error;
-  short  defaultVRefNum;
-  long   defaultDirID;
-  long   defaultProcID;
-
-  error = GetWDInfo (oldVRefNum, &defaultVRefNum, &defaultDirID, &defaultProcID);
-  if (error == noErr) {
-    if (defaultDirID != fsRtDirID) {
-      error = SetVol (NULL, oldVRefNum);
-    } else {
-      error = HSetVol (NULL, oldVRefNum, oldDirID);
-    }
-  }
-  return (error);
-}
 #endif
 
 /*****************************************************************************
@@ -257,87 +529,11 @@ static OSErr Nlm_RestoreDefault (short oldVRefNum, long oldDirID)
 *
 *****************************************************************************/
 
-#ifdef OS_MAC
-#ifndef COMP_THINKC
-#define USE_MPW_FILE_OPEN
-#endif
-#endif
+/* p_churchill 12/99 removed MPW conditional compilation
+ */
 
 static Nlm_FileOpenHook _hookFile = NULL;
 
-#ifdef USE_MPW_FILE_OPEN
-/*
-*  MPWOptimizationErrorBypass was called in order to avoid an apparent MPC C
-*  compiler optimization problem that resulted in the newDirID value sometimes
-*  appearing to be 0.  Placing any debugging statement (or this dummy function)
-*  after the statement that gets newDirID and before the statement that uses it
-*  originally appeared to fix the problem.  Upon further investigation, it turns
-*  out to be a problem with a UNIX to Mac file server product.  The problem did
-*  not occur when compiling under THINK C.
-*/
-
-static Nlm_Int2 Nlm_MPWOptimizationErrorBypass (short newVRefNum, long newDirID)
-{
-  return 0;
-}
-
-/*
-*  In MPW, if a temporary file is written first, without being created, it is
-*  created on the hard disk, rather than in the appropriate path.  This code
-*  creates the file in the desired location.
-*/
-
-static void Nlm_MPWCreateOutputFile (Nlm_CharPtr pathname, Nlm_CharPtr filename)
-
-{
-  OSErr     err;
-  FILE      *f;
-  Nlm_Char  temp [256];
-
-  f = fopen (filename, "r");
-  if (f == NULL) {
-    Nlm_StringNCpy_0(temp, pathname, sizeof(temp));
-    Nlm_CtoPstr ((Nlm_CharPtr) temp);
-    err = Create ((StringPtr) temp, 0, '    ', 'TEXT');
-  } else {
-    fclose (f);
-  }
-}
-
-static FILE * LIBCALL  Nlm_MPWFileOpen (Nlm_CharPtr pathname, Nlm_CharPtr mode)
-
-{
-  Nlm_Boolean  createfile;
-  OSErr        err;
-  FILE         *f;
-  long         newDirID;
-  short        newVRefNum;
-  long         oldDirID;
-  short        oldVRefNum;
-  Nlm_CharPtr  ptr;
-
-  newVRefNum = Nlm_MacGetVRefNum (pathname, &err);
-  newDirID = Nlm_MacGetDirID (pathname, newVRefNum, &err);
-  ptr = Nlm_StringRChr (pathname, (int) DIRDELIMCHR);
-  createfile = (Nlm_Boolean) (strchr (mode, 'w') != NULL);
-  if (ptr != NULL) {
-    ptr++;
-	Nlm_MPWOptimizationErrorBypass (newVRefNum, newDirID);
-    err = Nlm_SetDefault (newVRefNum, newDirID, &oldVRefNum, &oldDirID);
-	if (createfile) {
-	  Nlm_MPWCreateOutputFile (pathname, ptr);
-	}
-    f = fopen (ptr, mode);
-    err = Nlm_RestoreDefault (oldVRefNum, oldDirID);
-  } else {
-    if (createfile) {
-	  Nlm_MPWCreateOutputFile (pathname, pathname);
-	}
-    f = fopen (pathname, mode);
-  }
-  return f;
-}
-#endif
 
 NLM_EXTERN FILE * LIBCALL Nlm_FileOpen(const char *filename, const char *mode)
 {
@@ -368,44 +564,35 @@ NLM_EXTERN FILE * LIBCALL Nlm_FileOpen(const char *filename, const char *mode)
       f = fopen(filename, mode);
 
 #elif defined(OS_MAC)
-  {{
+  {
     OSType    fCreator;
     Nlm_Int2  fError;
     FInfo     fInfo;
     OSType    fType;
     Nlm_Char  temp [256];
+
     Nlm_StringNCpy_0(temp, filename, sizeof(temp));
     Nlm_CtoPstr ((Nlm_CharPtr) temp);
-    fError = GetFInfo ((StringPtr) temp, 0, &fInfo);
-    if (fError == 0) {
+    fError = HGetFInfo( 0, 0, (StringPtr) temp, &fInfo);
+    if (fError == noErr) {
       fCreator = fInfo.fdCreator;
       fType = fInfo.fdType;
     } else {
-      if (strchr(mode, 'b') != NULL)
-        fType = '    ';
-      else
-        fType = 'TEXT';
-      fCreator = '    ';
+        fCreator = '    ';
+        if (strchr(mode, 'b') != NULL)
+            fType = '    ';
+        else
+            fType = 'TEXT';
     }
-#ifdef USE_MPW_FILE_OPEN
-    {{
-      Nlm_Char localmode [16];
-      Nlm_Char path [PATH_MAX];
-      Nlm_StringNCpy_0(path, filename, sizeof(path));
-      Nlm_StringNCpy_0(localmode, mode, sizeof(localmode));
-      f = Nlm_MPWFileOpen(path, localmode);
-    }}
-#else
-    f = fopen (filename,mode);
-#endif /* USE_MPW_FILE_OPEN */
+    f = fopen( filename, mode);
 
-    fError = GetFInfo ((StringPtr) temp, 0, &fInfo);
-    if (fError == 0) {
+    fError = HGetFInfo( 0, 0, (StringPtr) temp, &fInfo);
+    if (fError == noErr) {
       fInfo.fdCreator = fCreator;
       fInfo.fdType = fType;
-      fError = SetFInfo ((StringPtr) temp, 0, &fInfo);
+      fError = HSetFInfo ( 0, 0, (StringPtr) temp,&fInfo);
     }
-  }} /* def OS_MAC */
+  } /* def OS_MAC */
 
 #elif defined(OS_VMS) && defined(DCC4DW12)
   /* never used */ 
@@ -467,10 +654,6 @@ NLM_EXTERN void LIBCALL  Nlm_FileClose (FILE *stream)
 /*****************************************************************************
 *   FileRead(buf, size, fp)
 *****************************************************************************/
-#ifdef WIN16
-#include <dos.h> /* dos.h defines the FP_SEG macro */
-#endif
-
 NLM_EXTERN size_t LIBCALL Nlm_FileRead
 (void *ptr, size_t size, size_t n, FILE *stream)
 {
@@ -490,19 +673,19 @@ NLM_EXTERN size_t LIBCALL Nlm_FileRead
 NLM_EXTERN size_t LIBCALL Nlm_FileWrite
 (const void *ptr, size_t size, size_t n, FILE *stream)
 {
-  size_t cnt;
-  if (n   &&  (SIZE_MAX / n)  <  size) {
-    ErrPostEx(SEV_WARNING,E_Programmer,0,"FileWrite:  size > SIZE_MAX");
-    return 0;
-  }
-  if (!ptr  ||  !stream)
-    return 0;
-
-  cnt = fwrite(ptr,size,n,stream);
-  if (cnt != n)
-    ErrPostEx(SEV_FATAL,E_File,E_FWrite,"File write error");
-
-  return cnt;
+    size_t cnt;
+    if (n   &&  (SIZE_MAX / n)  <  size) {
+        ErrPostEx(SEV_WARNING,E_Programmer,0,"FileWrite:  size > SIZE_MAX");
+        return 0;
+    }
+    if (!ptr  ||  !stream || !size)
+        return 0;
+    
+    cnt = fwrite(ptr,size,n,stream);
+    if (cnt != n)
+        ErrPostEx(SEV_FATAL,E_File,E_FWrite,"File write error");
+    
+    return cnt;
 }
 
 /*****************************************************************************
@@ -631,51 +814,76 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_FileNameFind (Nlm_CharPtr pathname)
   }
 }
 
+
+/*****************************************************************************
+*
+*   FilePathFind()
+*
+*****************************************************************************/
+NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_FilePathFind(const Nlm_Char* fullname)
+{
+  Nlm_CharPtr str;
+  size_t	     len = Nlm_StringLen(fullname);
+  if ( !len )
+    return 0;
+
+  while (len  &&  fullname[len] != DIRDELIMCHR)
+    len--;
+
+  str = (Nlm_Char*)Nlm_MemGet(len + 1, MGET_ERRPOST);
+  Nlm_MemCpy(str, fullname, len);
+  str[len] = '\0';
+  return str;
+}
+
+
 /*****************************************************************************
 *
 *   FileLength()
 *
 *****************************************************************************/
-NLM_EXTERN Nlm_Int4 LIBCALL Nlm_FileLength (Nlm_CharPtr fileName)
-
+NLM_EXTERN Nlm_Int8 LIBCALL Nlm_FileLength(Nlm_CharPtr fileName)
 {
-#ifdef OS_MAC
-  ParamBlockRec  params;
-  OSErr          err;
-  Nlm_Char       path [256];
-  Nlm_Int4       rsult;
+  Nlm_Int8 file_len = Nlm_FileLengthEx(fileName);
+  return (file_len > 0) ? file_len : 0;
+}
 
-  rsult = 0;
-  if (fileName != NULL && fileName [0] != '\0') {
+
+/*****************************************************************************
+*
+*   FileLengthEx()
+*
+*****************************************************************************/
+NLM_EXTERN Nlm_Int8 LIBCALL Nlm_FileLengthEx(const Nlm_Char* fileName)
+{
+  if (!fileName  ||  !*fileName)
+    return -1;
+
+#ifdef OS_MAC
+  {{
+    OSErr           err;
+    HParamBlockRec  params;
+    Nlm_Char        path[256];
+
+	Nlm_MemSet ((Nlm_VoidPtr) &params, 0, sizeof (HParamBlockRec));
     Nlm_StringNCpy_0(path, fileName, sizeof(path));
-    params.fileParam.ioNamePtr = (StringPtr) path;
+    Nlm_CtoPstr((Nlm_CharPtr) path);
+    params.fileParam.ioNamePtr = (StringPtr)path;
     params.fileParam.ioVRefNum = 0;
     params.fileParam.ioFDirIndex = 0;
-    Nlm_CtoPstr ((Nlm_CharPtr) path);
-    err = PBGetFInfo (&params, FALSE);
-    if (err == noErr) {
-      rsult = params.fileParam.ioFlLgLen;
-    }
-  }
-  return rsult;
+    err = PBHGetFInfo(&params, FALSE);
+    return (err == noErr) ?
+            params.fileParam.ioFlLgLen : -1;
+  }}
 #else
-  int          err;
-  Nlm_Char     local [256];
-  Nlm_Int4     rsult;
-  struct stat  sbuf;
-
-  rsult = 0;
-  if (fileName != NULL && fileName [0] != '\0') {
-    Nlm_StringNCpy_0(local, fileName, sizeof(local));
-    err = stat (local, &sbuf);
-    if (err == 0) {
-      rsult = sbuf.st_size;
-    }
-  }
-  return rsult;
+  {{
+    struct stat sbuf;
+    return (stat(fileName, &sbuf) == 0) ? sbuf.st_size : -1;
+  }}
 #endif
 }
 
+
 /*****************************************************************************
 *
 *   FileDelete()
@@ -737,27 +945,31 @@ static OSType Nlm_GetOSType (Nlm_CharPtr str, OSType dfault)
 NLM_EXTERN void LIBCALL Nlm_FileCreate (Nlm_CharPtr fileName, Nlm_CharPtr type, Nlm_CharPtr creator)
 
 {
-  FILE      *fp;
 #ifdef WIN_MAC
-  OSType    fCreator;
   Nlm_Int2  fError;
-  OSType    fType;
   Nlm_Char  temp [256];
+  OSType    fType;
+  OSType    fCreator;
+  FSSpec    spec;
+#else
+  FILE      *fp;
 #endif
 
   if (fileName != NULL && fileName [0] != '\0') {
+
 #ifdef WIN_MAC
-    if (type != NULL || creator != NULL) {
-      fp = Nlm_FileOpen (fileName, "r");
-      if (fp == NULL) {
+    // note: the following assumes either full pathname or that the current
+    // directory is the proper location to find/create the file
+
+    Nlm_StringNCpy_0(temp, fileName, sizeof(temp));
+    Nlm_CtoPstr ( temp);
+    fError = FSMakeFSSpec( 0, 0, (StringPtr)temp, &spec);
+    
+    // file not found, so create it...
+    if( fError == fnfErr){
         fType = Nlm_GetOSType (type, 'TEXT');
         fCreator = Nlm_GetOSType (creator, '    ');
-        Nlm_StringNCpy_0(temp, fileName, sizeof(temp));
-        Nlm_CtoPstr ((Nlm_CharPtr) temp);
-        fError = Create ((StringPtr) temp, 0, fCreator, fType);
-      } else {
-        Nlm_FileClose (fp);
-      }
+        FSpCreate( &spec, fCreator, fType, smSystemScript);
     }
 #else
     fp = Nlm_FileOpen (fileName, "w");
@@ -774,51 +986,31 @@ NLM_EXTERN void LIBCALL Nlm_FileCreate (Nlm_CharPtr fileName, Nlm_CharPtr type,
 *
 *****************************************************************************/
 
+#ifdef OS_MAC
+NLM_EXTERN Nlm_Boolean LIBCALL  Nlm_CreateDir (Nlm_CharPtr pathname)
+{
+  if (pathname != NULL && pathname [0] != '\0') {
+    OSErr err;
+    err = MacCreateDirectory(pathname);
+    return (Nlm_Boolean) (err == noErr || err == dupFNErr);
+  }
+  return FALSE;
+}
+#endif
+
+#ifndef OS_MAC
 NLM_EXTERN Nlm_Boolean LIBCALL  Nlm_CreateDir (Nlm_CharPtr pathname)
 {
 #ifndef OS_VMS
   size_t          len;
   Nlm_Char        path[PATH_MAX];
 #endif
-#ifdef OS_MAC
-  long            dirID;
-  Nlm_Char        dirname [FILENAME_MAX];
-  OSErr           err;
-  HParamBlockRec  pbh;
-  Nlm_CharPtr     ptr;
-  short           vRefNum;
-#endif
 #ifdef OS_UNIX
   mode_t          oldmask;
 #endif
   Nlm_Boolean     rsult = FALSE;
 
   if (pathname != NULL && pathname [0] != '\0') {
-#ifdef OS_MAC
-    Nlm_StringNCpy_0(path, pathname, sizeof(path));
-    len = Nlm_StringLen (path);
-    if (len > 0 && path [len - 1] == DIRDELIMCHR) {
-        path [len - 1] = '\0';
-    }
-    memset (&pbh, 0, sizeof (HParamBlockRec));
-    vRefNum = Nlm_MacGetVRefNum (path, &err);
-    if (err == noErr) {
-      dirID = Nlm_MacGetDirID (path, vRefNum, &err);
-      if (err == noErr) {
-        ptr = Nlm_StringRChr (path, (int) DIRDELIMCHR);
-        if (ptr != NULL) {
-          ptr++;
-          Nlm_StringNCpy_0(dirname, ptr, sizeof(dirname));
-          Nlm_CtoPstr ((Nlm_CharPtr) dirname);
-          pbh.fileParam.ioNamePtr = (StringPtr) dirname;
-          pbh.fileParam.ioVRefNum = vRefNum;
-          pbh.fileParam.ioDirID = dirID;
-          err = PBDirCreate (&pbh, FALSE);
-          rsult = (Nlm_Boolean) (err == noErr || err == dupFNErr);
-        }
-      }
-    }
-#endif
 #if defined(OS_MSWIN) || defined(OS_NT)
     Nlm_StringNCpy_0(path, pathname, sizeof(path));
     len = Nlm_StringLen (path);
@@ -849,6 +1041,7 @@ NLM_EXTERN Nlm_Boolean LIBCALL  Nlm_CreateDir (Nlm_CharPtr pathname)
   }
   return rsult;
 }
+#endif
 
 /*****************************************************************************
 *
@@ -938,22 +1131,6 @@ NLM_EXTERN ValNodePtr LIBCALL Nlm_DirCatalog (Nlm_CharPtr pathname)
       FindClose(hFindFile);
     }}
 #endif
-#if defined(WIN16)
-    {{
-      Nlm_Char x_path[PATH_MAX];
-      struct find_t fData;
-      Nlm_StringNCpy_0(x_path, pathname, sizeof(x_path) - 5);
-      Nlm_StringCat(x_path, "\\*.*");
-      if (_dos_findfirst(x_path, _A_SUBDIR|_A_RDONLY|_A_NORMAL|_A_ARCH, &fData)
-          != 0)
-        return 0;
-      do {
-        if (fData.name[0] != '.'  ||
-            (fData.name[1] != '.'  &&  fData.name[1] != '\0'))
-          ValNodeCopyStr(&vnp, (fData.attrib & _A_SUBDIR) ? 1 : 0, fData.name);
-      } while (_dos_findnext(&fData) == 0);
-    }}
-#endif
 #ifdef OS_UNIX
     sprintf (cmmd, "ls -1p %s 2>/dev/null", pathname);
     fp = popen (cmmd, "r");
@@ -998,7 +1175,7 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TmpNam (Nlm_CharPtr s)
     /* place the files in another directory specified by the environment   */
     /* variable TMPDIR                                                     */
 
-    filename = tempnam("", "");
+    filename = tempnam(NULL, NULL);
 
     if (s == NULL)
     { /* return pointer to static string */
@@ -1026,8 +1203,6 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TmpNam (Nlm_CharPtr s)
     long         gesResponse;
     long         newDirID;
     short        newVRefNum;
-    long         oldDirID;
-    short        oldVRefNum;
     CInfoPBRec   params;
     Nlm_Char     temp [PATH_MAX];
     Nlm_CharPtr  tmp;
@@ -1041,12 +1216,10 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TmpNam (Nlm_CharPtr s)
                        kCreateFolder, &newVRefNum, &newDirID);
       if (err == noErr) {
         useTempFolder = TRUE;
-        err = Nlm_SetDefault (newVRefNum, newDirID, &oldVRefNum, &oldDirID);
       }
     }
     filename = tmpnam (NULL);
     if (useTempFolder) {
-      err = Nlm_RestoreDefault (oldVRefNum, oldDirID);
       temp [0] = '\0';
       params.dirInfo.ioNamePtr = (StringPtr) directory;
       params.dirInfo.ioDrParID = newDirID;
@@ -1093,210 +1266,3 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TmpNam (Nlm_CharPtr s)
 #endif
 }
 
-/*****************************************************************************
-*
-*   CD-ROM Ejection Routines
-*
-*****************************************************************************/
-
-NLM_EXTERN Nlm_Boolean LIBCALL  Nlm_EjectCd(Nlm_CharPtr sVolume, Nlm_CharPtr deviceName,
-			Nlm_CharPtr rawDeviceName, 
-			Nlm_CharPtr mountPoint,
-			Nlm_CharPtr mountCmd)
-{
-    Nlm_Boolean retval = FALSE;
-#ifdef OS_MAC
-    OSErr err;
-    Nlm_CharPtr prob_area = "Ejection";
-    Nlm_Char    temp [64];
-
-    
-    Nlm_StringNCpy_0(temp, sVolume, sizeof(temp) - 1);
-    Nlm_StringCat (temp, ":");
-    if ((err = Eject((StringPtr) NULL, Nlm_MacGetVRefNum(temp, NULL))) == noErr)
-    {
-        if ((err = UnmountVol((StringPtr) NULL, Nlm_MacGetVRefNum(temp, NULL))) == noErr)
-        	return TRUE;
-        
-        /* We should still return TRUE if we Eject() successfully but failed to    */
-        /* unmount the volume; however, we need to warn them, because a subsequent */
-        /* GetFInfo() will result in a bus error, at least with System 7.0.        */
-        retval = TRUE;
-        prob_area = "Unmounting";
-    }
-
-    switch (err) {
-    case bdNamErr:
-	ErrPostEx(SEV_ERROR,E_File,E_CdEject,"%s error - bad volume name %s", prob_area, sVolume);
-	break;
-    case extFSErr:
-	ErrPostEx(SEV_ERROR,E_File,E_CdEject,"%s error - external file system %s", prob_area, sVolume);
-	break;
-    case ioErr:
-	ErrPostEx(SEV_ERROR,E_File,E_CdEject,"%s error - I/O error %s", prob_area, sVolume);
-	break;
-    case nsDrvErr:
-	ErrPostEx(SEV_ERROR,E_File,E_CdEject,"%s error - No such drive %s", prob_area, sVolume);
-	break;
-    case nsvErr:
-	ErrPostEx(SEV_ERROR,E_File,E_CdEject,"%s error - No such volume %s", prob_area, sVolume);
-	break;
-    case paramErr:
-	ErrPostEx(SEV_ERROR,E_File,E_CdEject,"%s error - No default volume %s", prob_area, sVolume);
-	break;
-    }
-    
-    return retval;
-#endif /* OS_MAC */
-
-#ifdef OS_UNIX
-	char cmd[100];
-#endif
-#ifdef OS_UNIX_SUN
-	int fd;
-
-	if (deviceName == NULL)
-	{
-		deviceName = DEFAULT_CDROM;
-	}
-
-	if (rawDeviceName == NULL)
-	{
-		rawDeviceName = DEFAULT_RAW_CDROM;
-	}
-
-	/* Open the CD-ROM character-based device */
-	if ((fd = open(rawDeviceName, O_RDONLY, 0)) < 0)
-	{
-		ErrPostEx(SEV_ERROR,E_File,E_CdEject,"Ejection error - Unable to open device %s", rawDeviceName);
-		return FALSE;
-	}
-
-	retval = ioctl(fd, CDROMEJECT, 0) >= 0;
-	close (fd);
-
-	if (! retval)
-	{
-		ErrPostEx(SEV_ERROR,E_File,E_CdEject,"Ejection error - Ioctl failure for %s", rawDeviceName);
-    	return FALSE;
-	}
-#endif /* OS_UNIX_SUN */
-
-#ifdef OS_UNIX
-	/* Now try to unmount device using (un)mount-script */
-	if (mountCmd != NULL)
-	{
-		sprintf(cmd, "%s -u %s >/dev/null 2>/dev/null", mountCmd,
-				deviceName);
-		retval = system(cmd) == 0;
-	}
-	else {
-		if (deviceName != NULL)
-		{
-			retval = Message(MSG_OKC,
-				            "Unmount device <%s> now; select OK when completed",
-						    deviceName) != ANS_CANCEL;
-		}
-		else
-		if (sVolume != NULL)
-		{
-			retval = Message(MSG_OKC,
-						    "Unmount volume <%s> now; select OK when completed",
-						    sVolume) != ANS_CANCEL;
-		}
-		else
-		{
-			retval = Message(MSG_OKC,
-							"Unmount CD-ROM now; select OK when completed") !=
-							ANS_CANCEL;
-		}
-	}
-#endif /* OS_UNIX */
-
-#ifdef OS_VMS
-	char  cmd[100];
-	char  tmp[100];
-	char* cPtr;
-
-
-	if ( mountPoint == NULL || *mountPoint == '\0' ) 
-		strcpy(tmp,DEFAULT_CDROM); 
-	else {
-		strcpy(tmp,mountPoint);
-		if ( cPtr = strchr(tmp,':') ) *(cPtr+1) = '\0';
-	}
-	/* 
-	** Try to mount device using mount-script 
-	*/
-
-	sprintf(cmd, "CD_DISMOUNT/UNLOAD %s",tmp);
-	retval = (system(cmd) == 0);
-
-	 Message(MSG_OK,
-		"Press the eject button on <%s>.",tmp);
-
-#endif
-	
-    return retval;
-}
-
-NLM_EXTERN Nlm_Boolean LIBCALL  Nlm_MountCd(Nlm_CharPtr sVolume, Nlm_CharPtr deviceName,
-			Nlm_CharPtr mountPoint, Nlm_CharPtr mountCmd)
-{
-	Nlm_Boolean retval = FALSE;
-
-#ifdef OS_UNIX
-	char cmd[100];
-
-	if (deviceName == NULL)
-	{
-		deviceName = DEFAULT_CDROM;
-	}
-
-	/* Try to mount device using mount-script */
-	if (mountCmd != NULL)
-	{
-		sprintf(cmd, "%s -m %s %s >/dev/null 2>/dev/null", mountCmd, deviceName,
-				mountPoint != NULL ? mountPoint : "");
-		retval = system(cmd) == 0;
-	}
-	else {
-		if (deviceName != NULL)
-		{
-		}
-		else
-		{
-			retval = Message(MSG_OKC,
-							"Mount CD-ROM now; select OK when completed") !=
-							ANS_CANCEL;
-		}
-	}
-#endif
-
-#ifdef OS_VMS
-	char  cmd[100];
-	char  tmp[100];
-	char* cPtr;
-
-
-	if ( mountPoint == NULL || *mountPoint == '\0' ) 
-		strcpy(tmp,DEFAULT_CDROM); 
-	else {
-		strcpy(tmp,mountPoint);
-		if ( cPtr = strchr(tmp,':') ) *(cPtr+1) = '\0';
-	}
-
-
-	/* Try to mount device using mount-script */
-
-	sprintf(cmd, "CD_MOUNT/MEDIA=CDROM/OVERRIDE=IDENTIFICATION/NOASSIST %s",
-          tmp);
-
-	retval = (system(cmd) == 0);
-
-#endif
-
-	return retval;
-}
-
-
