@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   10/23/91
 *
-* $Revision: 6.2 $
+* $Revision: 6.21 $
 *
 * File Description: 
 *   	miscellaneous functions
@@ -43,6 +43,64 @@
 * 02-16-94 Epstein     Retired Gestalt functions and definitions
 *
 * $Log: ncbimisc.c,v $
+* Revision 6.21  2001/04/05 03:03:28  juran
+* Defined our own C2PStr() and P2Cstr() for use with Carbon (which omits them).
+*
+* Revision 6.20  2000/10/30 18:11:41  beloslyu
+* FreeBSD was added
+*
+* Revision 6.19  2000/08/28 18:41:29  vakatov
+* Added type cast in ValNodeSort() to pass C++ compilation
+*
+* Revision 6.18  2000/03/10 17:29:12  kans
+* changed divray to Uint8 - fix supplied by DDBJ for Cray computer
+*
+* Revision 6.17  1999/12/21 17:52:40  kans
+* removed MPW/THINKC conditional code, starting upgrade to Carbon compatibility - Churchill
+*
+* Revision 6.16  1999/07/29 15:58:48  kans
+* added bigintvalue, ValNodeAddBigInt (PD)
+*
+* Revision 6.15  1999/06/07 18:22:20  beloslyu
+* NetBSD port
+*
+* Revision 6.14  1999/04/23 18:00:15  beloslyu
+* change Uint4 to Nlm_Uint4 and CharPtr to Nlm_CharPtr
+*
+* Revision 6.13  1999/04/23 17:28:50  shavirin
+* Changed Uint4 to the Nlm_Uint4
+*
+* Revision 6.12  1999/04/23 16:10:26  shavirin
+* Added new function Nlm_GetChecksum() which calculates
+* checksum from given string
+*
+* Revision 6.11  1999/04/14 19:37:23  madden
+* Add Nlm_ prefix to Int8
+*
+* Revision 6.10  1999/04/14 15:31:06  madden
+* add Nlm_Int8tostr function
+*
+* Revision 6.9  1999/03/11 16:21:51  kans
+* ValNodeSort is more visible copy of jzmisc SortValNode
+*
+* Revision 6.8  1999/01/29 19:23:05  kans
+* use C2PStr and P2CStr from <TextUtils.h>
+*
+* Revision 6.7  1999/01/21 20:08:37  ostell
+* added SwitchUint2 and 4, added integer bytestores
+*
+* Revision 6.6  1999/01/07 15:20:57  victorov
+* added Nlm_ prefix to MD5 functions to avoid conflicts
+*
+* Revision 6.5  1999/01/06 22:49:26  victorov
+* added MD5 hash calculator
+*
+* Revision 6.4  1998/10/13 20:49:38  vakatov
+* + Nlm_PlatformName()
+*
+* Revision 6.3  1998/07/02 18:24:28  vakatov
+* Cleaned the code & made it pass through the C++ compilation
+*
 * Revision 6.2  1998/04/08 16:49:56  kans
 * ValNodeLen returns Int4
 *
@@ -88,7 +146,7 @@
 #include <ncbiwin.h>
 
 #ifdef OS_MAC
-#include <Strings.h>
+#include <TextUtils.h>
 #endif
 
 /*
@@ -117,7 +175,7 @@ static int	bufno = 0; /* current buffer marker in the circular list */
 static char	buf[NBUFS][(CHAR_BIT)*sizeof(long)/2];
 
 /* divray[] is a fixed array of power-of-10 divisors which must be initialized*/
-static unsigned long divray[CHAR_BIT*sizeof(unsigned long)/2];
+static Uint8 divray[CHAR_BIT*sizeof(Uint8)/2];
 /* divray_max is related to the maximum precision available in a long int */
 static int	divray_max;
 
@@ -128,7 +186,7 @@ static char	commaray[DIM(divray)];
 static void divray_init PROTO((void));
 
 /* ncbi_ultostr() is the basic (unsigned) integer-to-ASCII conversion engine */
-static void ncbi_ultostr PROTO((char *buf, unsigned long value, int commas));
+static void ncbi_ultostr PROTO((char *buff, Uint8 value, int commas));
 
 /* ulwidth() is the basic length-determiner for integer-ASCII conversions */
 static int ulwidth PROTO((unsigned long value, int commas));
@@ -141,7 +199,7 @@ static void
 divray_init (void)
 
 {
-	unsigned long	j = ULONG_MAX, k = 1;
+	Uint8	j = UINT8_MAX, k = 1;
 
 	for (divray_max=0; divray_max < DIM(divray) && j != 0; ++divray_max) {
 		divray[divray_max] = k;
@@ -155,10 +213,9 @@ divray_init (void)
 
 /* ncbi_ultostr is the basic (unsigned) integer->ASCII conversion engine */
 static void
-ncbi_ultostr (char *buf, unsigned long value, int commas)
-
+ncbi_ultostr (char *buff, Uint8 value, int commas)
 {
-	unsigned long	value_orig = value;
+	Uint8 value_orig = value;
 	int	i, quotient;
 
 	if (divray_max == 0)
@@ -176,9 +233,9 @@ ncbi_ultostr (char *buf, unsigned long value, int commas)
 
 	for (; i >= 0; --i) {
 		if (commas && commaray[i] != NULLB && value != value_orig)
-			*buf++ = TRIPLE_MARK;
+			*buff++ = TRIPLE_MARK;
 		quotient = (int)(value / divray[i]);
-		*buf++ = (char) '0' + (char) quotient;
+		*buff++ = (char) '0' + (char) quotient;
 		switch (quotient) {
 		case 0: break;
 		case 1: value -=   divray[i]; break;
@@ -193,9 +250,31 @@ ncbi_ultostr (char *buf, unsigned long value, int commas)
 		default: value -= quotient*divray[i]; break; /* shouldn't be taken */
 		}
 	}
-	*buf = NULLB; /* tack on a NUL terminator */
+	*buff = NULLB; /* tack on a NUL terminator */
 	return;
 }
+/* Nlm_Int8tostr -- convert a signed long integer to ASCII */
+NLM_EXTERN char * LIBCALL Nlm_Int8tostr (Nlm_Int8 value, int opts)
+
+{
+	char	*bp0, *bp;
+
+    bp0 = bp = &buf[bufno][0];
+	if (++bufno >= NBUFS)
+		bufno = 0;
+
+	if (value < 0) {
+		*bp++ = '-';
+		value = -value;
+	}
+	else
+		if (opts&MISC_PLUSSIGNS && value > 0)
+			*bp++ = '+';
+
+	ncbi_ultostr(bp, (Uint8)value, opts&MISC_ANYCOMMAS);
+
+    return bp0;
+}
 
 /* Nlm_Ltostr -- convert a signed long integer to ASCII */
 NLM_EXTERN char * LIBCALL Nlm_Ltostr (long value, int opts)
@@ -215,7 +294,7 @@ NLM_EXTERN char * LIBCALL Nlm_Ltostr (long value, int opts)
 		if (opts&MISC_PLUSSIGNS && value > 0)
 			*bp++ = '+';
 
-	ncbi_ultostr(bp, (unsigned long)value, opts&MISC_ANYCOMMAS);
+	ncbi_ultostr(bp, (Uint8)value, opts&MISC_ANYCOMMAS);
 
     return bp0;
 }
@@ -233,7 +312,7 @@ NLM_EXTERN char * LIBCALL Nlm_Ultostr (unsigned long value, int opts)
 	if (opts&MISC_PLUSSIGNS && value > 0)
 		*bp++ = '+';
 
-	ncbi_ultostr(bp, value, opts&MISC_ANYCOMMAS);
+	ncbi_ultostr(bp, (Uint8) value, opts&MISC_ANYCOMMAS);
 
     return bp0;
 }
@@ -541,6 +620,28 @@ NLM_EXTERN ValNodePtr LIBCALL ValNodeAddInt (ValNodePtr PNTR head, Nlm_Int2 choi
 	return newnode;
 }
 
+/*****************************************************************************
+*
+*   ValNodeAddBigInt (head, choice, value)
+*      adds like ValNodeAdd()
+*      sets newnode->choice = choice (if choice does not matter, use 0)
+*      sets newnode->data.bigintvalue = value
+*   
+*****************************************************************************/
+NLM_EXTERN ValNodePtr LIBCALL ValNodeAddBigInt (ValNodePtr PNTR head, Nlm_Int2 choice, Nlm_Int8 value)
+{
+	ValNodePtr newnode;
+
+	newnode = ValNodeAdd(head);
+	if (newnode != NULL)
+	{
+		newnode->choice = (Nlm_Uint1)choice;
+		newnode->data.bigintvalue = value;
+	}
+
+	return newnode;
+}
+
 /*****************************************************************************
 *
 *   ValNodeAddBoolean (head, choice, value)
@@ -629,7 +730,6 @@ NLM_EXTERN ValNodePtr LIBCALL ValNodeAddFunction (ValNodePtr PNTR head, Nlm_Int2
 	return newnode;
 }
 
-
 /*****************************************************************************
 *
 *   ValNodeFree(vnp)
@@ -763,6 +863,38 @@ NLM_EXTERN ValNodePtr LIBCALL ValNodeFindNext (ValNodePtr head, ValNodePtr curr,
 	return curr;
 }
 
+/*****************************************************************************
+*
+*   ValNodeSort(list, compar)
+*   	Copied from SortValNode in jzcoll, renamed, for more general access
+*   	Makes array from ValNode list, calls HeapSort, reconnects ValNode list
+*
+*****************************************************************************/
+NLM_EXTERN ValNodePtr LIBCALL ValNodeSort (ValNodePtr list, int (LIBCALLBACK *compar )PROTO ((Nlm_VoidPtr, Nlm_VoidPtr )))
+{
+	ValNodePtr tmp, PNTR head;
+	Nlm_Int4 count, i;
+
+	if (list == NULL) return NULL;
+	
+	count = ValNodeLen (list);
+	head = (ValNodePtr *) MemNew (((size_t) count + 1) * sizeof (ValNodePtr));
+	for (tmp = list, i = 0; tmp != NULL && i < count; i++) {
+		head [i] = tmp;
+		tmp = tmp->next;
+	}
+
+	HeapSort (head, (size_t) count, sizeof (ValNodePtr), compar);
+	for (i = 0; i < count; i++) {
+		tmp = head [i];
+		tmp->next = head [i + 1];
+	}
+	list = head [0];
+	MemFree (head);
+
+	return list;
+}
+
 /*****************************************************************************
 *
 *   Start Of Node List Functions
@@ -985,39 +1117,47 @@ NLM_EXTERN Nlm_Boolean LIBCALL NodeListDelete (ValNodePtr head, Nlm_Int2 item)
 *****************************************************************************/
 
 #ifdef OS_MAC
+
+// 2001-03-22:  Joshua Juran
+// C2PStr() and P2CStr() do not exist in Carbon, so we roll our own.
+# if TARGET_API_MAC_CARBON
+static void C2PStr(char *ioStr)
+{
+	size_t len = strlen(ioStr);
+	if (len > 255) {
+		len = 255;
+	}
+	memmove(ioStr + 1, ioStr, len);
+	ioStr[0] = len;
+}
+
+static void P2CStr(StringPtr ioStr)
+{
+	Byte len = ioStr[0];
+	memmove(ioStr, ioStr + 1, len);
+	ioStr[len] = '\0';
+}
+# endif
+
+/*  p_churchill 12/99 removed conditional compilation support for 
+ *  THINKC and MPW
+ */
 void Nlm_CtoPstr (Nlm_CharPtr str)
 
 {
-#ifdef COMP_THINKC
-  c2pstr ((char *) str);
-#endif
-#ifdef COMP_MPW
-  c2pstr ((char *) str);
-#endif
-#ifdef COMP_METRO
-  c2pstr ((char *) str);
-#endif
+  C2PStr ((char *) str);
 }
 
 void Nlm_PtoCstr (Nlm_CharPtr str)
 
 {
-#ifdef COMP_THINKC
-  p2cstr ((StringPtr) str);
-#endif
-#ifdef COMP_MPW
-  p2cstr ((StringPtr) str);
-#endif
-#ifdef COMP_METRO
-  p2cstr ((StringPtr) str);
-#endif
+  P2CStr ((StringPtr) str);
 }
 #endif
 
 
-#ifdef IS_LITTLE_ENDIAN
 
-NLM_EXTERN Nlm_Uint2 Nlm_SwapUint2 (Nlm_Uint2 value)
+NLM_EXTERN Nlm_Uint2 Nlm_SwitchUint2 (Nlm_Uint2 value)
 
 {
 	Nlm_Uint2  m;
@@ -1026,7 +1166,7 @@ NLM_EXTERN Nlm_Uint2 Nlm_SwapUint2 (Nlm_Uint2 value)
 	return m;
 }
 
-NLM_EXTERN void Nlm_SwapUint2Buff (Nlm_Uint2 *buff, int count)
+NLM_EXTERN void Nlm_SwitchUint2Buff (Nlm_Uint2 *buff, int count)
 {
 	Nlm_Uint2 *ptr, n, m;
 	
@@ -1039,7 +1179,7 @@ NLM_EXTERN void Nlm_SwapUint2Buff (Nlm_Uint2 *buff, int count)
 	}
 }
 
-NLM_EXTERN unsigned long  Nlm_SwapLong (unsigned long value)
+NLM_EXTERN unsigned long  Nlm_SwitchLong (unsigned long value)
 {
 	unsigned long  m;
 	m  = ((value & (unsigned long)0xFF000000) >> 24);
@@ -1049,7 +1189,7 @@ NLM_EXTERN unsigned long  Nlm_SwapLong (unsigned long value)
 	return m;
 }
 
-NLM_EXTERN void Nlm_SwapLongBuff (unsigned long *buff, int count)
+NLM_EXTERN void Nlm_SwitchLongBuff (unsigned long *buff, int count)
 {
 	unsigned long *ptr, n, m;
 	
@@ -1064,7 +1204,7 @@ NLM_EXTERN void Nlm_SwapLongBuff (unsigned long *buff, int count)
 	}  
 }
 
-NLM_EXTERN Nlm_Uint4  Nlm_SwapUint4 (Nlm_Uint4 value)
+NLM_EXTERN Nlm_Uint4  Nlm_SwitchUint4 (Nlm_Uint4 value)
 {
 	Nlm_Uint4  m;
 	m  = ((value & (Nlm_Uint4)0xFF000000) >> 24);
@@ -1074,7 +1214,7 @@ NLM_EXTERN Nlm_Uint4  Nlm_SwapUint4 (Nlm_Uint4 value)
 	return m;
 }
 
-NLM_EXTERN void Nlm_SwapUint4Buff (Nlm_Uint4 *buff, int count)
+NLM_EXTERN void Nlm_SwitchUint4Buff (Nlm_Uint4 *buff, int count)
 {
 	Nlm_Uint4 *ptr, n, m;
 	
@@ -1088,7 +1228,7 @@ NLM_EXTERN void Nlm_SwapUint4Buff (Nlm_Uint4 *buff, int count)
 		*ptr++ = m;
 	}  
 }
-#endif /* IS_LITTLE_ENDIAN */
+
 
 /* the following ListXXX() functions previously resided in ni_list.c */
 
@@ -1511,3 +1651,401 @@ ListStrDel (NodePtr np)
         np = ListDelete(np);
     }
 }
+
+
+NLM_EXTERN const Nlm_Char* Nlm_PlatformName(void)
+{
+/* Mac */
+#if defined(OS_MAC)
+#  if defined(PROC_PPC)
+  return "PowerPC";
+#  elif defined(PROC_MC680X0)
+  return "MacOS_68K";
+#  else
+  return "MacOS";
+#  endif
+
+/* VMS */
+#elif defined(OS_VMS)
+#  if defined(OS_AXP_VMS)
+  return "AXP/OpenVMS";
+#  else
+  return "VMS";
+#  endif
+
+/* UNIX */
+#elif defined(OS_UNIX)
+#  if defined(PROC_IBM370)
+  return "IBM370_AIX";
+#  elif defined(OS_UNIX_SUN)
+  return "SunOS";
+#  elif defined(OS_UNIX_SOL)
+  return "Solaris";
+#  elif defined(OS_UNIX_OSF1) && defined(PROC_ALPHA)
+  return "Alpha_OSF1";
+#  elif defined(COMP_AUX)
+  return "Mac_AUX";
+#  elif defined(COMP_CRAY) && defined(PROC_YMP)
+  return "Cray";
+#  elif defined(PROC_CONVEX)
+  return "Convex";
+#  elif defined(PROC_HPPA)
+  return "HPUX";
+#  elif defined(OS_UNIX_NEXT)
+  return "NEXT";
+#  elif defined(PROC_MIPS)
+  return "SGI_MIPS";
+#  elif defined(OS_UNIX_ULTRIX)
+  return "ULTRIX";
+#  elif defined(OS_UNIX_SYSV) && defined(PROC_SPARC)
+  return "SystemV_SPARC";
+#  elif defined(OS_UNIX_AIX)
+  return "AIX";
+#  elif defined(OS_UNIX_LINUX)
+#    if defined(PROC_ALPHA)
+  return "LINUX_ALPHA";
+#    else
+  return "LINUX";
+#    endif
+#  elif defined(OS_UNIX_NETBSD)
+  return "NetBSD";
+#  elif defined(OS_UNIX_FREEBSD)
+  return "FreeBSD";
+#  else
+  return "UNIX";
+#  endif
+
+/* PC */
+#elif defined(OS_MSWIN)
+#  if defined(WIN16)
+  return "WIN16";
+#  elif defined(WIN32)
+  return "WIN32";
+#  else
+  return "MSWIN";
+#  endif
+
+/* NT */
+#elif defined(OS_WINNT)
+  return "WIN_NT";
+
+/* Unknown */
+#else
+  return "Unknown";
+
+#endif
+}
+
+
+/**
+ * MD5 stuff (used for security purposes)
+ */
+
+/*
+ * Note: this code is harmless on little-endian machines.
+ */
+static
+void
+byteReverse(Nlm_UcharPtr buf, Nlm_Uint4 longs)
+{
+    Nlm_Uint4 t;
+    do {
+	t = (Nlm_Uint4) ((unsigned) buf[3] << 8 | buf[2]) << 16 |
+	    ((unsigned) buf[1] << 8 | buf[0]);
+	*(Nlm_Uint4Ptr) buf = t;
+	buf += 4;
+    } while (--longs);
+}
+
+/*
+ * Start MD5 accumulation.  Set bit count to 0 and buffer to mysterious
+ * initialization constants.
+ */
+NLM_EXTERN void LIBCALL
+Nlm_MD5Init(Nlm_MD5ContextPtr ctx)
+{
+    ctx->buf[0] = 0x67452301;
+    ctx->buf[1] = 0xefcdab89;
+    ctx->buf[2] = 0x98badcfe;
+    ctx->buf[3] = 0x10325476;
+
+    ctx->bits[0] = 0;
+    ctx->bits[1] = 0;
+}
+
+/*
+ * Update context to reflect the concatenation of another buffer full
+ * of bytes.
+ */
+NLM_EXTERN void LIBCALL
+Nlm_MD5Update(Nlm_MD5ContextPtr ctx, Nlm_UcharPtr buf, Nlm_Uint4 len)
+{
+    Nlm_Uint4 t;
+
+    /* Update bitcount */
+
+    t = ctx->bits[0];
+    if ((ctx->bits[0] = t + ((Nlm_Uint4) len << 3)) < t)
+	ctx->bits[1]++;		/* Carry from low to high */
+    ctx->bits[1] += len >> 29;
+
+    t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */
+
+    /* Handle any leading odd-sized chunks */
+
+    if (t) {
+	Nlm_UcharPtr p = (Nlm_UcharPtr) ctx->in + t;
+
+	t = 64 - t;
+	if (len < t) {
+	    memcpy(p, buf, len);
+	    return;
+	}
+	memcpy(p, buf, t);
+#ifdef IS_BIG_ENDIAN
+	byteReverse(ctx->in, 16);
+#endif
+	Nlm_MD5Transform(ctx->buf, (Nlm_Uint4Ptr) ctx->in);
+	buf += t;
+	len -= t;
+    }
+    /* Process data in 64-byte chunks */
+
+    while (len >= 64) {
+	memcpy(ctx->in, buf, 64);
+#ifdef IS_BIG_ENDIAN
+	byteReverse(ctx->in, 16);
+#endif
+	Nlm_MD5Transform(ctx->buf, (Nlm_Uint4Ptr) ctx->in);
+	buf += 64;
+	len -= 64;
+    }
+
+    /* Handle any remaining bytes of data. */
+    memcpy(ctx->in, buf, len);
+}
+
+/*
+ * Final wrapup - pad to 64-byte boundary with the bit pattern 
+ * 1 0* (64-bit count of bits processed, MSB-first)
+ */
+NLM_EXTERN void LIBCALL
+Nlm_MD5Final(Nlm_MD5ContextPtr ctx, Nlm_Uchar digest[16])
+{
+    Nlm_Uint4 count;
+    Nlm_UcharPtr p;
+
+    /* Compute number of bytes mod 64 */
+    count = (ctx->bits[0] >> 3) & 0x3F;
+
+    /* Set the first char of padding to 0x80.  This is safe since there is
+       always at least one byte free */
+    p = ctx->in + count;
+    *p++ = 0x80;
+
+    /* Bytes of padding needed to make 64 bytes */
+    count = 64 - 1 - count;
+
+    /* Pad out to 56 mod 64 */
+    if (count < 8) {
+	/* Two lots of padding:  Pad the first block to 64 bytes */
+	memset(p, 0, count);
+#ifdef IS_BIG_ENDIAN
+	byteReverse(ctx->in, 16);
+#endif
+	Nlm_MD5Transform(ctx->buf, (Nlm_Uint4Ptr) ctx->in);
+
+	/* Now fill the next block with 56 bytes */
+	memset(ctx->in, 0, 56);
+    } else {
+	/* Pad block to 56 bytes */
+	memset(p, 0, count - 8);
+    }
+#ifdef IS_BIG_ENDIAN
+	byteReverse(ctx->in, 14);
+#endif
+
+    /* Append length in bits and transform */
+    ((Nlm_Uint4Ptr) ctx->in)[14] = ctx->bits[0];
+    ((Nlm_Uint4Ptr) ctx->in)[15] = ctx->bits[1];
+
+    Nlm_MD5Transform(ctx->buf, (Nlm_Uint4Ptr) ctx->in);
+#ifdef IS_BIG_ENDIAN
+    byteReverse((Nlm_UcharPtr) ctx->buf, 4);
+#endif
+    memcpy(digest, ctx->buf, 16);
+    memset(ctx, 0, sizeof(ctx));	/* In case it's sensitive */
+}
+
+/* The four core functions - F1 is optimized somewhat */
+
+/* #define F1(x, y, z) (x & y | ~x & z) */
+#define F1(x, y, z) (z ^ (x & (y ^ z)))
+#define F2(x, y, z) F1(z, x, y)
+#define F3(x, y, z) (x ^ y ^ z)
+#define F4(x, y, z) (y ^ (x | ~z))
+
+/* This is the central step in the MD5 algorithm. */
+#define MD5STEP(f, w, x, y, z, data, s) \
+	( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )
+
+/*
+ * The core of the MD5 algorithm, this alters an existing MD5 hash to
+ * reflect the addition of 16 longwords of new data.  MD5Update blocks
+ * the data and converts bytes into longwords for this routine.
+ */
+NLM_EXTERN void LIBCALL
+Nlm_MD5Transform(Nlm_Uint4 buf[4], Nlm_Uint4 in[16])
+{
+    register Nlm_Uint4 a, b, c, d;
+
+    a = buf[0];
+    b = buf[1];
+    c = buf[2];
+    d = buf[3];
+
+    MD5STEP(F1, a, b, c, d, in[0] + 0xd76aa478, 7);
+    MD5STEP(F1, d, a, b, c, in[1] + 0xe8c7b756, 12);
+    MD5STEP(F1, c, d, a, b, in[2] + 0x242070db, 17);
+    MD5STEP(F1, b, c, d, a, in[3] + 0xc1bdceee, 22);
+    MD5STEP(F1, a, b, c, d, in[4] + 0xf57c0faf, 7);
+    MD5STEP(F1, d, a, b, c, in[5] + 0x4787c62a, 12);
+    MD5STEP(F1, c, d, a, b, in[6] + 0xa8304613, 17);
+    MD5STEP(F1, b, c, d, a, in[7] + 0xfd469501, 22);
+    MD5STEP(F1, a, b, c, d, in[8] + 0x698098d8, 7);
+    MD5STEP(F1, d, a, b, c, in[9] + 0x8b44f7af, 12);
+    MD5STEP(F1, c, d, a, b, in[10] + 0xffff5bb1, 17);
+    MD5STEP(F1, b, c, d, a, in[11] + 0x895cd7be, 22);
+    MD5STEP(F1, a, b, c, d, in[12] + 0x6b901122, 7);
+    MD5STEP(F1, d, a, b, c, in[13] + 0xfd987193, 12);
+    MD5STEP(F1, c, d, a, b, in[14] + 0xa679438e, 17);
+    MD5STEP(F1, b, c, d, a, in[15] + 0x49b40821, 22);
+
+    MD5STEP(F2, a, b, c, d, in[1] + 0xf61e2562, 5);
+    MD5STEP(F2, d, a, b, c, in[6] + 0xc040b340, 9);
+    MD5STEP(F2, c, d, a, b, in[11] + 0x265e5a51, 14);
+    MD5STEP(F2, b, c, d, a, in[0] + 0xe9b6c7aa, 20);
+    MD5STEP(F2, a, b, c, d, in[5] + 0xd62f105d, 5);
+    MD5STEP(F2, d, a, b, c, in[10] + 0x02441453, 9);
+    MD5STEP(F2, c, d, a, b, in[15] + 0xd8a1e681, 14);
+    MD5STEP(F2, b, c, d, a, in[4] + 0xe7d3fbc8, 20);
+    MD5STEP(F2, a, b, c, d, in[9] + 0x21e1cde6, 5);
+    MD5STEP(F2, d, a, b, c, in[14] + 0xc33707d6, 9);
+    MD5STEP(F2, c, d, a, b, in[3] + 0xf4d50d87, 14);
+    MD5STEP(F2, b, c, d, a, in[8] + 0x455a14ed, 20);
+    MD5STEP(F2, a, b, c, d, in[13] + 0xa9e3e905, 5);
+    MD5STEP(F2, d, a, b, c, in[2] + 0xfcefa3f8, 9);
+    MD5STEP(F2, c, d, a, b, in[7] + 0x676f02d9, 14);
+    MD5STEP(F2, b, c, d, a, in[12] + 0x8d2a4c8a, 20);
+
+    MD5STEP(F3, a, b, c, d, in[5] + 0xfffa3942, 4);
+    MD5STEP(F3, d, a, b, c, in[8] + 0x8771f681, 11);
+    MD5STEP(F3, c, d, a, b, in[11] + 0x6d9d6122, 16);
+    MD5STEP(F3, b, c, d, a, in[14] + 0xfde5380c, 23);
+    MD5STEP(F3, a, b, c, d, in[1] + 0xa4beea44, 4);
+    MD5STEP(F3, d, a, b, c, in[4] + 0x4bdecfa9, 11);
+    MD5STEP(F3, c, d, a, b, in[7] + 0xf6bb4b60, 16);
+    MD5STEP(F3, b, c, d, a, in[10] + 0xbebfbc70, 23);
+    MD5STEP(F3, a, b, c, d, in[13] + 0x289b7ec6, 4);
+    MD5STEP(F3, d, a, b, c, in[0] + 0xeaa127fa, 11);
+    MD5STEP(F3, c, d, a, b, in[3] + 0xd4ef3085, 16);
+    MD5STEP(F3, b, c, d, a, in[6] + 0x04881d05, 23);
+    MD5STEP(F3, a, b, c, d, in[9] + 0xd9d4d039, 4);
+    MD5STEP(F3, d, a, b, c, in[12] + 0xe6db99e5, 11);
+    MD5STEP(F3, c, d, a, b, in[15] + 0x1fa27cf8, 16);
+    MD5STEP(F3, b, c, d, a, in[2] + 0xc4ac5665, 23);
+
+    MD5STEP(F4, a, b, c, d, in[0] + 0xf4292244, 6);
+    MD5STEP(F4, d, a, b, c, in[7] + 0x432aff97, 10);
+    MD5STEP(F4, c, d, a, b, in[14] + 0xab9423a7, 15);
+    MD5STEP(F4, b, c, d, a, in[5] + 0xfc93a039, 21);
+    MD5STEP(F4, a, b, c, d, in[12] + 0x655b59c3, 6);
+    MD5STEP(F4, d, a, b, c, in[3] + 0x8f0ccc92, 10);
+    MD5STEP(F4, c, d, a, b, in[10] + 0xffeff47d, 15);
+    MD5STEP(F4, b, c, d, a, in[1] + 0x85845dd1, 21);
+    MD5STEP(F4, a, b, c, d, in[8] + 0x6fa87e4f, 6);
+    MD5STEP(F4, d, a, b, c, in[15] + 0xfe2ce6e0, 10);
+    MD5STEP(F4, c, d, a, b, in[6] + 0xa3014314, 15);
+    MD5STEP(F4, b, c, d, a, in[13] + 0x4e0811a1, 21);
+    MD5STEP(F4, a, b, c, d, in[4] + 0xf7537e82, 6);
+    MD5STEP(F4, d, a, b, c, in[11] + 0xbd3af235, 10);
+    MD5STEP(F4, c, d, a, b, in[2] + 0x2ad7d2bb, 15);
+    MD5STEP(F4, b, c, d, a, in[9] + 0xeb86d391, 21);
+
+    buf[0] += a;
+    buf[1] += b;
+    buf[2] += c;
+    buf[3] += d;
+}
+
+/*----- Here is simplified MD4 algorithm, that used in Blast E-mail server
+  and formatdb ------ */
+
+#define F(x, y, z) (((x) & (y)) | ((~x) & (z)))
+#define G(x, y, z) (((x) & (y)) | ((x) & (z)) | ((y) & (z)))
+#define H(x, y, z) ((x) ^ (y) ^ (z))
+#define ROTATE_LEFT(x, n) (((x) << (n)) | ((x) >> (32-(n))))
+
+#define FF(a, b, c, d, x, s) { \
+    (a) += F ((b), (c), (d)) + (x); \
+	(a) = ROTATE_LEFT ((a), (s)); \
+}
+#define GG(a, b, c, d, x, s) { \
+    (a) += G ((b), (c), (d)) + (x) + (Nlm_Uint4)0x5a827999; \
+	(a) = ROTATE_LEFT ((a), (s)); \
+}
+#define HH(a, b, c, d, x, s) { \
+    (a) += H ((b), (c), (d)) + (x) + (Nlm_Uint4)0x6ed9eba1; \
+	(a) = ROTATE_LEFT ((a), (s)); \
+}
+
+/* This function calculates checksum from a buffer */
+Nlm_Uint4 Nlm_GetChecksum(Nlm_CharPtr p)
+{
+    Nlm_Int4 len;
+    struct Hash {
+	Nlm_Uint4 x1;
+	Nlm_Uint4 x2;
+	Nlm_Uint4 x3;
+	Nlm_Uint4 x4;
+	Nlm_Uint4 x5;
+	Nlm_Uint4 x6;
+	Nlm_Uint4 x7;
+	Nlm_Uint4 x8;
+	Nlm_Uint4 x9;
+	Nlm_Uint4 x10;
+	Nlm_Uint4 x11;
+	Nlm_Uint4 x12;
+    } Hash; 
+
+    Nlm_Uint4 a = 0x67452301;
+    Nlm_Uint4 b = 0xefcdab89;
+    Nlm_Uint4 c = 0x98badcfe;
+    Nlm_Uint4 d = 0x10325476;
+
+    if (p == NULL || p[0] == NULLB)
+	return 0;
+
+    MemSet(&Hash, 0, sizeof(Hash));
+    MemCopy(&Hash, p, 
+	    (len = StringLen(p)) > sizeof(Hash)? sizeof(Hash) : len);
+
+    FF (a, b, c, d, Hash.x1, 3);   /* 1 */
+    FF (d, a, b, c, Hash.x2, 7);   /* 2 */
+    FF (c, d, a, b, Hash.x3, 11);  /* 3 */
+    FF (b, c, d, a, Hash.x4, 19);  /* 4 */
+
+    GG (a, b, c, d, Hash.x5, 3);   /* 17 */
+    GG (d, a, b, c, Hash.x6, 5);   /* 18 */
+    GG (c, d, a, b, Hash.x7, 9);   /* 19 */
+    GG (b, c, d, a, Hash.x8, 13);  /* 20 */
+
+    HH (b, c, d, a, Hash.x9, 15);  /* 40 */
+    HH (a, b, c, d, Hash.x10, 3);  /* 41 */
+    HH (d, a, b, c, Hash.x11, 9);  /* 42 */
+    HH (c, d, a, b, Hash.x12, 11); /* 43 */
+
+    return (a+b+c+d);
+}
+
+/*  ----- End of simplified MD4 algorithm ------ */
