@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   10/23/91
 *
-* $Revision: 6.1 $
+* $Revision: 6.13 $
 *
 * File Description:
 *   	prototypes of miscellaneous functions
@@ -43,6 +43,42 @@
 * 06-15-93 Schuler     Added macros for Gestalt functins.
 *
 * $Log: ncbimisc.h,v $
+* Revision 6.13  1999/09/22 23:08:26  kans
+* added extended flag to valnode for SeqDesc, maybe later SeqId and SeqLoc
+*
+* Revision 6.12  1999/07/29 15:58:48  kans
+* added bigintvalue, ValNodeAddBigInt (PD)
+*
+* Revision 6.11  1999/04/23 16:25:36  shavirin
+* Added definition of the function Nlm_GetChecksum()
+*
+* Revision 6.10  1999/04/14 19:37:23  madden
+* Add Nlm_ prefix to Int8
+*
+* Revision 6.9  1999/04/14 15:31:07  madden
+* add Nlm_Int8tostr function
+*
+* Revision 6.8  1999/03/11 16:21:51  kans
+* ValNodeSort is more visible copy of jzmisc SortValNode
+*
+* Revision 6.7  1999/01/27 01:59:52  kans
+* cleaned up comment problem
+*
+* Revision 6.6  1999/01/21 20:08:37  ostell
+* added SwitchUint2 and 4, added integer bytestores
+*
+* Revision 6.5  1999/01/07 20:18:51  victorov
+* MD5 functions declarations fixed
+*
+* Revision 6.4  1999/01/07 15:20:58  victorov
+* added Nlm_ prefix to MD5 functions to avoid conflicts
+*
+* Revision 6.3  1999/01/06 22:49:27  victorov
+* added MD5 hash calculator
+*
+* Revision 6.2  1998/10/13 20:49:40  vakatov
+* + Nlm_PlatformName()
+*
 * Revision 6.1  1998/04/08 16:49:59  kans
 * ValNodeLen returns Int4
 *
@@ -104,10 +140,16 @@ NLM_EXTERN int LIBCALL Nlm_Ulwidth PROTO((unsigned long x, int opts));
 /* Same 1,2,4 opts as Nlm_Lwidth and Nlm_Ulwidth */
 NLM_EXTERN char * LIBCALL Nlm_Ltostr PROTO((long x, int opts));
 NLM_EXTERN char * LIBCALL Nlm_Ultostr PROTO((unsigned long x, int opts));
+/* Nlm_Int8tostr -- convert a signed long integer to ASCII */
+NLM_EXTERN CharPtr LIBCALL Nlm_Int8tostr PROTO((Nlm_Int8 value, int opts));
+
 
 /* Sorting */
 NLM_EXTERN void LIBCALL Nlm_HeapSort PROTO((VoidPtr base, size_t nel, size_t width, int (LIBCALLBACK *cmp) (VoidPtr, VoidPtr) ));
 
+/* Platform name */
+NLM_EXTERN const Nlm_Char* Nlm_PlatformName(void);
+
 
 /*****************************************************************************
 *
@@ -123,12 +165,14 @@ typedef union dataval {
 	Nlm_FloatHi realvalue;
 	Nlm_Boolean boolvalue;
 	Nlm_FnPtr	funcvalue;
+	Nlm_Int8    bigintvalue;
 }	DataVal, PNTR DataValPtr;
 
 typedef struct valnode {
-	Nlm_Uint1 choice;            /* to pick a choice */
-	DataVal data;            /* attached data */
-	struct valnode PNTR next;   /* next in linked list */
+	Nlm_Uint1 choice;          /* to pick a choice */
+	Nlm_Uint1 extended;        /* extra fields reserved to NCBI allocated in structure */
+	DataVal data;              /* attached data */
+	struct valnode PNTR next;  /* next in linked list */
 } ValNode, PNTR ValNodePtr;
 
 /*****************************************************************************
@@ -168,6 +212,11 @@ typedef struct valnode {
 *      sets newnode->choice = choice (if choice does not matter, use 0)
 *      sets newnode->data.intvalue = value
 *
+*   ValNodeAddBigInt (head, choice, value)
+*      adds like ValNodeAdd()
+*      sets newnode->choice = choice (if choice does not matter, use 0)
+*      sets newnode->data.bigintvalue = value
+*
 *   ValNodeAddBoolean (head, choice, value)
 *      adds like ValNodeAdd()
 *      sets newnode->choice = choice (if choice does not matter, use 0)
@@ -213,6 +262,10 @@ typedef struct valnode {
 *       If choice < 0 , returns all ValNodes
 *       Returns NULL, when no more found
 *
+*   ValNodeSort (list, compar)
+*   	Copied from SortValNode in jzcoll, renamed, for more general access
+*   	Makes array from ValNode list, calls HeapSort, reconnects ValNode list
+*
 *****************************************************************************/
 NLM_EXTERN ValNodePtr LIBCALL ValNodeNew PROTO((ValNodePtr vnp));
 NLM_EXTERN Nlm_Int4   LIBCALL ValNodeLen PROTO((ValNodePtr vnp));
@@ -221,6 +274,7 @@ NLM_EXTERN ValNodePtr LIBCALL ValNodeLink PROTO((ValNodePtr PNTR head, ValNodePt
 NLM_EXTERN ValNodePtr LIBCALL ValNodeAddStr PROTO((ValNodePtr PNTR head, Nlm_Int2 choice, Nlm_CharPtr str));
 NLM_EXTERN ValNodePtr LIBCALL ValNodeCopyStr PROTO((ValNodePtr PNTR head, Nlm_Int2 choice, Nlm_CharPtr str));
 NLM_EXTERN ValNodePtr LIBCALL ValNodeAddInt PROTO((ValNodePtr PNTR head, Nlm_Int2 choice, Nlm_Int4 value));
+NLM_EXTERN ValNodePtr LIBCALL ValNodeAddBigInt (ValNodePtr PNTR head, Nlm_Int2 choice, Nlm_Int8 value);
 NLM_EXTERN ValNodePtr LIBCALL ValNodeAddBoolean PROTO((ValNodePtr PNTR head, Nlm_Int2 choice, Nlm_Boolean value));
 NLM_EXTERN ValNodePtr LIBCALL ValNodeAddFloat PROTO((ValNodePtr PNTR head, Nlm_Int2 choice, Nlm_FloatHi value));
 NLM_EXTERN ValNodePtr LIBCALL ValNodeAddPointer PROTO((ValNodePtr PNTR head, Nlm_Int2 choice, Nlm_VoidPtr value));
@@ -230,6 +284,7 @@ NLM_EXTERN ValNodePtr LIBCALL ValNodeFreeData PROTO((ValNodePtr vnp));
 NLM_EXTERN ValNodePtr LIBCALL ValNodeExtract PROTO((ValNodePtr PNTR headptr, Nlm_Int2 choice));
 NLM_EXTERN ValNodePtr LIBCALL ValNodeExtractList PROTO((ValNodePtr PNTR headptr, Nlm_Int2 choice));
 NLM_EXTERN ValNodePtr LIBCALL ValNodeFindNext PROTO((ValNodePtr head, ValNodePtr curr, Nlm_Int2 choice));
+NLM_EXTERN ValNodePtr LIBCALL ValNodeSort PROTO((ValNodePtr list, int (LIBCALLBACK *compar) (VoidPtr, VoidPtr)));
 
 /*** old prototypes ******
 NLM_EXTERN ValNodePtr LIBCALL ValNodeLink PROTO((ValNodePtr vnp, ValNodePtr newnode));
@@ -302,13 +357,35 @@ NLM_EXTERN void Nlm_CtoPstr PROTO((Nlm_CharPtr str));
 NLM_EXTERN void Nlm_PtoCstr PROTO((Nlm_CharPtr str));
 #endif
 
+/* these functions reverse byte order in integers 
+   calling the same function a second time switches it back
+   the native ENDIAN nature of the machine is not considered
+*/
+NLM_EXTERN Uint2 Nlm_SwitchUint2 PROTO ((Uint2 value));
+NLM_EXTERN void Nlm_SwitchUint2Buff PROTO ((Uint2 *buff, int count));
+NLM_EXTERN unsigned long  Nlm_SwitchLong PROTO ((unsigned long value));
+NLM_EXTERN void Nlm_SwitchLongBuff PROTO ((unsigned long *buff, int count));
+NLM_EXTERN Uint4  Nlm_SwitchUint4 PROTO ((Uint4 value));
+NLM_EXTERN void Nlm_SwitchUint4Buff PROTO ((Uint4 *buff, int count));
+
+#define SwitchUint2 Nlm_SwitchUint2
+#define SwitchUint2Buff Nlm_SwitchUint2Buff
+#define SwitchLong Nlm_SwitchLong
+#define SwitchLongBuff Nlm_SwitchLongBuff
+#define SwitchUint4 Nlm_SwitchUint4
+#define SwitchUint4Buff Nlm_SwitchUint4Buff
+
+/** The following defines ALWAYS assume the value to switched is
+    BIG_ENDIAN. This is used to allow portable use of binary integers
+	in some NCBI applications such as BLAST and some indexes ****/
+
 #ifdef IS_LITTLE_ENDIAN
-NLM_EXTERN Uint2 Nlm_SwapUint2 PROTO ((Uint2 value));
-NLM_EXTERN void Nlm_SwapUint2Buff PROTO ((Uint2 *buff, int count));
-NLM_EXTERN unsigned long  Nlm_SwapLong PROTO ((unsigned long value));
-NLM_EXTERN void Nlm_SwapLongBuff PROTO ((unsigned long *buff, int count));
-NLM_EXTERN Uint4  Nlm_SwapUint4 PROTO ((Uint4 value));
-NLM_EXTERN void Nlm_SwapUint4Buff PROTO ((Uint4 *buff, int count));
+#define Nlm_SwapUint2(value)		Nlm_SwitchUint2(value)
+#define Nlm_SwapUint2Buff(buff, count) Nlm_SwitchUint2Buff(buff, count)
+#define Nlm_SwapLong(value)		Nlm_SwitchLong(value)
+#define Nlm_SwapLongBuff(buff, count) Nlm_SwitchLongBuff(buff, count)
+#define Nlm_SwapUint4(value)		Nlm_SwitchUint4(value)
+#define Nlm_SwapUint4Buff(buff, count) Nlm_SwitchUint4Buff(buff, count)
 #else
 #define Nlm_SwapUint2(value)		(value)
 #define Nlm_SwapUint2Buff(buff, count)
@@ -317,6 +394,7 @@ NLM_EXTERN void Nlm_SwapUint4Buff PROTO ((Uint4 *buff, int count));
 #define Nlm_SwapUint4(value)		(value)
 #define Nlm_SwapUint4Buff(buff, count)
 #endif
+
 #define SwapUint2 Nlm_SwapUint2
 #define SwapUint2Buff Nlm_SwapUint2Buff
 #define SwapLong Nlm_SwapLong
@@ -325,8 +403,31 @@ NLM_EXTERN void Nlm_SwapUint4Buff PROTO ((Uint4 *buff, int count));
 #define SwapUint4Buff Nlm_SwapUint4Buff
 
 
+/**
+ * MD5 stuff
+ */
+typedef struct md5context_ {
+	Nlm_Uint4 buf[4];
+	Nlm_Uint4 bits[2];
+	Nlm_Uchar in[64];
+} Nlm_MD5Context, PNTR Nlm_MD5ContextPtr;
+
+NLM_EXTERN void LIBCALL Nlm_MD5Init PROTO((Nlm_MD5ContextPtr context));
+NLM_EXTERN void LIBCALL Nlm_MD5Update PROTO((Nlm_MD5ContextPtr context, Nlm_UcharPtr buf, Nlm_Uint4 len));
+NLM_EXTERN void LIBCALL Nlm_MD5Final PROTO((Nlm_MD5ContextPtr context, Nlm_Uchar digest[16]));
+NLM_EXTERN void LIBCALL Nlm_MD5Transform PROTO((Nlm_Uint4 buf[4], Nlm_Uint4 in[16]));
+
+#define MD5Context Nlm_MD5Context
+#define MD5ContextPtr Nlm_MD5ContextPtr
+#define MD5Init Nlm_MD5Init
+#define MD5Update Nlm_MD5Update
+#define MD5Final Nlm_MD5Final
+#define MD5Transform Nlm_MD5Transform
+
 /* Error codes for the CTX_NCBIMISC context */
 
+Uint4 Nlm_GetChecksum(CharPtr p);
+
 #ifdef __cplusplus
 }
 #endif
@@ -339,5 +440,6 @@ NLM_EXTERN void Nlm_SwapUint4Buff PROTO ((Uint4 *buff, int count));
 #define NLM_EXTERN
 #endif
 
+
 #endif /* !_NCBIMISC_ */
 
