@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   2/13/91
 *
-* $Revision: 6.3 $
+* $Revision: 6.9 $
 *
 * File Description:
 *   	user alert and error messages
@@ -54,6 +54,25 @@
 *                      input will be read properly.
 *
 * $Log: ncbimsg.c,v $
+* Revision 6.9  2000/08/28 18:46:24  vakatov
+* Made internal hooks "static" and "C" -- to pass C++ compilation
+*
+* Revision 6.8  1999/10/01 14:41:40  kans
+* added SEV_REJECT between SEV_ERROR and SEV_FATAL
+*
+* Revision 6.7  1999/08/31 21:36:25  vakatov
+* Nlm_Message(), Nlm_MsgAlert() -- pre-save the temporary "scratch_str"
+* buffer to avoid its overriding on calling the "ncbierr.c"-related code
+*
+* Revision 6.6  1999/08/23 19:16:08  vakatov
+* Nlm_Message():  handle MSG_FATAL as SEV_MAX (was SEV_FATAL)
+*
+* Revision 6.5  1998/08/24 17:42:01  kans
+* fixed old style function definition warnings
+*
+* Revision 6.4  1998/06/11 21:02:29  shavirin
+* Fixed few warnings
+*
 * Revision 6.3  1997/11/26 21:26:23  vakatov
 * Fixed errors and warnings issued by C and C++ (GNU and Sun) compilers
 *
@@ -130,13 +149,21 @@ static char *_filename = __FILE__;
 #include "corepriv.h"
 #include <tsprintf.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-MsgAnswer PASCAL _DefMessageHook PROTO((MsgKey key, ErrSev sev,
-			const char *caption, const char *message));
-			
-void PASCAL _DefBeepHook PROTO((void));
+    static MsgAnswer PASCAL _DefMessageHook
+    (MsgKey key, ErrSev sev, const char *caption, const char *message);
+
+    static void PASCAL _DefBeepHook(void);
+
+    static int PASCAL _DefMonitorHook(MonitorPtr pMon, MonCode code);
+
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
 
-int PASCAL _DefMonitorHook PROTO((MonitorPtr pMon, MonCode code));
 
 
 /*************************************************************************\
@@ -260,7 +287,7 @@ NLM_EXTERN MsgAnswer CDECL Nlm_Message (Nlm_Int2 sevkey, const char *fmt, ...)
           break;
         case MSG_FATAL :
           Nlm_Beep();
-          sev = SEV_FATAL;
+          sev = SEV_MAX;
           break;
         case MSG_POSTERR :
           Nlm_Beep();
@@ -275,8 +302,11 @@ NLM_EXTERN MsgAnswer CDECL Nlm_Message (Nlm_Int2 sevkey, const char *fmt, ...)
 
   {{
     const Nlm_Char PNTR scratch_str = NULL;
+    Nlm_Char PNTR message;
     TSPRINTF(scratch_str, fmt);
-    ans = scratch_str ? MsgAlertStr(key, sev, caption, scratch_str) : ANS_NONE;
+    message = Nlm_StringSave(scratch_str);
+    ans = message ? MsgAlertStr(key, sev, caption, message) : ANS_NONE;
+    Nlm_MemFree(message);
   }}
 
   if (sevkey == MSG_FATAL)
@@ -301,12 +331,14 @@ NLM_EXTERN MsgAnswer CDECL Nlm_MsgAlert (key, sev, caption, fmt, va_alist)
 NLM_EXTERN MsgAnswer CDECL Nlm_MsgAlert (MsgKey key, ErrSev sev, const char *caption, const char *fmt, ...)
 #endif
 {
+  MsgAnswer ans;
   const Nlm_Char PNTR scratch_str = NULL;
+  Nlm_Char PNTR message;
   TSPRINTF(scratch_str, fmt);
-  if (scratch_str != NULL)
-    return MsgAlertStr(key, sev, caption, scratch_str);
-  else
-    return ANS_NONE;
+  message = Nlm_StringSave(scratch_str);
+  ans = message ? MsgAlertStr(key, sev, caption, message) : ANS_NONE;
+  Nlm_MemFree(message);
+  return ans;
 }
 
 
@@ -338,12 +370,14 @@ MsgAnswer PASCAL _DefMessageHook (MsgKey key, ErrSev sev,
 #ifdef OS_MSWIN
   if ( !Nlm_HasConsole )
     {
-      static UINT _sev_code[] = { 
+      static UINT _sev_code[SEV_MAX+1] = { 
         /* SEV_NONE */    MB_OK,
         /* SEV_INFO */    MB_ICONINFORMATION,
         /* SEV_WARNING */ MB_ICONASTERISK, /* same as MB_ICONINFORMATION */
         /* SEV_ERROR */   MB_ICONEXCLAMATION,
-        /* SEV_FATAL */   MB_ICONHAND
+        /* SEV_REJECT */  MB_ICONEXCLAMATION,
+        /* SEV_FATAL */   MB_ICONHAND,
+        /* SEV_MAX */     MB_ICONHAND
       };
 
       UINT flags = MB_TASKMODAL | _sev_code[(int)sev];
@@ -442,7 +476,7 @@ NLM_EXTERN MessageHook LIBCALL Nlm_SetMessageHook (MessageHook hook)
 #include <sys/types.h>
 #endif /* OS_UNIX */
 
-static int GetOneChar ()
+static int GetOneChar (void)
 {
   int value = 0;
 
@@ -494,7 +528,7 @@ static int GetOneChar ()
 * 01-13-94 Schuler   Modified to use BeepHook (old version preserved below)
 */
 
-NLM_EXTERN void LIBCALL  Nlm_Beep ()
+NLM_EXTERN void LIBCALL  Nlm_Beep (void)
 {
 	BeepHook hook = GetAppMsgInfo()->hookBeep;
 	(*hook)();
@@ -506,7 +540,7 @@ NLM_EXTERN void LIBCALL  Nlm_Beep ()
 *
 * Default beep function
 */
-void LIBCALLBACK _DefBeepHook ()
+void LIBCALLBACK _DefBeepHook (void)
 {
 #if   defined(OS_MAC)
   SysBeep( 60 );
@@ -772,7 +806,6 @@ int PASCAL _DefMonitorHook (MonitorPtr pMon, MonCode code)
 	{
 		case MonCode_Create :
 			return Dumb_MonCreate(pMon);
-			break;
 		case MonCode_Destroy :
 			Dumb_MonDestroy(pMon);
 			break;
@@ -790,8 +823,8 @@ static int Dumb_MonCreate (MonitorPtr pMon)
 {
 	char *buf;
 
-	if ( !(buf = (char *)Nlm_Calloc(81,1)) )
-		return FALSE;
+	if ((buf = (char *)Nlm_Calloc(81,1)) == NULL)
+            return FALSE;
 	buf[80] = '\0';
 	pMon->extra = buf;
 
