@@ -25,11 +25,11 @@
 *
 * File Name:  ncbistr.c
 *
-* Author:  Gish, Kans, Ostell, Schuler, Brylawski
+* Author:  Gish, Kans, Ostell, Schuler, Brylawski, Vakatov, Lavrentiev
 *
 * Version Creation Date:   3/4/91
 *
-* $Revision: 6.2 $
+* $Revision: 6.10 $
 *
 * File Description: 
 *   	portable string routines
@@ -37,15 +37,40 @@
 * Modifications:  
 * --------------------------------------------------------------------------
 * $Log: ncbistr.c,v $
+* Revision 6.10  2001/01/05 22:43:58  shavirin
+* Added functions, that transfer Uint8 values to platform-independent
+* objects and back.
+*
+* Revision 6.9  2000/12/04 23:48:02  kans
+* trim spaces around string now handles trailing tabs, newlines, etc.
+*
+* Revision 6.8  2000/11/30 22:46:07  lavr
+* Added the following functions for conversions of Int8 and Uint8
+* to strings and back; test suite attached at the end of the file.
+* Nlm_Int8ToString, Nlm_Uint8ToString, Nlm_StringToInt8, Nlm_StringToUint8
+*
+* Revision 6.7  2000/08/28 18:36:25  vakatov
+* un-const casts in some functions to pass C++ compilation
+*
+* Revision 6.6  1999/04/15 20:24:06  vakatov
+* Dont use "list" name as it can clash with the standard "list<>" template
+* on some raw C++ compilers
+*
+* Revision 6.5  1999/03/11 16:10:00  kans
+* StringHasNoText and TrimSpacesAroundString moved from vibforms
+*
+* Revision 6.4  1998/11/23 00:09:47  kans
+* fixed bug in StringTokMT (found by Hugues)
+*
+* Revision 6.3  1998/10/07 19:09:00  kans
+* added Nlm_StringTokMT, multithread-safe version
+*
 * Revision 6.2  1997/11/26 21:26:25  vakatov
 * Fixed errors and warnings issued by C and C++ (GNU and Sun) compilers
 *
 * Revision 6.1  1997/10/29 02:44:52  vakatov
 * Type castings to pass through the C++ compiler
 *
-* Revision 6.0  1997/08/25 18:17:07  madden
-* Revision changed to 6.0
-*
 * Revision 5.5  1997/07/16 19:49:18  vakatov
 * Added Nlm_StringPrintable() function
 *
@@ -63,9 +88,6 @@
  * Revision 5.1  1996/12/03  21:48:33  vakatov
  * Adopted for 32-bit MS-Windows DLLs
  *
- * Revision 5.0  1996/05/28  13:18:57  ostell
- * Set to revision 5.0
- *
  * Revision 4.12  1996/05/22  18:04:14  kans
  * changed nulls to '\0' in new string functions
  *
@@ -82,7 +104,7 @@
  * change String variables to theString to work around SGI4 problem
  *
  * Revision 4.7  1996/01/05  02:29:37  ostell
- * aprovided return value for TruncateStringCopy()
+ * provided return value for TruncateStringCopy()
  *
  * Revision 4.6  1996/01/03  21:04:46  epstein
  * modify StringSubString() API and add other new functions, per Brandon
@@ -102,18 +124,12 @@
  * Revision 4.1  1995/10/16  13:43:29  epstein
  * fix brain-dmanaged string-compare logic to handle null strings correctly
  *
- * Revision 4.0  1995/07/26  13:46:50  ostell
- * force revision to 4.0
- *
  * Revision 2.12  1995/07/18  19:56:10  tatiana
  * add Nlm_LabelCopyNext()
  *
  * Revision 2.11  1995/05/30  13:19:37  kans
  * fixed StringSearch algorithm - check until i <= strLen, not just < strLen
  *
- * Revision 2.10  1995/05/15  18:45:58  ostell
- * added Log line
- *
 * 3/4/91   Kans        Stricter typecasting for GNU C and C++.
 * 09-19-91 Schuler     Changed all types expressing sizes to size_t.
 * 09-19-91 Schuler     Changed return type for compare functions to int.
@@ -207,12 +223,12 @@ NLM_EXTERN int LIBCALL  Nlm_StringNICmp (const char FAR *a, const char FAR *b, s
 
 NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringChr (const char FAR *str, int chr)
 {
-	return str ? Nlm_StrChr(str,chr) : 0;
+	return (Nlm_CharPtr) (str ? Nlm_StrChr(str,chr) : 0);
 }
 
 NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringRChr (const char FAR *str, int chr)
 {
-	return str ? Nlm_StrRChr(str,chr) : 0;
+	return (Nlm_CharPtr) (str ? Nlm_StrRChr(str,chr) : 0);
 }
 
 NLM_EXTERN size_t LIBCALL  Nlm_StringSpn (const char FAR *a, const char FAR *b)
@@ -227,12 +243,12 @@ NLM_EXTERN size_t LIBCALL  Nlm_StringCSpn (const char FAR *a, const char FAR *b)
 
 NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringPBrk (const char FAR *a, const char FAR *b)
 {
-	return (a && b) ? Nlm_StrPBrk (a, b) : 0;
+	return (Nlm_CharPtr) ((a && b) ? Nlm_StrPBrk (a, b) : 0);
 }
 
 NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringStr (const char FAR *str1, const char FAR *str2)
 {
-	return (str1 && str2) ? Nlm_StrStr(str1,str2) : 0;
+	return (Nlm_CharPtr) ((str1 && str2) ? Nlm_StrStr(str1,str2) : 0);
 }
 
 NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringTok (char FAR *str1, const char FAR *str2)
@@ -240,6 +256,42 @@ NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringTok (char FAR *str1, const char FAR *s
     return str2 ? Nlm_StrTok(str1,str2) : 0;
 }
 
+NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringTokMT (char FAR *str1, const char FAR *str2, char FAR **tmp)
+{
+	char ch;
+	char FAR *ptr;
+	char FAR *rsult = NULL;
+
+	if (str2 == NULL || tmp == NULL) return NULL;
+	if (str1 != NULL) {
+  	  *tmp = str1;
+    }
+    ptr = *tmp;
+    if (ptr == NULL) return NULL;
+    ch = *ptr;
+    while (ch != '\0' && strchr (str2, ch) != NULL) {
+      ptr++;
+      ch = *ptr;
+    }
+    if (ch == '\0') {
+      *tmp = NULL;
+      return NULL;
+    }
+    rsult = ptr;
+    while (ch != '\0' && strchr (str2, ch) == NULL) {
+      ptr++;
+      ch = *ptr;
+    }
+    if (ch == '\0') {
+      *tmp = NULL;
+    } else {
+      *ptr = '\0';
+      ptr++;
+      *tmp = ptr;
+    }
+    return rsult; 
+}
+
 NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringMove (char FAR *to, const char FAR *from)
 {
     return (to && from) ? Nlm_StrMove (to, from) : to;
@@ -255,9 +307,9 @@ NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringSaveNoNull (const char FAR *from)
     return (from  && *from) ? Nlm_StrSave(from) : 0;
 }
 
-NLM_EXTERN size_t LIBCALL  Nlm_StringCnt (const char FAR *str, const char FAR *list)
+NLM_EXTERN size_t LIBCALL  Nlm_StringCnt (const char FAR *str, const char FAR *x_list)
 {
-    return (str && list) ? Nlm_StrCnt(str,list) : 0;
+    return (str && x_list) ? Nlm_StrCnt(str,x_list) : 0;
 }
 
 NLM_EXTERN char * LIBCALL Nlm_StringUpper (char *string)
@@ -427,15 +479,74 @@ NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StrMove (char FAR *to, const char FAR *from)
 	return to;
 }
 
-NLM_EXTERN size_t LIBCALL  Nlm_StrCnt (const char FAR *s, const char FAR *list)
+NLM_EXTERN Nlm_Boolean LIBCALL Nlm_StringHasNoText (Nlm_CharPtr str)
+
+{
+  Nlm_Uchar  ch;	/* to use 8bit characters in multibyte languages */
+
+  if (str != NULL) {
+    ch = *str;
+    while (ch != '\0') {
+      if (ch > ' ') {
+        return FALSE;
+      }
+      str++;
+      ch = *str;
+    }
+  }
+  return TRUE;
+}
+
+NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TrimSpacesAroundString (Nlm_CharPtr str)
+
+{
+  Nlm_Uchar    ch;	/* to use 8bit characters in multibyte languages */
+  Nlm_CharPtr  dst;
+  Nlm_CharPtr  ptr;
+
+  if (str != NULL && str [0] != '\0') {
+    dst = str;
+    ptr = str;
+    ch = *ptr;
+    while (ch != '\0' && ch <= ' ') {
+      ptr++;
+      ch = *ptr;
+    }
+    while (ch != '\0') {
+      *dst = ch;
+      dst++;
+      ptr++;
+      ch = *ptr;
+    }
+    *dst = '\0';
+    dst = NULL;
+    ptr = str;
+    ch = *ptr;
+    while (ch != '\0') {
+      if (ch > ' ') {
+        dst = NULL;
+      } else if (dst == NULL) {
+        dst = ptr;
+      }
+      ptr++;
+      ch = *ptr;
+    }
+    if (dst != NULL) {
+      *dst = '\0';
+    }
+  }
+  return str;
+}
+
+NLM_EXTERN size_t LIBCALL Nlm_StrCnt(const char FAR *s, const char FAR *x_list)
 {
 	size_t	cmap[1<<CHAR_BIT];
 	Nlm_Byte	c;
 	size_t	u, cnt;
 	const Nlm_Byte *bs = (const Nlm_Byte*)s;
-	const Nlm_Byte *blist = (const Nlm_Byte*)list;
+	const Nlm_Byte *blist = (const Nlm_Byte*)x_list;
 
-	if (s == NULL || list == NULL)
+	if (s == NULL || x_list == NULL)
 		return 0;
 
 	for (u = 0; u < DIM(cmap); ++u)
@@ -598,6 +709,160 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringISearch (const char FAR *str, const cha
   return Nlm_FindSubString (str, sub, FALSE);
 }
 
+NLM_EXTERN Nlm_Uint1Ptr LIBCALL Uint8ToBytes(Nlm_Uint8 value)
+{
+    Nlm_Uint1Ptr out_bytes;
+    Nlm_Int4 i, mask = 0xFF;
+    
+    out_bytes = MemNew(8);
+    
+    for(i = 0; i < 8; i++) {
+        out_bytes[i] = value & mask;
+        value >>= 8;
+    }
+    
+    return out_bytes;
+}
+
+NLM_EXTERN Nlm_Uint8 LIBCALL BytesToUint8(Nlm_Uint1Ptr bytes)
+{
+    Nlm_Uint8 value;
+    Nlm_Int4 i;
+
+    value = 0;
+    for(i = 7; i >= 0; i--) {
+        value += bytes[i];
+        if(i) value <<= 8;
+    }
+    
+    return value;
+}
+static Nlm_Uint8 s_StringToUint8(const char *str, const char **endptr, int *sgn)
+{
+    int sign = 0;  /* actual sign */
+    Nlm_Uint8 limdiv, limoff, result;
+    const char *s, *save;
+    char c;
+
+    /* assume error */
+    *endptr = 0;
+    if (!str)
+        return 0;
+
+    s = str;
+    while (IS_WHITESP(*s))
+        s++;
+    /* empty string - error */
+    if (*s == '\0')
+        return 0;
+
+    if (*sgn == 1) {
+        if (*s == '-') {
+            sign = 1;
+            s++;
+        } else if (*s == '+') {
+            s++;
+        }        
+    }
+    save = s;
+
+    limdiv = UINT8_MAX / 10;
+    limoff = UINT8_MAX % 10;
+    result = 0;
+    
+    for (c = *s; c; c = *++s) {
+        if (!IS_DIGIT(c)) {
+            break;
+        }
+        c -= '0';
+        if (result > limdiv || (result == limdiv && c > limoff)) {
+            /* overflow */
+            return 0;
+        }
+        result *= 10;
+        result += c;
+    }
+
+    /* there was no conversion - error */
+    if (save == s)
+        return 0;
+
+    *sgn = sign;
+    *endptr = s;
+    return result;
+}
+
+
+NLM_EXTERN Nlm_Uint8 LIBCALL Nlm_StringToUint8(const char* str, const char** endptr)
+{
+    int sign = 0; /* no sign allowed */
+    return s_StringToUint8(str, endptr, &sign);
+}
+
+
+NLM_EXTERN Nlm_Int8  LIBCALL Nlm_StringToInt8(const char* str, const char** endptr)
+{
+    int sign = 1; /* sign allowed */
+    Nlm_Uint8 result = s_StringToUint8(str, endptr, &sign);
+    if (*endptr) {
+        /* Check for overflow */
+        if (result > (sign
+                      ? -((Nlm_Uint8)(INT8_MIN + 1)) + 1
+                      : (Nlm_Uint8)(INT8_MAX)))
+            *endptr = 0;            
+    }
+    if (!*endptr)
+        return 0;
+    return sign ? -result : result;
+}
+
+
+static char *s_Uint8ToString(Nlm_Uint8 value, char *str, size_t str_size)
+{
+    char buf[32];
+    size_t i, j;
+
+    if (!str || str_size < 2)
+        return 0;
+
+    for (i = sizeof(buf) - 1; i > 0; i--) {
+        buf[i] = (char)(value % 10 + '0');
+        value /= 10;
+        if (!value)
+            break;
+    }
+    if (!i || (j = sizeof(buf) - i) >= str_size)
+        return 0;
+    memcpy(str, buf + i, j);
+    str[j] = '\0';
+    return str;
+}
+
+
+NLM_EXTERN char* LIBCALL Nlm_Uint8ToString(Nlm_Uint8 value, char* str, size_t str_size)
+{
+    return s_Uint8ToString(value, str, str_size);
+}
+
+
+NLM_EXTERN char* LIBCALL Nlm_Int8ToString (Nlm_Int8  value, char* str, size_t str_size)
+{
+    Nlm_Uint8 val;
+    char *save = str;
+
+    if (value < 0) {
+        if (!str || !str_size)
+            return 0;
+        *str++ = '-';
+        str_size--;
+        val = -(Nlm_Uint8)(value + 1) + 1;
+    } else
+        val = value;
+
+    return s_Uint8ToString(val, str, str_size) ? save : 0;
+}
+
+
 /*****************************************************************************
 *
 *   LabelCopy (to, from, buflen)
@@ -1401,7 +1666,7 @@ NLM_EXTERN size_t Nlm_stream2text(const Nlm_Char FAR PNTR str, size_t max_col,
   /* go to the beginning of the last completely fit word */
   for (sb = &str[len-1];
        sb != str  &&  !IS_WHITESP(*sb)  &&  !can_break(*sb, *(sb+1));
-       sb--);
+       sb--) continue;
   while (sb != str  &&  IS_WHITESP(*sb))
     sb--;
 
@@ -1416,8 +1681,8 @@ NLM_EXTERN size_t Nlm_stream2text(const Nlm_Char FAR PNTR str, size_t max_col,
   /* decide of whether and how to break the last alphabet word */
 
   /*  count the lead and the tail of the last non-fit word */
-  for (s = &str[len];  *s != '\0'  &&  IS_ALPHA(*s);  s++, n_tail++);
-  for (s = &str[len-1];  IS_ALPHA(*s);  s--, n_lead++);
+  for (s = &str[len];  *s != '\0'  &&  IS_ALPHA(*s);  s++, n_tail++) continue;
+  for (s = &str[len-1];  IS_ALPHA(*s);  s--, n_lead++) continue;
   ASSERT ( s > str );
 
   /* try to "move" symbols from lead in the sake of tail */
@@ -1455,7 +1720,7 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_rule_line(const Nlm_Char FAR PNTR str,
     str++;
   if ( !*str )
     return s;
-  for (str_len = Nlm_StringLen( str );  IS_WHITESP(str[str_len-1]); str_len--);
+  for (str_len = Nlm_StringLen( str );  IS_WHITESP(str[str_len-1]); str_len--) continue;
 
 
   /* truncate the original string if doesn't fit */
@@ -1521,7 +1786,7 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_rule_line(const Nlm_Char FAR PNTR str,
                     while ( n_add-- )
                       *_s++ = SPACE;
 
-                    for (_str++;  IS_WHITESP(*_str);  _str++);
+                    for (_str++;  IS_WHITESP(*_str);  _str++) continue;
                   }
                 else
                   break;
@@ -1644,3 +1909,133 @@ Nlm_Int2 Nlm_Main( void )
   return 0;
 }
 #endif  /* TEST_TEXT_FMT */
+
+
+#ifdef TEST_INT8_CONVERSION
+Nlm_Int2 Nlm_Main( void )
+{
+    char buffer[100];
+    char *s;
+    const char *p;
+    Nlm_Int8 i;
+    Nlm_Uint8 j;
+
+    s = Nlm_Int8ToString(0, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("0 = %s\n", s);
+    s = Nlm_Int8ToString(1, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("1 = %s\n", s);
+    s = Nlm_Int8ToString(1222222, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("1222222 = %s\n", s);
+    s = Nlm_Int8ToString(-15, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("-15 = %s\n", s);
+    s = Nlm_Int8ToString(-15555555, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("-15555555 = %s\n", s);
+    s = Nlm_Int8ToString(INT8_MAX, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("INT8_MAX = %s\n", s);
+    s = Nlm_Int8ToString(INT8_MIN, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("INT8_MIN = %s\n", s);
+    s = Nlm_Int8ToString(UINT8_MAX, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("UINT8_MAX = %s\n", s);
+    s = Nlm_Uint8ToString(UINT8_MAX, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("UINT8_MAX = %s\n", s);
+
+    strcpy(buffer, "9223372036854775807");
+    i = Nlm_StringToInt8(buffer, &p);
+    assert(p == buffer + strlen(buffer));
+    s = Nlm_Int8ToString(i, buffer + strlen(buffer) + 1,
+                         sizeof(buffer) - strlen(buffer) - 1);
+    assert(s != 0);
+    assert(strcmp(buffer, s) == 0);
+    printf("INT8_MAX input Ok\n");
+    i++;
+    s = Nlm_Int8ToString(i, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("INT8_MAX+1 = %s\n", s);
+
+    strcpy(buffer, "-9223372036854775808");
+    i = Nlm_StringToInt8(buffer, &p);
+    assert(p == buffer + strlen(buffer));
+    s = Nlm_Int8ToString(i, buffer + strlen(buffer) + 1,
+                         sizeof(buffer) - strlen(buffer) - 1);
+    assert(s != 0);
+    assert(strcmp(buffer, s) == 0);
+    printf("INT8_MIN input Ok\n");
+    i--;
+    s = Nlm_Int8ToString(i, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("INT8_MIN-1 = %s\n", s);
+
+    strcpy(buffer, "18446744073709551615");
+    j = Nlm_StringToUint8(buffer, &p);
+    assert(p == buffer + strlen(buffer));
+    s = Nlm_Uint8ToString(j, buffer + strlen(buffer) + 1,
+                          sizeof(buffer) - strlen(buffer) - 1);
+    assert(s != 0);
+    assert(strcmp(buffer, s) == 0);
+    printf("UINT8_MAX input Ok\n");
+    j++;
+    s = Nlm_Uint8ToString(j, buffer, sizeof(buffer));
+    assert(s != 0);
+    printf("UINT8_MAX+1 = %s\n", s);
+
+    strcpy(buffer, "1234567890abcdef0123546");
+    i = Nlm_StringToInt8(buffer, &p);
+    assert(p != 0);
+    s = Nlm_Int8ToString(i, buffer + strlen(buffer) + 1,
+                         sizeof(buffer) - strlen(buffer) - 1);
+    assert(s != 0);
+    printf("Out of %s only %.*s was accepted as input for Int8 %s\n",
+           buffer, (int)(p - buffer), buffer, s);
+
+    strcpy(buffer, "-987654321234567890abcdef0123546");
+    i = Nlm_StringToInt8(buffer, &p);
+    assert(p != 0);
+    s = Nlm_Int8ToString(i, buffer + strlen(buffer) + 1,
+                         sizeof(buffer) - strlen(buffer) - 1);
+    assert(s != 0);
+    printf("Out of %s only %.*s was accepted as input for Int8 %s\n",
+           buffer, (int)(p - buffer), buffer, s);
+
+    strcpy(buffer, "987654321234567890abcdef0123546");
+    j = Nlm_StringToUint8(buffer, &p);
+    assert(p != 0);
+    s = Nlm_Uint8ToString(j, buffer + strlen(buffer) + 1,
+                          sizeof(buffer) - strlen(buffer) - 1);
+    assert(s != 0);
+    printf("Out of %s only %.*s was accepted as input for Uint8 %s\n",
+           buffer, (int)(p - buffer), buffer, s);
+
+    strcpy(buffer, "-987654321234567890abcdef0123546");
+    j = Nlm_StringToUint8(buffer, &p);
+    assert(p == 0);
+    printf("Conversion of %s (negative) to Uint8 caused error\n", buffer);
+
+    strcpy(buffer, "9223372036854775808");
+    i = Nlm_StringToInt8(buffer, &p);
+    assert(p == 0);
+    printf("Conversion of %s (INT8_MAX + 1) to Int8 caused error\n", buffer);
+
+    strcpy(buffer, "-9223372036854775809");
+    i = Nlm_StringToInt8(buffer, &p);
+    assert(p == 0);
+    printf("Conversion of %s (INT8_MIN - 1) to Int8 caused error\n", buffer);
+
+    strcpy(buffer, "18446744073709551616");
+    j = Nlm_StringToUint8(buffer, &p);
+    assert(p == 0);
+    printf("Conversion of %s (UINT8_MAX + 1) to Uint8 caused error\n", buffer);
+
+    printf("All tests succeeded\n");
+
+    return 0;
+}
+#endif /* TEST_INT8_CONVERSION */
