@@ -25,11 +25,11 @@
 *
 * File Name:  ncbistr.h
 *
-* Author:  Gish, Kans, Ostell, Schuler
+* Author:  Gish, Kans, Ostell, Schuler, Vakatov, Lavrentiev
 *
 * Version Creation Date:   1/1/91
 *
-* $Revision: 6.0 $
+* $Revision: 6.6 $
 *
 * File Description:
 *   	prototypes for portable string routines
@@ -37,6 +37,29 @@
 * Modifications:
 * --------------------------------------------------------------------------
 * $Log: ncbistr.h,v $
+* Revision 6.6  2001/01/05 22:43:58  shavirin
+* Added functions, that transfer Uint8 values to platform-independent
+* objects and back.
+*
+* Revision 6.5  2000/11/30 22:46:08  lavr
+* Added the following functions for conversions of Int8 and Uint8
+* to strings and back; test suite attached at the end of the file.
+* Nlm_Int8ToString, Nlm_Uint8ToString, Nlm_StringToInt8, Nlm_StringToUint8
+*
+* Revision 6.4  1999/04/15 20:24:07  vakatov
+* Dont use "list" name as it can clash with the standard "list<>" template
+* on some raw C++ compilers
+*
+* Revision 6.3  1999/03/11 16:10:00  kans
+* StringHasNoText and TrimSpacesAroundString moved from vibforms
+*
+* Revision 6.2  1998/11/27 19:08:59  vakatov
+* Removed proto of StringAppend() -- it is not implemented anyway, and
+* there are more standard functions from the "String[N]Cat" family
+*
+* Revision 6.1  1998/10/07 19:09:01  kans
+* added Nlm_StringTokMT, multithread-safe version
+*
 * Revision 6.0  1997/08/25 18:17:10  madden
 * Revision changed to 6.0
 *
@@ -99,7 +122,7 @@ NLM_EXTERN int LIBCALL Nlm_StrIPCmp PROTO((const char FAR *a, const char FAR *b)
 NLM_EXTERN int LIBCALL Nlm_StrNIPCmp PROTO((const char FAR *a, const char FAR *b, size_t max));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StrMove PROTO((char FAR *to, const char FAR *from));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StrSave PROTO((const char FAR *from));
-NLM_EXTERN size_t LIBCALL Nlm_StrCnt PROTO((const char FAR *str, const char FAR *list));
+NLM_EXTERN size_t LIBCALL Nlm_StrCnt PROTO((const char FAR *str, const char FAR *x_list));
 
 NLM_EXTERN size_t LIBCALL Nlm_StringLen PROTO((const char *str));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringCpy PROTO((char FAR *to, const char FAR *from));
@@ -132,10 +155,11 @@ NLM_EXTERN size_t LIBCALL Nlm_StringSpn PROTO((const char FAR *str1, const char
 NLM_EXTERN size_t LIBCALL Nlm_StringCSpn PROTO((const char FAR *str1, const char FAR *str2));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringStr PROTO((const char FAR *str1, const char FAR *str2));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringTok PROTO((char FAR *str1, const char FAR *str2));
+NLM_EXTERN Nlm_CharPtr LIBCALL  Nlm_StringTokMT PROTO((char FAR *str1, const char FAR *str2, char FAR **tmp));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringMove PROTO((char FAR *to, const char FAR *from));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringSave PROTO((const char FAR *from));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringSaveNoNull PROTO((const char FAR *from));
-NLM_EXTERN size_t LIBCALL Nlm_StringCnt PROTO((const char FAR *str, const char FAR *list));
+NLM_EXTERN size_t LIBCALL Nlm_StringCnt PROTO((const char FAR *str, const char FAR *x_list));
 NLM_EXTERN char* LIBCALL Nlm_StringUpper PROTO((char *string));
 NLM_EXTERN char* LIBCALL Nlm_StringLower PROTO((char *string));
 
@@ -144,6 +168,32 @@ NLM_EXTERN Nlm_Int2 LIBCALL Nlm_MeshStringICmp PROTO((const char FAR *str1, cons
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringSearch PROTO((const char FAR *str, const char FAR *sub));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringISearch PROTO((const char FAR *str, const char FAR *sub));
 
+NLM_EXTERN Nlm_Boolean LIBCALL Nlm_StringHasNoText PROTO((Nlm_CharPtr str));
+NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TrimSpacesAroundString PROTO((Nlm_CharPtr str));
+
+/* Printing 8-byte integer into platform-independent array of 8 bytes
+   memory allocated for this storage */
+NLM_EXTERN Nlm_Uint1Ptr LIBCALL Uint8ToBytes(Nlm_Uint8 value);
+
+/* Reading platform-independent array of 8 bytes and transfer it to
+   8-bytes integer */
+NLM_EXTERN Nlm_Uint8 LIBCALL BytesToUint8(Nlm_Uint1Ptr bytes);
+
+/* Retrieve integer value from "str".
+ * On success, "*endptr" will point to the symbol of "str" that
+ * appears right after the read number.
+ * On conversion error, "*endptr" will be assigned NULL.
+ */
+NLM_EXTERN Nlm_Int8  LIBCALL Nlm_StringToInt8 (const char* str, const char** endptr);
+NLM_EXTERN Nlm_Uint8 LIBCALL Nlm_StringToUint8(const char* str, const char** endptr);
+
+/* Print "value" to "str".
+ * Return '\0'-terminated "str" on success.
+ * Return NULL if "str_size" is not enough to put the number and '\0'.
+ */
+NLM_EXTERN char* LIBCALL Nlm_Int8ToString (Nlm_Int8  value, char* str, size_t str_size);
+NLM_EXTERN char* LIBCALL Nlm_Uint8ToString(Nlm_Uint8 value, char* str, size_t str_size);
+
 /*****************************************************************************
 *
 *   LabelCopy (to, from, buflen)
@@ -194,7 +244,6 @@ NLM_EXTERN Nlm_Boolean LIBCALL StringSub PROTO ((char FAR *String, char Find, ch
 NLM_EXTERN Nlm_Boolean LIBCALL StringSubSet PROTO ((char FAR *String,char FAR *FindSet, char Replace));
 NLM_EXTERN Nlm_Boolean LIBCALL StringSubString PROTO ((char FAR *String, char FAR *Find, char FAR *Replace, Nlm_Int4 MaxLength));
 NLM_EXTERN Nlm_CharPtr LIBCALL StringEnd PROTO ((char FAR *String));
-NLM_EXTERN Nlm_CharPtr LIBCALL StringAppend PROTO ((char FAR *String, char FAR *Append));
 NLM_EXTERN Int4 LIBCALL CountChar PROTO ((char FAR *String, char Char));
 NLM_EXTERN Int4 LIBCALL CountStrings PROTO ((char FAR *String, char FAR *Find));
 NLM_EXTERN Nlm_CharPtr LIBCALL StripSpaces PROTO ((char FAR *Line));
@@ -345,6 +394,7 @@ NLM_EXTERN char * LIBCALL Nlm_StrLower PROTO((char *string));
 #define StringCSpn  Nlm_StringCSpn
 #define StringStr   Nlm_StringStr
 #define StringTok   Nlm_StringTok
+#define StringTokMT Nlm_StringTokMT
 #define StringMove	Nlm_StringMove
 #define StringSave	Nlm_StringSave
 #define StringSaveNoNull Nlm_StringSaveNoNull
@@ -357,9 +407,17 @@ NLM_EXTERN char * LIBCALL Nlm_StrLower PROTO((char *string));
 #define StringSearch Nlm_StringSearch
 #define StringISearch Nlm_StringISearch
 
+#define StringHasNoText Nlm_StringHasNoText
+#define TrimSpacesAroundString Nlm_TrimSpacesAroundString
+
 #define LabelCopy Nlm_LabelCopy
 #define LabelCopyExtra Nlm_LabelCopyExtra
 
+#define StringToInt8  Nlm_StringToInt8
+#define StringToUint8 Nlm_StringToUint8
+#define Int8ToString  Nlm_Int8ToString
+#define Uint8ToString Nlm_Uint8ToString
+
 /*----------------------------------------*/
 /*      Misc Text Oriented Macros         */
 /*----------------------------------------*/
