@@ -29,13 +29,73 @@
 *
 * Version Creation Date:   8/5/96
 *
-* $Revision: 6.14 $
+* $Revision: 6.34 $
 *
 * File Description: 
 *
 * Modifications:  
 * --------------------------------------------------------------------------
 * $Log: entrez.c,v $
+* Revision 6.34  2000/12/05 17:52:58  kans
+* removed cn3d, which caused ddv link problem
+*
+* Revision 6.33  2000/04/21 23:13:56  kans
+* added qualPageData
+*
+* Revision 6.32  2000/03/02 17:49:24  kans
+* call ID1 fetch enable/disable in preference to old Entrez service for sequence fetch
+*
+* Revision 6.31  2000/03/01 23:52:25  kans
+* comment out ID1 fetch for now
+*
+* Revision 6.30  2000/03/01 22:20:11  kans
+* raised version number, calls ID1BioseqFetchEnable to fetch sequences
+*
+* Revision 6.29  2000/02/11 23:04:49  kans
+* remove loadUidItem and saveUidItem - use import and export
+*
+* Revision 6.28  1999/11/12 23:06:56  kans
+* DoLoadNamedUidList takes Int4 num argument
+*
+* Revision 6.27  1999/11/04 15:53:20  kans
+* raised version number for public release
+*
+* Revision 6.26  1999/08/27 15:59:09  kans
+* removed obsolete load and save uid list commands
+*
+* Revision 6.25  1999/08/24 18:05:58  kans
+* has target control, use delayed neighbor policy
+*
+* Revision 6.24  1999/05/27 16:39:24  kans
+* preferences to options menu
+*
+* Revision 6.23  1999/05/12 17:41:48  kans
+* raised version number for public release
+*
+* Revision 6.22  1999/01/14 19:07:51  kans
+* new parameters to Cn3DWin_Entrez
+*
+* Revision 6.21  1999/01/13 20:09:18  kans
+* added neighbor policy control
+*
+* Revision 6.20  1999/01/08 18:27:35  kans
+* commented out makerpt.prt load - not used
+*
+* Revision 6.19  1999/01/03 22:47:05  kans
+* raised version number
+*
+* Revision 6.18  1998/12/08 16:52:41  kans
+* new params to ShowNetConfigForm to simplify configuration
+*
+* Revision 6.17  1998/12/08 03:19:56  kans
+* changes for web dispatcher only config form
+*
+* Revision 6.16  1998/12/04 23:35:45  kans
+* comment out requirement for ncbi config file, now that cdconfig.c is kludged with hard-coded values to psrventr
+*
+* Revision 6.15  1998/11/11 02:06:31  kans
+* set browser path from config file for PubMed launch
+*
 * Revision 6.14  1998/05/04 15:46:06  kans
 * obsoleted load/save uid list, in favor of import/export
 *
@@ -131,6 +191,7 @@
 #include <medview.h>
 #include <bspview.h>
 #include <accentr.h>
+#include <accid1.h>
 #include <lsqfetch.h>
 #include <objsub.h>
 #include <gather.h>
@@ -139,13 +200,15 @@
 
 #include <objmime.h>
 #include <mmdbapi.h>
+/*
 #ifndef WIN16
 #include <cn3dentr.h>
 #endif
+*/
 
 #include <entrez.h>
 
-#define ENTREZ_APP_VERSION "6.40"
+#define ENTREZ_APP_VERSION "7.50"
 
 static ForM  termListForm = NULL;
 static ForM  docSumForm = NULL;
@@ -159,8 +222,6 @@ static IteM  importItem = NULL;
 static IteM  saveItem = NULL;
 static IteM  saveAsItem = NULL;
 */
-static IteM  loadUidItem = NULL;
-static IteM  saveUidItem = NULL;
 static IteM  printItem = NULL;
 static IteM  cutItem = NULL;
 static IteM  copyItem = NULL;
@@ -172,6 +233,7 @@ static IteM  preferencesItem = NULL;
 static IteM  clearUnusedItem = NULL;
 static IteM  legendItem = NULL;
 static ChoicE  queryChoice = NULL;
+static ChoicE  neighborChoice = NULL;
 
 static Boolean  termListUp = FALSE;
 static Boolean  docSumUp = FALSE;
@@ -179,7 +241,6 @@ static Boolean  docSumUp = FALSE;
 
 static Boolean  macLike = FALSE;
 static Boolean  showAsnPage = FALSE;
-static Boolean  loadSaveUidListOK = FALSE;
 
 static FonT  titleFont = NULL;
 
@@ -356,8 +417,6 @@ static void TermSelectionActivateProc (WindoW w)
                    (HANDLE) pasteItem,
                    (HANDLE) deleteItem,
                    (HANDLE) preferencesItem,
-                   (HANDLE) loadUidItem,
-                   (HANDLE) saveUidItem,
                    (HANDLE) queryChoice,
                    (HANDLE) clearUnusedItem,
                    NULL);
@@ -368,6 +427,11 @@ static void DocumentSummaryActivateProc (WindoW w)
 {
   docSumUp = TRUE;
   currentFormDataPtr = (VoidPtr) GetObjectExtra (w);
+  if (UsingDelayedNeighbor ((ForM) w)) {
+    SafeSetValue (neighborChoice, 2);
+  } else {
+    SafeSetValue (neighborChoice, 1);
+  }
   RepeatProcOnHandles (Enable,
                    (HANDLE) closeItem,
                    (HANDLE) importItem,
@@ -376,8 +440,7 @@ static void DocumentSummaryActivateProc (WindoW w)
                    (HANDLE) copyItem,
                    (HANDLE) docsumfontItem,
                    (HANDLE) preferencesItem,
-                   (HANDLE) loadUidItem,
-                   (HANDLE) saveUidItem,
+                   (HANDLE) neighborChoice,
                    NULL);
 }
 
@@ -414,6 +477,7 @@ static void MacDeactProc (WindoW w)
   SafeSetTitle (exportItem, "Export...");
   SafeSetTitle (importItem, "Import...");
   SafeSetValue (queryChoice, 0);
+  SafeSetValue (neighborChoice, 0);
   RepeatProcOnHandles (Disable,
                    (HANDLE) closeItem,
                    (HANDLE) duplicateItem,
@@ -429,9 +493,8 @@ static void MacDeactProc (WindoW w)
                    (HANDLE) preferencesItem,
                    (HANDLE) queryChoice,
                    (HANDLE) clearUnusedItem,
+                   (HANDLE) neighborChoice,
                    (HANDLE) legendItem,
-                   (HANDLE) loadUidItem,
-                   (HANDLE) saveUidItem,
                    NULL);
 }
 #endif
@@ -761,30 +824,21 @@ static void StyleManagerProc (IteM i)
 }
 
 
+/*
 #ifndef WIN16
 static void Cn3DWinShowProc (IteM i)
 {
   WindoW  w;
 
-  w = Cn3DWin_Entrez();
+  w = Cn3DWin_Entrez(NULL, TRUE);
   if (w == NULL) return;
   Show (w);
   Select (w);
 }
 #endif
+*/
 
 
-static CharPtr obsoletemsg =
-"Please use Import and Export items to load and save Uid lists.\n\
-The Load and Save Uid List items are obsolete, and will disappear\n\
-in the next release.";
-
-static void ObsoleteUidListProc (IteM i)
-
-{
-  Message (MSG_OK, "%s", obsoletemsg);
-}
-
 #ifndef WIN_MAC
 static void MedlineViewFormMenus (WindoW w)
 
@@ -866,16 +920,9 @@ static void TermListFormMenus (WindoW w)
     AddAboutAndHelpMenuItems (m);
     FormCommandItem (m, "Close", bfp, VIB_MSG_CLOSE);
     SeparatorItem (m);
-    if (loadSaveUidListOK) {
-      FormCommandItem (m, "Import Uid List...", bfp, VIB_MSG_IMPORT);
-      FormCommandItem (m, "Export Uid List...", bfp, VIB_MSG_EXPORT);
-      SeparatorItem (m);
-      i = CommandItem (m, "Load Uid List...", ObsoleteUidListProc);
-      SetObjectExtra (i, bfp, NULL);
-      i = CommandItem (m, "Save Uid List...", ObsoleteUidListProc);
-      SetObjectExtra (i, bfp, NULL);
-      SeparatorItem (m);
-    }
+    FormCommandItem (m, "Import Uid List...", bfp, VIB_MSG_IMPORT);
+    FormCommandItem (m, "Export Uid List...", bfp, VIB_MSG_EXPORT);
+    SeparatorItem (m);
     FormCommandItem (m, "Quit/Q", bfp, VIB_MSG_QUIT);
 
     m = PulldownMenu (w, "Edit");
@@ -885,20 +932,22 @@ static void TermListFormMenus (WindoW w)
     FormCommandItem (m, CLEAR_MENU_ITEM, bfp, VIB_MSG_DELETE);
 
     m = PulldownMenu (w, "Options");
+    CommandItem (m, "Preferences...", PreferencesProc);
+    SeparatorItem (m);
     sub = SubMenu (m, "Query Style");
     CreateQueryTypeChoice (sub, bfp);
     CreateClearUnusedItem (m, bfp);
 
     m = PulldownMenu (w, "Misc");
-    CommandItem (m, "Preferences...", PreferencesProc);
-    SeparatorItem (m);
     VSMAddToMenu (m, VSM_DESKTOP);
     SeparatorItem (m);
     CommandItem (m, "Style Manager...", StyleManagerProc);
+/*
 #ifndef WIN16
     SeparatorItem (m);
     CommandItem (m, "Cn3D Window...", Cn3DWinShowProc);
 #endif
+*/
   }
 }
 
@@ -906,6 +955,7 @@ static void DocSumFormMenus (WindoW w)
 
 {
   BaseFormPtr  bfp;
+  ChoicE       c;
   IteM         i;
   MenU         m;
   MenU         sub;
@@ -923,25 +973,22 @@ static void DocSumFormMenus (WindoW w)
     FormCommandItem (m, "Save As...", bfp, VIB_MSG_SAVE_AS);
     SeparatorItem (m);
     */
-    if (loadSaveUidListOK) {
-      i = CommandItem (m, "Load Uid List...", ObsoleteUidListProc);
-      SetObjectExtra (i, bfp, NULL);
-      i = CommandItem (m, "Save Uid List...", ObsoleteUidListProc);
-      SetObjectExtra (i, bfp, NULL);
-      SeparatorItem (m);
-    }
     FormCommandItem (m, "Print...", bfp, VIB_MSG_PRINT);
 
     m = PulldownMenu (w, "Edit");
     FormCommandItem (m, COPY_MENU_ITEM, bfp, VIB_MSG_COPY);
 
     m = PulldownMenu (w, "Options");
+    CommandItem (m, "Preferences...", PreferencesProc);
+    SeparatorItem (m);
+    sub = SubMenu (m, "Neighbor Policy");
+    c = CreateNeighborDelayChoice (sub, bfp);
+    SetValue (c, 2); /* UseDelayedNeighbor */
+    SeparatorItem (m);
     LoadDocsumOptionsMenu (m);
     seqviewprocs.alignWithChecked = entrezglobals.alignWithChecked;
 
     m = PulldownMenu (w, "Misc");
-    CommandItem (m, "Preferences...", PreferencesProc);
-    SeparatorItem (m);
     sub = SubMenu (m, "Font Selection");
     i = CommandItem (sub, "DocSum Font...", DocSumFontChangeProc);
     SetObjectExtra (i, bfp, NULL);
@@ -1059,7 +1106,7 @@ static void DoRetrieveDocuments (ForM f, Int2 num, Int2 parents, Int4Ptr uids, I
   RetrieveDocuments (docSumForm, num, parents, uids, db);
 }
 
-static void DoLoadNamedUidList (ForM f, CharPtr term, Int2 num, Int4Ptr uids, Int2 db)
+static void DoLoadNamedUidList (ForM f, CharPtr term, Int4 num, Int4Ptr uids, Int2 db)
 
 {
   LoadNamedUidList (termListForm, term, num, uids, db);
@@ -1083,17 +1130,10 @@ static void SetupAppProperties (void)
   Char     str [32];
   Int2     val;
 
-  showAsnPage = FALSE;
+  showAsnPage = TRUE;
   if (GetEntrezAppParam ("PREFERENCES", "SHOWASNPAGE", "", str, sizeof (str) - 1)) {
-    if (StringICmp (str, "TRUE") == 0) {
-      showAsnPage = TRUE;
-    }
-  }
-
-  loadSaveUidListOK = TRUE;
-  if (GetEntrezAppParam ("PREFERENCES", "LOADSAVEUIDLIST", "", str, sizeof (str) - 1)) {
     if (StringICmp (str, "FALSE") == 0) {
-      loadSaveUidListOK = FALSE;
+      showAsnPage = FALSE;
     }
   }
 
@@ -1101,7 +1141,7 @@ static void SetupAppProperties (void)
   medviewprocs.cleanupObjectPtr = FALSE;
   medviewprocs.activateForm = MedlineViewFormActivated;
   medviewprocs.closeForm = NULL;
-  medviewprocs.useFolderTabs = CHANGE_VIEW_RADIOBUTTONS;
+  medviewprocs.useFolderTabs = CHANGE_VIEW_POPUP;
   /*
   medviewprocs.initPage = CITATION_PAGE;
   */
@@ -1115,9 +1155,9 @@ static void SetupAppProperties (void)
   SetAppProperty ("MedlineDisplayForm", &medviewprocs);
 
   MemSet ((Pointer) (&seqviewprocs), 0, sizeof (SeqViewProcs));
-  seqviewprocs.hasTargetControl = FALSE;
+  seqviewprocs.hasTargetControl = TRUE;
   seqviewprocs.hasDoneButton = FALSE;
-  seqviewprocs.hasDuplicateButton = TRUE;
+  seqviewprocs.hasDuplicateButton = FALSE;
   seqviewprocs.launchEditors = FALSE;
   seqviewprocs.launchSubviewers = TRUE;
   seqviewprocs.sendSelectMessages = TRUE;
@@ -1125,7 +1165,7 @@ static void SetupAppProperties (void)
   seqviewprocs.cleanupObjectPtr = FALSE;
   seqviewprocs.activateForm = BioseqViewFormActivated;
   seqviewprocs.closeForm = NULL;
-  seqviewprocs.useFolderTabs = CHANGE_VIEW_RADIOBUTTONS;
+  seqviewprocs.useFolderTabs = CHANGE_VIEW_POPUP;
   /*
   seqviewprocs.initNucPage = NUCASN2FF_PAGE_1;
   seqviewprocs.initProtPage = PROTGENPEPT_PAGE;
@@ -1148,8 +1188,10 @@ static void SetupAppProperties (void)
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &emblPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &gnptPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &fstaPageData);
+  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &qualPageData);
   if (showAsnPage) {
     AddBioseqPageToList (&(seqviewprocs.pageSpecs), &asnPageData);
+    AddBioseqPageToList (&(seqviewprocs.pageSpecs), &dskPageData);
   }
 
   SetAppProperty ("SeqDisplayForm", &seqviewprocs);
@@ -1255,6 +1297,10 @@ static void SetupAppProperties (void)
     entrezglobals.initMode = StringSaveNoNull (str);
   }
 
+  if (GetEntrezAppParam ("SETTINGS", "BROWSER", NULL, str, sizeof (str))) {
+    SetAppProperty ("MedviewBrowserPath", (void *) StringSave (str));
+  }
+
   MemSet ((Pointer) (&stdedprocs), 0, sizeof (StdEditorProcs));
 #ifdef WIN_MAC
   stdedprocs.activateForm = StdEditorFormActivated;
@@ -1385,11 +1431,6 @@ static void SetupMacMenus (void)
   saveAsItem = FormCommandItem (m, "Save As...", NULL, VIB_MSG_SAVE_AS);
   SeparatorItem (m);
   */
-  if (loadSaveUidListOK) {
-    loadUidItem = CommandItem (m, "Load Uid List...", ObsoleteUidListProc);
-    saveUidItem = CommandItem (m, "Save Uid List...", ObsoleteUidListProc);
-    SeparatorItem (m);
-  }
   printItem = FormCommandItem (m, "Print...", NULL, VIB_MSG_PRINT);
   SeparatorItem (m);
   FormCommandItem (m, "Quit/Q", NULL, VIB_MSG_QUIT);
@@ -1406,15 +1447,19 @@ static void SetupMacMenus (void)
   deleteItem = FormCommandItem (m, CLEAR_MENU_ITEM, NULL, VIB_MSG_DELETE);
 
   m = PulldownMenu (NULL, "Options");
+  preferencesItem = CommandItem (m, "Preferences...", PreferencesProc);
+  SeparatorItem (m);
   sub = SubMenu (m, "Query Style");
   queryChoice = CreateQueryTypeChoice (sub, NULL);
   clearUnusedItem = CreateClearUnusedItem (m, NULL);
   SeparatorItem (m);
+  sub = SubMenu (m, "Neighbor Policy");
+  neighborChoice = CreateNeighborDelayChoice (sub, NULL);
+  SeparatorItem (m);
   LoadDocsumOptionsMenu (m);
   seqviewprocs.alignWithChecked = entrezglobals.alignWithChecked;
 
   m = PulldownMenu (NULL, "Misc");
-  preferencesItem = CommandItem (m, "Preferences...", PreferencesProc);
   sub = SubMenu (m, "Font Selection");
   docsumfontItem = CommandItem (sub, "DocSum Font...", DocSumFontChangeProc);
   displayfontItem = CommandItem (sub, "Display Font...", DisplayFontChangeProc);
@@ -1424,10 +1469,12 @@ static void SetupMacMenus (void)
   VSMAddToMenu (m, VSM_DESKTOP);
   SeparatorItem (m);
   CommandItem (m, "Style Manager...", StyleManagerProc);
+/*
 #ifndef WIN16
   SeparatorItem (m);
   CommandItem (m, "Cn3D Window...", Cn3DWinShowProc);
 #endif
+*/
 }
 #endif
 
@@ -1444,6 +1491,7 @@ static void CleanupEntrez (void)
   FreePrintOptions ();
 
   LocalSeqFetchDisable ();
+  ID1BioseqFetchDisable ();
   EntrezBioseqFetchDisable ();
 }
 
@@ -1487,9 +1535,15 @@ static void ConfigCancelled (void)
   QuitProgram ();
 }
 
+/*
 static CharPtr configMessage =
 "Entrez cannot find ncbi configuration file.\n\
 Do you wish to run the configuration program?";
+*/
+
+static CharPtr configMessage =
+"Entrez cannot connect to the data server.\n\
+Do you wish to run the configuration program?";
 
 Int2 Main (void)
 
@@ -1500,15 +1554,18 @@ Int2 Main (void)
   Char           str [16];
   Int2           val;
   WindoW         w;
+/*
 #if defined(WIN16)
 #else
   PRGD           prgdDict;
 #endif
+*/
 
   ErrSetFatalLevel (SEV_MAX);
   ErrClearOptFlags (EO_SHOW_USERSTR);
   ProcessUpdatesFirst (FALSE);
 
+  /*
   if (! GetAppParam ("NCBI", "NCBI", NULL, NULL, str, sizeof (str) - 1)) {
     if (Message (MSG_YN, configMessage) == ANS_YES) {
       ShowNetConfigForm (ConfigFormActivated, ConfigFormMessage,
@@ -1525,6 +1582,7 @@ Int2 Main (void)
     Message (MSG_FATAL, "Entrez cannot find ncbi configuration file");
     return 0;
   }
+  */
 
   UseLocalAsnloadDataAndErrMsg ();
   SetupAppProperties ();
@@ -1550,9 +1608,18 @@ Int2 Main (void)
   SetTitle (w, "Finding Entrez Data");
   if (! EntrezBioseqFetchEnable ("Entrez", TRUE)) {
     ArrowCursor ();
-    Message (MSG_FATAL, "Unable to connect to Entrez databases");
+    /* Message (MSG_FATAL, "Unable to connect to Entrez databases"); */
+    if (Message (MSG_YN, configMessage) == ANS_YES) {
+      ShowNetConfigForm (ConfigFormActivated, ConfigFormMessage,
+                         ConfigAccepted, ConfigCancelled, NULL, TRUE);
+      ProcessEvents ();
+      return 0;
+    } else {
+      Message (MSG_FATAL, "Entrez cannot run connection to the data server");
+    }
     return 0;
   }
+  ID1BioseqFetchEnable ("Entrez", TRUE);
   LocalSeqFetchInit (FALSE);
 
 #ifdef WIN_MAC
@@ -1603,11 +1670,13 @@ Int2 Main (void)
     Message (MSG_FATAL, "PrintTemplateSetLoad asn2ff.prt failed");
     return 0;
   }
+  /*
   if (! PrintTemplateSetLoad ("makerpt.prt")) {
     ArrowCursor ();
     Message (MSG_FATAL, "PrintTemplateSetLoad makerpt.prt failed");
     return 0;
   }
+  */
 
   SetTitle (w, "Loading sequence alphabet converter");
   if (! SeqCodeSetLoad ()) {
@@ -1629,13 +1698,15 @@ Int2 Main (void)
     return 0;
   }
 
+/*
 #ifndef WIN16
   SetTitle (w, "Loading structure dictionary");
   if (OpenMMDBAPI ((POWER_VIEW ^ FETCH_ENTREZ), NULL)) {
     prgdDict = GetPRGDDictionary ();
-    Cn3DWin_Entrez();
+    Cn3DWin_Entrez(NULL, TRUE);
   }
 #endif
+*/
 
   SetTitle (w, "Creating menus");
 
@@ -1678,6 +1749,7 @@ Int2 Main (void)
     docSumForm = CreateDocSumForm (-10, -90, "Document",
                                    DocumentSummaryActivateProc,
                                    DocumentSummaryFormMessage);
+    UseDelayedNeighbor (docSumForm, TRUE);
     if (docSumForm != NULL) {
       ProcessEvents ();
     } else {
