@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.7 $
+* $Revision: 6.32 $
 *
 * File Description: 
 *
@@ -50,57 +50,71 @@
 extern EnumFieldAssoc  orgmod_subtype_alist [];
 ENUM_ALIST(orgmod_subtype_alist)
   {" ",                 0},
-  {"Strain",            2},
-  {"Substrain",         3},
-  {"Type",              4},
-  {"Subtype",           5},
-  {"Variety",           6},
-  {"Serotype",          7},
-  {"Serogroup",         8},
-  {"Serovar",           9},
-  {"Cultivar",         10},
-  {"Pathovar",         11},
-  {"Chemovar",         12},
-  {"Biovar",           13},
-  {"Biotype",          14},
-  {"Group",            15},
-  {"Subgroup",         16},
-  {"Isolate",          17},
-  {"Common",           18},
   {"Acronym",          19},
+  {"Anamorph",         29},
+  {"Authority",        24},
+  {"Biotype",          14},
+  {"Biovar",           13},
+  {"Breed",            31},
+  {"Chemovar",         12},
+  {"Common",           18},
   {"Dosage",           20},
+  {"Cultivar",         10},
+  {"Ecotype",          27},
+  {"Forma",            25},
+  {"Forma-specialis",  26},
+  {"Group",            15},
+  {"Isolate",          17},
   {"Natural-host",     21},
-  {"Sub-species",      22},
-  {"Specimen-voucher", 23},
+  {"Old Lineage",     253},
   {"Old Name",        254},
+  {"Pathovar",         11},
+  {"Serogroup",         8},
+  {"Serotype",          7},
+  {"Serovar",           9},
+  {"Specimen-voucher", 23},
+  {"Strain",            2},
+  {"Subgroup",         16},
+  {"Sub-species",      22},
+  {"Substrain",         3},
+  {"Subtype",           5},
+  {"Synonym",          28},
+  {"Teleomorph",       30},
+  {"Type",              4},
+  {"Variety",           6},
 END_ENUM_ALIST
 
 extern EnumFieldAssoc  subsource_subtype_alist [];
 ENUM_ALIST(subsource_subtype_alist)
-  {" ",                 0},
-  {"Chromosome",        1},
-  {"Map",               2},
-  {"Clone",             3},
-  {"Subclone",          4},
-  {"Haplotype",         5},
-  {"Genotype",          6},
-  {"Sex",               7},
-  {"Cell-line",         8},
-  {"Cell-type",         9},
-  {"Tissue-type",      10},
-  {"Clone-lib",        11},
-  {"Dev-stage",        12},
-  {"Frequency",        13},
-  {"Germline",         14},
-  {"Rearranged",       15},
-  {"Lab-host",         16},
-  {"Pop-variant",      17},
-  {"Tissue-lib",       18},
-  {"Plasmid-name",     19},
-  {"Transposon-name",  20},
-  {"Ins-seq-name",     21},
-  {"Plastid-name",     22},
-  {"Country",          23},
+  {" ",                      0},
+  {"Cell-line",              8},
+  {"Cell-type",              9},
+  {"Chromosome",             1},
+  {"Clone",                  3},
+  {"Clone-lib",             11},
+  {"Country",               23},
+  {"Dev-stage",             12},
+  {"Endogenous-virus-name", 25},
+  {"Environmental-sample",  27},
+  {"Frequency",             13},
+  {"Genotype",               6},
+  {"Germline",              14},
+  {"Haplotype",              5},
+  {"Ins-seq-name",          21},
+  {"Isolation-source",      28},
+  {"Lab-host",              16},
+  {"Map",                    2},
+  {"Plasmid-name",          19},
+  {"Plastid-name",          22},
+  {"Pop-variant",           17},
+  {"Segment",               24},
+  {"Sex",                    7},
+  {"Subclone",               4},
+  {"Rearranged",            15},
+  {"Tissue-lib",            18},
+  {"Tissue-type",           10},
+  {"Transgenic",            26},
+  {"Transposon-name",       20},
 END_ENUM_ALIST
 
 static ENUM_ALIST(biosource_genome_alist)
@@ -119,6 +133,11 @@ static ENUM_ALIST(biosource_genome_alist)
   {"Cyanelle",            12},
   {"Proviral",            13},
   {"Virion",              14},
+  {"Nucleomorph",         15},
+  {"Apicoplast",          16},
+  {"Leucoplast",          17},
+  {"Proplastid",          18},
+  {"Endogenous-virus",    19},
 END_ENUM_ALIST
 
 extern EnumFieldAssoc  biosource_genome_simple_alist [];
@@ -128,6 +147,7 @@ ENUM_ALIST(biosource_genome_simple_alist)
   {"Chloroplast",          2},
   {"Kinetoplast",          4},
   {"Mitochondrion",        5},
+  {"Plastid",              6},
   {"Macronuclear",         7},
   {"Extrachromosomal",     8},
   {"Plasmid",              9},
@@ -136,6 +156,11 @@ ENUM_ALIST(biosource_genome_simple_alist)
   {"Cyanelle",            12},
   {"Proviral",            13},
   {"Virion",              14},
+  {"Nucleomorph",         15},
+  {"Apicoplast",          16},
+  {"Leucoplast",          17},
+  {"Proplastid",          18},
+  {"Endogenous-virus",    19},
 END_ENUM_ALIST
 
 static ENUM_ALIST(biosource_origin_alist)
@@ -280,7 +305,7 @@ extern Boolean LoadOrganismTable (void)
 #endif
         tmp = ptr;
         ch = *tmp;
-        while (ch != '0' && ch != '\n') {
+        while (ch != '\0' && ch != '\n') {
           tmp++;
           ch = *tmp;
         }
@@ -392,11 +417,11 @@ extern void SetupGeneticCodes (void)
         }
       }
       if (id != 7 && id != 8) {
-        if (id > 0 && id < 20) {
+        if (id > 0 /* && id < 30 */ ) {
           i = 0;
           if (StringLen (str + i) > 0 && index < NUM_GENETIC_CODES - 1) {
             ch = str [i];
-            while (ch == ' ' | ch == ';') {
+            while (ch == ' ' || ch == ';') {
               i++;
               ch = str [i];
             }
@@ -563,10 +588,20 @@ static void ChangeGencodePopups (GenBioPagePtr gbp)
       if (GetEnumPopup (gbp->genome, gbp->genomeAlist, &genome)) {
         if (genome == 4 || genome == 5) {
           SafeSetValue (gbp->simplecode, gcIdToIndex [gbp->mitoGC]);
+        } else if (genome == GENOME_chloroplast ||
+                   genome == GENOME_chromoplast ||
+                   genome == GENOME_plastid ||
+                   genome == GENOME_cyanelle ||
+                   genome == GENOME_apicoplast ||
+                   genome == GENOME_leucoplast ||
+                   genome == GENOME_proplastid) {
+          SafeSetValue (gbp->simplecode, gcIdToIndex [11]);
         } else {
           SafeSetValue (gbp->simplecode, gcIdToIndex [gbp->nuclGC]);
         }
       }
+      SafeSetValue (gbp->gcode, gcIdToIndex [gbp->nuclGC]);
+      SafeSetValue (gbp->mgcode, gcIdToIndex [gbp->mitoGC]);
     } else {
       SafeSetValue (gbp->gcode, gcIdToIndex [gbp->nuclGC]);
       SafeSetValue (gbp->mgcode, gcIdToIndex [gbp->mitoGC]);
@@ -580,13 +615,13 @@ static void SetCodes (GenBioPagePtr gbp, Int2 row, Boolean changepopups)
   Char  str [256];
 
   if (gbp != NULL && row > 0) {
-    CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 4);
+    CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 3);
     StrToInt (str, &gbp->nuclGC);
-    CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 5);
+    CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 4);
     StrToInt (str, &gbp->mitoGC);
-    CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 6);
+    CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 5);
     SafeSetTitle (gbp->gbDiv, str);
-    CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 7);
+    CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 6);
     StrToLong (str, &gbp->taxID);
     if (changepopups) {
       ChangeGencodePopups (gbp);
@@ -660,9 +695,8 @@ static void AutoScrollTax (GenBioPagePtr gbp, TexT t, Boolean isSciName,
           }
           return;
         }
-        CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 2);
-        if (str [0] == 'S' && isSciName) {
-          CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 3);
+        if (isSciName) {
+          CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 2);
           if (! gbp->typedComName) {
             if (setText) {
               SafeSetTitle (gbp->commonName, str);
@@ -676,23 +710,6 @@ static void AutoScrollTax (GenBioPagePtr gbp, TexT t, Boolean isSciName,
               return;
             }
           }
-        } else if (str [0] == 'C' && (! isSciName)) {
-          CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 3);
-          if (! gbp->typedSciName) {
-            if (setText) {
-              /*
-              SafeSetTitle (gbp->taxName, str);
-              */
-            }
-          } else {
-            GetTitle (gbp->taxName, txt, sizeof (txt));
-            if (StringICmp (txt, str) != 0) {
-              if (changepopups) {
-                ChangeGencodePopups (gbp);
-              }
-              return;
-            }
-          }
         } else {
           if (changepopups) {
             ChangeGencodePopups (gbp);
@@ -714,12 +731,40 @@ static void AutoScrollTax (GenBioPagePtr gbp, TexT t, Boolean isSciName,
 static void TaxNameText (TexT t)
 
 {
-  GenBioPagePtr  gbp;
+  DbtagPtr         dbt;
+  GenBioPagePtr    gbp;
+  ValNodePtr       head;
+  ValNodePtr       nextvnp;
+  ValNodePtr PNTR  prevvnp;
+  ValNodePtr       vnp;
 
   gbp = (GenBioPagePtr) GetObjectExtra (t);
   if (gbp != NULL) {
     gbp->typedSciName = TRUE;
     AutoScrollTax (gbp, t, TRUE, TRUE, TRUE);
+    SafeSetTitle (gbp->lineage, "");
+    SafeSetTitle (gbp->gbDiv, "");
+    head = DialogToPointer (gbp->db);
+    if (head != NULL) {
+      prevvnp = &head;
+      vnp = head;
+      while (vnp != NULL) {
+        nextvnp = vnp->next;
+        dbt = (DbtagPtr) vnp->data.ptrvalue;
+        if (dbt != NULL) {
+          if (StringICmp (dbt->db, "taxon") == 0) {
+            *prevvnp = vnp->next;
+            vnp->next = NULL;
+            DbtagFree (dbt);
+            ValNodeFree (vnp);
+          } else {
+            prevvnp = (ValNodePtr PNTR) &(vnp->next);
+          }
+        }
+        vnp = nextvnp;
+      }
+      PointerToDialog (gbp->db, head);
+    }
   }
 }
 
@@ -779,22 +824,11 @@ static void ReleaseTaxName (DoC d, PoinT pt)
     if (item > 0 && row > 0 && row == gbp->clickedOrg) {
       ResetClip ();
       if (row == gbp->clickedOrg) {
+        CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 1);
+        SafeSetTitle (gbp->taxName, str);
         CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 2);
-        if (str [0] == 'S') {
-          CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 1);
-          SafeSetTitle (gbp->taxName, str);
-          CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 3);
-          SafeSetTitle (gbp->commonName, str);
-          Select (gbp->taxName);
-        } else if (str [0] == 'C') {
-          CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 3);
-          SafeSetTitle (gbp->taxName, str);
-          CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 1);
-          SafeSetTitle (gbp->commonName, str);
-          Select (gbp->taxName);
-        } else {
-          return;
-        }
+        SafeSetTitle (gbp->commonName, str);
+        Select (gbp->taxName);
         SetCodes (gbp, row, TRUE);
       }
     }
@@ -830,22 +864,11 @@ extern Boolean SetBioSourceDialogTaxName (DialoG d, CharPtr taxname)
         ChangeGencodePopups (gbp);
         return FALSE;
       }
+      CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 1);
+      SafeSetTitle (gbp->taxName, str);
       CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 2);
-      if (str [0] == 'S') {
-        CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 1);
-        SafeSetTitle (gbp->taxName, str);
-        CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 3);
-        SafeSetTitle (gbp->commonName, str);
-        Select (gbp->taxName);
-      } else if (str [0] == 'C') {
-        CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 3);
-        SafeSetTitle (gbp->taxName, str);
-        CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 1);
-        SafeSetTitle (gbp->commonName, str);
-        Select (gbp->taxName);
-      } else {
-        return FALSE;
-      }
+      SafeSetTitle (gbp->commonName, str);
+      Select (gbp->taxName);
       SetCodes (gbp, row, TRUE);
       return TRUE;
     }
@@ -999,6 +1022,9 @@ static void BioSourcePtrToGenBioPage (DialoG d, Pointer data)
       SetEnumPopup (gbp->genome, gbp->genomeAlist, (UIEnum) biop->genome);
       SetEnumPopup (gbp->origin, biosource_origin_alist, (UIEnum) biop->origin);
       SafeSetStatus (gbp->is_focus, biop->is_focus);
+      if (biop->is_focus) {
+        SafeEnable (gbp->is_focus);
+      }
       orp = biop->org;
       if (orp != NULL) {
         gbp->origTaxName = StringSave (orp->taxname);
@@ -1073,11 +1099,21 @@ static void BioSourcePtrToGenBioPage (DialoG d, Pointer data)
       if (gbp->simplecode != NULL) {
         if (GetEnumPopup (gbp->genome, gbp->genomeAlist, &genome)) {
           if (genome == 4 || genome == 5) {
-            SafeSetValue (gbp->simplecode, gcIdToIndex [onp->gcode]);
+            SafeSetValue (gbp->simplecode, gcIdToIndex [onp->mgcode]);
+          } else if (genome == GENOME_chloroplast ||
+                     genome == GENOME_chromoplast ||
+                     genome == GENOME_plastid ||
+                     genome == GENOME_cyanelle ||
+                     genome == GENOME_apicoplast ||
+                     genome == GENOME_leucoplast ||
+                     genome == GENOME_proplastid) {
+            SafeSetValue (gbp->simplecode, gcIdToIndex [11]);
           } else {
             SafeSetValue (gbp->simplecode, gcIdToIndex [onp->gcode]);
           }
         }
+        SafeSetValue (gbp->gcode, gcIdToIndex [onp->gcode]);
+        SafeSetValue (gbp->mgcode, gcIdToIndex [onp->mgcode]);
       } else {
         SafeSetValue (gbp->gcode, gcIdToIndex [onp->gcode]);
         SafeSetValue (gbp->mgcode, gcIdToIndex [onp->mgcode]);
@@ -1144,8 +1180,10 @@ static Pointer GenBioPageToBioSourcePtr (DialoG d)
             if (GetEnumPopup (gbp->genome, gbp->genomeAlist, &genome)) {
               if (genome == 4 || genome == 5) {
                 onp->mgcode = gcIndexToId [GetValue (gbp->simplecode)];
+                onp->gcode = gcIndexToId [GetValue (gbp->gcode)];
               } else {
                 onp->gcode = gcIndexToId [GetValue (gbp->simplecode)];
+                onp->mgcode = gcIndexToId [GetValue (gbp->mgcode)];
               }
             }
           } else {
@@ -1413,6 +1451,17 @@ extern DialoG CreateSimpleBioSourceDialog (GrouP h, CharPtr title)
     gbp->gbDiv = DialogText (x, "", 4, NULL);
     Hide (gbp->gbDiv);
 
+/* superimpose two hidden genetic code controls to save both in resulting biosource */
+
+    gbp->gcode = PopupList (x, TRUE, NULL);
+    PopulateGeneticCodePopup (gbp->gcode);
+    SetValue (gbp->gcode, 1);
+    gbp->mgcode = PopupList (x, TRUE, NULL);
+    PopulateGeneticCodePopup (gbp->mgcode);
+    SetValue (gbp->mgcode, 1);
+    Hide (gbp->gcode);
+    Hide (gbp->mgcode);
+
     SelectFont (systemFont);
   }
 
@@ -1611,11 +1660,19 @@ static Pointer SubsourceDialogToSubSourcePtr (DialoG d)
         if (tmp != NULL && sscanf (tmp, "%d", &val) == 1 && val != 0) {
           MemFree (tmp);
           tmp = ExtractTagListColumn ((CharPtr) vnp->data.ptrvalue, 1);
-          if ((val == 14 || val == 15) && StringHasNoText (tmp)) {
+          if ((val == SUBSRC_germline ||
+               val == SUBSRC_rearranged ||
+               val == SUBSRC_transgenic ||
+               val == SUBSRC_environmental_sample) &&
+              StringHasNoText (tmp)) {
             MemFree (tmp);
             tmp = StringSave ("");
           }
-          if (! StringHasNoText (tmp) || val == 14 || val == 15) {
+          if ((! StringHasNoText (tmp)) ||
+              val == SUBSRC_germline ||
+              val == SUBSRC_rearranged ||
+              val == SUBSRC_transgenic ||
+              val == SUBSRC_environmental_sample) {
             ssp = SubSourceNew ();
             if (ssplast == NULL) {
               head = ssp;
@@ -1764,7 +1821,8 @@ static void CleanupBioSourceDialog (GraphiC g, VoidPtr data)
 }
 
 static DialoG CreateBioSourceDialog (GrouP h, CharPtr title, GrouP PNTR pages,
-                                     BioSourcePtr biop, GenBioFormPtr gfp)
+                                     BioSourcePtr biop, GenBioFormPtr gfp,
+                                     Boolean diableFocusControl)
 
 {
   ButtoN         b;
@@ -1881,6 +1939,9 @@ static DialoG CreateBioSourceDialog (GrouP h, CharPtr title, GrouP PNTR pages,
     SetValue (gbp->origin, 0);
 
     gbp->is_focus = CheckBox (gbp->orgGrp [1], "Biological focus (if multiple source features)", NULL);
+    if (diableFocusControl) {
+      Disable (gbp->is_focus);
+    }
     AlignObjects (ALIGN_CENTER, (HANDLE) g, (HANDLE) f, NULL);
 
     Hide (gbp->orgGrp [1]);
@@ -2060,10 +2121,10 @@ static void SetBioSourceImportExportItems (GenBioFormPtr gfp)
     exportItm = FindFormMenuItem ((BaseFormPtr) gfp, VIB_MSG_EXPORT);
     switch (gfp->currentPage) {
       case ORGANISM_PAGE :
-        SafeSetTitle (importItm, "Import...");
-        SafeSetTitle (exportItm, "Export...");
-        SafeDisable (importItm);
-        SafeDisable (exportItm);
+        SafeSetTitle (importItm, "Import BioSource...");
+        SafeSetTitle (exportItm, "Export BioSource...");
+        SafeEnable (importItm);
+        SafeEnable (exportItm);
         break;
       case MODIFIERS_PAGE :
         SafeSetTitle (importItm, "Import...");
@@ -2128,14 +2189,33 @@ static void ChangeBioSourcePage (VoidPtr data, Int2 newval, Int2 oldval)
 static Boolean ImportBioSourceForm (ForM f, CharPtr filename)
 
 {
+  AsnIoPtr       aip;
+  BioSourcePtr   biop;
   GenBioFormPtr  gfp;
+  Char           path [PATH_MAX];
 
+  path [0] = '\0';
+  StringNCpy_0 (path, filename, sizeof (path));
   gfp = (GenBioFormPtr) GetObjectExtra (f);
   if (gfp != NULL) {
     switch (gfp->currentPage) {
+      case ORGANISM_PAGE :
+        if (path [0] != '\0' || GetInputFileName (path, sizeof (path), "", "TEXT")) {
+          aip = AsnIoOpen (path, "r");
+          if (aip != NULL) {
+            biop = BioSourceAsnRead (aip, NULL);
+            AsnIoClose (aip);
+            if (biop != NULL) {
+              PointerToDialog (gfp->data, (Pointer) biop);
+              biop = BioSourceFree (biop);
+              Update ();
+              return TRUE;
+            }
+          }
+        }
+        break;
       case LOCATION_PAGE :
         return ImportDialog (gfp->location, filename);
-        break;
       default :
         break;
     }
@@ -2146,14 +2226,41 @@ static Boolean ImportBioSourceForm (ForM f, CharPtr filename)
 static Boolean ExportBioSourceForm (ForM f, CharPtr filename)
 
 {
+  AsnIoPtr       aip;
+  BioSourcePtr   biop;
   GenBioFormPtr  gfp;
+  Char           path [PATH_MAX];
+#ifdef WIN_MAC
+  FILE           *fp;
+#endif
 
+  path [0] = '\0';
+  StringNCpy_0 (path, filename, sizeof (path));
   gfp = (GenBioFormPtr) GetObjectExtra (f);
   if (gfp != NULL) {
     switch (gfp->currentPage) {
+      case ORGANISM_PAGE :
+        if (path [0] != '\0' || GetOutputFileName (path, sizeof (path), NULL)) {
+#ifdef WIN_MAC
+          fp = FileOpen (path, "r");
+          if (fp != NULL) {
+            FileClose (fp);
+          } else {
+            FileCreate (path, "TEXT", "ttxt");
+          }
+#endif
+          aip = AsnIoOpen (path, "w");
+          if (aip != NULL) {
+            biop = DialogToPointer (gfp->data);
+            BioSourceAsnWrite (biop, aip, NULL);
+            AsnIoClose (aip);
+            biop = BioSourceFree (biop);
+            return TRUE;
+          }
+        }
+        break;
       case LOCATION_PAGE :
         return ExportDialog (gfp->location, filename);
-        break;
       default :
         break;
     }
@@ -2325,7 +2432,7 @@ extern ForM CreateBioSourceDescForm (Int2 left, Int2 top, Int2 width,
     if (sdp != NULL && sdp->choice == Seq_descr_source) {
       biop = sdp->data.ptrvalue;
     }
-    gfp->data = CreateBioSourceDialog (g, NULL, gfp->pages, biop, gfp);
+    gfp->data = CreateBioSourceDialog (g, NULL, gfp->pages, biop, gfp, FALSE);
 
     AlignObjects (ALIGN_CENTER, (HANDLE) gfp->foldertabs, (HANDLE) gfp->data, NULL);
 
@@ -2459,7 +2566,7 @@ extern ForM CreateBioSourceFeatForm (Int2 left, Int2 top, Int2 width,
     if (sfp != NULL && sfp->data.choice == SEQFEAT_BIOSRC) {
       biop = sfp->data.value.ptrvalue;
     }
-    gfp->data = CreateBioSourceDialog (h, NULL, gfp->pages, biop, gfp);
+    gfp->data = CreateBioSourceDialog (h, NULL, gfp->pages, biop, gfp, TRUE);
  
     s = HiddenGroup (h, -1, 0, NULL);
     CreateCommonFeatureGroup (s, (FeatureFormPtr) gfp, sfp, FALSE, TRUE);
@@ -2544,7 +2651,6 @@ extern Int2 LIBCALLBACK BioSourceGenFunc (Pointer data)
       break;
     default :
       return OM_MSG_RET_ERROR;
-      break;
   }
   omudp = ItemAlreadyHasEditor (ompcp->input_entityID, ompcp->input_itemID,
                                 ompcp->input_itemtype, ompcp->proc->procid);
@@ -2593,6 +2699,7 @@ extern Int2 LIBCALLBACK BioSourceGenFunc (Pointer data)
 	                           OMPROC_EDIT, gfp->userkey);
     if (omudp != NULL) {
       omudp->userdata.ptrvalue = (Pointer) gfp;
+      omudp->messagefunc = StdVibrantEditorMsgFunc;
     }
     SendMessageToForm (gfp->form, VIB_MSG_INIT);
     if (sdp != NULL) {
