@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   4/30/95
 *
-* $Revision: 6.23 $
+* $Revision: 6.35 $
 *
 * File Description: 
 *
@@ -45,15 +45,23 @@
 #ifndef _SEQVIEW_
 #define _SEQVIEW_
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include <dlogutil.h>
 #include <document.h>
 #include <viewer.h>
 #include <glbpic.h>
 
+
+#undef NLM_EXTERN
+#ifdef NLM_IMPORT
+#define NLM_EXTERN NLM_IMPORT
+#else
+#define NLM_EXTERN extern
+#endif
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* bioseqviewdata pointer is passed to callbacks to display views */
 
 typedef struct bioseqviewdata {
@@ -64,15 +72,27 @@ typedef struct bioseqviewdata {
   DoC             doc;
   TexT            text;
   PaneL           pnl;
+
+  PaneL           udv;
+  PaneL           ddv;
+
   GrouP           styleControlGrp;
   GrouP           scaleControlGrp;
   GrouP           findGeneGrp;
   GrouP           docTxtControlGrp;
+  GrouP           baseCtgControlGrp;
   GrouP           modeControlGrp;
+  GrouP           newGphControlGrp;
   GrouP           pnlParentGrp;
+
+  GrouP           udvParentGrp;
+  /*
+  GrouP           ddvParentGrp;
+  */
   PrompT          clickMe;
 
   Boolean         useScrollText;
+  Boolean         showContigJoin;
   Boolean         launchEditors;
   Boolean         launchSubviewers;
   Boolean         sendSelectMessages;
@@ -96,6 +116,11 @@ typedef struct bioseqviewdata {
 
   PopuP           ffModeCtrl;
 
+  PopuP           newGphStyle;
+  PopuP           newGphFilter;
+  PopuP           newGphScale;
+  PopuP           newGphLayout;
+
   ValNodePtr      slp_list;
   ValNodePtr      g_list;
   ValNodePtr      anp_node;
@@ -141,7 +166,7 @@ typedef struct bioseqpagedata {
   BioseqSelectProc       highlight;
   BioseqViewProc         toClipboard;
   BioseqViewProc         print;
-  BioseqExportProc       export;
+  BioseqExportProc       exprt;
   BioseqExportProc       togif;
   BioseqViewProc         resize;
   struct bioseqpagedata  PNTR next;
@@ -154,6 +179,10 @@ extern BioseqPageData sumPageData;
 extern BioseqPageData gphPageData;
 extern BioseqPageData alnPageData;
 extern BioseqPageData seqPageData;
+/*
+extern BioseqPageData udvPageData;
+extern BioseqPageData ddvPageData;
+*/
 
 extern BioseqPageData gbgnPageData;
 extern BioseqPageData gnbkPageData;
@@ -162,7 +191,9 @@ extern BioseqPageData ddbjPageData;
 extern BioseqPageData gnptPageData;
 
 extern BioseqPageData fstaPageData;
+extern BioseqPageData qualPageData;
 extern BioseqPageData asnPageData;
+extern BioseqPageData xmlPageData;
 extern BioseqPageData dskPageData;
 
 /*
@@ -189,6 +220,7 @@ typedef struct seqviewprocs {
   Boolean          sendSelectMessages;
   Boolean          highlightSelections;
   Boolean          forceSeparateViewer;
+  Boolean          keepSmartViewerVisible;
 
   Boolean          cleanupObjectPtr;
   WndActnProc      activateForm;
@@ -219,6 +251,8 @@ typedef struct seqviewprocs {
 
   IteM             alignWithChecked;  /* application sets to EntrezGlobalsPtr->alignWithChecked */
   Boolean          alignDefault;  /* application sets to EntrezGlobalsPtr->alignDefault */
+
+  Boolean          lockFarComponents;
 } SeqViewProcs, PNTR SeqViewProcsPtr;
 
 #define REGISTER_NEW_SEQENTRY_VIEW ObjMgrProcLoad(OMPROC_VIEW,"View Bioseq Report","Bioseq Report",OBJ_BIOSEQ,0,OBJ_BIOSEQ,0,NULL,NewSeqEntryViewGenFunc,PROC_PRIORITY_DEFAULT)
@@ -244,6 +278,7 @@ extern ForM MakeToolFormForBioseqView (BaseFormPtr bfp, GrpActnProc createToolBa
 extern void SetBioseqViewTarget (BaseFormPtr fp, CharPtr seqId);
 
 extern BioseqViewPtr GetBioseqViewPtrFromBaseFormPtr (BaseFormPtr fp);
+NLM_EXTERN void Nlm_LaunchWebPage (Char *url);
 
 extern Boolean BioseqViewCanSaveFasta (ForM f, Boolean nucs, Boolean prots, Boolean onlyTarget);
 extern Boolean ExportBioseqViewFasta (ForM f, CharPtr filename, Boolean nucs, Boolean prots, Boolean onlyTarget);
@@ -275,4 +310,11 @@ extern void EnableDisableLegendItem (BioseqViewPtr bvp, Boolean enable);
 }
 #endif
 
+#undef NLM_EXTERN
+#ifdef NLM_EXPORT
+#define NLM_EXTERN NLM_EXPORT
+#else
+#define NLM_EXTERN
+#endif
+
 #endif /* ndef _SEQVIEW_ */
