@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.10 $
+* $Revision: 6.29 $
 *
 * File Description: 
 *
@@ -53,76 +53,9 @@
 #include <objfeat.h>
 #include <objseq.h>
 #include <toasn3.h>
-
-static void CollectGeneFeatures (SeqEntryPtr sep, Pointer mydata, Int4 index, Int2 indent)
-
-{
-  BioseqPtr        bsp;
-  BioseqSetPtr     bssp;
-  SeqAnnotPtr      sap;
-  SeqFeatPtr       sfp;
-  ValNodePtr PNTR  vnpp;
-
-  if (sep == NULL || sep->data.ptrvalue == NULL || mydata == NULL) return;
-  vnpp = (ValNodePtr PNTR) mydata;
-  sap = NULL;
-  if (IS_Bioseq (sep)) {
-    bsp = (BioseqPtr) sep->data.ptrvalue;
-    sap = bsp->annot;
-  } else if (IS_Bioseq_set (sep)) {
-    bssp = (BioseqSetPtr) sep->data.ptrvalue;
-    sap = bssp->annot;
-  } else return;
-  while (sap != NULL) {
-    if (sap->type == 1 && sap->data != NULL) {
-      sfp = (SeqFeatPtr) sap->data;
-      while (sfp != NULL) {
-        if (sfp->data.choice == SEQFEAT_GENE) {
-          ValNodeAddPointer (vnpp, 0, (Pointer) sfp);
-        }
-        sfp = sfp->next;
-      }
-    }
-    sap = sap->next;
-  }
-}
-
-static void ExtendGeneWithinNucProt (SeqEntryPtr sep)
-
-{
-  BioseqSetPtr  bssp;
-  ValNodePtr    vnp;
-
-  if (sep == NULL) return;
-  if (IS_Bioseq_set (sep)) {
-    bssp = (BioseqSetPtr) sep->data.ptrvalue;
-    if (bssp == NULL) return;
-    if (bssp->_class == 7 || bssp->_class == 13 ||
-        bssp->_class == 14 || bssp->_class == 15) {
-      for (sep = bssp->seq_set; sep != NULL; sep = sep->next) {
-        ExtendGeneWithinNucProt (sep);
-      }
-      return;
-    }
-  }
-  vnp = NULL;
-  SeqEntryExplore (sep, (Pointer) (&vnp), CollectGeneFeatures);
-  if (vnp != NULL && vnp->next == NULL) {
-    SeqEntryExplore (sep, NULL, CorrectGeneFeatLocation);
-  }
-  ValNodeFree (vnp);
-}
-
-extern void ExtendGeneFeatIfOnMRNA (Uint2 entityID, SeqEntryPtr sep)
-
-{
-  if (entityID < 1 && sep == NULL) return;
-  if (entityID > 0 && sep == NULL) {
-    sep = GetTopSeqEntryForEntityID (entityID);
-  }
-  if (sep == NULL) return;
-  ExtendGeneWithinNucProt (sep);
-}
+#ifdef WIN_MOTIF
+#include <netscape.h>
+#endif
 
 typedef struct datepage {
   DIALOG_MESSAGE_BLOCK
@@ -989,8 +922,9 @@ static Boolean ShouldBeAGBQual (Int2 qual, Boolean allowProductGBQual)
   if (qual < 0) return FALSE;
   if (allowProductGBQual && qual == GBQUAL_product) return TRUE;
   if (qual == GBQUAL_citation || qual == GBQUAL_db_xref || qual == GBQUAL_evidence ||
-      qual == GBQUAL_gene || qual == GBQUAL_label || qual == GBQUAL_map ||
-      qual == GBQUAL_note || qual == GBQUAL_partial || qual == GBQUAL_product) {
+      qual == GBQUAL_exception || qual == GBQUAL_gene || qual == GBQUAL_label ||
+      qual == GBQUAL_map || qual == GBQUAL_note || qual == GBQUAL_partial ||
+      qual == GBQUAL_product || qual == GBQUAL_pseudo) {
     return FALSE;
   }
   if (Nlm_GetAppProperty ("SequinUseEMBLFeatures") == NULL) {
@@ -1036,6 +970,36 @@ static void GBQualPtrToFieldPage (DialoG d, Pointer data)
   }
 }
 
+extern void SeqFeatPtrToFieldPage (DialoG d, SeqFeatPtr sfp);
+extern void SeqFeatPtrToFieldPage (DialoG d, SeqFeatPtr sfp)
+
+{
+  /*
+  FieldPagePtr  fpf;
+  Int2          i;
+
+  fpf = (FieldPagePtr) GetObjectExtra (d);
+  if (fpf != NULL && sfp != NULL) {
+        for (i = 0; i < fpf->numfields; i++) {
+          if (StringICmp ("exception", fpf->fields [i]) == 0) {
+            if (fpf->values [i] != NULL) {
+              if (sfp->except_text == NULL || *(sfp->except_text) == '\0') {
+                SetTitle (fpf->values [i], "");
+              } else {
+                SetTitle (fpf->values [i], sfp->except_text);
+              }
+            }
+          } else if (StringICmp ("pseudo", fpf->fields [i]) == 0) {
+            if (fpf->boxes [i] != NULL && (! GetStatus (fpf->boxes [i]))) {
+              SetStatus (fpf->boxes [i], sfp->pseudo);
+            }
+          }
+        }
+  }
+  */
+
+}
+
 static Pointer FieldPageToGBQualPtr (DialoG d)
 
 {
@@ -1162,6 +1126,11 @@ extern DialoG CreateImportFields (GrouP h, CharPtr name, SeqFeatPtr sfp, Boolean
           }
         }
       }
+    } else if (sfp != NULL && sfp->data.choice == SEQFEAT_CDREGION) {
+      /*
+      seen [GBQUAL_exception] = LEGAL_FEATURE;
+      seen [GBQUAL_pseudo] = LEGAL_FEATURE;
+      */
     }
 
     while (gbq != NULL) {
@@ -1249,6 +1218,48 @@ extern DialoG CreateImportFields (GrouP h, CharPtr name, SeqFeatPtr sfp, Boolean
   return (DialoG) p;
 }
 
+static void ChangeCannedMessage (PopuP p)
+
+{
+  FeatureFormPtr  ffp;
+  Int2            val;
+
+  ffp = (FeatureFormPtr) GetObjectExtra (p);
+  if (ffp == NULL) return;
+  val = GetValue (p);
+  switch (val) {
+    case 1 :
+      if (Message (MSG_YN, "Clear the explanation field?") == ANS_YES) {
+        SetTitle (ffp->exceptText, "");
+        if (Message (MSG_YN, "Clear the exception flag?") == ANS_YES) {
+          SetStatus (ffp->exception, FALSE);
+        }
+      }
+      break;
+    case 2 :
+      SetTitle (ffp->exceptText, "RNA editing");
+      SetStatus (ffp->exception, TRUE);
+      break;
+    case 3 :
+      SetTitle (ffp->exceptText, "reasons given in citation");
+      SetStatus (ffp->exception, TRUE);
+      break;
+    case 4 :
+      SetTitle (ffp->exceptText, "ribosomal slippage");
+      SetStatus (ffp->exception, TRUE);
+      break;
+    case 5 :
+      SetTitle (ffp->exceptText, "trans splicing");
+      SetStatus (ffp->exception, TRUE);
+      break;
+    case 6 :
+      SetTitle (ffp->exceptText, "artificial frameshift");
+      SetStatus (ffp->exception, TRUE);
+    default :
+      break;
+  }
+}
+
 static CharPtr crossRefWarn =
 "A gene mapped by cross-reference does not extend the range\n\
 of the indicated gene feature.  Overlap is the usual case, and\n\
@@ -1280,23 +1291,27 @@ static CharPtr  commonNoCitFormTabs [] = {
 };
 
 extern void Nlm_LaunchGeneFeatEd (ButtoN b);
-extern GrouP CreateCommonFeatureGroup (GrouP h, FeatureFormPtr ffp,
-                                       SeqFeatPtr sfp, Boolean hasGeneControl,
-                                       Boolean hasCitationTab)
+extern GrouP CreateCommonFeatureGroupEx (GrouP h, FeatureFormPtr ffp,
+                                         SeqFeatPtr sfp, Boolean hasGeneControl,
+                                         Boolean hasCitationTab, Boolean hasGeneSuppress)
 
 {
   ButtoN   b;
   GrouP    c;
+  PopuP    canned;
   Boolean  cdsQuals;
   GrouP    f;
   GrouP    g;
   Boolean  hasQuals;
+  Boolean  indexerVersion;
   Char     just;
   GrouP    k;
   GrouP    m;
   GrouP    p;
   Int2     page;
+  ButtoN   pseudo = NULL;
   GrouP    q;
+  GrouP    r;
   GrouP    t;
   GrouP    v;
   GrouP    x;
@@ -1308,11 +1323,13 @@ extern GrouP CreateCommonFeatureGroup (GrouP h, FeatureFormPtr ffp,
     cdsQuals = FALSE;
     if (ffp->gbquals == NULL && sfp != NULL && sfp->qual != NULL) {
       hasQuals = TRUE;
+      /*
       if (GetAppProperty ("InternalNcbiSequin") != NULL) {
         if (sfp->data.choice == SEQFEAT_CDREGION) {
           cdsQuals = TRUE;
         }
       }
+      */
     }
     m = HiddenGroup (h, -1, 0, NULL);
     SetGroupSpacing (m, 10, 10);
@@ -1337,20 +1354,67 @@ extern GrouP CreateCommonFeatureGroup (GrouP h, FeatureFormPtr ffp,
 
     c = HiddenGroup (p, -1, 0, NULL);
     SetGroupSpacing (c, 10, 10);
-    f = HiddenGroup (c, 7, 0, NULL);
-    StaticPrompt (f, "Flags", 0, popupMenuHeight, programFont, 'l');
-    ffp->partial = CheckBox (f, "Partial", NULL);
-    ffp->exception = CheckBox (f, "Exception", NULL);
-    StaticPrompt (f, "Evidence", 0, popupMenuHeight, programFont, 'l');
-    ffp->evidence = PopupList (f, TRUE, NULL);
+    f = HiddenGroup (c, -1, 0, NULL);
+    r = HiddenGroup (f, 7, 0, NULL);
+    StaticPrompt (r, "Flags", 0, popupMenuHeight, programFont, 'l');
+    ffp->partial = CheckBox (r, "Partial", NULL);
+    indexerVersion = (Boolean) (GetAppProperty ("InternalNcbiSequin") != NULL);
+    if (! indexerVersion) {
+      Disable (ffp->partial);
+    }
+    if (ffp->pseudo == NULL) {
+      ffp->pseudo = CheckBox (r, "Pseudo", NULL);
+      pseudo = ffp->pseudo; /* allows pseudo control on earlier feature-specific page */
+    }
+    StaticPrompt (r, "Evidence", 0, popupMenuHeight, programFont, 'l');
+    ffp->evidence = PopupList (r, TRUE, NULL);
     PopupItem (ffp->evidence, " ");
     PopupItem (ffp->evidence, "Experimental");
     PopupItem (ffp->evidence, "Non-Experimental");
     AlignObjects (ALIGN_MIDDLE, (HANDLE) ffp->partial,
-                  (HANDLE) ffp->exception, (HANDLE) ffp->evidence, NULL);
+                  (HANDLE) ffp->evidence, (HANDLE) pseudo, NULL);
+    r = HiddenGroup (f, -3, 0, NULL);
+    ffp->exception = CheckBox (r, "Exception", NULL);
+    StaticPrompt (r, "Explanation", 0, dialogTextHeight, programFont, 'l');
+    ffp->exceptText = DialogText (r, "", 12, NULL);
+    AlignObjects (ALIGN_MIDDLE, (HANDLE) ffp->exception,
+                 (HANDLE) ffp->exceptText, NULL);
+    if (ffp->this_subtype == FEATDEF_CDS) {
+      StaticPrompt (r, "Standard explanation", 0, popupMenuHeight, programFont, 'l');
+      canned = PopupList (r, TRUE, ChangeCannedMessage);
+      SetObjectExtra (canned, (Pointer) ffp, NULL);
+      PopupItem (canned, " ");
+      PopupItem (canned, "RNA editing");
+      PopupItem (canned, "reasons given in citation");
+      PopupItem (canned, "ribosomal slippage");
+      PopupItem (canned, "trans splicing");
+      PopupItem (canned, "artificial frameshift");
+      if (sfp != NULL && sfp->excpt) {
+        if (StringICmp (sfp->except_text, "RNA editing") == 0) {
+          SetValue (canned, 2);
+        } else if (StringICmp (sfp->except_text, "reasons given in citation") == 0 ||
+                   StringICmp (sfp->except_text, "reasons cited in publication") == 0) {
+          SetValue (canned, 3);
+        } else if (StringICmp (sfp->except_text, "ribosomal slippage") == 0) {
+          SetValue (canned, 4);
+        } else if (StringICmp (sfp->except_text, "ribosome slippage") == 0) {
+          SetValue (canned, 4);
+        } else if (StringICmp (sfp->except_text, "trans splicing") == 0) {
+          SetValue (canned, 5);
+        } else if (StringICmp (sfp->except_text, "trans-splicing") == 0) {
+          SetValue (canned, 5);
+        } else if (StringICmp (sfp->except_text, "artificial frameshift") == 0) {
+          SetValue (canned, 6);
+        }
+      } else {
+        SetValue (canned, 1);
+      }
+    }
 
     if (cdsQuals) {
+      /*
       ffp->gbquals = CreateImportFields (c, NULL, sfp, FALSE);
+      */
     }
 
     g = NULL;
@@ -1402,6 +1466,17 @@ extern GrouP CreateCommonFeatureGroup (GrouP h, FeatureFormPtr ffp,
       ffp->editGeneBtn = PushButton (y, "Edit Gene Feature", Nlm_LaunchGeneFeatEd);
       SetObjectExtra (ffp->editGeneBtn, ffp, NULL);
       Hide (ffp->editGeneBtn);
+    } else if (hasGeneSuppress) {
+      k = HiddenGroup (c, 3, 0, NULL);
+      StaticPrompt (k, "Map by", 0, stdLineHeight, programFont, 'l');
+      ffp->useGeneXref = HiddenGroup (k, 3, 0, GeneXrefWarn);
+      SetObjectExtra (ffp->useGeneXref, ffp, NULL);
+      RadioButton (ffp->useGeneXref, "Overlap");
+      b = RadioButton (ffp->useGeneXref, "Cross-reference");
+      Disable (b);
+      RadioButton (ffp->useGeneXref, "Suppress");
+      SetValue (ffp->useGeneXref, 1);
+      y = HiddenGroup (c, 0, 0, NULL);
     }
     ffp->commonSubGrp [page] = c;
     page++;
@@ -1477,6 +1552,14 @@ extern GrouP CreateCommonFeatureGroup (GrouP h, FeatureFormPtr ffp,
   return c;
 }
 
+extern GrouP CreateCommonFeatureGroup (GrouP h, FeatureFormPtr ffp,
+                                       SeqFeatPtr sfp, Boolean hasGeneControl,
+                                       Boolean hasCitationTab)
+
+{
+  return CreateCommonFeatureGroupEx (h, ffp, sfp, hasGeneControl, hasCitationTab, FALSE);
+}
+
 static Boolean DlgutilFindBspItem (GatherContextPtr gcp)
 
 {
@@ -1535,7 +1618,7 @@ extern void SetNewFeatureDefaultInterval (FeatureFormPtr ffp)
   }
 }
 
-extern void FileToScrollText (TexT t, CharPtr path)
+extern Boolean FileToScrollText (TexT t, CharPtr path)
 
 {
   FILE     *fp;
@@ -1590,9 +1673,11 @@ extern void FileToScrollText (TexT t, CharPtr path)
           SetTitle (t, str);
         }
         MemFree (str);
+        return TRUE;
       }
     }
   }
+  return FALSE;
 }
 
 extern void ScrollTextToFile (TexT t, CharPtr path)
@@ -1878,6 +1963,9 @@ extern void LaunchGeneralTextViewer (CharPtr path, CharPtr title)
       if (tvpp->displayFont != NULL) {
         fnt = tvpp->displayFont;
       }
+      if (tvpp->activateForm != NULL) {
+        SetActivate (w, tvpp->activateForm);
+      }
     }
 
 #ifndef WIN_MAC
@@ -1900,7 +1988,9 @@ extern void LaunchGeneralTextViewer (CharPtr path, CharPtr title)
                               fnt, FALSE, NULL);
       SetObjectExtra (tfp->text, tfp, NULL);
       RealizeWindow (w);
-      FileToScrollText (tfp->text, path);
+      if (! FileToScrollText (tfp->text, path)) {
+        SetTitle (tfp->text, "(Text is too large to be displayed in this control.)");
+      }
     } else {
       tfp->doc = DocumentPanel (w, pixwidth, pixheight);
       SetObjectExtra (tfp->doc, tfp, NULL);
@@ -2403,3 +2493,56 @@ void GetRidOfEmptyFeatsDescStrings (Uint2 entityID, SeqEntryPtr sep)
   SeqEntryExplore (sep, NULL, GetRidOfEmptyFeatsDescCallback);
 }
 
+extern Int2 LIBCALLBACK StdVibrantEditorMsgFunc (OMMsgStructPtr ommsp)
+
+{
+  BaseFormPtr    bfp;
+  OMUserDataPtr  omudp;
+
+  omudp = (OMUserDataPtr)(ommsp->omuserdata);
+  if (omudp == NULL) return OM_MSG_RET_ERROR;
+  bfp = (BaseFormPtr) omudp->userdata.ptrvalue;
+  if (bfp == NULL) return OM_MSG_RET_ERROR;
+  switch (ommsp->message) {
+    case OM_MSG_DEL:
+      Remove (bfp->form);
+      Update ();
+      break;
+    default :
+      break;
+  }
+  return OM_MSG_RET_OK;
+}
+
+/* launch url section */
+
+NLM_EXTERN void LaunchEntrezURL (CharPtr database, Int4 uid, CharPtr format)
+
+{
+#ifdef WIN_MOTIF
+  NS_Window  window = NULL;
+#endif
+
+  Char  url [256];
+
+  if (uid < 1 || StringHasNoText (database) || StringHasNoText (format)) return;
+  sprintf (url,
+           "http://www.ncbi.nlm.nih.gov:80/entrez/query.fcgi?cmd=Retrieve&db=%s&list_uids=%ld&dopt=%s",
+            database, (long) uid, format);
+
+#ifdef WIN_MAC
+  Nlm_SendURLAppleEvent (url, "MOSS", NULL);
+#endif
+#ifdef WIN_MSWIN
+  if (! Nlm_MSWin_OpenDocument (url)) {
+    Message (MSG_POST, "Unable to launch browser");
+  }
+#endif
+#ifdef WIN_MOTIF
+  if (! NS_OpenURL (&window, url, NULL, TRUE)) {
+    Message (MSG_POST, "Unable to launch browser");
+  }
+  NS_WindowFree (window);
+#endif
+}
+
