@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.7 $
+* $Revision: 6.16 $
 *
 * File Description: 
 *
@@ -45,10 +45,6 @@
 #ifndef _DLOGUTIL_
 #define _DLOGUTIL_
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include <vibrant.h>
 #include <sqnutils.h>
 #include <prtutil.h>
@@ -57,6 +53,11 @@ extern "C" {
 #include <objfdef.h>
 #include <sequtil.h>
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #define CHANGE_VIEW_NOTABS        0
 #define CHANGE_VIEW_FOLDERTABS    1
 #define CHANGE_VIEW_TEXTTABS      2
@@ -101,11 +102,13 @@ extern ValNodePtr AddStringToValNodeChain (ValNodePtr head, CharPtr str, Uint1 c
   TexT            geneSymbol;       \
   TexT            geneDesc;         \
   ButtoN          editGeneBtn;      \
+  TexT            protXrefName;     \
   DialoG          location;         \
   DialoG          product;          \
   DialoG          featcits;         \
   DialoG          dbxrefs;          \
-  DialoG          gbquals;
+  DialoG          gbquals;          \
+  DialoG          usrobjext;
 
 typedef struct descform {
   DESCRIPTOR_FORM_BLOCK
@@ -131,6 +134,7 @@ extern void StdFeatFormAcceptButtonProc (ButtoN b);
 extern void ExtendGeneFeatIfOnMRNA (Uint2 entityID, SeqEntryPtr sep);
 
 extern OMUserDataPtr ItemAlreadyHasEditor (Uint2 entityID, Uint2 itemID, Uint2 itemtype, Uint2 procid);
+extern Int2 LIBCALLBACK StdVibrantEditorMsgFunc (OMMsgStructPtr ommsp);
 
 /*****************************************************************************
 *
@@ -159,6 +163,9 @@ extern DialoG CreateExtProceedingsDialog (GrouP prnt, CharPtr title,
 extern GrouP CreateCommonFeatureGroup (GrouP h, FeatureFormPtr ffp,
                                        SeqFeatPtr sfp, Boolean hasGeneControl,
                                        Boolean hasCitationTab);
+extern GrouP CreateCommonFeatureGroupEx (GrouP h, FeatureFormPtr ffp,
+                                         SeqFeatPtr sfp, Boolean hasGeneControl,
+                                         Boolean hasCitationTab, Boolean hasGeneSuppress);
 extern void PopulateGenePopup (FeatureFormPtr ffp);
 extern void SeqFeatPtrToCommon (FeatureFormPtr ffp, SeqFeatPtr sfp);
 extern void SetNewFeatureDefaultInterval (FeatureFormPtr ffp);
@@ -213,7 +220,7 @@ typedef struct textviewprocs {
 
 extern void LaunchGeneralTextViewer (CharPtr path, CharPtr title);
 
-extern void FileToScrollText (TexT t, CharPtr path);
+extern Boolean FileToScrollText (TexT t, CharPtr path);
 extern void ScrollTextToFile (TexT t, CharPtr path);
 
 extern void FileToClipboard (CharPtr path);
@@ -265,6 +272,25 @@ extern CharPtr SaveStringFromTextAndStripNewlines (TexT t);
 
 extern void GetRidOfEmptyFeatsDescStrings (Uint2 entityID, SeqEntryPtr sep);
 
+/*****************************************************************************
+*
+*   LaunchEntrezURL constructs a web-Entrez URL query.  Parameter choices are:
+*
+*     Database      Report Formats
+*
+*      PubMed        DocSum, Brief, Abstract, Citation, MEDLINE, ASN.1, ExternalLink
+*      Nucleotide    DocSum, Brief, GenBank, ASN.1, FASTA, ExternalLink
+*      Protein       DocSum, Brief, GenPept, ASN.1, FASTA, ExternalLink
+*      Genome        DocSum, Brief, ASN.1, ExternalLink
+*      Popset        DocSum, Brief, ASN.1, FASTA, ExternalLink
+*      Structure     DocSum, Brief
+*      OMIM          DocSum, Text, Synopsis, Variants, MiniMIM, ASN.1, ExternalLink
+*      Taxonomy      DocSum, Brief, TxInfo, TxTree, ExternalLink
+*
+*****************************************************************************/
+
+NLM_EXTERN void LaunchEntrezURL (CharPtr database, Int4 uid, CharPtr format);
+
 
 #ifdef __cplusplus
 }
