@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   2/5/97
 *
-* $Revision: 6.14 $
+* $Revision: 6.39 $
 *
 * File Description: 
 *
@@ -50,11 +50,18 @@
 static ParData ffParFmt = {FALSE, FALSE, FALSE, FALSE, TRUE, 0, 0};
 static ColData ffColFmt = {0, 0, 80, 0, NULL, 'l', FALSE, FALSE, FALSE, FALSE, TRUE};
 
+typedef struct docdescrstruct {
+  Uint2  entityID, itemID, itemtype;
+  Int2   docitem;
+} DocDescrStruct, PNTR DocDescrPtr;
+
 typedef struct flatstruct {
-  Asn2ffJobPtr     ajp;
-  FFPrintArrayPtr  pap;
-  SeqEntryPtr      sep;
-  Int4             pap_size;
+  Asn2ffJobPtr      ajp;
+  FFPrintArrayPtr   pap;
+  SeqEntryPtr       sep;
+  Int4              pap_size;
+  Int4              numdescr;
+  DocDescrPtr       descr;
 } FlatStruct, PNTR FlatStructPtr;
 
 static Boolean GetIDsFromDoc (DoC d, Int2 item, Uint2Ptr entityPtr,
@@ -296,6 +303,7 @@ static Boolean MatchItemInFlatFileProc (GatherContextPtr gcp)
   msp = (MatchStrucPtr) gcp->userdata;
   if (msp == NULL) return FALSE;
   sfp = (SeqFeatPtr) gcp->thisitem;
+  /*
   if (msp->slashgene) {
     itemID = GetBestGeneOrProteinFeature (gcp->entityID, NULL, sfp->location, SEQFEAT_GENE);
     if (itemID != 0) {
@@ -310,6 +318,7 @@ static Boolean MatchItemInFlatFileProc (GatherContextPtr gcp)
   if (msp->found) {
     return TRUE;
   }
+  */
   if (sfp->data.choice == SEQFEAT_CDREGION) {
     itemID = GetBestGeneOrProteinFeature (gcp->entityID, NULL, sfp->product, SEQFEAT_PROT);
     if (itemID != 0) {
@@ -320,6 +329,7 @@ static Boolean MatchItemInFlatFileProc (GatherContextPtr gcp)
       }
     }
   }
+  /*
   if (! msp->slashgene) {
     itemID = GetBestGeneOrProteinFeature (gcp->entityID, NULL, sfp->location, SEQFEAT_GENE);
     if (itemID != 0) {
@@ -330,6 +340,7 @@ static Boolean MatchItemInFlatFileProc (GatherContextPtr gcp)
       }
     }
   }
+  */
   return FALSE;
 }
 
@@ -391,7 +402,7 @@ static void ReleaseIcon (DoC d, PoinT pt)
           WatchCursor ();
           Update ();
           editItemID = itemID;
-          if (itemtype == OBJ_SEQFEAT) {
+          if (itemtype == OBJ_SEQFEAT && GetAppProperty ("InternalNcbiSequin") != NULL) {
             str = GetDocText (d, item, row, 1);
             TrimSpacesAroundString (str);
             src = str;
@@ -460,6 +471,7 @@ static void DocFreeFlat (DoC d, VoidPtr data)
     fsp = (FlatStructPtr) data;
     asn2ff_cleanup (fsp->ajp);
     MemFree (fsp->ajp);
+    MemFree (fsp->descr);
     MemFree (fsp);
   }
 }
@@ -473,17 +485,26 @@ static CharPtr FFPrintFunc (DoC d, Int2 index, Pointer data)
   DescrStructPtr   descr;
   FlatStructPtr    fsp;
   ErrSev           level;
+  SeqEntryPtr      oldsep = NULL;
   OMUserDataPtr    omudp;
   FFPrintArrayPtr  pap;
   CharPtr          str = NULL;
+  SeqEntryPtr      topsep;
 
   bvp = (BioseqViewPtr) GetObjectExtra (d);
   fsp = (FlatStructPtr) data;
   if (bvp == NULL || bvp->bsp == NULL || fsp == NULL) return StringSave ("?\n");
   level = ErrSetMessageLevel (SEV_MAX);
   bsp = BioseqLock (bvp->bsp);
-  str = FFPrint (fsp->pap, (Int4) (index - 1), fsp->pap_size);
   bfp = (BaseFormPtr) GetObjectExtra (bvp->form);
+  if (bfp != NULL) {
+    topsep = GetTopSeqEntryForEntityID (bfp->input_entityID);
+    oldsep = SeqEntrySetScope (topsep);
+  }
+  str = FFPrint (fsp->pap, (Int4) (index - 1), fsp->pap_size);
+  if (oldsep != NULL) {
+    SeqEntrySetScope (oldsep);
+  }
   if (bfp != NULL) {
     pap = fsp->pap;
     if (pap != NULL) {
@@ -518,16 +539,64 @@ static Boolean DeltaLitOnly (BioseqPtr bsp)
   return TRUE;
 }
 
-static Boolean PopulateFF (DoC d, SeqEntryPtr sep, BioseqPtr bsp, Uint1 format, Uint1 mode)
+static int LIBCALLBACK SortDescrProc (VoidPtr vp1, VoidPtr vp2)
 
 {
-  Asn2ffJobPtr   ajp;
-  BioseqViewPtr  bvp;
-  FonT           fnt;
-  FlatStructPtr  fsp;
-  Int4           index;
-  ErrSev         level;
-  Boolean        rsult;
+  DocDescrPtr  descr1;
+  DocDescrPtr  descr2;
+
+  descr1 = (DocDescrPtr) vp1;
+  descr2 = (DocDescrPtr) vp2;
+  if (descr1 == NULL || descr2 == NULL) return 0;
+
+  if (descr1->entityID > descr2->entityID) return 1;
+  if (descr1->entityID < descr2->entityID) return -1;
+
+  if (descr1->itemtype > descr2->itemtype) return 1;
+  if (descr1->itemtype < descr2->itemtype) return -1;
+
+  if (descr1->itemID > descr2->itemID) return 1;
+  if (descr1->itemID < descr2->itemID) return -1;
+
+  if (descr1->docitem > descr2->docitem) return 1;
+  if (descr1->docitem < descr2->docitem) return -1;
+
+  return 0;
+}
+
+static void LookForGEDseqID (BioseqPtr bsp, Pointer userdata)
+
+{
+  BoolPtr   isGEDPtr;
+  SeqIdPtr  sip;
+
+  isGEDPtr = (BoolPtr) userdata;
+  for (sip = bsp->id; sip != NULL; sip = sip->next) {
+    switch (sip->choice) {
+      case SEQID_GENBANK :
+      case SEQID_EMBL :
+      case SEQID_DDBJ :
+        *isGEDPtr = TRUE;
+        return;
+      default :
+        break;
+    }
+  }
+}
+
+static Boolean PopulateFF (DoC d, SeqEntryPtr sep, BioseqPtr bsp, Uint1 format, Uint1 mode, Boolean show_gene, Boolean showContigJoin)
+
+{
+  Asn2ffJobPtr    ajp;
+  BioseqViewPtr   bvp;
+  DescrStructPtr  descr;
+  DocDescrPtr     doscr;
+  DocDescrPtr     doscrp;
+  FonT            fnt;
+  FlatStructPtr   fsp;
+  Int4            index;
+  ErrSev          level;
+  Boolean         rsult;
 
   rsult = FALSE;
   if (d != NULL && sep != NULL && bsp != NULL && spop != NULL) {
@@ -547,29 +616,56 @@ static Boolean PopulateFF (DoC d, SeqEntryPtr sep, BioseqPtr bsp, Uint1 format,
       ajp->sep = sep;
       ajp->mode = mode;
       ajp->format = format;
+      ajp->show_version = TRUE;
       ajp->gb_style = TRUE;
       ajp->show_seq = TRUE;
       ajp->show_gi = TRUE;
       ajp->error_msgs = FALSE;
       ajp->non_strict = TRUE;
       ajp->Spop = spop;
-      ajp->show_gene = TRUE;
-      if (IsAGenomeRecord (sep) ||
+      ajp->show_gene = show_gene;
+      if (/* IsAGenomeRecord (sep) || */
           IsSegmentedBioseqWithoutParts (sep)) {
         ajp->only_one = TRUE;
         ajp->genome_view = TRUE;
+        ajp->contig_view = (Boolean) (! showContigJoin);
       } else if (IsADeltaBioseq (sep) && (! DeltaLitOnly (bsp))) {
         ajp->only_one = TRUE;
         ajp->genome_view = TRUE;
+        ajp->contig_view = (Boolean) (! showContigJoin);
+      }
+      if (GetAppProperty ("InternalNcbiSequin") != NULL) {
+        ajp->bankit = TRUE;
       }
       fsp->pap_size = asn2ff_setup (ajp, &(fsp->pap));
       if (fsp->pap_size > 0) {
         asn2ff_set_output (NULL, "\n");
+        fsp->numdescr = 0;
         for (index = 0; index < fsp->pap_size; index++) {
           AppendItem (d, FFPrintFunc, (Pointer) fsp, FALSE,
                       fsp->pap [index].estimate, &ffParFmt,
                       &ffColFmt, fnt);
+          if (fsp->pap [index].descr != NULL) {
+            (fsp->numdescr)++;
+          }
         }
+        doscrp = (DocDescrPtr) MemNew (sizeof (DocDescrStruct) * (size_t) (fsp->numdescr + 1));
+        fsp->descr = doscrp;
+        fsp->numdescr = 0;
+        if (doscrp != NULL) {
+          for (index = 0; index < fsp->pap_size; index++) {
+            descr = fsp->pap [index].descr;
+            if (descr != NULL) {
+              doscr = &(doscrp [fsp->numdescr]);
+              doscr->entityID = descr->entityID;
+              doscr->itemID = descr->itemID;
+              doscr->itemtype = descr->itemtype;
+              doscr->docitem = index + 1;
+              (fsp->numdescr)++;
+            }
+          }
+        }
+        HeapSort (doscrp, (size_t) fsp->numdescr, sizeof (DocDescrStruct), SortDescrProc);
         rsult = TRUE;
       }
       ErrSetMessageLevel (level);
@@ -578,21 +674,27 @@ static Boolean PopulateFF (DoC d, SeqEntryPtr sep, BioseqPtr bsp, Uint1 format,
   return rsult;
 }
 
-static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format)
+static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format, Boolean show_gene)
 
 {
-  BioseqPtr    bsp;
-  DoC          doc;
-  Uint2        entityID;
-  FILE         *fp;
-  Int2         into;
-  Int2         item;
-  Uint1        mode;
-  Char         path [PATH_MAX];
-  BaR          sb;
-  SeqEntryPtr  sep;
-  Int4         startsAt;
-  TexT         txt;
+  Asn2ffJobPtr  ajp;
+  BioseqPtr     bsp;
+  DoC           doc;
+  Uint2         entityID;
+  FILE          *fp;
+  Int2          into;
+  Boolean       isGenBankEMBLorDDBJ;
+  Boolean       showContigJoin;
+  Int2          item;
+  ErrSev        level;
+  Uint1         mode;
+  SeqEntryPtr   oldsep;
+  Char          path [PATH_MAX];
+  BaR           sb = NULL;
+  SeqEntryPtr   sep;
+  Int4          startsAt;
+  SeqEntryPtr   topsep;
+  TexT          txt;
 
   if (bvp == NULL) return;
   if (bvp->hasTargetControl) {
@@ -633,39 +735,79 @@ static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format)
     }
   }
   sep = SeqMgrGetSeqEntryForData (bsp);
+  entityID = ObjMgrGetEntityIDForChoice (sep);
   if (bvp->hasTargetControl) {
     if (bvp->viewWholeEntity) {
-      entityID = ObjMgrGetEntityIDForChoice (sep);
       sep = GetTopSeqEntryForEntityID (entityID);
     } else if (ISA_na (bsp->mol) && bsp->repr == Seq_repr_seg) {
-      entityID = ObjMgrGetEntityIDForChoice (sep);
+      sep = GetBestTopParentForData (entityID, bsp);
+    } else if (ISA_aa (bsp->mol) && bsp->repr == Seq_repr_seg) {
       sep = GetBestTopParentForData (entityID, bsp);
     }
   } else {
     if (ISA_na (bsp->mol) || bsp->repr == Seq_repr_seg) {
-      entityID = ObjMgrGetEntityIDForChoice (sep);
       sep = GetBestTopParentForData (entityID, bsp);
     }
   }
   if (sep == NULL) return;
 
+  topsep = GetTopSeqEntryForEntityID (entityID);
+  oldsep = SeqEntrySetScope (topsep);
+
   WatchCursor ();
   ffColFmt.pixWidth = screenRect.right - screenRect.left;
   ffColFmt.pixInset = 8;
+  isGenBankEMBLorDDBJ = FALSE;
+  VisitBioseqsInSep (topsep, (Pointer) &isGenBankEMBLorDDBJ, LookForGEDseqID);
+  /* now using control instead of seqid type */
+  showContigJoin = bvp->showContigJoin;
   if (bvp->useScrollText) {
-    TmpNam (path);
-    fp = FileOpen (path, "w");
-    if (fp != NULL) {
-      if (SeqEntryToFlat (sep, fp, format, mode)) {
-        FileClose (fp);
-        FileToScrollText (txt, path);
-      } else {
-        FileClose (fp);
+    ajp = MemNew (sizeof (Asn2ffJob));
+    if (ajp != NULL) {
+      TmpNam (path);
+      fp = FileOpen (path, "w");
+      if (fp != NULL) {
+        level = ErrSetMessageLevel (SEV_MAX);
+        ajp->sep = sep;
+        ajp->mode = mode;
+        ajp->format = format;
+        ajp->show_version = TRUE;
+        ajp->gb_style = TRUE;
+        ajp->show_seq = TRUE;
+        ajp->show_gi = TRUE;
+        ajp->error_msgs = FALSE;
+        ajp->non_strict = TRUE;
+        ajp->Spop = spop;
+        ajp->show_gene = show_gene;
+        if (/* IsAGenomeRecord (sep) || */
+            IsSegmentedBioseqWithoutParts (sep)) {
+          ajp->only_one = TRUE;
+          ajp->genome_view = TRUE;
+          ajp->contig_view = (Boolean) (! showContigJoin);
+        } else if (IsADeltaBioseq (sep) && (! DeltaLitOnly (bsp))) {
+          ajp->only_one = TRUE;
+          ajp->genome_view = TRUE;
+          ajp->contig_view = (Boolean) (! showContigJoin);
+        }
+        if (GetAppProperty ("InternalNcbiSequin") != NULL) {
+          ajp->bankit = TRUE;
+        }
+        ajp->fp = fp;
+        if (SeqEntryToFlatAjp (ajp, sep, fp, format, mode)) {
+          FileClose (fp);
+          if (! FileToScrollText (txt, path)) {
+            SetTitle (txt, "(Text is too large to be displayed in this control.)");
+          }
+        } else {
+          FileClose (fp);
+        }
+        ErrSetMessageLevel (level);
       }
+      FileRemove (path);
     }
-    FileRemove (path);
+    MemFree (ajp);
   } else {
-    PopulateFF (doc, sep, bsp, format, mode);
+    PopulateFF (doc, sep, bsp, format, mode, show_gene, showContigJoin);
     SetDocShade (doc, DrawIcon, NULL, NULL, NULL);
     SetDocProcs (doc, ClickIcon, NULL, ReleaseIcon, NULL);
     SetDocCache (doc, StdPutDocCache, StdGetDocCache, StdResetDocCache);
@@ -677,6 +819,9 @@ static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format)
     CorrectBarValue (sb, startsAt + into);
     UpdateDocument (doc, 0, 0);
   }
+
+  SeqEntrySetScope (oldsep);
+
   ArrowCursor ();
   Update ();
 }
@@ -684,7 +829,7 @@ static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format)
 static void PopulateGenBank (BioseqViewPtr bvp)
 
 {
-  PopulateFlatFile (bvp, GENBANK_FMT);
+  PopulateFlatFile (bvp, GENBANK_FMT, TRUE);
 }
 
 static void PopulateEMBL (BioseqViewPtr bvp)
@@ -692,22 +837,22 @@ static void PopulateEMBL (BioseqViewPtr bvp)
 {
   if (bvp == NULL) return;
   if (bvp->hasTargetControl) {
-    PopulateFlatFile (bvp, PSEUDOEMBL_FMT);
+    PopulateFlatFile (bvp, PSEUDOEMBL_FMT, TRUE);
   } else {
-    PopulateFlatFile (bvp, EMBL_FMT);
+    PopulateFlatFile (bvp, EMBL_FMT, TRUE);
   }
 }
 
 static void PopulateDDBJ (BioseqViewPtr bvp)
 
 {
-  PopulateFlatFile (bvp, GENBANK_FMT);
+  PopulateFlatFile (bvp, GENBANK_FMT, FALSE);
 }
 
 static void PopulateGenPept (BioseqViewPtr bvp)
 
 {
-  PopulateFlatFile (bvp, GENPEPT_FMT);
+  PopulateFlatFile (bvp, GENPEPT_FMT, TRUE);
 }
 
 static void PopulateFasta (BioseqViewPtr bvp)
@@ -725,7 +870,7 @@ static void PopulateFasta (BioseqViewPtr bvp)
   Int2         into;
   Int2         item;
   Char         path [PATH_MAX];
-  BaR          sb;
+  BaR          sb = NULL;
   SeqEntryPtr  sep;
   Int4         startsAt;
   TexT         txt;
@@ -807,7 +952,9 @@ static void PopulateFasta (BioseqViewPtr bvp)
     if (fastaOK) {
       FileClose (fp);
       if (bvp->useScrollText) {
-        FileToScrollText (txt, path);
+        if (! FileToScrollText (txt, path)) {
+          SetTitle (txt, "(Text is too large to be displayed in this control.)");
+        }
       } else {
         DisplayFancy (doc, path, &ffParFmt, &ffColFmt, fnt, 4);
         SetDocCache (doc, StdPutDocCache, StdGetDocCache, StdResetDocCache);
@@ -828,10 +975,44 @@ static void PopulateFasta (BioseqViewPtr bvp)
   Update ();
 }
 
-static void PopulateAsn (BioseqViewPtr bvp)
+static void PrintQualProc (CharPtr buf, Uint4 buflen, Pointer userdata)
+
+{
+  FILE  *fp;
+
+  fp = (FILE*) userdata;
+  fprintf (fp, "%s", buf);
+}
+
+static void PrintQualScoresProc (SeqEntryPtr sep, Pointer mydata, Int4 index, Int2 indent)
+
+{
+  BioseqPtr  bsp;
+  FILE       *fp;
+
+  if (! IS_Bioseq (sep)) return;
+  bsp = (BioseqPtr) sep->data.ptrvalue;
+  if (bsp == NULL) return;
+  fp = (FILE*) mydata;
+  PrintQualityScoresToBuffer (bsp, FALSE, fp, PrintQualProc);
+}
+
+static void PrintFarQualScoresProc (SeqEntryPtr sep, Pointer mydata, Int4 index, Int2 indent)
+
+{
+  BioseqPtr  bsp;
+  FILE       *fp;
+
+  if (! IS_Bioseq (sep)) return;
+  bsp = (BioseqPtr) sep->data.ptrvalue;
+  if (bsp == NULL) return;
+  fp = (FILE*) mydata;
+  PrintQualityScoresForContig (bsp, FALSE, fp);
+}
+
+static void PopulateQuality (BioseqViewPtr bvp)
 
 {
-  AsnIoPtr     aipout;
   BioseqPtr    bsp;
   DoC          doc;
   Uint2        entityID;
@@ -840,7 +1021,7 @@ static void PopulateAsn (BioseqViewPtr bvp)
   Int2         into;
   Int2         item;
   Char         path [PATH_MAX];
-  BaR          sb;
+  BaR          sb = NULL;
   SeqEntryPtr  sep;
   Int4         startsAt;
   TexT         txt;
@@ -889,26 +1070,28 @@ static void PopulateAsn (BioseqViewPtr bvp)
     if (bvp != NULL && bvp->displayFont != NULL) {
       fnt = bvp->displayFont;
     }
-    aipout = AsnIoNew (ASNIO_TEXT_OUT, fp, NULL, NULL, NULL);
-    if (aipout != NULL) {
-      if (SeqEntryAsnWrite (sep, aipout, NULL)) {
-        AsnIoClose (aipout);
-        if (bvp->useScrollText) {
-          FileToScrollText (txt, path);
-        } else {
-          DisplayFancy (doc, path, &ffParFmt, &ffColFmt, fnt, 4);
-          SetDocCache (doc, StdPutDocCache, StdGetDocCache, StdResetDocCache);
-          SetDocAutoAdjust (doc, FALSE);
-          ForceFormat (doc, item);
-          SetDocAutoAdjust (doc, TRUE);
-          AdjustDocScroll (doc);
-          GetItemParams4 (doc, item, &startsAt, NULL, NULL, NULL, NULL);
-          CorrectBarValue (sb, startsAt + into);
-          UpdateDocument (doc, 0, 0);
-        }
-      } else {
-        AsnIoClose (aipout);
+    if (VisitGraphsInSep (sep, NULL, NULL) > 0) {
+      SeqEntryExplore (sep, (Pointer) fp, PrintQualScoresProc);
+    } else if (bsp->repr == Seq_repr_delta) {
+      SeqEntryExplore (sep, (Pointer) fp, PrintFarQualScoresProc);
+    } else {
+      SeqEntryExplore (sep, (Pointer) fp, PrintQualScoresProc);
+    }
+    FileClose (fp);
+    if (bvp->useScrollText) {
+      if (! FileToScrollText (txt, path)) {
+        SetTitle (txt, "(Text is too large to be displayed in this control.)");
       }
+    } else {
+      DisplayFancy (doc, path, &ffParFmt, &ffColFmt, fnt, 4);
+      SetDocCache (doc, StdPutDocCache, StdGetDocCache, StdResetDocCache);
+      SetDocAutoAdjust (doc, FALSE);
+      ForceFormat (doc, item);
+      SetDocAutoAdjust (doc, TRUE);
+      AdjustDocScroll (doc);
+      GetItemParams4 (doc, item, &startsAt, NULL, NULL, NULL, NULL);
+      CorrectBarValue (sb, startsAt + into);
+      UpdateDocument (doc, 0, 0);
     }
   }
   FileRemove (path);
@@ -916,6 +1099,104 @@ static void PopulateAsn (BioseqViewPtr bvp)
   Update ();
 }
 
+static void PopulateAsnOrXML (BioseqViewPtr bvp, CharPtr mode)
+
+{
+  AsnIoPtr     aipout;
+  BioseqPtr    bsp;
+  DoC          doc;
+  Uint2        entityID;
+  FonT         fnt;
+  Int2         into;
+  Int2         item;
+  Char         path [PATH_MAX];
+  BaR          sb = NULL;
+  SeqEntryPtr  sep;
+  Int4         startsAt;
+  TexT         txt;
+
+  if (bvp == NULL) return;
+  doc = NULL;
+  txt = NULL;
+  bsp = bvp->bsp;
+  if (bvp->useScrollText) {
+    txt = bvp->text;
+    Reset (txt);
+    Update ();
+  } else {
+    doc = bvp->doc;
+    GetScrlParams (doc, NULL, &item, &into);
+    sb = GetSlateVScrollBar ((SlatE) doc);
+    Reset (doc);
+    SetDocShade (doc, NULL, NULL, NULL, NULL);
+    SetDocProcs (doc, NULL, NULL, NULL, NULL);
+    SetDocCache (doc, NULL, NULL, NULL);
+    Update ();
+    SetDocAutoAdjust (doc, FALSE);
+  }
+  if (bsp == NULL) return;
+  sep = SeqMgrGetSeqEntryForData (bsp);
+  if (bvp->hasTargetControl) {
+    if (bvp->viewWholeEntity) {
+      entityID = ObjMgrGetEntityIDForChoice (sep);
+      sep = GetTopSeqEntryForEntityID (entityID);
+    }
+  } else {
+    if (ISA_na (bsp->mol) || bsp->repr == Seq_repr_seg) {
+      entityID = ObjMgrGetEntityIDForChoice (sep);
+      sep = GetBestTopParentForData (entityID, bsp);
+    }
+  }
+  if (sep == NULL) return;
+
+  WatchCursor ();
+  ffColFmt.pixWidth = screenRect.right - screenRect.left;
+  ffColFmt.pixInset = 8;
+  TmpNam (path);
+  aipout = AsnIoOpen (path, mode);
+  if (aipout != NULL) {
+    fnt = programFont;
+    if (bvp != NULL && bvp->displayFont != NULL) {
+      fnt = bvp->displayFont;
+    }
+    if (SeqEntryAsnWrite (sep, aipout, NULL)) {
+      AsnIoClose (aipout);
+      if (bvp->useScrollText) {
+        if (! FileToScrollText (txt, path)) {
+          SetTitle (txt, "(Text is too large to be displayed in this control.)");
+        }
+      } else {
+        DisplayFancy (doc, path, &ffParFmt, &ffColFmt, fnt, 4);
+        SetDocCache (doc, StdPutDocCache, StdGetDocCache, StdResetDocCache);
+        SetDocAutoAdjust (doc, FALSE);
+        ForceFormat (doc, item);
+        SetDocAutoAdjust (doc, TRUE);
+        AdjustDocScroll (doc);
+        GetItemParams4 (doc, item, &startsAt, NULL, NULL, NULL, NULL);
+        CorrectBarValue (sb, startsAt + into);
+        UpdateDocument (doc, 0, 0);
+      }
+    } else {
+      AsnIoClose (aipout);
+    }
+  }
+  FileRemove (path);
+  ArrowCursor ();
+  Update ();
+}
+
+static void PopulateXML (BioseqViewPtr bvp)
+
+{
+  PopulateAsnOrXML (bvp, "wx");
+}
+
+static void PopulateAsn (BioseqViewPtr bvp)
+
+{
+  PopulateAsnOrXML (bvp, "w");
+}
+
 static void ShowFlatFile (BioseqViewPtr bvp, Boolean show)
 
 {
@@ -926,6 +1207,7 @@ static void ShowFlatFile (BioseqViewPtr bvp, Boolean show)
     } else {
       SafeShow (bvp->doc);
     }
+    SafeShow (bvp->baseCtgControlGrp);
     SafeShow (bvp->docTxtControlGrp);
     SafeShow (bvp->modeControlGrp);
     SafeShow (bvp->clickMe);
@@ -942,7 +1224,9 @@ static void ShowFlatFile (BioseqViewPtr bvp, Boolean show)
     EnableDisableLegendItem (bvp, FALSE);
     SafeHide (bvp->findGeneGrp);
     SafeHide (bvp->docTxtControlGrp);
+    SafeHide (bvp->baseCtgControlGrp);
     SafeHide (bvp->modeControlGrp);
+    SafeHide (bvp->newGphControlGrp);
     SafeHide (bvp->clickMe);
   }
 }
@@ -959,6 +1243,7 @@ static void ShowFastaOrAsn (BioseqViewPtr bvp, Boolean show)
     }
     SafeHide (bvp->modeControlGrp);
     SafeShow (bvp->docTxtControlGrp);
+    SafeHide (bvp->baseCtgControlGrp);
     SafeHide (bvp->clickMe);
   } else {
     SafeHide (bvp->text);
@@ -973,31 +1258,82 @@ static void ShowFastaOrAsn (BioseqViewPtr bvp, Boolean show)
     EnableDisableLegendItem (bvp, FALSE);
     SafeHide (bvp->findGeneGrp);
     SafeHide (bvp->docTxtControlGrp);
+    SafeHide (bvp->baseCtgControlGrp);
     SafeHide (bvp->modeControlGrp);
+    SafeHide (bvp->newGphControlGrp);
     SafeHide (bvp->clickMe);
   }
 }
 
+static Int2 FindFFParagraph (DocDescrPtr doscr, Int4 numdescr,
+                             Uint2 entityID, Uint2 itemID, Uint2 itemtype)
+
+{
+  DocDescrPtr  dsp;
+  Int4         L, R, mid;
+  Int2         rsult;
+
+  L = 0;
+  R = numdescr - 1;
+  while (L <= R) {
+    mid = (L + R) / 2;
+    dsp = &(doscr [mid]);
+    if (dsp == NULL) return 0;
+    if (dsp->entityID > entityID) {
+      R = mid - 1;
+    } else if (dsp->entityID < entityID) {
+      L = mid + 1;
+    } else if (dsp->itemtype > itemtype) {
+      R = mid - 1;
+    } else if (dsp->itemtype < itemtype) {
+      L = mid + 1;
+    } else if (dsp->itemID > itemID) {
+      R = mid - 1;
+    } else if (dsp->itemID < itemID) {
+      L = mid + 1;
+    } else {
+      rsult = dsp->docitem;
+      /* scan to first paragraph for item */
+      while (mid >= 0) {
+        dsp = &(doscr [mid]);
+        if (dsp != NULL) {
+          if (dsp->entityID == entityID &&
+              dsp->itemtype == itemtype &&
+              dsp->itemID == itemID) {
+            rsult = dsp->docitem;
+          }
+        }
+        mid--;
+      }
+      return rsult;
+    }
+  }
+
+  return 0;
+}
+
+
 static void SelectFlatFile (BioseqViewPtr bvp, Uint2 selentityID, Uint2 selitemID,
                             Uint2 selitemtype, SeqLocPtr region,
                             Boolean select, Boolean scrollto)
 
 {
-  Int2     bottom;
-  DoC      doc;
-  Uint2    entityID;
-  Int2     item;
-  Uint2    itemID;
-  Uint2    itemtype;
-  Boolean  needToScroll;
-  Int2     numItems;
-  RecT     rct;
-  BaR      sb;
-  Int2     scrollhere;
-  Int4     startsAt;
-  WindoW   tempPort;
-  CharPtr  text;
-  Int2     top;
+  Int2           bottom = 0;
+  DoC            doc;
+  FlatStructPtr  fsp;
+  Uint2          entityID;
+  Uint2          itemID;
+  Uint2          itemtype;
+  Int2           item;
+  Boolean        needToScroll;
+  Int2           numItems;
+  RecT           rct;
+  BaR            sb;
+  Int2           scrollhere;
+  Int4           startsAt;
+  WindoW         tempPort;
+  CharPtr        text;
+  Int2           top = 0;
 
   if (bvp == NULL) return;
   if (! bvp->highlightSelections) return;
@@ -1009,22 +1345,75 @@ static void SelectFlatFile (BioseqViewPtr bvp, Uint2 selentityID, Uint2 selitemI
   GetDocParams (doc, &numItems, NULL);
   needToScroll = TRUE;
   scrollhere = 0;
-  for (item = 1; item <= numItems; item++) {
-    if (GetIDsFromDoc (doc, item, &entityID, &itemID, &itemtype)) {
-      if (entityID == selentityID &&
-          itemID == selitemID &&
-          itemtype == selitemtype) {
-        if (ItemIsVisible (doc, item, &top, &bottom, NULL)) {
-          needToScroll = FALSE;
-          ObjectRect (doc, &rct);
-          InsetRect (&rct, 4, 4);
-          rct.right = rct.left + 4;
-          rct.top = top;
-          rct.bottom = bottom;
-          InsetRect (&rct, -1, -1);
-          InvalRect (&rct);
-        } else if (scrollhere == 0) {
-          scrollhere = item;
+  fsp = (FlatStructPtr) GetDocData (doc);
+  if (fsp != NULL && fsp->descr != NULL && fsp->numdescr > 0) {
+    if (GetScrlParams (doc, NULL, &item, NULL)) {
+      while (item <= numItems && ItemIsVisible (doc, item, &top, &bottom, NULL)) {
+        if (GetIDsFromDoc (doc, item, &entityID, &itemID, &itemtype)) {
+          if (entityID == selentityID &&
+              itemID == selitemID &&
+              itemtype == selitemtype) {
+            needToScroll = FALSE;
+            ObjectRect (doc, &rct);
+            InsetRect (&rct, 4, 4);
+            rct.right = rct.left + 4;
+            rct.top = top;
+            rct.bottom = bottom;
+            InsetRect (&rct, -1, -1);
+            InvalRect (&rct);
+          }
+        }
+        item++;
+      }
+    }
+    scrollhere = FindFFParagraph (fsp->descr, fsp->numdescr,
+                                  selentityID, selitemID, selitemtype);
+    if (scrollhere > 0) {
+      if (ItemIsVisible (doc, scrollhere, &top, &bottom, NULL)) {
+        needToScroll = FALSE;
+      }
+      ObjectRect (doc, &rct);
+      InsetRect (&rct, 4, 4);
+      rct.right = rct.left + 4;
+      rct.top = top;
+      rct.bottom = bottom;
+      InsetRect (&rct, -1, -1);
+      InvalRect (&rct);
+    } else if (selitemtype == OBJ_BIOSEQ) { /* not preindexed in fsp descr */
+      ObjectRect (doc, &rct);
+      InsetRect (&rct, 4, 4);
+      rct.right = rct.left + 4;
+      rct.top = top;
+      rct.bottom = bottom;
+      InsetRect (&rct, -1, -1);
+      InvalRect (&rct);
+    }
+  }
+  /* for items currently not preindexed (LOCUS, ORGANISM), do a short linear search */
+  if (/* scrollhere == 0 && */ selitemID > 0 /* && needToScroll */) {
+    for (item = 1; item <= numItems && item < 10; item++) {
+      if (GetIDsFromDoc (doc, item, &entityID, &itemID, &itemtype)) {
+        if (entityID == selentityID &&
+            itemID == selitemID &&
+            itemtype == selitemtype) {
+          /*
+          if (ItemIsVisible (doc, item, &top, &bottom, NULL)) {
+            needToScroll = FALSE;
+            ObjectRect (doc, &rct);
+            InsetRect (&rct, 4, 4);
+            rct.right = rct.left + 4;
+            rct.top = top;
+            rct.bottom = bottom;
+            InsetRect (&rct, -1, -1);
+            InvalRect (&rct);
+          } else if (scrollhere == 0) {
+            scrollhere = item;
+          }
+          */
+          if (needToScroll) {
+            scrollhere = item;
+            item = 10; /* break the for loop */
+          }
         }
       }
     }
@@ -1078,7 +1467,6 @@ static void PrintFlatFileFastaOrAsn (BioseqViewPtr bvp)
   TexT     txt;
 #ifdef WIN_MOTIF
   Char     cmmd [256];
-  FILE     *f;
   Int2     len;
   CharPtr  printCmd;
   Char     str [PATH_MAX];
@@ -1274,6 +1662,13 @@ BioseqPageData fstaPageData = {
   ExportFlatFileFastaOrAsn, NULL, ResizeFlatFileFastaOrAsn, NULL
 };
 
+BioseqPageData qualPageData = {
+  "Quality", TRUE, TRUE, FALSE, FALSE, -1,
+  PopulateQuality, ShowFastaOrAsn, NULL,
+  CopyFlatFileFastaOrAsn, PrintFlatFileFastaOrAsn,
+  ExportFlatFileFastaOrAsn, NULL, ResizeFlatFileFastaOrAsn, NULL
+};
+
 BioseqPageData asnPageData = {
   "ASN.1", TRUE, TRUE, TRUE, FALSE, -1,
   PopulateAsn, ShowFastaOrAsn, NULL,
@@ -1281,3 +1676,10 @@ BioseqPageData asnPageData = {
   ExportAsnAfterConfirming, NULL, ResizeFlatFileFastaOrAsn, NULL
 };
 
+BioseqPageData xmlPageData = {
+  "XML", TRUE, TRUE, TRUE, FALSE, -1,
+  PopulateXML, ShowFastaOrAsn, NULL,
+  CopyFlatFileFastaOrAsn, PrintFlatFileFastaOrAsn,
+  ExportAsnAfterConfirming, NULL, ResizeFlatFileFastaOrAsn, NULL
+};
+
