@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   11-29-94
 *
-* $Revision: 6.1 $
+* $Revision: 6.13 $
 *
 * File Description: 
 *
@@ -58,9 +58,6 @@ static void NEAR VSMWinDelete PROTO((VSMWinPtr vsmwp, Uint2 entityID, Uint2 item
 static void NEAR VSMWinRefreshData PROTO((VSMWinPtr vsmwp, Uint2 entityID,
                                      Uint2 itemID, Uint2 itemtype));
 
-static void NEAR VSMWinSelect PROTO((VSMWinPtr vsmwp,
-               Uint2 entityID, Uint2 itemID, Uint2 itemtype, Boolean select));
-
 static void NEAR VSMPictSelect PROTO((VSMPictPtr vsmpp,
                Uint2 entityID, Uint2 itemID, Uint2 itemtype, Boolean select));
 
@@ -70,9 +67,6 @@ static Boolean NEAR VSeqMgrCreateMainWindow PROTO((VSeqMgrPtr vsmp, Boolean prog
 
 static void NEAR VSeqMgrPopulateMain PROTO((VSMWinPtr vsmwp));
 
-static Boolean NEAR VSeqMgrRefreshData PROTO((Uint2 entityID, Uint2 itemID,
-                                                        Uint2 itemtype));
-
 #define VSM_PICT_UP_BUTTON  65001
 #define VSM_PICT_DOWN_BUTTON 65002
 SegmenT VSMEntityDraw PROTO((ObjMgrDataPtr omdp, VSMPictPtr vsmpp, VSeqMgrPtr vsmp));
@@ -133,7 +127,7 @@ Boolean LIBCALL VSeqMgrInit (Boolean show)
 *   	initializes if not already done
 *
 *****************************************************************************/
-Boolean LIBCALL VSeqMgrShow()
+Boolean LIBCALL VSeqMgrShow(void)
 {
 	VSeqMgrPtr vsmp;
 	VSMWinPtr tmp, next=NULL;
@@ -160,7 +154,7 @@ Boolean LIBCALL VSeqMgrShow()
 *      keeps settings for the main window
 *
 *****************************************************************************/
-Boolean LIBCALL VSeqMgrDelete()
+Boolean LIBCALL VSeqMgrDelete(void)
 {
 	VSeqMgrPtr vsmp;
 	VSMWinPtr tmp, next=NULL;
@@ -245,11 +239,6 @@ static void QuitVSeqMgr (WindoW w)
 	return;
 }
 
-static void CleanupViewerProc (WindoW w, VoidPtr data)
-{
-	return;
-}
-
 static void ResizeVSeqMgr (WindoW w)
 {
 	VSMWinPtr vsmwp;
@@ -385,6 +374,7 @@ static void VSeqMgrSaveProc (ChoicE c)
 	return;
 }
 
+/*
 static Boolean ObjMgrProcMatch (ObjMgrProcPtr ompp, OMProcControlPtr ompcp, Boolean showerror)
 {
 	ObjMgrTypePtr omtp1, omtp2;
@@ -396,7 +386,7 @@ static Boolean ObjMgrProcMatch (ObjMgrProcPtr ompp, OMProcControlPtr ompcp, Bool
 	if ((ompp == NULL) || (ompcp == NULL))
 		return FALSE;
 
-	if (ompp->inputtype == 0)   /* takes any type */
+	if (ompp->inputtype == 0)
 		return TRUE;
 
 	if (ompp->inputtype == ompcp->input_itemtype)
@@ -452,6 +442,7 @@ static Boolean ObjMgrProcMatch (ObjMgrProcPtr ompp, OMProcControlPtr ompcp, Bool
 	}
 	return TRUE;
 }
+*/
 
 static void ShowVSMProc (IteM i)
 {
@@ -599,7 +590,9 @@ Boolean LIBCALL VSMAddMenu (WindoW w, Int2 menutype)
 	SbstrucPtr sbp;
 	Uint2 subtype;
 
+#ifndef WIN_MAC
 	if (w == NULL) return FALSE;
+#endif
 	switch (menutype)
 	{
 		case VSM_FILTER_MENU:
@@ -649,7 +642,6 @@ Boolean LIBCALL VSMAddMenu (WindoW w, Int2 menutype)
 			while ((ompp = ObjMgrProcFindNext(omp, proctype, 0,0, ompp)) != NULL);
 			ValNodeFree (submenulist);
 			return TRUE;
-			break;
 		case VSM_EDIT_MENU:
 			proctype = proctypes[menutype];
 			vsmp = VSeqMgrGet();
@@ -710,6 +702,9 @@ Boolean LIBCALL VSMAddMenu (WindoW w, Int2 menutype)
                           subtype != FEATDEF_sig_peptide &&
                           subtype != FEATDEF_transit_peptide &&
                           subtype != FEATDEF_source &&
+                          subtype != FEATDEF_virion &&
+                          subtype != FEATDEF_mutation &&
+                          subtype != FEATDEF_allele &&
                           subtype != FEATDEF_site_ref) {
                         i = CommandItem (sub2, ompp->proclabel, VSeqMgrStdMenuProc);
                         SetObjectExtra(i, (VoidPtr)ompp, NULL);
@@ -993,6 +988,7 @@ static void VSMDragAndDrop(VSMWinPtr vsmwp, Uint2 entityID, Uint2 itemID, Uint2
 
 	if (entityID == 0)   /* going into desktop */
 	{
+		ObjMgrSendMsg(OM_MSG_UPDATE, ompc.input_entityID, ompc.input_itemID, ompc.input_itemtype);
 		ObjMgrRegister(ompc.input_itemtype, ompc.input_data);
 		return;
 	}
@@ -1114,6 +1110,7 @@ static void VSeqMgrClickProc (VieweR v, SegmenT s, PoinT p)
 	}
 	
 	vsmwp->dblclick = dblClick;
+	vsmwp->shftkey = shftKey;
 	vsmwp->dragged = FALSE;
 	if (dblClick)
 	{
@@ -1166,7 +1163,7 @@ static void VSeqMgrReleaseProc (VieweR v, SegmenT s, PoinT p)
 {
 	VSMWinPtr vsmwp;
 	SegmenT sp, top, next, prev;
-	Uint2 segID, primID, primCt, entityID=0, itemtype, itemID;
+	Uint2 segID, primID, primCt, entityID=0, itemtype = 0, itemID = 0;
 	Uint1 highlight = PLAIN_SEGMENT;
 	Int2 expand = 0;
 	Boolean do_drag = FALSE;
@@ -1186,13 +1183,6 @@ static void VSeqMgrReleaseProc (VieweR v, SegmenT s, PoinT p)
 		vsmwp->marquee = FALSE;
 	}
 		
-	/*
-	if (vsmwp->dblclick)
-	{
-		Message(MSG_OK,"DoubleClick");
-		return;
-	}
-	*/
 	sp = FindSegment(v, p, &segID, &primID, &primCt);
 	if (sp == NULL)
 	{
@@ -1244,8 +1234,13 @@ static void VSeqMgrReleaseProc (VieweR v, SegmenT s, PoinT p)
         	else if (itemID == VSM_PICT_DOWN_BUTTON)
         		expand = -1;
         		
-        	if (! expand)
-				ObjMgrSelect(entityID, itemID, itemtype,0,NULL);
+        	if (! expand) {
+        		if (vsmwp->shftkey) {
+					ObjMgrAlsoSelect(entityID, itemID, itemtype,0,NULL);
+        		} else {
+					ObjMgrSelect(entityID, itemID, itemtype,0,NULL);
+				}
+			}
 			else
 			{
 				WatchCursor();
@@ -1588,8 +1583,6 @@ static Boolean NEAR VSeqMgrCreateMainWindow(VSeqMgrPtr vsmp, Boolean progwindow)
 	return TRUE;
 }
 
-static void BuildList (SeqEntryPtr sep, Pointer data, Int4 index, Int2 indent);
-
 /*****************************************************************************
 *
 *   VSeqMgrPopulateMain(vsmwp)
@@ -1736,36 +1729,6 @@ static void NEAR VSeqMgrPopulateMain(VSMWinPtr vsmwp)
 
 }
 
-/*****************************************************************************
-*
-*   
-*
-*****************************************************************************/
-static Boolean NEAR VSeqMgrRefreshData (Uint2 entityID, Uint2 itemID, Uint2 itemtype)
-{
-	VSeqMgrPtr vsmp;
-	VSMWinPtr vsmwp;
-
-	vsmp = VSeqMgrGet();
-	for (vsmwp = vsmp->wins; vsmwp != NULL; vsmwp = vsmwp->next)
-	{
-		if ((! vsmwp->entityID) || (! entityID) ||
-			(vsmwp->entityID == entityID))
-		{
-			if ((! vsmwp->itemID) || (! itemID) ||
-				(vsmwp->itemID == itemID))
-			{
-				if ((! vsmwp->itemtype) || (! itemtype) ||
-					(vsmwp->itemtype == itemtype))
-				{
-					VSMWinRefreshData(vsmwp, entityID, itemID, itemtype);
-				}
-			}
-		}
-	}
-	return TRUE;
-}
-
 /*****************************************************************************
 *
 *   VSMWinNew(vsmp)
@@ -2066,51 +2029,6 @@ static Boolean VSMHighlightProc (SegmenT seg, PrimitivE prim, Uint2 segid, Uint2
 	return TRUE;
 }
 
-/*****************************************************************************
-*
-*	VSMWinSelect(vsmwp, entityID, itemID, itemtype, select)
-*
-*****************************************************************************/
-static void NEAR VSMWinSelect(VSMWinPtr vsmwp, Uint2 entityID, Uint2 itemID, Uint2 itemtype, Boolean select)
-{
-	PrimitivE p;
-	SegmenT s;
-	Int2 i, j;
-	VieweR v;
-	VSMHlp vh;
-
-
-	if (vsmwp == NULL) return;
-	v = GetWindowExtra(vsmwp->w);
-
-	i = 1;
-	vh.h_v = v;
-	vh.h_item = itemID;
-	vh.h_type = itemtype;
-	if (select)
-		vh.highlight = OUTLINE_SEGMENT;
-	else
-		vh.highlight = PLAIN_SEGMENT;
-
-
-	while ((p = GetPrimitive(vsmwp->picture, i)) != NULL)
-	{
-		s = (SegmenT)p;
-		j = SegmentID(s);
-		if (j == (Int2)entityID)
-		{
-			if (! itemID)
-				HighlightSegment(v, s, vh.highlight);
-			else
-				ExploreSegment(s, (Pointer)(&vh), VSMHighlightProc);
-			break;
-		}
-		i++;
-	}
-
- 	return;
-}
-
 /*****************************************************************************
 *
 *	VSMPictSelect(vspp, entityID, itemID, itemtype, select)
@@ -2178,7 +2096,14 @@ static Boolean VSMGatherPictProc (GatherContextPtr gcp)
 	ValNode vn;
 	SeqFeatPtr sfp;
 	SeqAnnotPtr annot;
+	SeqGraphPtr sgp;
 	ValNodePtr desc;
+	ObjectIdPtr oip;
+	SeqIdPtr sip;
+	UserFieldPtr ufp;
+	UserObjectPtr uop;
+	SeqEntryPtr sep;
+	SeqEntryPtr oldsep;
 
 	vsmgp = (VSMGatherProcSTPtr)(gcp->userdata);
 	vsmp = vsmgp->vsmp;
@@ -2191,6 +2116,9 @@ static Boolean VSMGatherPictProc (GatherContextPtr gcp)
 	SelectFont(vsmp->font);
 	omtp = ObjMgrTypeFind(vsmp->omp, gcp->thistype, NULL, NULL);
 
+	sep = GetTopSeqEntryForEntityID (gcp->entityID);
+	oldsep = SeqEntrySetScope (sep);
+
 	switch (gcp->thistype)
 	{
 		case OBJ_SEQSUB:
@@ -2372,8 +2300,22 @@ static Boolean VSMGatherPictProc (GatherContextPtr gcp)
 			width = StringWidth(buf) + (2 * vsmp->charw);
 			AddAttribute(seg, COLOR_ATT , BLUE_COLOR, 0, 0,0,0);
 			AddTextLabel(seg, left + vsmp->charw, top, buf, vsmp->font,0, LOWER_RIGHT,(Uint2)(gcp->itemID));
-			add_frame(seg, left+vsmp->charw, top, (left+width), (top-lineheight), (Uint2)(gcp->itemID));
 			vsmgp->currline--;
+			maxwidth = width;
+			k = 1;
+			sgp = (SeqGraphPtr) gcp->thisitem;
+			if (vsmgp->level[OBJ_SEQGRAPH] > 1 && sgp != NULL && sgp->loc != NULL) {
+				k++;
+				SeqLocLabel(sgp->loc, buf, buflen, OM_LABEL_CONTENT);
+				SelectFont(vsmp->font);
+				width = StringWidth(buf) + (3 * vsmp->charw);
+				if (width > maxwidth)
+					maxwidth = width;
+				AddTextLabel(seg, left + (2*vsmp->charw), (top-lineheight), buf, vsmp->font,0, LOWER_RIGHT,(Uint2)(gcp->itemID));
+				vsmgp->currline--;
+			}
+			add_frame(seg, left, top-2, (left+width), (top-(k*lineheight)+2), (Uint2)(gcp->itemID));
+			/* add_frame(seg, left+vsmp->charw, top, (left+width), (top-lineheight), (Uint2)(gcp->itemID)); */
 			break;
 		case OBJ_SEQFEAT:
 			if (vsmgp->level[OBJ_SEQFEAT] < 2)
@@ -2411,6 +2353,35 @@ static Boolean VSMGatherPictProc (GatherContextPtr gcp)
 					AddTextLabel(seg, left + (2*vsmp->charw), (top-lineheight*2), buf, vsmp->font,0, LOWER_RIGHT,(Uint2)(gcp->itemID));
 					vsmgp->currline--;
 				}
+				if (vsmp != NULL && vsmp->extraLevel && sfp->data.choice == SEQFEAT_RNA && sfp->ext != NULL) {
+					uop = sfp->ext;
+					if (uop->type != NULL && StringICmp (uop->type->str, "MrnaProteinLink") == 0) {
+						ufp = uop->data;
+						if (ufp != NULL && ufp->choice == 1) {
+							oip = ufp->label;
+							if (oip != NULL && oip->str != NULL && StringICmp (oip->str, "protein seqID") == 0) {
+								tmp = (CharPtr) ufp->data.ptrvalue;
+								if (tmp != NULL) {
+									sip = MakeSeqID (tmp);
+									if (sip != NULL) {
+										vn.choice = SEQLOC_WHOLE;
+										vn.data.ptrvalue = (Pointer) sip;
+										k++;
+										StringCpy (buf, "protein ");
+										SeqLocLabel(&vn, buf + 8, buflen - 8, OM_LABEL_CONTENT);
+										SelectFont(vsmp->font);
+										width = StringWidth(buf) + (3 * vsmp->charw);
+										if (width > maxwidth)
+											maxwidth = width;
+										AddTextLabel(seg, left + (2*vsmp->charw), (top-lineheight*(k-1)), buf, vsmp->font,0, LOWER_RIGHT,(Uint2)(gcp->itemID));
+										vsmgp->currline--;
+										SeqIdFree (sip);
+									}
+								}
+							}
+						}
+					}
+				}
 			}
 			add_frame(seg, left, top-2, (left+width), (top-(k*lineheight)+2), (Uint2)(gcp->itemID));
 			break;
@@ -2432,6 +2403,9 @@ static Boolean VSMGatherPictProc (GatherContextPtr gcp)
 			break;
 
 	}
+
+	SeqEntrySetScope (oldsep);
+
 	return TRUE;
 }
 
@@ -2705,6 +2679,7 @@ static OMUserDataPtr NEAR VSMAddPictureToEntity (VSMWinPtr vsmwp, Uint2 entityID
 	ValNodePtr vnp;
 	Pointer ptr;
 	SelStructPtr ssp;
+	VSeqMgrPtr vsmp;
 
 	if (vsmwp == NULL) return NULL;
 											  /* already have it? */
@@ -2734,6 +2709,11 @@ static OMUserDataPtr NEAR VSMAddPictureToEntity (VSMWinPtr vsmwp, Uint2 entityID
 	
 	vsmpp->expansion += expand;
 
+	vsmp = vsmwp->vsmp;
+	if (vsmp != NULL) {
+		vsmp->extraLevel = TRUE;
+	}
+
 	seg = VSMEntityDraw(omdp, vsmpp, vsmwp->vsmp);
 
 	if (seg == NULL)
