@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   3/3/95
 *
-* $Revision: 6.6 $
+* $Revision: 6.24 $
 *
 * File Description: 
 *
@@ -71,8 +71,12 @@ typedef struct validextra {
   FonT           font;
   ButtoN         find;
   TexT           searchfor;
+  PrompT         showncount;
   PrompT         summary;
-  Int2           counts [6];
+  Int4           counts [6];
+  Int4           totalcount;
+  Int4           addedcount;
+  Int4           remaining;
   Int2           clicked;
   Int2           selected;
   Boolean        dblClick;
@@ -150,10 +154,14 @@ static void ClickValid (DoC d, PoinT pt)
 
   vep = GetObjectExtra (d);
   if (vep != NULL) {
-    vep->clicked = 0;
-    vep->dblClick = dblClick;
-    vep->shftKey = shftKey;
     MapDocPoint (d, pt, &item, &row, NULL, NULL);
+    if (item > 0 && row > 0 && vep->clicked == item) {
+      vep->dblClick = dblClick;
+    } else {
+      vep->dblClick = FALSE;
+    }
+    vep->clicked = 0;
+    vep->shftKey = shftKey;
     if (item > 0 && row > 0) {
       vep->clicked = item;
     }
@@ -285,6 +293,7 @@ static void ReleaseValid (DoC d, PoinT pt)
   }
 }
 
+/*
 static void RemoveProc (ButtoN b)
 
 {
@@ -317,6 +326,7 @@ static void RemoveProc (ButtoN b)
     }
   }
 }
+*/
 
 static void CloseValidWindow (WindoW w)
 
@@ -342,6 +352,7 @@ extern void ClearValidateWindow (void)
     vep = GetObjectExtra (validWindow);
     if (vep != NULL) {
       Reset (vep->doc);
+      vep->selected = 0;
       vep->messages = ValNodeFreeData (vep->messages);
       for (vnp = vep->errorfilter; vnp != NULL; vnp = vnp->next) {
         efp = (ErrFltrPtr) vnp->data.ptrvalue;
@@ -360,9 +371,13 @@ extern void ClearValidateWindow (void)
       SetValue (vep->filter, 1);
       Show (vep->filter);
       SetTitle (vep->summary, "");
+      SetTitle (vep->showncount, "");
       for (sev = SEV_NONE; sev <= SEV_MAX; sev++) {
         vep->counts [sev] = 0;
       }
+      vep->totalcount = 0;
+      vep->addedcount = 0;
+      vep->remaining = 0;
       SetTitle (vep->searchfor, "");
       if (Enabled (vep->find)) {
         Disable (vep->find);
@@ -385,7 +400,7 @@ static CharPtr CharPrtProc (DoC d, Int2 item, Pointer ptr)
   if (vep != NULL) {
     if (ptr != NULL) {
       str = StringSave (ptr);
-      if (GetValue (vep->verbose) == 2) {
+      if (GetValue (vep->verbose) == 3) {
         tabcount = 0;
         tmp = str;
         ch = *tmp;
@@ -427,12 +442,15 @@ static void RepopVal (PopuP p)
   int            sev;
   int            subcode;
   CharPtr        str;
+  Char           tmp [32];
   ValidExtraPtr  vep;
   ValNodePtr     vnp;
 
   vep = GetObjectExtra (p);
   if (vep != NULL) {
     Reset (vep->doc);
+    vep->addedcount = 0;
+    vep->selected = 0;
     minlev = GetValue (vep->minlevel);
     filt = GetValue (vep->filter);
     efp = NULL;
@@ -463,6 +481,7 @@ static void RepopVal (PopuP p)
             okay = TRUE;
           }
           if (okay) {
+            (vep->addedcount)++;
             AppendItem (vep->doc, CharPrtProc, vnp->data.ptrvalue, FALSE,
                         (StringLen (vnp->data.ptrvalue) / 50) + 1,
                         &valParFmt, valColFmt, vep->font);
@@ -471,6 +490,14 @@ static void RepopVal (PopuP p)
       }
       MemFree (str);
     }
+    if (vep->addedcount > 1) {
+      sprintf (tmp, " %ld items shown", (long) vep->addedcount);
+    } else if (vep->addedcount > 0) {
+      sprintf (tmp, " %ld item shown", (long) vep->addedcount);
+    } else {
+      StringCpy (tmp, "");
+    }
+    SafeSetTitle (vep->showncount, tmp);
     UpdateDocument (vep->doc, 0, 0);
   }
 }
@@ -627,6 +654,47 @@ static void RevalidateProc (ButtoN b)
     for (i = SEV_NONE; i <= SEV_MAX; i++) {
       vep->counts [i] = 0;
     }
+    vep->totalcount = 0;
+    vep->addedcount = 0;
+    vep->remaining = 0;
+    vep->clicked = 0;
+    vep->selected = 0;
+    vep->dblClick = FALSE;
+    bfp = vep->bfp;
+    if (bfp != NULL && vep->revalProc != NULL) {
+      vep->revalProc (bfp->form);
+    }
+  }
+}
+
+static Boolean doSuppressContext = TRUE;
+
+extern Boolean ShouldSetSuppressContext (void)
+
+{
+  return doSuppressContext;
+}
+
+static void SetVerbosityAndRevalidate (PopuP p)
+
+{
+  BaseFormPtr    bfp;
+  int            i;
+  ValidExtraPtr  vep;
+
+  vep = GetObjectExtra (p);
+  if (vep != NULL) {
+    if (GetValue (vep->verbose) > 1) {
+      doSuppressContext = TRUE;
+    } else {
+      doSuppressContext = FALSE;
+    }
+    for (i = SEV_NONE; i <= SEV_MAX; i++) {
+      vep->counts [i] = 0;
+    }
+    vep->totalcount = 0;
+    vep->addedcount = 0;
+    vep->remaining = 0;
     vep->clicked = 0;
     vep->selected = 0;
     vep->dblClick = FALSE;
@@ -758,6 +826,16 @@ extern void CreateStdValidatorFormMenus (WindoW w)
 }
 #endif
 
+static CharPtr canStillSubmitText =
+"Please review and correct the following errors. If you are unable to resolve " \
+"an error, submit your sequences, and our annotation staff may contact you " \
+"to help resolve the remaining errors when your sequences are processed.";
+
+static CharPtr howToClickText =
+"Click on an error item to select and scroll to that feature. " \
+"Shift click to choose target sequence and feature if in a set of multiple sequences. " \
+"Double click on an error item to launch the appropriate feature editor.";
+
 /* CreateValidateWindowEx is hidden, allowing a revalidate button */
 extern void CreateValidateWindowEx (ErrNotifyProc notify, CharPtr title,
                                     FonT font, ErrSev sev, Boolean verbose,
@@ -771,9 +849,10 @@ extern void CreateValidateWindowEx (ErrNotifyProc notify, CharPtr title,
   GrouP              g;
   GrouP              h;
   int                i;
+  GrouP              ppt;
   GrouP              q;
   StdEditorProcsPtr  sepp;
-  PrompT             t;
+  GrouP              t;
   ValidExtraPtr      vep;
   WindoW             w;
 
@@ -805,6 +884,8 @@ extern void CreateValidateWindowEx (ErrNotifyProc notify, CharPtr title,
       if (w != NULL) {
         vep->notify = notify;
 
+        ppt = MultiLinePrompt (w, canStillSubmitText, stdCharWidth * 30, systemFont);
+
         c = HiddenGroup (w, 8, 0, NULL);
         SetGroupSpacing (c, 10, 2);
         /*
@@ -812,22 +893,28 @@ extern void CreateValidateWindowEx (ErrNotifyProc notify, CharPtr title,
         Disable (vep->remove);
         */
         StaticPrompt (c, "Message", 0, popupMenuHeight, programFont, 'l');
-        vep->verbose = PopupList (c, FALSE, RepopVal);
+        vep->verbose = PopupList (c, FALSE, SetVerbosityAndRevalidate);
         SetObjectExtra (vep->verbose, vep, NULL);
         PopupItem (vep->verbose, "Verbose");
+        PopupItem (vep->verbose, "Normal");
         PopupItem (vep->verbose, "Terse");
-        if (verbose) {
+        if (GetAppProperty ("InternalNcbiSequin") != NULL) {
           SetValue (vep->verbose, 1);
-        } else {
+          doSuppressContext = FALSE;
+        } else if (verbose) {
           SetValue (vep->verbose, 2);
+          doSuppressContext = TRUE;
+        } else {
+          SetValue (vep->verbose, 3);
+          doSuppressContext = TRUE;
         }
         StaticPrompt (c, "Severity", 0, popupMenuHeight, programFont, 'l');
-        vep->minlevel = PopupList (c, FALSE, RepopVal);
+        vep->minlevel = PopupList (c, FALSE, RepopVal); /* was SetVerbosityAndRevalidate */
         SetObjectExtra (vep->minlevel, vep, NULL);
         PopupItem (vep->minlevel, "INFO");
         PopupItem (vep->minlevel, "WARN");
         PopupItem (vep->minlevel, "ERROR");
-        PopupItem (vep->minlevel, "FATAL");
+        PopupItem (vep->minlevel, "REJECT");
         if (sev >= 1 && sev <= 4) {
           SetValue (vep->minlevel, (Int2) sev);
         } else {
@@ -849,12 +936,14 @@ extern void CreateValidateWindowEx (ErrNotifyProc notify, CharPtr title,
         SetValue (vep->filter, 1);
         Break (w);
 
-        f = HiddenGroup (w, 4, 0, NULL);
+        f = HiddenGroup (w, 6, 0, NULL);
         vep->find = PushButton (f, "Find", ValFindProc);
         SetObjectExtra (vep->find, vep, NULL);
         Disable (vep->find);
         vep->searchfor = DialogText (f, "", 15, ValTextProc);
         SetObjectExtra (vep->searchfor, vep, NULL);
+        vep->showncount = StaticPrompt (f, " 0000000 items shown", 0, dialogTextHeight, systemFont, 'l');
+        SetTitle (vep->showncount, "");
 
         g = HiddenGroup (w, 0, -5, NULL);
         h = HiddenGroup (g, 5, 0, NULL);
@@ -866,8 +955,7 @@ extern void CreateValidateWindowEx (ErrNotifyProc notify, CharPtr title,
         StaticPrompt (h, "Subcode", stdCharWidth * 9, 0, programFont, 'l');
         StaticPrompt (h, "Message", stdCharWidth * 20, 0, programFont, 'l');
         */
-        t = StaticPrompt (g, "Double click on an error item to launch the appropriate editor.",
-                          0, 0, programFont, 'c');
+        t = MultiLinePrompt (g, howToClickText, stdCharWidth * 30, systemFont);
         vep->doc = DocumentPanel (g, stdCharWidth * 30, stdLineHeight * 20);
         SetObjectExtra (vep->doc, vep, NULL);
         SetDocAutoAdjust (vep->doc, FALSE);
@@ -878,6 +966,9 @@ extern void CreateValidateWindowEx (ErrNotifyProc notify, CharPtr title,
         for (i = SEV_NONE; i <= SEV_MAX; i++) {
           vep->counts [i] = 0;
         }
+        vep->totalcount = 0;
+        vep->addedcount = 0;
+        vep->remaining = 0;
         vep->clicked = 0;
         vep->selected = 0;
         vep->dblClick = FALSE;
@@ -901,7 +992,7 @@ extern void CreateValidateWindowEx (ErrNotifyProc notify, CharPtr title,
         b = PushButton (w, "Dismiss", CloseValidButton);
 #endif
 
-        AlignObjects (ALIGN_CENTER, (HANDLE) vep->doc, (HANDLE) t,
+        AlignObjects (ALIGN_CENTER, (HANDLE) ppt, (HANDLE) vep->doc, (HANDLE) t,
                       (HANDLE) vep->summary, (HANDLE) b, NULL);
 
         RealizeWindow (w);
@@ -933,6 +1024,48 @@ extern void CreateValidateWindow (ErrNotifyProc notify, CharPtr title,
   CreateValidateWindowEx (notify, title, font, sev, verbose, NULL, NULL, FALSE);
 }
 
+extern void ShowValidateDoc (void)
+
+{
+  Char           str [64];
+  ValidExtraPtr  vep;
+
+  if (validWindow == NULL) return;
+  vep = GetObjectExtra (validWindow);
+  if (vep == NULL) return;
+  RepopVal (vep->filter);
+  SafeShow (vep->doc);
+  SafeShow (vep->searchfor);
+  sprintf (str, "INFO: %ld      WARNING: %ld      ERROR: %ld      REJECT: %ld",
+           (long) vep->counts [SEV_INFO], (long) vep->counts [SEV_WARNING],
+           (long) vep->counts [SEV_ERROR], (long) vep->counts [SEV_REJECT]);
+  SafeSetTitle (vep->summary, str);
+  if (vep->addedcount > 1) {
+    sprintf (str, " %ld items shown", (long) vep->addedcount);
+  } else if (vep->addedcount > 0) {
+    sprintf (str, " %ld item shown", (long) vep->addedcount);
+  } else {
+    StringCpy (str, "");
+  }
+  SafeSetTitle (vep->showncount, str);
+  Update ();
+  ShowValidateWindow ();
+  Update ();
+}
+
+extern void HideValidateDoc (void)
+
+{
+  ValidExtraPtr  vep;
+
+  if (validWindow == NULL) return;
+  vep = GetObjectExtra (validWindow);
+  if (vep == NULL) return;
+  SafeHide (vep->doc);
+  SafeHide (vep->searchfor);
+  Update ();
+}
+
 extern void ShowValidateWindow (void)
 
 {
@@ -951,9 +1084,15 @@ extern void ShowValidateWindow (void)
       }
     }
     if (! Visible (validWindow)) {
-      Show (validWindow);
+      if (vep->totalcount > 0) {
+        Show (validWindow);
+      }
+    }
+    if (Visible (validWindow)) {
+      if (Visible (vep->doc)) {
+        Select (validWindow);
+      }
     }
-    Select (validWindow);
   }
 }
 
@@ -984,7 +1123,7 @@ static CharPtr StringMoveAndConvertTabs (CharPtr dst, CharPtr src, CharPtr suffi
 }
 
 static CharPtr severityLabel [] = {
-  "NONE", "INFO", "WARN", "ERROR", "FATAL", "MAX", NULL
+  "NONE", "INFO", "WARN", "ERROR", "REJECT", "FATAL", "MAX", NULL
 };
 
 static void FillValidBuffer (CharPtr buf, CharPtr text1, CharPtr text2,
@@ -1046,6 +1185,7 @@ extern void RepopulateValidateFilter (void)
     Hide (vep->filter);
     Update ();
     Reset (vep->filter);
+    vep->selected = 0;
     PopupItem (vep->filter, "ALL");
     SetValue (vep->filter, 1);
     vnp = vep->errorfilter;
@@ -1152,7 +1292,7 @@ static void ProcessValidMessage (ValidExtraPtr vep, CharPtr text1, CharPtr text2
 {
   CharPtr     buf;
   size_t      len;
-  Int2        numItems;
+  /* Int2        numItems; */
   Char        str [64];
   ValNodePtr  vnp;
 
@@ -1175,11 +1315,16 @@ static void ProcessValidMessage (ValidExtraPtr vep, CharPtr text1, CharPtr text2
       if (vnp != NULL) {
         vnp->data.ptrvalue = buf;
         if (sev >= minlev || sev == SEV_NONE) {
+          (vep->addedcount)++;
+          /*
           AppendItem (vep->doc, CharPrtProc, vnp->data.ptrvalue, FALSE,
                       (StringLen (vnp->data.ptrvalue) / 50) + 1,
                       &valParFmt, valColFmt, vep->font);
           GetDocParams (vep->doc, &numItems, NULL);
-          UpdateDocument (vep->doc, numItems, numItems);
+          if (Visible (vep->doc) && AllParentsVisible (vep->doc)) {
+            UpdateDocument (vep->doc, numItems, numItems);
+          }
+          */
         }
       }
     }
@@ -1207,10 +1352,34 @@ extern void AppendValidMessage (CharPtr text1, CharPtr text2, CharPtr text3,
         sev = SEV_MAX;
       }
       (vep->counts [sev])++;
-      sprintf (str, "INFO: %d      WARNING: %d      ERROR: %d      FATAL: %d",
-               (int) vep->counts [SEV_INFO], (int) vep->counts [SEV_WARNING],
-               (int) vep->counts [SEV_ERROR], (int) vep->counts [SEV_FATAL]);
-      SetTitle (vep->summary, str);
+      (vep->totalcount)++;
+      if (vep->remaining > 0) {
+        (vep->remaining)--;
+      }
+      if (vep->remaining < 1) {
+        sprintf (str, "INFO: %ld      WARNING: %ld      ERROR: %ld      REJECT: %ld",
+                 (long) vep->counts [SEV_INFO], (long) vep->counts [SEV_WARNING],
+                 (long) vep->counts [SEV_ERROR], (long) vep->counts [SEV_REJECT]);
+        if (vep->totalcount < 30) {
+          vep->remaining = 1;
+        } else if (vep->totalcount < 100) {
+          vep->remaining = 5;
+        } else if (vep->totalcount < 300) {
+          vep->remaining = 20;
+        } else if (vep->totalcount < 1000) {
+          vep->remaining = 50;
+        } else if (vep->totalcount < 3000) {
+          vep->remaining = 100;
+        } else if (vep->totalcount < 10000) {
+          vep->remaining = 200;
+        } else if (vep->totalcount < 30000) {
+          vep->remaining = 500;
+        } else {
+          vep->remaining = 1000;
+	    }
+        SetTitle (vep->summary, str);
+        Update ();
+      }
       if (text1 == NULL) {
         text1 = severityLabel [sev];
       }
@@ -1228,7 +1397,7 @@ extern void AppendValidMessage (CharPtr text1, CharPtr text2, CharPtr text3,
         vnp = vnp->next;
       }
       /*
-      if (GetValue (vep->verbose) == 2) {
+      if (GetValue (vep->verbose) == 3) {
         expanded = NULL;
       }
       */
@@ -1238,7 +1407,11 @@ extern void AppendValidMessage (CharPtr text1, CharPtr text2, CharPtr text3,
       ProcessValidMessage (vep, text1, text2, text3, sev, errcode,
                            subcode, entityID, itemID, itemtype,
                            message, expanded, context, ctxlen, minlev); 
-      Select (validWindow);
+      /*
+      if (Visible (validWindow)) {
+        Select (validWindow);
+      }
+      */
     }
   }
 }
@@ -1255,6 +1428,7 @@ extern int LIBCALLBACK ValidErrHook (const ErrDesc *err)
   ErrSev      sev;
 
   if (err != NULL) {
+    /* cannot suppress FileOpen report here because it is same as ERR_SEQ_DESCR_Inconsistent */
     /* if (err->errcode == E_File && err->subcode == E_FOpen) return 1; */
     name1 = NULL;
     name2 = NULL;
@@ -1366,6 +1540,7 @@ static void SearchTextProc (TexT t)
   }
 }
 
+/*
 static void SearchFeatProc (GrouP g)
 
 {
@@ -1440,6 +1615,7 @@ static void SearchPubProc (GrouP g)
     }
   }
 }
+*/
 
 typedef struct searchlist {
   Uint2             subtype;
@@ -1504,7 +1680,6 @@ static Boolean SearchGatherFunc (GatherContextPtr gcp)
         break;
       default :
         return TRUE;
-        break;
     }
     if (success && searchSpop->ptr != NULL &&
         *((CharPtr) (searchSpop->ptr)) != '\0') {
