@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   06/05/92
 *
-* $Revision: 6.0 $
+* $Revision: 6.1 $
 *
 * File Description: 
 *       miscellaneous library for network Entrez
@@ -44,6 +44,9 @@
 *
 * RCS Modification History:
 * $Log: netlib.c,v $
+* Revision 6.1  1998/08/24 21:00:51  kans
+* fixed -v -fd warnings
+*
 * Revision 6.0  1997/08/25 18:35:01  madden
 * Revision changed to 6.0
 *
@@ -132,7 +135,7 @@ static void NEAR FindAsnType (AsnTypePtr PNTR atp, AsnModulePtr amp, CharPtr str
   }
 }
 
-static Boolean InitLoad()
+static Boolean InitLoad(void)
 {
     AsnModulePtr amp;
     static Boolean loaded = FALSE;
