@@ -43,15 +43,13 @@ static Boolean ForceNetInit PROTO((void));
 static Boolean NetFini PROTO((void));
 static Boolean GenericReestablishNet PROTO((CharPtr svcName, Boolean showErrs));
 static SeqIdPtr s_ID1ArchSeqIdsGet PROTO((Uint4 gi,AsnIoPtr aiopr));
-static SeqEntryPtr s_ID1ArchSeqEntryGet PROTO((Int4 uid, Int4 ent,Int4Ptr status, Int2 maxplex));
+static SeqEntryPtr s_ID1ArchSeqEntryGet PROTO((Int4 uid, CharPtr db, Int4 ent,Int4Ptr status, Int2 maxplex));
 static Int4 s_ID1ArcgGIStateGet PROTO((Uint4 gi));
 static ID1SeqHistPtr s_ID1ArchGIHistGet PROTO((Uint4 gi,Boolean rev,AsnIoPtr aiopr));
 
 static NI_HandPtr svcp = NULL;
 static AsnIoPtr   asnin = NULL;
 static AsnIoPtr   asnout = NULL;
-ID1serverRequestPtr id1reqp;
-ID1serverBackPtr id1bp;
 static Boolean num_attached = 0;
 static Boolean reallyFinal = TRUE;
 static NI_DispatcherPtr dispatcher;
@@ -67,7 +65,7 @@ static Boolean (*myNetInit) PROTO((void));
 Boolean ID1ArchInit (void)
 
 {
-    DataVal    av;
+    
 
     myNetInit = ID1ArchInit;
 
@@ -96,11 +94,8 @@ Boolean ID1ArchInit (void)
 static Boolean s_ID1ArchFini (void)
 
 {
-    Boolean retval = TRUE;
-
-    ID1serverBackFree (id1bp);
     NetFini();
-    return retval;
+    return TRUE;
 }
 
 /* the only thing done here is to suppress errors */
@@ -124,12 +119,14 @@ Boolean ID1ArchFini (void)
 }
 
 static SeqEntryPtr
-s_ID1ArchSeqEntryGet (Int4 uid, Int4 ent,Int4Ptr status, Int2 maxplex)
+s_ID1ArchSeqEntryGet (Int4 uid, CharPtr db, Int4 ent,Int4Ptr status, Int2 maxplex)
 
 {
     ID1serverRequestPtr id1reqp;
     SeqEntryPtr sep;
     ID1serverMaxcomplexPtr mcp;
+    ID1serverBackPtr id1bp;
+
 
     if(status)
 	*status=0;
@@ -140,8 +137,12 @@ s_ID1ArchSeqEntryGet (Int4 uid, Int4 ent,Int4Ptr status, Int2 maxplex)
     id1reqp->data.ptrvalue = mcp;
 		mcp -> gi = uid;
 		mcp -> maxplex = maxplex;
-		if(ent > 0)
+		if(ent > 0){
 			mcp->ent = ent;
+			if(db){
+				mcp->sat=StringSave(db);
+			}
+		}
     ID1serverRequestAsnWrite (id1reqp, asnout, NULL);
     AsnIoReset(asnout);
     ID1serverRequestFree (id1reqp);
@@ -173,10 +174,10 @@ s_ID1ArchSeqEntryGet (Int4 uid, Int4 ent,Int4Ptr status, Int2 maxplex)
 }
 
 SeqEntryPtr
-ID1ArchSeqEntryGet (Int4 gi,Int4 ent, Int4Ptr status, Int2 maxplex)
+ID1ArchSeqEntryGet (Int4 gi,CharPtr db, Int4 ent, Int4Ptr status, Int2 maxplex)
 {
     Int4 i;
-    SeqEntryPtr sep;
+    SeqEntryPtr sep=NULL;
     short erract;
     ErrDesc err;
     Int4 retval = 0;
@@ -196,7 +197,7 @@ ID1ArchSeqEntryGet (Int4 gi,Int4 ent, Int4Ptr status, Int2 maxplex)
 			ErrGetOpts(&erract, NULL);
 			ErrSetOpts(ERR_CONTINUE, 0);
 			ErrFetch(&err);
-			sep = s_ID1ArchSeqEntryGet (gi, ent, status, maxplex);
+			sep = s_ID1ArchSeqEntryGet (gi, db, ent, status, maxplex);
 			ErrSetOpts(erract, 0);
 			if (! ErrFetch(&err))
 				 break; /* success */
@@ -208,6 +209,8 @@ static SeqIdPtr
 s_ID1ArchSeqIdsGet (Uint4 gi,AsnIoPtr aiopr)
 {
     ID1serverRequestPtr id1reqp;
+    ID1serverBackPtr id1bp;
+
     SeqIdPtr sip;
 
     id1reqp = ValNodeNew(NULL);
@@ -267,6 +270,8 @@ static ID1SeqHistPtr
 s_ID1ArchGIHistGet(Uint4 gi,Boolean rev,AsnIoPtr aiopr)
 {
     ID1serverRequestPtr id1reqp;
+    ID1serverBackPtr id1bp;
+
     ID1SeqHistPtr ishp;
 
     id1reqp = ValNodeNew(NULL);
@@ -326,6 +331,8 @@ static Int4
 s_ID1ArcgGIStateGet(Uint4 gi)
 {
     ID1serverRequestPtr id1reqp;
+    ID1serverBackPtr id1bp;
+
     Int4 state;
 
     id1reqp = ValNodeNew(NULL);
@@ -351,7 +358,7 @@ s_ID1ArcgGIStateGet(Uint4 gi)
 Int4    
 ID1ArcgGIStateGet(Uint4 gi)
 {
-	Int4 state,i;
+	Int4 state=0,i;
 	short erract;
 	ErrDesc err;
 	for (i = 0; i < ID_SERV_RETRIES; i++)
@@ -494,6 +501,7 @@ static Int4
 s_ID1ArchGIGet (SeqIdPtr sip)
 {
     ID1serverRequestPtr id1reqp;
+    ID1serverBackPtr id1bp;
     Int4 gi;
 
     id1reqp = ValNodeNew(NULL);
@@ -521,7 +529,7 @@ s_ID1ArchGIGet (SeqIdPtr sip)
 Int4
 ID1ArchGIGet (SeqIdPtr sip)
 {
-    Int4 gi, i;
+    Int4 gi=0, i;
     short erract;
     ErrDesc err;
     Int4 retval = 0;
@@ -552,7 +560,8 @@ void    SeqHistPrintTable (ID1SeqHistPtr ishp,FILE PNTR fp)
         Char    repl_date[20];
         SeqHistPtr shp;
 	SeqIdPtr   sip;
-	Uint4      old_gi=0,ent;
+	Uint4      old_gi=0,ent=0;
+	CharPtr	   sat=NULL;
 #ifdef IDFETCH_HTML_OUTPUT
 	Boolean    checked=FALSE;
 	if(!ishp){
@@ -572,8 +581,8 @@ void    SeqHistPrintTable (ID1SeqHistPtr ishp,FILE PNTR fp)
 		fprintf(fp,"%-20s%-20s%-20s\n","--","------","------");
 		fprintf(fp,"</b>");
 #else
-		fprintf(fp,"%-20s%-20s%-20s\n","GI","Loaded","Retrieval No.");
-		fprintf(fp,"%-20s%-20s%-20s\n","--","------","------------");
+		fprintf(fp,"%-20s%-20s%-10s%-20s\n","GI","Loaded","DB","Retrieval No.");
+		fprintf(fp,"%-20s%-20s%-10s%-20s\n","--","------","--","------------");
 
 #endif
 		
@@ -586,14 +595,18 @@ void    SeqHistPrintTable (ID1SeqHistPtr ishp,FILE PNTR fp)
 					}
 				}else if(sip->choice == SEQID_LOCAL){
 					ent = ((ObjectIdPtr)sip->data.ptrvalue)->id;
+					sat = "ID";
+				} else if(sip->choice == SEQID_GENERAL) {
+					ent = ((DbtagPtr)sip->data.ptrvalue)->tag->id;
+					sat = ((DbtagPtr)sip->data.ptrvalue)->db;
 				}
 			}
                         if(shp->replace_date){
                                 if(shp->replace_date->data[0]){
-                                        sprintf(repl_date,"%0.2d/%0.2d/%0.2d",
+                                        sprintf(repl_date,"%0.2d/%0.2d/%0.4d",
                                                         shp->replace_date->data[2],
                                                         shp->replace_date->data[3],
-                                                        shp->replace_date->data[1]);
+                                                        1900+shp->replace_date->data[1]);
                                 }else{
                                         sprintf(repl_date,"%-20s",shp->replace_date->str);
                                 }
@@ -608,7 +621,7 @@ void    SeqHistPrintTable (ID1SeqHistPtr ishp,FILE PNTR fp)
                                 old_gi,repl_date,ent,checked?"":"CHECKED");
 			checked = TRUE;
 #else
-			fprintf(fp,"%-20d\n",ent);
+			fprintf(fp,"%-10s%-20d\n",sat,ent);
 #endif
                 }
 	}
@@ -654,11 +667,11 @@ void   id_print_gi_state(Int4 state,CharPtr buf,Uint1 len)
         }
         dlen = strlen(buf);
         if(state & GI_IS_SUPPRESSED){
-                strncpy(buf+dlen,"|SUPPRESSED BY RULE",len - dlen - 1);
+                strncpy(buf+dlen,"|SUPPRESSED",len - dlen - 1);
         }
         dlen = strlen(buf);
         if(state & GI_IS_OVERRIDEN){
-                strncpy(buf+dlen,"|MANUALLY SUPPRESSED",len - dlen - 1);
+                strncpy(buf+dlen,"|WITHDRAWN",len - dlen - 1);
         }
         dlen = strlen(buf);
         if(state & GI_IS_CONFIDENTIAL){
