@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   07/15/97
 *
-* $Revision: 1.3 $
+* $Revision: 1.12 $
 *
 * File Description: 
 *       API for Taxonomy service
@@ -44,6 +44,33 @@
 *
 * RCS Modification History:
 * $Log: txcdproc.c,v $
+* Revision 1.12  2001/09/28 15:53:15  soussov
+* tax1e_maxTaxId() added
+*
+* Revision 1.11  2001/09/18 16:55:38  soussov
+* switching to the new ASN
+*
+* Revision 1.10  2001/09/18 14:42:26  soussov
+* Fixes error processing in case of reconnection
+*
+* Revision 1.9  2001/09/17 22:27:34  soussov
+* ErrPost call added if nothing was read from the service
+*
+* Revision 1.8  2001/06/13 21:23:19  soussov
+* makes purify happy
+*
+* Revision 1.7  1999/12/20 17:05:16  soussov
+* taxid4gi added
+*
+* Revision 1.6  1999/10/21 19:20:57  soussov
+* time tracking added
+*
+* Revision 1.5  1998/07/27 16:54:04  soussov
+* bug in txc_getChildren fixed
+*
+* Revision 1.4  1998/06/11 20:42:11  soussov
+* kill some warnings
+*
 * Revision 1.3  1998/04/01 17:34:02  soussov
 * changed tp include <>
 *
@@ -74,6 +101,7 @@ static char *this_file = __FILE__;
 #include <objtax1.h>
 #include <txclient.h>
 #include <taxinc.h>
+#include <time.h>
 
 #define TAXARCH_SERV_RETRIES 4
 
@@ -93,6 +121,22 @@ static Boolean reallyFinal = TRUE;
 static NI_DispatcherPtr dispatcher;
 static Boolean (*myNetInit) PROTO((void));
 
+static int tax_track= 0;
+static clock_t tax_time1;
+
+static void report_service_error(CharPtr proc_name, Taxon1RespPtr taxbp)
+{
+    if(taxbp == NULL) {
+	ErrPostEx(SEV_ERROR, 0, 0, "%s got NULL responce from service");
+    }
+    else if(taxbp->choice == Taxon1Resp_error) {
+	Taxon1ErrorPtr tep= taxbp->data.ptrvalue;
+	if(tep != NULL) {
+	    ErrPostEx(tep->level, 0, 0, "%s got %s", proc_name, tep->msg? tep->msg : "Empty message");
+	}
+    }
+}
+
 /*****************************************************************************
 *
 *   NetTaxArchReadAsn ()
@@ -111,7 +155,7 @@ static Taxon1RespPtr NetTaxArchReadAsn(void)
 
     taxbp = Taxon1RespAsnRead(asnin, NULL);
 
-    if (ErrFetch(&err))
+    if ((taxbp == NULL) || ErrFetch(&err))
     {
         ErrPost (CTX_UNKNOWN, 1, "Null message read from server");
     }
@@ -148,10 +192,15 @@ static Boolean TaxServInit(void)
     asnin= svcp->raip;
     asnout= svcp->waip;
 
+    if(getenv("TAX_TRACK_TIME") != NULL) {
+	tax_track= 1;
+    }
+
     /**********************************************************/
 
     taxrp= ValNodeNew(NULL);
     taxrp->choice = Taxon1Req_init;
+    if(tax_track) tax_time1= clock();
     Taxon1ReqAsnWrite(taxrp, asnout, NULL);
     AsnIoReset(asnout);
     Taxon1ReqFree (taxrp);
@@ -162,6 +211,11 @@ static Boolean TaxServInit(void)
     else  {
         taxbp->data.ptrvalue = NULL;
         Taxon1RespFree (taxbp);
+	if(tax_track) {
+	    double diff= (double)(clock() - tax_time1)/CLOCKS_PER_SEC;
+	    printf("tax1_init() %f\n", diff);
+	}
+	    
         return TRUE;
     }
 }
@@ -249,6 +303,7 @@ static Taxon1NamePtr s_findname(CharPtr sname)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
     if(taxbp->choice != Taxon1Resp_findname) {
+	report_service_error("tax_findByName", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -267,6 +322,7 @@ static Taxon1NamePtr findname(CharPtr sname)
     ErrDesc err;
     Taxon1NamePtr tnp = NULL;
 
+    if(tax_track) tax_time1= clock();
     for (i= TAXARCH_SERV_RETRIES; i >= 0; --i) {
         ErrGetOpts(&erract, NULL);
         ErrSetOpts(ERR_IGNORE, 0);
@@ -279,6 +335,11 @@ static Taxon1NamePtr findname(CharPtr sname)
 	
 	if(!ReestablishNetTaxArch()) break;
     }
+    if(tax_track) {
+	double diff= (double)(clock() - tax_time1)/CLOCKS_PER_SEC;
+	printf("tax1_findByName() %f\n", diff);
+    }
+	
 
     return(tnp);
 }
@@ -332,6 +393,7 @@ static Taxon1NamePtr s_getorgnames(Int4 tax_id)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
     if(taxbp->choice != Taxon1Resp_getorgnames) {
+	report_service_error("tax_getOrgNames", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -417,6 +479,7 @@ static Int4 s_getdesignator(CharPtr sname)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return 0;
     
     if(taxbp->choice != Taxon1Resp_getdesignator) {
+	report_service_error("tax_getDesignator", taxbp);
 	Taxon1RespFree(taxbp);
 	return 0;
     }
@@ -472,6 +535,7 @@ static Int4 s_getunique(CharPtr sname)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return 0;
     
     if(taxbp->choice != Taxon1Resp_getunique) {
+	report_service_error("tax_uniqueName", taxbp);
 	Taxon1RespFree(taxbp);
 	return 0;
     }
@@ -528,6 +592,7 @@ static Int4 s_getidbyorg(OrgRefPtr orgRef)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return 0;
     
     if(taxbp->choice != Taxon1Resp_getidbyorg) {
+	report_service_error("txc_getTaxIdByOrgRef", taxbp);
 	Taxon1RespFree(taxbp);
 	return 0;
     }
@@ -545,6 +610,7 @@ static Int4 getidbyorg(OrgRefPtr orgRef)
     ErrDesc err;
     Int4 tax_id= 0;
 
+    if(tax_track) tax_time1= clock();
     for (i= TAXARCH_SERV_RETRIES; i >= 0; --i) {
         ErrGetOpts(&erract, NULL);
         ErrSetOpts(ERR_IGNORE, 0);
@@ -558,6 +624,12 @@ static Int4 getidbyorg(OrgRefPtr orgRef)
 	if(!ReestablishNetTaxArch()) break;
     }
 
+    if(tax_track) {
+	double diff= (double)(clock() - tax_time1)/CLOCKS_PER_SEC;
+	printf("tax1_getTaxIdByOrgRef() %f\n", diff);
+    }
+    
+
     return tax_id;
 }
 
@@ -588,14 +660,14 @@ Int4 tax_getIdByName(char* sname, char* qualif, Int1 id)
     return tax_id;
 }
 
-static Taxon1InfoPtr s_getlineage(Int4 tax_id)
+static Taxon1NamePtr s_getlineage(Int4 tax_id)
 {
     Taxon1ReqPtr taxrp;
     Taxon1RespPtr taxbp;
-    Taxon1InfoPtr tnp;
+    Taxon1NamePtr tnp;
 
     if((taxrp= ValNodeNew(NULL)) == NULL) return NULL;
-    taxrp->choice= Taxon1Req_getlineage;
+    taxrp->choice= Taxon1Req_taxalineage;
     taxrp->data.intvalue= tax_id;
     Taxon1ReqAsnWrite(taxrp, asnout, NULL);
     AsnIoReset(asnout);
@@ -603,7 +675,8 @@ static Taxon1InfoPtr s_getlineage(Int4 tax_id)
 
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
-    if(taxbp->choice != Taxon1Resp_getlineage) {
+    if(taxbp->choice != Taxon1Resp_taxalineage) {
+	report_service_error("txc_getLineage", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -616,13 +689,14 @@ static Taxon1InfoPtr s_getlineage(Int4 tax_id)
 }
 
 
-static Taxon1InfoPtr getlineage(Int4 tax_id)
+static Taxon1NamePtr getlineage(Int4 tax_id)
 {
     Int4 i;
     short erract;
     ErrDesc err;
-    Taxon1InfoPtr tnp = NULL;
+    Taxon1NamePtr tnp = NULL;
 
+    if(tax_track) tax_time1= clock();
     for (i= TAXARCH_SERV_RETRIES; i >= 0; --i) {
         ErrGetOpts(&erract, NULL);
         ErrSetOpts(ERR_IGNORE, 0);
@@ -636,14 +710,19 @@ static Taxon1InfoPtr getlineage(Int4 tax_id)
 	if(!ReestablishNetTaxArch()) break;
     }
 
+    if(tax_track) {
+	double diff= (double)(clock() - tax_time1)/CLOCKS_PER_SEC;
+	printf("tax1_getLineage() %f\n", diff);
+    }
+
     return(tnp);
 }
 
 TXC_TreeNodePtr* txc_getLineage(Int4 lin_id, Int4Ptr lin_len)
 {
     TXC_TreeNodePtr* lin;
-    Int4 n= 0, i, k;
-    Taxon1InfoPtr tnp, tnp_list= (lin_id > 0)? getlineage(lin_id) : NULL;
+    Int4 n= 0, i, k, k1;
+    Taxon1NamePtr tnp, tnp_list= (lin_id > 0)? getlineage(lin_id) : NULL;
 
     if(lin_len != NULL) *lin_len= 0;
 
@@ -655,7 +734,7 @@ TXC_TreeNodePtr* txc_getLineage(Int4 lin_id, Int4Ptr lin_len)
 	/* no memory, just free tnp_list */
 	for(i= 0; i < n; i++) {
 	    tnp= tnp_list->next;
-	    Taxon1InfoFree(tnp_list);
+	    Taxon1NameFree(tnp_list);
 	    tnp_list= tnp;
 	}
 	return NULL;
@@ -663,13 +742,18 @@ TXC_TreeNodePtr* txc_getLineage(Int4 lin_id, Int4Ptr lin_len)
 
     for(i= 0; i < n; i++) {
 	tnp= tnp_list->next;
-	k= StringLen(tnp_list->sval);
-	if((lin[i]= MemNew(k+10)) != NULL) {
-	    lin[i]->tax_id= tnp_list->ival1;
-	    lin[i]->flags= tnp_list->ival2;
-	    StringCpy(lin[i]->node_label, tnp_list->sval);
+	k= StringLen(tnp_list->oname);
+	k1= tnp_list->uname? (StringLen(tnp_list->uname) + 2) : 0;
+	if((lin[i]= MemNew(k+k1+10)) != NULL) {
+	    lin[i]->tax_id= tnp_list->taxid;
+	    lin[i]->flags= tnp_list->cde | TXC_STHIDE;
+	    memcpy(lin[i]->node_label, tnp_list->oname, k+1);
+	    if(k1) {
+		memcpy(lin[i]->node_label + (k+1), tnp_list->uname, k1 - 1);
+		lin[i]->flags^= TXC_STHIDE;
+	    }
 	}
-	Taxon1InfoFree(tnp_list);
+	Taxon1NameFree(tnp_list);
 	tnp_list= tnp;
     }
 	    
@@ -677,14 +761,14 @@ TXC_TreeNodePtr* txc_getLineage(Int4 lin_id, Int4Ptr lin_len)
     return lin;
 }
 		
-static Taxon1InfoPtr s_getchildren(Int4 tax_id)
+static Taxon1NamePtr s_getchildren(Int4 tax_id)
 {
     Taxon1ReqPtr taxrp;
     Taxon1RespPtr taxbp;
-    Taxon1InfoPtr tnp;
+    Taxon1NamePtr tnp;
 
     if((taxrp= ValNodeNew(NULL)) == NULL) return NULL;
-    taxrp->choice= Taxon1Req_getchildren;
+    taxrp->choice= Taxon1Req_taxachildren;
     taxrp->data.intvalue= tax_id;
     Taxon1ReqAsnWrite(taxrp, asnout, NULL);
     AsnIoReset(asnout);
@@ -692,7 +776,8 @@ static Taxon1InfoPtr s_getchildren(Int4 tax_id)
 
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
-    if(taxbp->choice != Taxon1Resp_getchildren) {
+    if(taxbp->choice != Taxon1Resp_taxachildren) {
+	report_service_error("txc_getChildren", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -705,12 +790,12 @@ static Taxon1InfoPtr s_getchildren(Int4 tax_id)
 }
 
 
-static Taxon1InfoPtr getchildren(Int4 tax_id)
+static Taxon1NamePtr getchildren(Int4 tax_id)
 {
     Int4 i;
     short erract;
     ErrDesc err;
-    Taxon1InfoPtr tnp = NULL;
+    Taxon1NamePtr tnp = NULL;
 
     for (i= TAXARCH_SERV_RETRIES; i >= 0; --i) {
         ErrGetOpts(&erract, NULL);
@@ -731,8 +816,8 @@ static Taxon1InfoPtr getchildren(Int4 tax_id)
 TXC_TreeNodePtr* txc_getChildren(Int4 node_id, Int4Ptr nof_children)
 {
     TXC_TreeNodePtr* lin;
-    Int4 n= 0, i, k;
-    Taxon1InfoPtr tnp, tnp_list= (node_id > 0)? getchildren(node_id) : NULL;
+    Int4 n= 0, i, k, k1;
+    Taxon1NamePtr tnp, tnp_list= getchildren(node_id);
 
     if(nof_children != NULL) *nof_children= 0;
 
@@ -744,7 +829,7 @@ TXC_TreeNodePtr* txc_getChildren(Int4 node_id, Int4Ptr nof_children)
 	/* no memory, just free tnp_list */
 	for(i= 0; i < n; i++) {
 	    tnp= tnp_list->next;
-	    Taxon1InfoFree(tnp_list);
+	    Taxon1NameFree(tnp_list);
 	    tnp_list= tnp;
 	}
 	return NULL;
@@ -752,13 +837,27 @@ TXC_TreeNodePtr* txc_getChildren(Int4 node_id, Int4Ptr nof_children)
 
     for(i= 0; i < n; i++) {
 	tnp= tnp_list->next;
-	k= StringLen(tnp_list->sval);
-	if((lin[i]= MemNew(k+10)) != NULL) {
-	    lin[i]->tax_id= tnp_list->ival1;
-	    lin[i]->flags= tnp_list->ival2;
-	    StringCpy(lin[i]->node_label, tnp_list->sval);
+	if(tnp_list->cde == 0) {
+	    k= 0;
+	    k1= 0;
 	}
-	Taxon1InfoFree(tnp_list);
+	else {
+	    k= StringLen(tnp_list->oname);
+	    k1= tnp_list->uname? (StringLen(tnp_list->uname) + 2) : 0;
+	}
+	if((lin[i]= MemNew(k+k1+10)) != NULL) {
+	    lin[i]->tax_id= tnp_list->taxid;
+	    lin[i]->flags= k? (tnp_list->cde | TXC_STHIDE) : 0;
+	    if(k) {
+		memcpy(lin[i]->node_label, tnp_list->oname, k+1);
+		if(k1) {
+		    memcpy(lin[i]->node_label + (k+1), tnp_list->uname, k1 - 1);
+		    lin[i]->flags^= TXC_STHIDE;
+		}
+	    }
+	    
+	}
+	Taxon1NameFree(tnp_list);
 	tnp_list= tnp;
     }
 	    
@@ -782,6 +881,7 @@ static Taxon1InfoPtr s_getcde(void)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
     if(taxbp->choice != Taxon1Resp_getcde) {
+	report_service_error("txc_loadNameClasses", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -817,7 +917,7 @@ static Taxon1InfoPtr getcde(void)
     return(tnp);
 }
 
-Boolean txc_loadNameClasses()
+Boolean txc_loadNameClasses(void)
 {
     Taxon1InfoPtr tnp, tnp_list= getcde();
 
@@ -849,6 +949,7 @@ static Taxon1InfoPtr s_getranks(void)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
     if(taxbp->choice != Taxon1Resp_getranks) {
+	report_service_error("txc_loadRanks", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -884,7 +985,7 @@ static Taxon1InfoPtr getranks(void)
     return(tnp);
 }
 
-Boolean txc_loadRanks()
+Boolean txc_loadRanks(void)
 {
     Taxon1InfoPtr tnp, tnp_list= getranks();
 
@@ -915,6 +1016,7 @@ static Taxon1InfoPtr s_getdivs(void)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
     if(taxbp->choice != Taxon1Resp_getdivs) {
+	report_service_error("txc_loadDivisions", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -950,7 +1052,7 @@ static Taxon1InfoPtr getdivs(void)
     return(tnp);
 }
 
-Boolean txc_loadDivisions()
+Boolean txc_loadDivisions(void)
 {
     Taxon1InfoPtr tnp, tnp_list= getdivs();
     Uint4 w;
@@ -990,6 +1092,7 @@ static Taxon1InfoPtr s_getgcs(void)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
     if(taxbp->choice != Taxon1Resp_getgcs) {
+	report_service_error("txc_loadGCs", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -1025,7 +1128,7 @@ static Taxon1InfoPtr getgcs(void)
     return(tnp);
 }
 
-Boolean txc_loadGCs()
+Boolean txc_loadGCs(void)
 {
     Taxon1InfoPtr tnp, tnp_list= getgcs();
 
@@ -1049,7 +1152,7 @@ Boolean txc_loadGCs()
 
 static Boolean ReestablishNetTaxArch(void)
 {
-    return GenericReestablishNet("Taxon2", TRUE);
+    return GenericReestablishNet("Taxon2", FALSE);
 }
 
 /*****************************************************************************
@@ -1077,16 +1180,20 @@ static Boolean GenericReestablishNet(CharPtr svcName, Boolean showErrs)
 
     if (!myNetInit())
     {
-        sprintf (buf, "%s get failed; re-contacting dispatcher", svcName);
-        MonitorStrValue(mon, buf);
+	if(mon) {
+	    sprintf (buf, "%s get failed; re-contacting dispatcher", svcName);
+	    MonitorStrValue(mon, buf);
+	}
         retval = FALSE;
         if (ForceNetInit())
         { /* successfully established contact w/dispatcher */
-            sprintf (buf, "%s get failed; re-requesting %s service",
-                     svcName, svcName);
-            MonitorStrValue(mon, buf);
-            retval = myNetInit();
-        }
+	    if(mon) {
+		sprintf (buf, "%s get failed; re-requesting %s service",
+			 svcName, svcName);
+		MonitorStrValue(mon, buf);
+	    }
+	    retval = myNetInit();
+	}
         else {
             ErrPost(CTX_UNKNOWN, 1, "Unable to re-contact dispatcher");
             if (showErrs) {
@@ -1095,7 +1202,7 @@ static Boolean GenericReestablishNet(CharPtr svcName, Boolean showErrs)
         }
     }
 
-    MonitorFree(mon);
+    if(mon) MonitorFree(mon);
 
     if (! retval )
     {
@@ -1192,6 +1299,7 @@ static _subspecPtr s_SSget(Int4 tax_id, _subspecPtr ss)
     if((taxbp= NetTaxArchReadAsn()) == NULL) return NULL;
     
     if(taxbp->choice != Taxon1Resp_getorgmod) {
+	report_service_error("tax_SSget", taxbp);
 	Taxon1RespFree(taxbp);
 	return NULL;
     }
@@ -1221,6 +1329,7 @@ _subspecPtr tax_SSget(Int4 tax_id, _subspecPtr ss)
     ErrDesc err;
     _subspecPtr res= NULL;
 
+    if(tax_track) tax_time1= clock();
     for (i= TAXARCH_SERV_RETRIES; i >= 0; --i) {
         ErrGetOpts(&erract, NULL);
         ErrSetOpts(ERR_IGNORE, 0);
@@ -1233,6 +1342,114 @@ _subspecPtr tax_SSget(Int4 tax_id, _subspecPtr ss)
 	
 	if(!ReestablishNetTaxArch()) break;
     }
+    if(tax_track) {
+	double diff= (double)(clock() - tax_time1)/CLOCKS_PER_SEC;
+	printf("tax1_SSget() %f\n", diff);
+    }
 
     return res;
 }
+
+static Int4 s_getTaxId4GI(Int4 gi)
+{
+    Taxon1ReqPtr taxrp;
+    Taxon1RespPtr taxbp;
+    Int4 tax_id;
+
+    if((taxrp= ValNodeNew(NULL)) == NULL) return 0;
+    taxrp->choice= Taxon1Req_id4gi;
+    taxrp->data.intvalue= gi;
+    Taxon1ReqAsnWrite(taxrp, asnout, NULL);
+    AsnIoReset(asnout);
+    Taxon1ReqFree(taxrp);
+
+    if((taxbp= NetTaxArchReadAsn()) == NULL) return 0;
+    
+    if(taxbp->choice != Taxon1Resp_id4gi) {
+	report_service_error("tax_getTaxId4GI", taxbp);
+	Taxon1RespFree(taxbp);
+	return 0;
+    }
+
+    tax_id= taxbp->data.intvalue;
+    Taxon1RespFree(taxbp);
+
+    return tax_id;
+}
+
+static Int4 getTaxId4GI(Int4 gi)
+{
+    Int4 i;
+    short erract;
+    ErrDesc err;
+    Int4 tax_id= 0;
+
+    for (i= TAXARCH_SERV_RETRIES; i >= 0; --i) {
+        ErrGetOpts(&erract, NULL);
+        ErrSetOpts(ERR_IGNORE, 0);
+        ErrFetch(&err);
+
+        tax_id = s_getTaxId4GI(gi);
+
+        ErrSetOpts(erract, 0);
+        if (!ErrFetch(&err))  break; /* success */
+	
+	if(!ReestablishNetTaxArch()) break;
+    }
+
+    return tax_id;
+}
+
+Int4 tax_getTaxId4GI(Int4 gi)
+{
+    return getTaxId4GI(gi);
+}
+
+static Int4 s_maxTaxId()
+{
+    Taxon1ReqPtr taxrp;
+    Taxon1RespPtr taxbp;
+    Int4 tax_id;
+
+    if((taxrp= ValNodeNew(NULL)) == NULL) return 0;
+    taxrp->choice= Taxon1Req_maxtaxid;
+    Taxon1ReqAsnWrite(taxrp, asnout, NULL);
+    AsnIoReset(asnout);
+    Taxon1ReqFree(taxrp);
+
+    if((taxbp= NetTaxArchReadAsn()) == NULL) return 0;
+    
+    if(taxbp->choice != Taxon1Resp_maxtaxid) {
+	report_service_error("tax1e_maxTaxId", taxbp);
+	Taxon1RespFree(taxbp);
+	return 0;
+    }
+
+    tax_id= taxbp->data.intvalue;
+    Taxon1RespFree(taxbp);
+
+    return tax_id;
+}
+
+Int4 tax1e_maxTaxId()
+{
+    Int4 i;
+    short erract;
+    ErrDesc err;
+    Int4 tax_id= 0;
+
+    for (i= TAXARCH_SERV_RETRIES; i >= 0; --i) {
+        ErrGetOpts(&erract, NULL);
+        ErrSetOpts(ERR_IGNORE, 0);
+        ErrFetch(&err);
+
+        tax_id = s_maxTaxId();
+
+        ErrSetOpts(erract, 0);
+        if (!ErrFetch(&err))  break; /* success */
+	
+	if(!ReestablishNetTaxArch()) break;
+    }
+
+    return tax_id;
+}
