@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.4 $
+* $Revision: 6.8 $
 *
 * File Description:  Object manager for module NCBI-Seqalign
 *
@@ -104,7 +104,7 @@ static Int2 NEAR SeqAlignLabelContent (SeqAlignPtr sap, CharPtr buf, Int2 buflen
 		case 1 :
 			ddp = (DenseDiagPtr) sap->segs;
 			if (ddp != NULL) {
-				for (sip = ddp->id, j = 0; sip != NULL, j < 2; sip = sip->next, j++) {
+				for (sip = ddp->id, j = 0; sip != NULL && j < 2; sip = sip->next, j++) {
 					SeqIdWrite (sip, str, PRINTID_FASTA_SHORT, sizeof (str));
 					if (j > 0) {
 						StringCat (label, ",");
@@ -119,7 +119,7 @@ static Int2 NEAR SeqAlignLabelContent (SeqAlignPtr sap, CharPtr buf, Int2 buflen
 		case 2 :
 			dsp = (DenseSegPtr) sap->segs;
 			if (dsp != NULL) {
-				for (sip = dsp->ids, j = 0; sip != NULL, j < 2; sip = sip->next, j++) {
+				for (sip = dsp->ids, j = 0; sip != NULL && j < 2; sip = sip->next, j++) {
 					SeqIdWrite (sip, str, PRINTID_FASTA_SHORT, sizeof (str));
 					if (j > 0) {
 						StringCat (label, ",");
@@ -134,7 +134,7 @@ static Int2 NEAR SeqAlignLabelContent (SeqAlignPtr sap, CharPtr buf, Int2 buflen
 		case 3 :
 			ssp = (StdSegPtr) sap->segs;
 			if (ssp != NULL && ssp->loc != NULL) {
-				for (slp = ssp->loc, j = 0; slp != NULL, j < 2; slp = slp->next, j++) {
+				for (slp = ssp->loc, j = 0; slp != NULL && j < 2; slp = slp->next, j++) {
 					sip = SeqLocId (slp);
 					SeqIdWrite (sip, str, PRINTID_FASTA_SHORT, sizeof (str));
 					if (j > 0) {
@@ -252,7 +252,7 @@ static Uint2 LIBCALLBACK SeqAlignSubTypeFunc (Pointer ptr)
 *   SeqAlignAsnLoad()
 *
 *****************************************************************************/
-NLM_EXTERN Boolean LIBCALL SeqAlignAsnLoad ()
+NLM_EXTERN Boolean LIBCALL SeqAlignAsnLoad (void)
 {
     if (loaded)
         return TRUE;
@@ -291,12 +291,35 @@ NLM_EXTERN Boolean LIBCALL SeqAlignAsnLoad ()
 *   SeqAlign Routines
 *
 *****************************************************************************/
+NLM_EXTERN SeqAlignIndexPtr LIBCALL SeqAlignIndexFree (SeqAlignIndexPtr saip)
+{
+	Boolean retval;
+	VoidPtr ptr;
+	SeqAlignIndexPtr tmp = NULL;
+
+	if (saip == NULL)
+		return saip;
+
+	if (saip->freefunc != NULL)
+	{
+		ptr = (VoidPtr)(saip);
+		retval = (*(saip->freefunc))(ptr);
+		if (! retval)
+			ErrPostEx(SEV_ERROR,0,0,"SeqAlignFreeFunc: saip->freefunc returned FALSE");
+		return tmp;
+	}
+
+	ErrPostEx(SEV_ERROR,0,0,"SeqAlignIndexFree: saip lacking a freefunc");
+
+	return tmp;
+}
+
 /*****************************************************************************
 *
 *   SeqAlignNew()
 *
 *****************************************************************************/
-NLM_EXTERN SeqAlignPtr LIBCALL SeqAlignNew ()
+NLM_EXTERN SeqAlignPtr LIBCALL SeqAlignNew (void)
 {
     return (SeqAlignPtr)MemNew(sizeof(SeqAlign));
 }
@@ -314,6 +337,7 @@ NLM_EXTERN SeqAlignPtr LIBCALL SeqAlignFree (SeqAlignPtr sap)
     if (sap == NULL)
         return (SeqAlignPtr)NULL;
 
+    sap->saip = SeqAlignIndexFree (sap->saip);  /* free index elements */
     
     switch (sap->segtype)
     {
@@ -345,6 +369,7 @@ NLM_EXTERN SeqAlignPtr LIBCALL SeqAlignFree (SeqAlignPtr sap)
     }
     ScoreSetFree(sap->score);
 	SeqLocSetFree(sap->bounds);
+        SeqIdFree(sap->master);
 
 	ObjMgrDelete(OBJ_SEQALIGN, (Pointer)sap);
 
@@ -612,7 +637,7 @@ erret:
 *   ScoreNew()
 *
 *****************************************************************************/
-NLM_EXTERN ScorePtr LIBCALL ScoreNew ()
+NLM_EXTERN ScorePtr LIBCALL ScoreNew (void)
 {
     return (ScorePtr)MemNew(sizeof(Score));
 }
@@ -1650,7 +1675,7 @@ NLM_EXTERN SeqAlignPtr LIBCALL SeqAlignSetFree (SeqAlignPtr sap)
     return NULL;
 }
 
-NLM_EXTERN SeqAlignPtr LIBCALL SeqAlignSetNew ()
+NLM_EXTERN SeqAlignPtr LIBCALL SeqAlignSetNew (void)
 {
     return SeqAlignNew();
 }
