@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.0 $
+* $Revision: 6.6 $
 *
 * File Description:  Object manager for module NCBI-Seqloc
 *
@@ -41,6 +41,24 @@
 *
 *
 * $Log: objloc.c,v $
+* Revision 6.6  2001/08/06 20:07:26  ostell
+* added SEQID_TPG, TPE, TPD types
+*
+* Revision 6.5  2001/01/31 15:24:20  kans
+* PatentSeqId.seqid is now an Int4 (JO)
+*
+* Revision 6.4  2000/04/05 21:42:33  hurwitz
+* made SeqIdSetDup consistent with declaration
+*
+* Revision 6.3  2000/04/05 18:11:54  dondosha
+* Moved SeqIdSetDup from mblast.c
+*
+* Revision 6.2  1998/08/26 17:46:12  kans
+* fixed -v -fd warnings in label functions
+*
+* Revision 6.1  1998/08/24 18:28:05  kans
+* removed solaris -v -fd warnings
+*
 * Revision 6.0  1997/08/25 18:50:03  madden
 * Revision changed to 6.0
 *
@@ -167,6 +185,7 @@ NLM_EXTERN Int2 LIBCALL SeqIdLabel ( SeqIdPtr sip, CharPtr buffer, Int2 buflen,
 {
 	Char label[40];
 	Int2 diff, len;
+	Int2 rsult = 0;
 
 	if ((sip == NULL) || (buflen < 1))
 		return 0;
@@ -186,9 +205,9 @@ NLM_EXTERN Int2 LIBCALL SeqIdLabel ( SeqIdPtr sip, CharPtr buffer, Int2 buflen,
 			return len - buflen;
 		case OM_LABEL_TYPE:
 		default:
-			return LabelCopy(buffer, seqidname, buflen);
+			rsult = LabelCopy(buffer, seqidname, buflen);
 	}
-	return 0;
+	return rsult;
 }
 
 static Uint2 LIBCALLBACK SeqIdSubTypeFunc (Pointer ptr)
@@ -212,7 +231,7 @@ static Boolean loaded = FALSE;
 *   Seqloc loader
 *
 *****************************************************************************/
-NLM_EXTERN Boolean LIBCALL SeqLocAsnLoad ()
+NLM_EXTERN Boolean LIBCALL SeqLocAsnLoad (void)
 {
     if (loaded)
         return TRUE;
@@ -271,6 +290,9 @@ NLM_EXTERN Boolean LIBCALL SeqLocAsnLoad ()
     13 = ddbj Textseq-id    -- ddbj
 	14 = prf Textseq-id ,         -- PRF SEQDB
 	15 = pdb PDB-seq-id          -- PDB sequence
+    16 = tpg Textseq-id ,         -- Third Party Annot/Seq Genbank
+    17 = tpe Textseq-id ,         -- Third Party Annot/Seq EMBL
+    18 = tpd Textseq-id }         -- Third Party Annot/Seq DDBJ
 *
 *****************************************************************************/
 /*****************************************************************************
@@ -305,6 +327,9 @@ NLM_EXTERN SeqIdPtr LIBCALL SeqIdFree (SeqIdPtr anp)
         case SEQID_OTHER:     /* other */
         case SEQID_DDBJ:
 		case SEQID_PRF:
+	case SEQID_TPG:
+	case SEQID_TPE:
+	case SEQID_TPD:
             TextSeqIdFree((TextSeqIdPtr)pnt);
             break;
         case SEQID_PATENT:      /* patent seq id */
@@ -420,6 +445,18 @@ NLM_EXTERN Boolean LIBCALL SeqIdAsnWrite (SeqIdPtr anp, AsnIoPtr aip, AsnTypePtr
             writetype = SEQ_ID_pdb;
             func = (AsnWriteFunc) PDBSeqIdAsnWrite;
             break;
+        case SEQID_TPG:      /* tpg   */
+            writetype = SEQ_ID_tpg;
+            func = (AsnWriteFunc) TextSeqIdAsnWrite;
+            break;
+        case SEQID_TPE:      /* tpe   */
+            writetype = SEQ_ID_tpe;
+            func = (AsnWriteFunc) TextSeqIdAsnWrite;
+            break;
+        case SEQID_TPD:      /* tpd   */
+            writetype = SEQ_ID_tpd;
+            func = (AsnWriteFunc) TextSeqIdAsnWrite;
+            break;
     }
     if (writetype != NULL)
         retval = (* func)(pnt, aip, writetype);   /* write it out */
@@ -550,6 +587,21 @@ NLM_EXTERN SeqIdPtr LIBCALL SeqIdAsnRead (AsnIoPtr aip, AsnTypePtr orig)
         choice = SEQID_PDB;
         func = (AsnReadFunc) PDBSeqIdAsnRead;
     }
+    else if (atp == SEQ_ID_tpg)
+    {
+        choice = SEQID_TPG;
+        func = (AsnReadFunc) TextSeqIdAsnRead;
+    }
+    else if (atp == SEQ_ID_tpe)
+    {
+        choice = SEQID_TPE;
+        func = (AsnReadFunc) TextSeqIdAsnRead;
+    }
+    else if (atp == SEQ_ID_tpd)
+    {
+        choice = SEQID_TPD;
+        func = (AsnReadFunc) TextSeqIdAsnRead;
+    }
     else
         goto erret;
 
@@ -625,6 +677,9 @@ NLM_EXTERN SeqIdPtr LIBCALL SeqIdDup (SeqIdPtr oldid)
 		case SEQID_OTHER:
 		case SEQID_DDBJ:
 		case SEQID_PRF:
+		case SEQID_TPG:
+		case SEQID_TPE:
+		case SEQID_TPD:
 			at = (TextSeqIdPtr)oldid->data.ptrvalue;
             bt = TextSeqIdNew();
 			if (bt == NULL) return NULL;
@@ -650,6 +705,33 @@ NLM_EXTERN SeqIdPtr LIBCALL SeqIdDup (SeqIdPtr oldid)
 	return newid;
 }
 
+/*****************************************************************************
+*
+*   SeqIdPtr SeqIdSetDup(oldid)
+*       duplicates a chain of SeqId's
+*
+*****************************************************************************/
+NLM_EXTERN SeqIdPtr LIBCALL 
+SeqIdSetDup(SeqIdPtr seqid)
+{
+   SeqIdPtr sid_head, sid, seqid_var;
+
+   if (seqid == NULL)
+      return seqid;
+   else {
+      seqid_var = seqid;
+      sid_head = sid = SeqIdDup(seqid);
+   }
+
+   while ((seqid_var = seqid_var->next) != NULL) {
+      sid->next = SeqIdDup(seqid_var);
+      sid = sid->next;
+   }
+
+   return sid_head;
+}
+
+
 /*****************************************************************************
 *
 *   SeqIdSetFree()
@@ -1034,7 +1116,7 @@ NLM_EXTERN PatentSeqIdPtr LIBCALL PatentSeqIdAsnRead (AsnIoPtr aip, AsnTypePtr o
         goto erret;
     if (AsnReadVal(aip, atp, &av) <= 0)
         goto erret;
-    psip->seqid = (Int2)av.intvalue;
+    psip->seqid = av.intvalue;
     atp = AsnReadId(aip, amp, atp);   /* read the cit */
     if (atp == NULL)
         goto erret;
@@ -1059,7 +1141,7 @@ erret:
 *   GiimNew()
 *
 *****************************************************************************/
-NLM_EXTERN GiimPtr LIBCALL GiimNew ()
+NLM_EXTERN GiimPtr LIBCALL GiimNew (void)
 {
     return (GiimPtr)MemNew(sizeof(Giim));
 }
@@ -2720,5 +2802,3 @@ erret:
     goto ret;
 }
 
-
-
