@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.0 $
+* $Revision: 6.4 $
 *
 * File Description:  Object manager for module NCBI-Pub
 *
@@ -41,6 +41,20 @@
 *
 *
 * $Log: objpub.c,v $
+* Revision 6.4  2001/11/20 14:50:54  kans
+* PubLabelUnique return immediately if PMID or MUID, already unique
+*
+* Revision 6.3  2000/06/15 19:13:28  yaschenk
+* PubAsnWrite should not fail when pmid is stripped
+*
+* Revision 6.2  1998/12/30 20:28:34  ostell
+* fixed bug in PubLabelUnique() for author list. If alp->names is NULL,
+* diff was not initialized. So operations using diff moved inside
+* if (vnp2 != NULL)
+*
+* Revision 6.1  1998/08/24 18:28:08  kans
+* removed solaris -v -fd warnings
+*
 * Revision 6.0  1997/08/25 18:50:27  madden
 * Revision changed to 6.0
 *
@@ -267,7 +281,7 @@ static Uint2 LIBCALLBACK PubSetSubTypeFunc (Pointer ptr)
 *   PubAsnLoad()
 *
 *****************************************************************************/
-NLM_EXTERN Boolean LIBCALL PubAsnLoad ()
+NLM_EXTERN Boolean LIBCALL PubAsnLoad (void)
 {
     if (loaded)
         return TRUE;
@@ -427,6 +441,7 @@ NLM_EXTERN Boolean LIBCALL PubAsnWrite (ValNodePtr anp, AsnIoPtr aip, AsnTypePtr
 	if ((aip->spec_version > 0 && aip->spec_version < 5) && anp->choice >= 13)  /* ASN4 strip new value */
 	{
 		ErrPostEx(SEV_ERROR,0,0,"ASN4: PubMedId stripped");
+		retval=TRUE;
 		goto erret;
 	}
 
@@ -1336,11 +1351,13 @@ NLM_EXTERN Int2 LIBCALL PubLabelUnique (ValNodePtr pub, CharPtr buf, Int2 buflen
 			sprintf(tbuf, "NLM%ld", (long)(pub->data.intvalue));
 			diff = LabelCopy(buf, tbuf, buflen);
 			buflen -= diff;
+			return (len - buflen); /* already unique */
 			break;
 		case PUB_PMid:
 			sprintf(tbuf, "PM%ld", (long)(pub->data.intvalue));
 			diff = LabelCopy(buf, tbuf, buflen);
 			buflen -= diff;
+			return (len - buflen); /* already unique */
 			break;
 		case PUB_Equiv:
 			for (i = 0; i < 5; i++)
@@ -1489,7 +1506,7 @@ cit_book:   imp = cbp->imp;
 				if ((cit = StringSave(cgp->cit)) != NULL) {
 					if (!unique) {
 						for (s = cit + StringLen(cit) -1; 
-									s > cit && *s !='|';   s--);
+									s > cit && *s !='|';   s--) continue;
 						if (*s == '|' ) {
 							*s = '\0';
 						}
@@ -1533,9 +1550,9 @@ cit_book:   imp = cbp->imp;
 			}
 			else
 				diff = LabelCopy(buf, (CharPtr)(vnp2->data.ptrvalue), buflen);
-		}
 		buflen -= diff;
 		buf += diff;
+		}
 	}
 
 	if (dp != NULL)
