@@ -30,8 +30,27 @@ Author: Gennadiy Savchuk, Jinqhui Zhang, Tom Madden
 Contents: Functions to perform both local and global banded alignments.
 
 ****************************************************************************/
-/* $Revision: 6.10 $ */
-/* $Log: bandalgn.c,v $
+/* $Revision: 6.16 $
+ *
+ * $Log: bandalgn.c,v $
+ * Revision 6.16  2001/05/25 19:32:48  vakatov
+ * Nested comment typo fixed
+ *
+ * Revision 6.15  2000/02/10 22:47:06  vakatov
+ * DLL'zation for MSVC on PC, Win-NT
+ *
+ * Revision 6.14  1999/09/01 21:46:49  chappey
+ * SeqPortFree in load_data
+ *
+ * Revision 6.13  1999/07/13 15:39:48  sicotte
+ * Fixed casting warnings
+ *
+ * Revision 6.12  1999/04/06 15:21:00  sicotte
+ * Fixed prototype problems and lint casting and.
+ *
+ * Revision 6.11  1998/08/24 20:20:32  kans
+ * fixed -v -fd warnings
+ *
  * Revision 6.10  1998/05/04 19:58:21  tatiana
  * change SEV_ERROR to SEV_WARNING
  *
@@ -96,9 +115,10 @@ Contents: Functions to perform both local and global banded alignments.
 #include <salutil.h>
 #include <bandalgn.h>
 
-PSUGapOptionsPtr LIBCALL
+
+NLM_EXTERN PSUGapOptionsPtr
 PSUGapOptionsDelete(PSUGapOptionsPtr options)
-     
+
 {
   options = MemFree(options);
   
@@ -109,7 +129,7 @@ PSUGapOptionsDelete(PSUGapOptionsPtr options)
 	Create the PSUGapOptionsPtr and fill in with default values.
 */
 
-PSUGapOptionsPtr LIBCALL
+NLM_EXTERN PSUGapOptionsPtr
 PSUGapOptionsCreate(Uint1 search_type)
 
 {
@@ -138,7 +158,7 @@ PSUGapOptionsCreate(Uint1 search_type)
 */
 
 
-GlobalBandStructPtr LIBCALL
+NLM_EXTERN GlobalBandStructPtr
 GlobalBandStructDelete(GlobalBandStructPtr gbsp)
 
 {
@@ -158,7 +178,7 @@ GlobalBandStructDelete(GlobalBandStructPtr gbsp)
 	Create the GlobalBandStructPtr and defaults for a search_type.
 */
 
-GlobalBandStructPtr LIBCALL
+NLM_EXTERN GlobalBandStructPtr
 GlobalBandStructCreate(Uint1 search_type)
 
 {
@@ -182,7 +202,7 @@ GlobalBandStructCreate(Uint1 search_type)
 /*
         Performs a global alignment, producing a SeqAlign.
 */
-SeqAlignPtr LIBCALL 
+NLM_EXTERN SeqAlignPtr
 GlobalBandToSeqAlign (GlobalBandStructPtr gbsp)
 {
 
@@ -227,7 +247,7 @@ GlobalBandToSeqAlign (GlobalBandStructPtr gbsp)
         can be made into a SeqAlign.
 */
 
-Boolean LIBCALL 
+NLM_EXTERN Boolean
 GlobalBandToEditBlock(GlobalBandStructPtr gbsp)
 {
   GapXEditBlockPtr edit_block;
@@ -277,7 +297,6 @@ GlobalBandToEditBlock(GlobalBandStructPtr gbsp)
     score = 0;
     ErrPostEx(SEV_ERROR, 0, 0, "Unknown method.");
     return FALSE;
-    break;
   }
   if (score == 0) {
 	return FALSE;
@@ -287,7 +306,7 @@ GlobalBandToEditBlock(GlobalBandStructPtr gbsp)
   S = MemFree(S);
 
   gbsp->edit_block = edit_block;
-  gbsp->score = score;
+  gbsp->score = (Int4) score;
   gbsp->alignment_length = Slen;
 
   return TRUE;
@@ -317,7 +336,7 @@ GlobalBandToEditBlock(GlobalBandStructPtr gbsp)
 	Does not delete the sequence or the ID's.
 */
 
-LocalBandStructPtr LIBCALL
+NLM_EXTERN LocalBandStructPtr
 LocalBandStructDelete(LocalBandStructPtr lbsp)
 
 {
@@ -337,7 +356,7 @@ LocalBandStructDelete(LocalBandStructPtr lbsp)
 	Create the LocalBandStructPtr and defaults for a search_type.
 */
 
-LocalBandStructPtr LIBCALL
+NLM_EXTERN LocalBandStructPtr
 LocalBandStructCreate(Uint1 search_type)
 
 {
@@ -361,7 +380,7 @@ LocalBandStructCreate(Uint1 search_type)
 /*
         Performs local alignment, producing a SeqAlign.
 */
-SeqAlignPtr LIBCALL 
+NLM_EXTERN SeqAlignPtr
 LocalBandToSeqAlign (LocalBandStructPtr lbsp)
 {
 
@@ -405,7 +424,7 @@ LocalBandToSeqAlign (LocalBandStructPtr lbsp)
         can be made into a SeqAlign.
 */
 
-Boolean LIBCALL 
+NLM_EXTERN Boolean
 LocalBandToEditBlock(LocalBandStructPtr lbsp)
 {
   GapXEditBlockPtr edit_block;
@@ -437,12 +456,12 @@ LocalBandToEditBlock(LocalBandStructPtr lbsp)
   S = MemFree(S);
 
   lbsp->edit_block = edit_block;
-  lbsp->score = score;
+  lbsp->score = (Int4)score;
 
-  return TRUE;;
+  return TRUE;
 }
 
-Int4 LIBCALL
+Int4
 BAND_LOCAL_ALIGN(Uint1Ptr A, Uint1Ptr B,
 		 Int4 M, Int4 N,
 		 Int4Ptr PNTR matrix, PSUGapOptionsPtr options,
@@ -528,13 +547,15 @@ BAND_LOCAL_ALIGN(Uint1Ptr A, Uint1Ptr B,
       if (e > c) {
 	if (e > 0) {
 	  dp->CC = e; e-=H;
-	  *((Int4Ptr) (dp++)-1) = d-H;
+	  *(((Int4Ptr) (dp++))-1) = d-H;
 	} else {
-	  dp->CC = *((Int4Ptr) (dp++)-1) = 0;
+	  dp->CC = *(((Int4Ptr) (dp))-1) = 0;
+          dp++;
 	}
       } else {
 	if (c <= 0) {
-	  dp->CC = *((Int4Ptr) (dp++)-1) = 0;
+	  dp->CC = *(((Int4Ptr) (dp))-1) = 0;
+          dp++;
 	} else {
 	  if (c > best_score) {
 	    best_score = c;
@@ -616,30 +637,31 @@ BAND_LOCAL_ALIGN(Uint1Ptr A, Uint1Ptr B,
   options->width = up - (startj - starti) - low + 1;
 
   switch(align_type) {
+      /* XXX Float Score is converted to Int4 --> Round-off Errors */
   case L_BAND_LINEAR:
-    return gband_linear(A + starti - 1, B + startj - 1,
+    return (Int4) (gband_linear(A + starti - 1, B + startj - 1,
 			endi - starti + 1, endj - startj + 1,
-			matrix, options, S, &Slen) / BND_DIGIT;
+			matrix, options, S, &Slen) / BND_DIGIT);
   case L_BAND_QUADRATIC:
-    return gband_quadratic(A + starti - 1, B + startj - 1,
+    return (Int4) ( gband_quadratic(A + starti - 1, B + startj - 1,
 			   endi - starti + 1, endj - startj + 1,
-			   matrix, options, S, &Slen) / BND_DIGIT; 
+			   matrix, options, S, &Slen) / BND_DIGIT); 
   case L_BAND_LGAP:
-    return gband_linear_gap(A + starti - 1, B + startj - 1,
+    return (Int4) ( gband_linear_gap(A + starti - 1, B + startj - 1,
 			    endi - starti + 1, endj - startj + 1,
-			    matrix, options, S, &Slen) / BND_DIGIT; 
+			    matrix, options, S, &Slen) / BND_DIGIT); 
   case L_BAND_QGAP:
-    return gband_linear_qgap(A + starti - 1, B + startj - 1,
+    return (Int4) ( gband_linear_qgap(A + starti - 1, B + startj - 1,
 			     endi - starti + 1, endj - startj + 1,
-			     matrix, options, S, &Slen) / BND_DIGIT; 
+			     matrix, options, S, &Slen) / BND_DIGIT); 
   case L_BAND_L3GAP:
-    return gband_l3gap(A + starti - 1, B + startj - 1,
+    return (Int4) ( gband_l3gap(A + starti - 1, B + startj - 1,
 		       endi - starti + 1, endj - startj + 1,
-		       matrix, options, S, &Slen) / BND_DIGIT; 
+		       matrix, options, S, &Slen) / BND_DIGIT); 
   case L_BAND_Q3GAP:
-    return gband_q3gap(A + starti - 1, B + startj - 1,
+    return (Int4) ( gband_q3gap(A + starti - 1, B + startj - 1,
 		       endi - starti + 1, endj - startj + 1,
-		       matrix, options, S, &Slen) / BND_DIGIT; 
+		       matrix, options, S, &Slen) / BND_DIGIT); 
   default:
     ErrPostEx(SEV_ERROR, 0, 0, "Unknown method.");
     return -1;
@@ -687,7 +709,7 @@ static Uint1 HSblastna_to_ncbi4na[] = {   1, /* A, 0 */
                                  0, /* Gap, 15 */
                         };
 
-void SetGlobaltOptions(GlobalBandStructPtr gbsp, Int4 lg1_ext, Int4 rg1_ext, Int4 lg2_ext, Int4 rg2_ext, Int4 lg1_open, Int4 lg2_open, Int4 rg1_open, Int4 rg2_open, Int2 gopen, Int2 gext)
+NLM_EXTERN void SetGlobaltOptions(GlobalBandStructPtr gbsp, Int4 lg1_ext, Int4 rg1_ext, Int4 lg2_ext, Int4 rg2_ext, Int4 lg1_open, Int4 lg2_open, Int4 rg1_open, Int4 rg2_open, Int2 gopen, Int2 gext)
 {
 	PSUGapOptionsPtr opt;
 
@@ -705,7 +727,7 @@ void SetGlobaltOptions(GlobalBandStructPtr gbsp, Int4 lg1_ext, Int4 rg1_ext, Int
 	opt->rg2_open = rg2_open;
 }
 
-static Int4Ptr PNTR LIBCALL BlastStyleMatCreate(void){
+static Int4Ptr PNTR BlastStyleMatCreate(void){
 /* Allocate Matrix as one contigous memory block to increase cache-hits&speed
  */
   Int4Ptr PNTR matrix;
@@ -717,7 +739,7 @@ static Int4Ptr PNTR LIBCALL BlastStyleMatCreate(void){
    return matrix;
 }
 
-Int2 ChangeGlobalBandMatrix(GlobalBandStructPtr gbsp, Boolean is_prot, CharPtr matrix_name, Int4 penalty, Int4 reward)
+NLM_EXTERN Int2 ChangeGlobalBandMatrix(GlobalBandStructPtr gbsp, Boolean is_prot, CharPtr matrix_name, Int4 penalty, Int4 reward)
 {
    BLAST_ScoreBlkPtr   sbp;
    Int4                i, j;
@@ -819,9 +841,9 @@ static Uint1Ptr load_data (SeqLocPtr slp,Boolean is_prot)
 {
   SeqPortPtr       spp;
   Uint1Ptr         seq = NULL;
-  Int2             code;
   Int4 len;
   Int4 index;
+  Uint1             code;
 
   if (is_prot)
     code = Seq_code_ncbistdaa;
@@ -834,17 +856,17 @@ static Uint1Ptr load_data (SeqLocPtr slp,Boolean is_prot)
     {
       seq[index] = SeqPortGetResidue(spp);
     }
+  SeqPortFree (spp);
   return seq;
 }
 
-GlobalBandStructPtr CreatBandStruct(SeqLocPtr slp1, SeqLocPtr slp2, Int4Ptr PNTR W, Boolean is_prot, Int2 method)
+NLM_EXTERN GlobalBandStructPtr CreatBandStruct(SeqLocPtr slp1, SeqLocPtr slp2, Int4Ptr PNTR W, Boolean is_prot, Int2 method)
 {
 	GlobalBandStructPtr gbsp;
 	Uint1Ptr seq1,seq2;
 	Int4 len1, len2;
 	Int2 gopen, gext;  /* default values */
 	Int4 ma=1, ms = -2; /* default values */
-	Uint1 code = Seq_code_ncbi4na;
   
 	if (is_prot) {
 		gopen = 5;
@@ -858,7 +880,7 @@ GlobalBandStructPtr CreatBandStruct(SeqLocPtr slp1, SeqLocPtr slp2, Int4Ptr PNTR
 
 	seq1 = load_data(slp1, is_prot);
 	seq2 = load_data(slp2, is_prot);
-	gbsp = GlobalBandStructCreate(method);
+	gbsp = GlobalBandStructCreate((Uint1)method);
 	gbsp->seq1 = seq1;
 	gbsp->seq2 = seq2;
 	gbsp->seq1_length = len1;
@@ -904,14 +926,14 @@ GlobalBandStructPtr CreatBandStruct(SeqLocPtr slp1, SeqLocPtr slp2, Int4Ptr PNTR
    */
 
 
-void SetLowUpFromBlast(PSUGapOptionsPtr opt, Boolean is_prot, Int2 type, Int2 width, SeqLocPtr slp1, SeqLocPtr slp2)
+NLM_EXTERN void SetLowUpFromBlast(PSUGapOptionsPtr opt, Boolean is_prot, Int2 type, Int2 width, SeqLocPtr slp1, SeqLocPtr slp2)
 {
 	BLAST_OptionsBlkPtr options;
 	SeqAlignPtr seqalign, sap;
 	Int2 index1, i2;
 	Int4 x,y,maxy,miny,mindi,maxdi,mindi_low,maxdi_up;
 	Int4 len,len1,len2,maxlen=0,alnlen,lastx,lowestx,diag;
-	DenseDiagPtr  ddp, maxddp = NULL;
+	DenseDiagPtr  ddp;
 	ScorePtr score;
 	FloatHi val, maxsc = 0;
 	DenseSegPtr	dsp;
@@ -1054,7 +1076,7 @@ void SetLowUpFromBlast(PSUGapOptionsPtr opt, Boolean is_prot, Int2 type, Int2 wi
 }
 
 
-SeqAlignPtr GlobalBandByLoc(GlobalBandStructPtr gbsp, SeqLocPtr slp1, SeqLocPtr slp2,  Boolean is_prot, Int2 band_method)
+NLM_EXTERN SeqAlignPtr GlobalBandByLoc(GlobalBandStructPtr gbsp, SeqLocPtr slp1, SeqLocPtr slp2,  Boolean is_prot, Int2 band_method)
 {
 	PSUGapOptionsPtr opt;
 	SeqAlignPtr seqalign;
@@ -1069,9 +1091,8 @@ SeqAlignPtr GlobalBandByLoc(GlobalBandStructPtr gbsp, SeqLocPtr slp1, SeqLocPtr
 	return seqalign;
 }
 
-SeqAlignPtr ExtendSeqAlign(SeqAlignPtr sap, Int4 start1, Int4 start2, Int4 stop1, Int4 stop2, Int4 x1, Int4 y1, Int4 x2, Int4 y2)
+NLM_EXTERN SeqAlignPtr ExtendSeqAlign(SeqAlignPtr sap, Int4 start1, Int4 start2, Int4 stop1, Int4 stop2, Int4 x1, Int4 y1, Int4 x2, Int4 y2)
 {
-   DenseDiagPtr  ddp;
    DenseSegPtr   dsp;
    Int4          index1;
    Int4Ptr       n_starts, n_lens;
@@ -1084,7 +1105,9 @@ SeqAlignPtr ExtendSeqAlign(SeqAlignPtr sap, Int4 start1, Int4 start2, Int4 stop1
    if (start1==x1 && start2==y1 && stop1==x2 && stop2==y2)
       return sap;
    if (sap->segtype == 1) {
-      ddp = sap->segs;
+       DenseDiagPtr  ddp;
+       ddp = sap->segs;
+       ErrPostEx(SEV_WARNING,0,0,"ExtendSeqAlign doesn't support segtype==1\n");
    } else if (sap->segtype == 2) {
       dsp = sap->segs;
       n = dsp->numseg;
@@ -1142,9 +1165,8 @@ SeqAlignPtr ExtendSeqAlign(SeqAlignPtr sap, Int4 start1, Int4 start2, Int4 stop1
    return sap;
 }
 
-SeqAlignPtr CC_ExtendSeqAlign(SeqAlignPtr sap, Int4 start1, Int4 start2, Int4 stop1, Int4 stop2, Int4 x1, Int4 y1, Int4 x2, Int4 y2, Uint1 strand1, Uint1 strand2)
+NLM_EXTERN SeqAlignPtr CC_ExtendSeqAlign(SeqAlignPtr sap, Int4 start1, Int4 start2, Int4 stop1, Int4 stop2, Int4 x1, Int4 y1, Int4 x2, Int4 y2, Uint1 strand1, Uint1 strand2)
 {
-   DenseDiagPtr  ddp;
    DenseSegPtr   dsp;
    Int4Ptr       n_starts, n_lens;
    Uint1Ptr      n_strands;
@@ -1158,7 +1180,9 @@ SeqAlignPtr CC_ExtendSeqAlign(SeqAlignPtr sap, Int4 start1, Int4 start2, Int4 st
    if (start1==x1 && start2==y1 && stop1==x2 && stop2==y2)
       return sap;
    if (sap->segtype == 1) {
-      ddp = (DenseDiagPtr) sap->segs;
+       DenseDiagPtr  ddp; 
+     ddp = (DenseDiagPtr) sap->segs;
+       ErrPostEx(SEV_WARNING,0,0,"ExtendSeqAlign doesn't support segtype==1\n");
    } else if (sap->segtype == 2) {
       dsp = (DenseSegPtr) sap->segs;
       n = dsp->numseg;
@@ -1245,7 +1269,7 @@ SeqAlignPtr CC_ExtendSeqAlign(SeqAlignPtr sap, Int4 start1, Int4 start2, Int4 st
 
 /* doesn't work for minus strands in SeqAlign */
 /* deals with segtypes 1 and 2 only*/
-void GetAlignExtremes(SeqAlignPtr sap, Int4Ptr xx1, Int4Ptr yy1, Int4Ptr xx2, Int4Ptr yy2)
+NLM_EXTERN void GetAlignExtremes(SeqAlignPtr sap, Int4Ptr xx1, Int4Ptr yy1, Int4Ptr xx2, Int4Ptr yy2)
 {
 	DenseDiagPtr  	ddp;
 	DenseSegPtr		dsp;
