@@ -37,6 +37,9 @@
 * Date     Name        Description of modification
 *
 * $Log: csim.c,v $
+* Revision 6.1  1998/09/09 11:47:43  sirotkin
+* MAXSEG to NLM_MAX_SEG to avoid RS6000 occasional conflict
+*
 * Revision 6.0  1997/08/25 18:52:57  madden
 * Revision changed to 6.0
 *
@@ -478,8 +481,8 @@ static SeqAlignPtr SIM(SeqLocPtr loc1, SeqLocPtr loc2, CharPtr A, CharPtr B, Int
 
 
 	/**allocate the space for recording the result**/
-	rec_a1 = MemNew((size_t)MAXSEG * sizeof(Int4));
-	rec_a2 = MemNew((size_t)MAXSEG * sizeof(Int4));
+	rec_a1 = MemNew((size_t)NLM_MAXSEG * sizeof(Int4));
+	rec_a2 = MemNew((size_t)NLM_MAXSEG * sizeof(Int4));
 
 	/*get the matrix value and gap penalties*/
 	value.v=V;
