@@ -56,6 +56,12 @@
 * Date     Name        Description of modification
 *
 * $Log: falign.c,v $
+* Revision 6.2  2000/11/02 20:56:17  vakatov
+* Renamed "join()" to "static s_Join()" to avoid name conflicts
+*
+* Revision 6.1  1998/06/16 18:30:55  kans
+* fixed unix compiler warnings
+*
 * Revision 6.0  1997/08/25 18:53:08  madden
 * Revision changed to 6.0
 *
@@ -217,7 +223,7 @@ void bfound(Int4, Int4);
 void setup(void);
 void global_setup(fragment PNTR, fragment PNTR);
 void global(fragment PNTR, fragment PNTR, Int4,  SeqLocPtr,  SeqLocPtr, SeqAlignPtr);
-Int4 join(fragment PNTR, fragment PNTR);
+static Int4 s_Join(fragment PNTR, fragment PNTR);
 void recom_setup(kbclass PNTR);
 void recompute(kbclass PNTR);
 Int4 disjoint(void);
@@ -454,7 +460,7 @@ Int4 encoding[128];
 Int4 mask;
 
 /* hash table initialization */
-void table_init()
+void table_init(void)
 {
 
 	/* perform initializations */
@@ -466,13 +472,12 @@ void table_init()
 }
 
 /* build hash table */
-void fbld_table()
+void fbld_table(void)
 {
 	Int4 ecode;
 	Int4 i, pos;
 	CharPtr q, endp;
 	Int4 s5;
-	CharPtr ckalloc();
 
 	q = seq2 + sb2 - 1;
 	ecode = 0L;
@@ -494,13 +499,12 @@ void fbld_table()
 }
 
 /* build hash table */
-void bbld_table()
+void bbld_table(void)
 {
 	Int4 ecode;
 	Int4 i, pos;
 	CharPtr q, endp;
 	Int4 s5;
-	CharPtr ckalloc();
 
 	q = seq2 + se2 + 1;
 	ecode = 0L;
@@ -528,7 +532,6 @@ Int4 rcbld_table(Int4Ptr buc, Int4Ptr cbls, Int4Ptr cblink, CharPtr seq, Int4 b,
 	Int4 i, pos;
 	CharPtr q, endp;
 	Int4 s5;
-	CharPtr ckalloc();
 
 	q = seq + e + 1;
 	ecode = 0L;
@@ -554,7 +557,6 @@ Int4 rcbld_table(Int4Ptr buc, Int4Ptr cbls, Int4Ptr cblink, CharPtr seq, Int4 b,
 Int4 ex_table(Int4Ptr buc, Int4Ptr cbls, Int4Ptr cblink, CharPtr seq, Int4 pos,Int4 ecode)
 {
 	CharPtr q;
-	CharPtr ckalloc();
 
 	q = seq + (pos-W+1);
 	ecode = ((ecode & mask) << 2) + encoding[*q];
@@ -667,7 +669,7 @@ void free_all(Int4 i, Int4 j1, Int4 j2, fragment PNTR cdiag_prev[], list cri, db
 	db_freeList(cli_d);
 }
 
-void scan()	/* scan sequence 1 */
+void scan(void)	/* scan sequence 1 */
 {
 	Int4 ecode;
 	Int4 i, l, m, pos;
@@ -715,7 +717,7 @@ void scan()	/* scan sequence 1 */
 		}
 }
 
-void fscan_init()
+void fscan_init(void)
 {
 	Int4 i,j;
 
@@ -728,7 +730,7 @@ void fscan_init()
 	cur_end = mid1;
 }
 
-void bscan_init()
+void bscan_init(void)
 {
 	Int4 i,j;
 
@@ -742,7 +744,7 @@ void bscan_init()
 	cur_end = len1 - mid2;
 }
 
-void fscan()	/* scan sequence 1 */
+void fscan(void)	/* scan sequence 1 */
 {
 	Int4 ecode;
 	Int4 i, pos;
@@ -773,7 +775,7 @@ void fscan()	/* scan sequence 1 */
 	}
 }
 
-void bscan() /* scan sequence 1 */
+void bscan(void) /* scan sequence 1 */
 {
 	Int4 ecode;
 	Int4 i, pos, dd;
@@ -820,7 +822,6 @@ void found(Int4 i, Int4 j)
 {
 	Int4 k;
 	fragment PNTR f;
-	CharPtr ckalloc();
 
 	if (i && j && seq1[i-1] == seq2[j-1])
 		return;
@@ -894,7 +895,6 @@ void rffound(Int4 i, Int4 j)
 {
 	Int4 k;
 	fragment PNTR f;
-	CharPtr ckalloc();
 
 	if (i && j && seq1[i-1] == seq2[j-1])
 		return;
@@ -918,7 +918,6 @@ void enter_endrow(fragment PNTR f)
 {
 	Int4 row,col;
 	frag_pt PNTR e, PNTR e1, PNTR e2;
-	CharPtr ckalloc();
 
 	row = f->i + f->k - 1;
 	col = f->j + f->k - 1;
@@ -952,7 +951,6 @@ void enter_endcol(fragment PNTR f)
 {
 	Int4 row,col;
 	frag_pt PNTR e, PNTR e1, PNTR e2;
-	CharPtr ckalloc();
 
 	row = f->i + f->k - 1;
 	col = f->j + f->k - 1;
@@ -985,7 +983,6 @@ void ffound(Int4 i, Int4 j)
 {
 	Int4 k;
 	fragment PNTR f;
-	CharPtr ckalloc();
 
 	if (i && j && seq1[i-1] == seq2[j-1])
 		return;
@@ -1014,7 +1011,6 @@ void bfound(Int4 i, Int4 j)
 {
 	Int4 k;
 	fragment PNTR f;
-	CharPtr ckalloc();
 
 	if (seq1[i+1] && seq2[j+1] && seq1[i+1] == seq2[j+1])
 		return;
@@ -1041,10 +1037,8 @@ void bfound(Int4 i, Int4 j)
 }
 
 /* setup - initialization */
-void setup()
+void setup(void)
 {
-	Char *ckalloc();
-
 	/* skip lists initialization */
 	init();
 	db_init();
@@ -1102,8 +1096,8 @@ void global(fragment PNTR bf, fragment PNTR ef, Int4 gs, SeqLocPtr slp1,  SeqLoc
 {
 	Int4 i, ll;
 	fragment PNTR fdf, PNTR bdf;
-	fragment PNTR fsb, PNTR fse, PNTR bsb, PNTR bse, PNTR mf;
-	Int4 js;
+	fragment PNTR fsb, PNTR fse = NULL, PNTR bsb, PNTR bse = NULL, PNTR mf;
+	Int4 js = 0;
 
 	global_setup(bf,ef);
 	if ((se1-sb1+1 < W) || (se2-sb2+1 < W))
@@ -1128,7 +1122,7 @@ void global(fragment PNTR bf, fragment PNTR ef, Int4 gs, SeqLocPtr slp1,  SeqLoc
 		if (cut[pff->j]) {
 			fdf = cut[pff->j]->ff;
 			bdf = cut[pff->j]->bf;
-			js = join(fdf,bdf)- (Int4) (DIGIT*(bdf->k));
+			js = s_Join(fdf,bdf)- (Int4) (DIGIT*(bdf->k));
 
 			if (js == gs) {
 				mf = fdf;
@@ -1145,7 +1139,7 @@ void global(fragment PNTR bf, fragment PNTR ef, Int4 gs, SeqLocPtr slp1,  SeqLoc
 		db_reset_pos(li_d);
 		malign(pfo,diag_prev,ri,li_c,li_d,pff);
 		malign(pbo,bdiag_prev,bri,bli_c,bli_d,pbf);
-		js = join(pff,pbf);
+		js = s_Join(pff,pbf);
 		if ((pff->j-pff->i != (pff->bgf)->j-(pff->bgf)->i) &&
 			(pbf->j-pbf->i != (pbf->bgf)->j-(pbf->bgf)->i))
 				js = js + IO;
@@ -1172,7 +1166,7 @@ void global(fragment PNTR bf, fragment PNTR ef, Int4 gs, SeqLocPtr slp1,  SeqLoc
 			db_reset_pos(li_d);
 			malign1(pfo,ri,li_c,li_d,pff);
 			malign1(pbo,bri,bli_c,bli_d,pbf);
-			js = join(pff,pbf);
+			js = s_Join(pff,pbf);
 			if ((pff->j-pff->i != (pff->bgf)->j-(pff->bgf)->i) &&
 				(pbf->j-pbf->i != (pbf->bgf)->j-(pbf->bgf)->i))
 					js = js + IO;
@@ -1247,7 +1241,7 @@ void global(fragment PNTR bf, fragment PNTR ef, Int4 gs, SeqLocPtr slp1,  SeqLoc
 }
 
 /* join - return the cost after join */
-Int4 join(fragment PNTR ff, fragment PNTR bf)
+static Int4 s_Join(fragment PNTR ff, fragment PNTR bf)
 {
 	return(ff->score + bf->score);
 }
@@ -1329,7 +1323,7 @@ void recompute(kbclass PNTR sp)
 
 /* disjoint - return TRUE if [tt,bb]X[ll,rr]-[tt1,bb]X[ll1,rr] shares
 	      no vertices with the areas bounded by S */
-Int4 disjoint()
+Int4 disjoint(void)
 {
 	kbclass PNTR sp;
 	Int4 i,j;
@@ -1368,7 +1362,7 @@ Int4 disjoint()
 }
 
 /* syn_area - synchronize [tt,bb]X[ll,rr] with all other areas */
-void syn_area()
+void syn_area(void)
 {
 	kbclass PNTR sp;
 	Int4 i,si,sj;
@@ -1392,7 +1386,7 @@ void syn_area()
 }
 
 /* ex_row - extend one row (incrementally) */
-void ex_row()
+void ex_row(void)
 {
 	Int4 s5;
 	Int4 tpos;
@@ -1414,7 +1408,7 @@ void ex_row()
 }
 
 /* ex_col - extend one col (incrementally) */
-void ex_col()
+void ex_col(void)
 {
 	Int4 s5;
 	Int4 tpos;
@@ -1441,7 +1435,6 @@ void rcfound(Int4 i, Int4 j)
 {
 	Int4 k;
 	fragment PNTR f;
-	CharPtr ckalloc();
 
 	if (seq1[i+1] && seq2[j+1] && seq1[i+1] == seq2[j+1])
 		return;
@@ -1465,7 +1458,7 @@ void rcfound(Int4 i, Int4 j)
 }
 
 /* trcscan - scan seq1 and seq2 backward (columnwise) */
-void trcscan()
+void trcscan(void)
 {
 	Int4 ecode;
 	Int4 i, pos, tpos;
@@ -1499,7 +1492,6 @@ void trcfound(Int4 i, Int4 j)
 {
 	Int4 k;
 	fragment PNTR f;
-	CharPtr ckalloc();
 
 	if (seq1[i+1] && seq2[j+1] && seq1[i+1] == seq2[j+1])
 		return;
@@ -1633,7 +1625,7 @@ void ractive_ext(fragment PNTR f)
 	}
 }
 
-void rcfree()
+void rcfree(void)
 {
 	Int4 i,j;
 	frag_pt PNTR p, PNTR q;
@@ -1789,7 +1781,7 @@ SeqAlignPtr class_print(SeqLocPtr slp1, SeqLocPtr slp2)
 }
 
 /* findmaxS - return the class with the maximum score */
-kbclass PNTR findmaxS()
+kbclass PNTR findmaxS(void)
 {
 	Int4 i, ms, mi;
 	kbclass PNTR sp;
@@ -1871,8 +1863,6 @@ void replaceS(fragment PNTR f)
 /* insertS - insert a new class */
 void insertS(fragment PNTR f)
 {
-	CharPtr ckalloc();
-
 	incre(f);
 	S[lastS] = (kbclass *) ckalloc(sizeof(kbclass));
 	set_class(lastS,f);
@@ -1881,13 +1871,13 @@ void insertS(fragment PNTR f)
 }
 
 /* sizeS - return the number of current equivalent classes */
-Int4 sizeS()
+Int4 sizeS(void)
 {
 	return lastS;
 }
 
 /* minscoreS - return the minimum score in all classes */
-Int4 minscoreS()
+Int4 minscoreS(void)
 {
 	return S[minS]->score;
 }
@@ -2216,7 +2206,6 @@ frag_pt PNTR otr_survive(frag_pt PNTR rcopy, Int4 col)
 {
 	frag_pt PNTR p, PNTR q, PNTR r, PNTR t1, PNTR t2;
 	Int4 cur_score;
-	CharPtr ckalloc();
 
 	if (!rcopy) return NULL;
 	r = NULL;
@@ -2313,7 +2302,6 @@ frag_pt PNTR otl_survive(frag_pt PNTR rcopy, Int4 col)
 	frag_pt PNTR p, PNTR q, PNTR r, PNTR t;
 	Int4 cur_score,col_diff;
 	fragment PNTR pf;
-	CharPtr ckalloc();
 
 	if (!rcopy) return NULL;
 	r = NULL;
