@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   3/17/94
 *
-* $Revision: 6.5 $
+* $Revision: 6.8 $
 *
 * File Description: 
 *
@@ -66,6 +66,15 @@
 *     
 *     A detailed change log follows:
 *     $Log: suggest.c,v $
+*     Revision 6.8  2001/05/25 19:43:53  vakatov
+*     Nested comment typo fixed
+*
+*     Revision 6.7  1998/06/17 21:55:47  kans
+*     fixed 64-bit SGI compiler warnings
+*
+*     Revision 6.6  1998/06/12 20:42:18  kans
+*     fixed most unix compiler warnings
+*
 *     Revision 6.5  1997/12/02 17:43:52  kans
 *     added cast to sprintf, explicit int return values
 *
@@ -397,6 +406,8 @@ for any particular segment
 /* with the cursor on this line, the following
 command adds comments on defines
 :+6,+25s/#define DEBUG\([ _A-Z]*\)/ \/\* #define DEBUG\1 \*\/
+*/
+
 /* with the cursor on this line, the following
 command removes comments on defines
 :+3,+22s/ \/\* #define DEBUG\([ _A-Z]*\) \*\//#define DEBUG\1/ */
@@ -438,8 +449,7 @@ static Int4
 	* nt_dna_len,
 	* nt_len_offsets ,
 	orientation,
-	shift,
-	DNA_ID;
+	shift;
 
 Match * m_stk = NULL;
 Int4 m_top;
@@ -465,20 +475,21 @@ static DBTINYINT last_orient;
 
 static char	*module = "APT_suggest";
 static DBSMALLINT	exception;
-static Int4	aa_id_seq, * nt_id_seq = NULL;
-static num_nts; /* number of nucleotides > 1 if segmented */
-static max_num_nts;
+static Int4 * nt_id_seq = NULL;
+#ifdef DEBUG_TOO_SHORT 	
+static Int4	aa_id_seq;
+#endif
+static Int4 num_nts; /* number of nucleotides > 1 if segmented */
+static Int4 max_num_nts;
 static DBTINYINT * nt_orient = NULL;
 static SeqPtr		dbaa_seq, * dbnt_seq= NULL;
 static	Int4	aori, ahz, aasc;
 
-static Int4 Weirdness_found;
-
 Int4 Total_len;
 
 #define fatal(m)	{ErrPostEx(SEV_FATAL, 0, 0, "%s:  %s", module, m); goto Error;}
 
-static char * suggest_revision = "$Revision: 6.5 $";
+static char * suggest_revision = "$Revision: 6.8 $";
 
 static Int4	suggest_get_frames_init = 1;
 
@@ -1035,7 +1046,10 @@ static Int4 AddInterval (Int4 lonum, Int4 hinum, Int4 orient)
 static int
 report(void)
 {
-	Int4 i, j, k, last, first, from, to;
+	Int4 i, from, to;
+#ifdef OLD_UNASSIGNED_CODE
+	Int4 j, k, last, first;
+#endif
 
 #ifdef OLD_CHECKING
 	for (i = 1; i < m_top; ++i) {
@@ -1208,18 +1222,12 @@ cleanup(void)
 
 static Int4 APT_suggest(SuggestOutputPtr pSuggestOut, Int4 num_nuc)
 {
-	Int4		i, j, rc;
+	Int4		i;
 	Int4 suggest_revision_len = strlen(suggest_revision);
 	Int4 nt;
-	Int4 len;
-	char formbuf[12];
 	DBTINYINT frame_1 =1;
 	char nt_msg[512];
-	Int4 combined;
-	Int4 lookstoplen;
 	double combined_length;
-	IntPtr  thisInt;
-	Char str [128];
 
 	/* Get the amino acid sequence's id_seq from the form */
 
@@ -1459,6 +1467,7 @@ fprintf(fp,
 }
 
 
+/*
 static Int4
 dump_m_stk(FILE *fp, char *msg)
 {
@@ -1468,6 +1477,7 @@ for (index=0; index < m_top; index ++ ) {
 }
 return index;
 }
+*/
 
 
 
@@ -1681,52 +1691,6 @@ static void Suggest_Intervals (SuggestRecPtr sugrp,
   }
 }
 
-/*****************************************************************************
-*
-*   Build_Table (sugrp)
-*       Builds table of residue triplets for fast lookup.  Each residue triplet
-*       is encoded as:
-*
-*       tval = (word [0] - 'A') * 676 + (word [1] - 'A') * 26 + word [2] - 'A';
-*     9/22/95 Karl Sirotkin
-*   changed to 
-*
-*       Stops and unknown use "ZZZ" + 1
-*
-*****************************************************************************/
-
-static Boolean Build_Table (SuggestRecPtr sugrp)
-
-{
-  TripletNodePtr  h;
-  Int4            tval;
-  CharPtr         p;
-  Int4            position;
-  SeqRecPtr       protein;
-
-  if (sugrp != NULL && tripletTable != NULL) {
-    protein = &(sugrp->protein);
-    if (protein != NULL && protein->sequence != NULL) {
-      position = 0;
-      for (p = protein->sequence; p [WORD_SIZE - 1] != '\0'; p++) {
-        tval = (p [0] - 'A') * 676 + (p [1] - 'A') * 26 + (p [2] - 'A');
-        if (tval > 17575 || tval < 0) {
-          tval = 17576;
-        }
-        h = (TripletNodePtr) MemNew (sizeof (TripletNode));
-        if (h == NULL) {
-          return FALSE;
-        }
-        h->position = position;
-        h->link = tripletTable [tval];
-        tripletTable [tval] = h;
-        position++;
-      }
-    }
-  }
-  return TRUE;
-}
-
 /* default code (code 0) means use the standard code (code 1) */
 
 static CharPtr geneticCodes [16] = {
