@@ -1,3 +1,62 @@
+/*
+* ===========================================================================
+*
+*                            PUBLIC DOMAIN NOTICE
+*               National Center for Biotechnology Information
+*
+*  This software/database is a "United States Government Work" under the
+*  terms of the United States Copyright Act.  It was written as part of
+*  the author's official duties as a United States Government employee and
+*  thus cannot be copyrighted.  This software/database is freely available
+*  to the public for use. The National Library of Medicine and the U.S.
+*  Government have not placed any restriction on its use or reproduction.
+*
+*  Although all reasonable efforts have been taken to ensure the accuracy
+*  and reliability of the software and data, the NLM and the U.S.
+*  Government do not and cannot warrant the performance or results that
+*  may be obtained by using this software or data. The NLM and the U.S.
+*  Government disclaim all warranties, express or implied, including
+*  warranties of performance, merchantability or fitness for any particular
+*  purpose.
+*
+*  Please cite the author in any work or product based on this material.
+*
+* ===========================================================================
+*
+* File Name: urkptpf.c
+*
+* Author(s): John Kuzio
+*
+* Version Creation Date: 98-01-01
+*
+* $Revision: 6.32 $
+*
+* File Description: patterns and profiles
+*
+* Modifications:
+* --------------------------------------------------------------------------
+* Date       Name        Description of modification
+* --------------------------------------------------------------------------
+* $Log: urkptpf.c,v $
+* Revision 6.32  1999/07/13 14:08:46  sicotte
+* prefix functions SeqAlignSort* with URK to resolve toolkit conflicts
+*
+* Revision 6.31  1998/12/23 21:34:03  kuzio
+* SeqPortStuff
+*
+* Revision 6.30  1998/11/16 14:29:53  kuzio
+* flagBoundaryCondition
+*
+* Revision 6.29  1998/09/21 14:38:24  kuzio
+* check for cr in comments
+*
+* Revision 6.28  1998/09/16 18:03:36  kuzio
+* cvs logging
+*
+*
+* ==========================================================================
+*/
+
 #include <objres.h>
 #include <objseq.h>
 #include <sequtil.h>
@@ -27,7 +86,7 @@ static Uint1Ptr GetSeqChunk (SeqPortPtr spp, Int4 start, Int4 len,
   SeqPortSeek (spp, start, SEEK_SET);
   for (i = 0; i < size; i++)
     *sequence++ = (Uint1) SeqPortGetResidue (spp);
-  *sequence = '\0';
+  *sequence = 0;
 
   return seqhead;
 }
@@ -128,14 +187,22 @@ extern ValNodePtr ReadPatternNames (CharPtr filename)
 {
   FILE        *fiop;
   ValNodePtr  namenew, namelist = NULL;
-  Char        buf[128];
+  Char        buf[PATH_MAX];
   CharPtr     cptr;
 
-  if (filename != NULL)
+  if (filename != NULL && StrLen (filename) > 0)
   {
-    if ((fiop = FileOpen (filename, "r")) == NULL)
+    if (FindPath ("ncbi", "ncbi", "data", buf, sizeof (buf)))
+      FileBuildPath (buf, NULL, filename);
+    else
+      StrCpy (buf, filename);
+
+    if ((fiop = FileOpen (buf, "r")) == NULL)
     {
-      return namelist;
+      if ((fiop = FileOpen (filename, "r")) == NULL)
+      {
+        return NULL;
+      }
     }
     while ((FileGets (buf, sizeof (buf), fiop)) != NULL)
     {
@@ -172,6 +239,9 @@ extern ComPatPtr ReadRENPattern (CharPtr filename, Boolean flagNeedCutPos,
 
   ComPatPtr cpph, cppn, cpp;
 
+  if (filename == NULL || !(StrLen (filename) > 0))
+    return NULL;
+
   if (FindPath ("ncbi", "ncbi", "data", bufin, sizeof (bufin)))
     FileBuildPath (bufin, NULL, filename);
   else
@@ -253,7 +323,7 @@ extern ComPatPtr ReadRENPattern (CharPtr filename, Boolean flagNeedCutPos,
         if (StrStr (bufin, echar))
           break;
       }
-      if ((cpp = urkCompilePattern (bufpat, 0)) != NULL)
+      if ((cpp = CompilePattern (bufpat, 0)) != NULL)
       {
         PalindromeCheck (cpp);
         StrCpy (cpp->name, bufname);
@@ -303,6 +373,9 @@ extern ComPatPtr ReadPrositePattern (CharPtr filename, Boolean flagStringent,
   if (cutoff < 0)
     cutoff = INT4_MAX;
 
+  if (filename == NULL || !(StrLen (filename) > 0))
+    return NULL;
+
   if (FindPath ("ncbi", "ncbi", "data", buf, sizeof (buf)))
     FileBuildPath (buf, NULL, filename);
   else
@@ -401,10 +474,10 @@ extern ComPatPtr ReadPrositePattern (CharPtr filename, Boolean flagStringent,
       }
       if (!flagTaxonHit)
         continue;
-      urkCleanPattern (psbuff);
+      CleanPattern (psbuff);
       if ((gbbuff = PrositeToGBPattern (psbuff)) != NULL)
       {
-        if ((cpp = urkCompilePattern (gbbuff, 1)) != NULL)
+        if ((cpp = CompilePattern (gbbuff, 1)) != NULL)
         {
           StrCpy (cpp->name, nabuff);
           if (cpph == NULL)
@@ -630,6 +703,9 @@ extern CharPtr ReadPattern (CharPtr filename)
   CharPtr bptr, tptr;
   Int4    blen = 128;
 
+  if (filename == NULL || !(StrLen (filename) > 0))
+    return NULL;
+
   if ((fin = FileOpen (filename, "r")) == NULL)
     return NULL;
 
@@ -685,7 +761,7 @@ extern CharPtr ReadPattern (CharPtr filename)
   return bptr;
 }
 
-extern void urkCleanPattern (CharPtr inbuff)
+extern void CleanPattern (CharPtr inbuff)
 {
   CharPtr headptr, outbuff;
 
@@ -696,7 +772,7 @@ extern void urkCleanPattern (CharPtr inbuff)
   {
     if (*inbuff == '#' || *inbuff == ';')
     {
-      while (*inbuff != '\n')
+      while (*inbuff != '\n' || *inbuff != '\r')
         inbuff++;
     }
     if (!IS_WHITESP (*inbuff))
@@ -738,7 +814,7 @@ extern ComPatPtr ComPatNew (ComPatPtr curcpp)
   return cpp;
 }
 
-extern ComPatPtr urkComPatFree (ComPatPtr headcpp)
+extern ComPatPtr ComPatFree (ComPatPtr headcpp)
 {
   ComPatPtr   cpp, cppp;
 
@@ -799,7 +875,7 @@ extern ComPatPtr ComPatDup (ComPatPtr cppold)
   return cppheadpat;
 }
 
-extern ComPatPtr urkInvertPattern (ComPatPtr cppold)
+extern ComPatPtr InvertPattern (ComPatPtr cppold)
 {
   ComPatPtr cpp, cpphead, cppnew = NULL, cppnewhead = NULL;
   CharPtr   pat;
@@ -883,7 +959,7 @@ extern ComPatPtr urkInvertPattern (ComPatPtr cppold)
         cppnew->res[n] = cpp->res[n];
     }
   }
-  urkComPatFree (cpphead);
+  ComPatFree (cpphead);
   return cppnewhead;
 }
 
@@ -895,7 +971,7 @@ extern void PalindromeCheck (ComPatPtr cpp)
   while (cpp != NULL)
   {
     cpp->flagPalindrome = TRUE;
-    cppi = cpphi = urkInvertPattern (cpp);
+    cppi = cpphi = InvertPattern (cpp);
     cppt = cpp;
     while (cppt != NULL)
     {
@@ -914,7 +990,7 @@ extern void PalindromeCheck (ComPatPtr cpp)
       cppi = cppi->next;
       cppt = cppt->next;
     }
-    urkComPatFree (cpphi);
+    ComPatFree (cpphi);
     cpp = cpp->nextpattern;
   }
   return;
@@ -1039,7 +1115,7 @@ static void AddResidues (CharPtr res, Char pat, Int2 moltype)
   return;
 }
 
-extern ComPatPtr urkCompilePattern (CharPtr pattern, Int2 moltype)
+extern ComPatPtr CompilePattern (CharPtr pattern, Int2 moltype)
 {
   ComPatPtr    cpp = NULL, headcpp = NULL;
   CharPtr      head;
@@ -1051,7 +1127,7 @@ extern ComPatPtr urkCompilePattern (CharPtr pattern, Int2 moltype)
   if (*pattern == '\0')
     return headcpp;
 
-  urkCleanPattern (pattern);
+  CleanPattern (pattern);
   flagCount = FALSE;
   flagFirst = TRUE;
   flagNTerm = FALSE;
@@ -1073,14 +1149,14 @@ extern ComPatPtr urkCompilePattern (CharPtr pattern, Int2 moltype)
      case '<':         /* just a check for spurious N signals */
       if (!flagFirst)
       {
-        headcpp = urkComPatFree (headcpp);
+        headcpp = ComPatFree (headcpp);
         return headcpp;
       }
       break;
      case '>':         /* just a check for spurious C signals */
       if (*(pattern+1) != '\0')
       {
-        headcpp = urkComPatFree (headcpp);
+        headcpp = ComPatFree (headcpp);
         return headcpp;
       }
       break;
@@ -1119,7 +1195,7 @@ extern ComPatPtr urkCompilePattern (CharPtr pattern, Int2 moltype)
       }
       if (*pattern == '\0')
       {
-        headcpp = urkComPatFree (headcpp);
+        headcpp = ComPatFree (headcpp);
         return headcpp;
       }
       if (cpp->min < 1)
@@ -1151,13 +1227,13 @@ extern ComPatPtr urkCompilePattern (CharPtr pattern, Int2 moltype)
       }
       if (*pattern == '\0')
       {
-        headcpp = urkComPatFree (headcpp);
+        headcpp = ComPatFree (headcpp);
         return headcpp;
       }
       break;
      case ')':
      case ']':
-      headcpp = urkComPatFree (headcpp);
+      headcpp = ComPatFree (headcpp);
       return headcpp;
      default:
       if (!flagCount)
@@ -1183,7 +1259,7 @@ extern ComPatPtr urkCompilePattern (CharPtr pattern, Int2 moltype)
   {
     if (*(cpp->res) == '\0')
     {
-      headcpp = urkComPatFree (headcpp);
+      headcpp = ComPatFree (headcpp);
       return headcpp;
     }
     cpp = cpp->next;
@@ -1444,13 +1520,13 @@ extern SeqAlignPtr PatternMatchBioseq (BioseqPtr bsp, ComPatPtr cpp,
     SeqAlignLink (&saphead, sap);
     if (ISA_na (bsp->mol) && !cpp->flagPalindrome)
     {
-      cppi = urkInvertPattern (cpp);
+      cppi = InvertPattern (cpp);
       misses = 0;
       sap = PatternMatchX (seq, start, Seq_strand_minus, bsp->id, cppi, 0, 0,
                            mismatch, &misses, bsp->topology, TRUE, TRUE);
       SeqAlignLink (&saphead, sap);
-      SeqAlignSortByStart (&saphead);
-      urkComPatFree (cppi);
+      URK_SeqAlignSortByStart (&saphead);
+      ComPatFree (cppi);
     }
     MemFree (seq);
     start += (seqchunk - patlenmax + 1);
@@ -1478,16 +1554,16 @@ extern SeqAlignPtr PatternMatchBioseq (BioseqPtr bsp, ComPatPtr cpp,
           SeqAlignLink (&saphead, sap);
           if (ISA_na (bsp->mol) && !cpp->flagPalindrome)
           {
-            cppi = urkInvertPattern (cpp);
+            cppi = InvertPattern (cpp);
             misses = 0;
             sap = PatternMatchX (seq, start, Seq_strand_minus, bsp->id, cppi,
                                  0, 0, mismatch, &misses, bsp->topology,
                                  TRUE, TRUE);
             WrapSeqAlign (sap, bsp->length);
             SeqAlignLink (&saphead, sap);
-            urkComPatFree (cppi);
+            ComPatFree (cppi);
           }
-          SeqAlignSortByStart (&saphead);
+          URK_SeqAlignSortByStart (&saphead);
           MemFree (seq);
         }
         MemFree (seqr);
@@ -1501,95 +1577,139 @@ extern SeqAlignPtr PatternMatchBioseq (BioseqPtr bsp, ComPatPtr cpp,
 
 /* Profiles */
 
+static Boolean IS_COMMENT (Char c)
+{
+  if (c == '#' || c == ';')
+    return TRUE;
+  else
+    return FALSE;
+}
+
+/* format
+;
+# comment lines are ; or #
+;
+
+A 1.5   2
+G 1      2.5
+T 0.000   3
+C -15.3    3.5
+
+*/
+
 extern ComProfPtr ReadProfile (CharPtr filename)
 {
   FILE       *fin;
+  Char       bufin[PATH_MAX];
   CharPtr    inbuff;
-  CharPtr    hptr, tptr;
+  CharPtr    hptr;
   FloatHiPtr weight;
-  Int4       blen = 2048;
-  Int4       i, size = 0;
+  Int4       i;
+  Int4       blen = 2048; /* sort've determines max size of profile */
+  Int4       size = 0;
+  Char       residue;
   ComProfPtr hppp = NULL, ppp;
 
-  if ((fin = FileOpen (filename, "r")) == NULL)
+  if (filename == NULL || !(StrLen (filename) > 0))
     return NULL;
 
+  if (FindPath ("ncbi", "ncbi", "data", bufin, sizeof (bufin)))
+    FileBuildPath (bufin, NULL, filename);
+  else
+    StrCpy (bufin, filename);
+
+  if ((fin = FileOpen (bufin, "r")) == NULL)
+  {
+    if ((fin = FileOpen (filename, "r")) == NULL)
+    {
+      return NULL;
+    }
+  }
+
   inbuff = MemNew (sizeof (Char) * blen);
-  weight = MemNew (sizeof (FloatHi) * MAXPROFSIZE);
-  if (inbuff == NULL || weight == NULL)
+  if (inbuff == NULL)
   {
-    MemFree (inbuff);
-    MemFree (weight);
     FileClose (fin);
     return NULL;
   }
 
   while ((FileGets (inbuff, blen, fin)) != NULL)
   {
-    if (*inbuff == '#' || *inbuff == ';')
-      continue;
-
-    if (size == 0)
-    {
-      hptr = inbuff;
-      while (*hptr != ';' && *hptr != '#' && *hptr != '\n' && *hptr != '\0')
-      {
-        while (*hptr == ' ')
-         hptr++;
-        while (*hptr != ' ' && *hptr != ';' && *hptr != '#' &&
-               *hptr != '\n' && *hptr != '\0')
-          hptr++;
-        if (*hptr != ';' && *hptr != '#')
-          size++;
-      }
-      if (size > 1)
-      {
-        hppp = ppp = ComProfNew (hppp);
-        for (i = 1; i < size; i++)
-          ComProfNew (hppp);
-      }
-      else
-      {
-        MemFree (inbuff);
-        MemFree (weight);
-        FileClose (fin);
-        return hppp;
-      }
-    }
-    if ((hptr = StrChr (inbuff, ';')) == NULL)
-    {
-      MemFree (inbuff);
-      MemFree (weight);
-      FileClose (fin);
-      hppp = ComProfFree (hppp);
-      return hppp;
-    }
-    if (*hptr == ';')
-      hptr++;
-    while (*hptr == ' ' && *hptr != '\0')
-      hptr++;
-    if (*hptr == '\0')
-    {
-      MemFree (inbuff);
-      MemFree (weight);
-      FileClose (fin);
-      hppp = ComProfFree (hppp);
-      return hppp;
-    }
-    tptr = hptr;
     hptr = inbuff;
-    for (i = 0; i < size; i++)
+    while (IS_WHITESP (*hptr))
+      hptr++;
+    if (IS_COMMENT (*hptr))
+      continue;
+    if (*hptr == '\0')
+      continue;
+    if (!IS_ALPHA (*hptr) && *hptr != '*')
+      continue;
+    hptr++;
+/* first real line determine the matrix width (or size) */
+    while (!IS_COMMENT (*hptr) && *hptr != '\0')
     {
-      while (*hptr == ' ')
+      while (IS_WHITESP (*hptr))
         hptr++;
+      if (IS_COMMENT (*hptr) || *hptr == '\0')
+        continue;
+      while (!IS_WHITESP (*hptr) && !IS_COMMENT (*hptr) && *hptr != '\0')
+        hptr++;
+      size++;
+    }
+    if (size > 1)
+    {
+      hppp = ComProfNew (NULL);
+      for (i = 1; i < size; i++)
+        ComProfNew (hppp);
+    }
+    else
+    {
+      MemFree (inbuff);
+      FileClose (fin);
+      return NULL;
+    }
+    break;
+  }
+
+  rewind (fin);
+  weight = MemNew (sizeof (FloatHi) * size);
+  while ((FileGets (inbuff, blen, fin)) != NULL)
+  {
+    hptr = inbuff;
+    while (IS_WHITESP (*hptr))
+      hptr++;
+    if (IS_COMMENT (*hptr))
+      continue;
+    if (*hptr == '\0')
+      continue;
+    if (!IS_ALPHA (*hptr) && *hptr != '*')
+      continue;
+    residue = TO_UPPER (*hptr);
+    hptr++;
+    i = 0;
+    while (i < size)
+    {
+      while (IS_WHITESP (*hptr))
+        hptr++;
+      if (IS_COMMENT (*hptr) || *hptr == '\0')
+        continue;
       weight[i] = atof (hptr);
-      while (*hptr != ' ')
+      i++;
+      while (!IS_WHITESP (*hptr) && !IS_COMMENT (*hptr) && *hptr != '\0')
         hptr++;
     }
+    if (i != size)
+    {
+      MemFree (inbuff);
+      MemFree (weight);
+      ComProfFree (hppp);
+      FileClose (fin);
+      return NULL;
+    }
     ppp = hppp;
     for (i = 0; i < size; i++)
     {
-      ppp->score[*tptr-'A'] = weight[i];
+      ppp->score[residue-' '] = weight[i];
       ppp = ppp->next;
     }
   }
@@ -1609,7 +1729,7 @@ extern ComProfPtr ComProfNew (ComProfPtr curppp)
   ppp->next = NULL;
   ppp->prevprofile = NULL;
   ppp->nextprofile = NULL;
-  for (i = 0; i < 32; i++)
+  for (i = 0; i < 64; i++)
     ppp->score[i] = 0.0;
 
   if (curppp == NULL)
@@ -1637,42 +1757,142 @@ extern ComProfPtr ComProfFree (ComProfPtr headppp)
   return headppp;
 }
 
-extern ComProfPtr InvertProfile (ComProfPtr ppp)
+extern ComProfPtr ComProfDup (ComProfPtr pppold)
 {
-  ComProfPtr ppphead;
+  ComProfPtr   ppp, pppt, pppnew, ppphead, pppheadprof = NULL;
+  Int4         i;
+
+  while (pppold != NULL)
+  {
+    ppp = pppold;
+    ppphead = NULL;
+    pppnew = NULL;
+    while (ppp != NULL)
+    {
+      pppnew = ComProfNew (pppnew);
+      if (ppphead == NULL)
+        ppphead = pppnew;
+      StrCpy (pppnew->name, ppp->name);
+      for (i = 0; i < 64; i++)
+        pppnew->score[i] = ppp->score[i];
+      ppp = ppp->next;
+    }
+    if (pppheadprof == NULL)
+    {
+      pppheadprof = ppphead;
+    }
+    else
+    {
+      pppt = pppheadprof;
+      while (pppt->nextprofile != NULL)
+        pppt = pppt->nextprofile;
+      pppt->nextprofile = ppphead;
+      ppphead->prevprofile = pppt;
+    }
+    pppold = pppold->nextprofile;
+  }
+
+  return pppheadprof;
+}
+
+extern ComProfPtr ComProfDupThis (ComProfPtr pppthis)
+{
+  ComProfPtr   pppdup = NULL;
+  Int4         i;
+
+  if (pppthis != NULL)
+  {
+    pppdup = ComProfNew (NULL);
+    StrCpy (pppdup->name, pppthis->name);
+    for (i = 0; i < 64; i++)
+      pppdup->score[i] = pppthis->score[i];
+  }
+  return pppdup;
+}
+
+extern void ComProfLink (ComProfPtr PNTR ppphead, ComProfPtr pppdup)
+{
+  ComProfPtr   ppp;
+
+  ppp = *ppphead;
+  if (ppp == NULL)
+  {
+    *ppphead = pppdup;
+    return;
+  }
+  else
+  {
+    while (ppp->next != NULL)
+      ppp = ppp->next;
+    ppp->next = pppdup;
+    pppdup->prev = ppp;
+    return;
+  }
+}
+
+extern void IntegerProfile (ComProfPtr ppp)
+{
+  Int4 i;
+
+  while (ppp != NULL)
+  {
+    for (i = 0; i < 64; i++)
+    {
+      ppp->iscore[i] = (Int4) ppp->score[i];
+    }
+    ppp = ppp->next;
+  }
+  return;
+}
+
+extern ComProfPtr InvertProfile (ComProfPtr pppold)
+{
+  ComProfPtr ppphead, ppp, ppptemp;
   FloatHi    score;
 
-  ppphead = ppp;
+  ppphead = ppp = ComProfDup (pppold);
   while (ppp != NULL)
   {
 /* A T */
-    score = ppp->score['A'-'A'];
-    ppp->score['A'-'A'] = ppp->score['T'-'A'];
-    ppp->score['T'-'A'] = score;
+    score = ppp->score['A'-' '];
+    ppp->score['A'-' '] = ppp->score['T'-' '];
+    ppp->score['T'-' '] = score;
 /* G C */
-    score = ppp->score['G'-'A'];
-    ppp->score['G'-'A'] = ppp->score['C'-'A'];
-    ppp->score['C'-'A'] = score;
+    score = ppp->score['G'-' '];
+    ppp->score['G'-' '] = ppp->score['C'-' '];
+    ppp->score['C'-' '] = score;
 /* purine pyrimidine */
-    score = ppp->score['R'-'A'];
-    ppp->score['R'-'A'] = ppp->score['Y'-'A'];
-    ppp->score['Y'-'A'] = score;
+    score = ppp->score['R'-' '];
+    ppp->score['R'-' '] = ppp->score['Y'-' '];
+    ppp->score['Y'-' '] = score;
 /* AC GT */
-    score = ppp->score['K'-'A'];
-    ppp->score['K'-'A'] = ppp->score['M'-'A'];
-    ppp->score['M'-'A'] = score;
+    score = ppp->score['K'-' '];
+    ppp->score['K'-' '] = ppp->score['M'-' '];
+    ppp->score['M'-' '] = score;
 /* GCA CGT */
-    score = ppp->score['V'-'A'];
-    ppp->score['V'-'A'] = ppp->score['B'-'A'];
-    ppp->score['B'-'A'] = score;
+    score = ppp->score['V'-' '];
+    ppp->score['V'-' '] = ppp->score['B'-' '];
+    ppp->score['B'-' '] = score;
 /* ATG TAC */
-    score = ppp->score['D'-'A'];
-    ppp->score['D'-'A'] = ppp->score['H'-'A'];
-    ppp->score['H'-'A'] = score;
-
+    score = ppp->score['D'-' '];
+    ppp->score['D'-' '] = ppp->score['H'-' '];
+    ppp->score['H'-' '] = score;
     ppp = ppp->next;
   }
 
+  ppp = ppphead;
+  while (ppp->next != NULL)
+    ppp = ppp->next;
+  ppphead = ppp;
+  ppptemp = ppp->next;
+  while (ppp != NULL)
+  {
+    ppp->next = ppp->prev;
+    ppp->prev = ppptemp;
+    ppptemp = ppp;
+    ppp = ppp->next;
+  }
+  ppphead->prev = NULL;
   return ppphead;
 }
 
@@ -1684,7 +1904,7 @@ extern FloatHi ProfScoreMax (ComProfPtr ppp)
   while (ppp != NULL)
   {
     score = 0.0;
-    for (i = 0; i < 32; i++)
+    for (i = 0; i < 64; i++)
     {
       if (ppp->score[i] > score)
         score = ppp->score[i];
@@ -1695,7 +1915,7 @@ extern FloatHi ProfScoreMax (ComProfPtr ppp)
   return scoremax;
 }
 
-static Int4 ProfLenMax (ComProfPtr ppp)
+extern Int4 ProfLenMax (ComProfPtr ppp)
 {
   Int4 i = 0;
 
@@ -1708,22 +1928,56 @@ static Int4 ProfLenMax (ComProfPtr ppp)
   return i;
 }
 
+extern Int4 CatenateProfile (ComProfPtr ppphead, Int4 proflen, Int4 icount)
+{
+  Int4       i, countthis, lenthis;
+  ComProfPtr pppthis, pppdup;
+
+  lenthis = proflen;
+  if (ppphead == NULL)
+    return lenthis;
+
+  pppthis = ppphead;
+  countthis = 0;
+  for (i = 0; i < icount; i++)
+  {
+    pppdup = ComProfDupThis (pppthis);
+    ComProfLink (&ppphead, pppdup);
+    lenthis++;
+    countthis++;
+    if (countthis == proflen)
+    {
+      pppthis = ppphead;
+      countthis = 0;
+    }
+    else
+    {
+      pppthis = pppthis->next;
+    }
+  }
+  return lenthis;
+}
+
 /*
- flagFirstInvocation *MUST* be TRUE on callers' invocation
-                     this sets prlnmx and scoremax
- flagFindAllMatches  if FALSE on intial invocation then
-                     just find the first hit
+ flagFirstInvocation   *MUST* be TRUE on callers' invocation
+                       this sets prlnmx and scoremax
+ flagFindAllMatches    if FALSE on intial invocation then
+                       just find the first hit
+ flagBoundaryCondition just finds the fuzzy entries and ignores
+                       the clear Yays and Nays
 */
 static SeqAlignPtr ProfileMatchX (Uint1Ptr seq, Int4 seqpos, FloatHi cutoff,
                                   Uint1 strand, SeqIdPtr sip, ComProfPtr ppp,
                                   FloatHi scoremax, FloatHi score,
                                   Int4 prlnmx, Int2 topology,
                                   Boolean flagFirstInvocation,
-                                  Boolean flagFindAllMatches)
+                                  Boolean flagFindAllMatches,
+                                  Boolean flagBoundaryCondition)
 {
   SeqAlignPtr  saphead = NULL, sap;
+  FloatHi      lowcut, highcut;
 
-  if (ppp == NULL || *seq == '\0')
+  if (ppp == NULL || *seq == 0)
     return saphead;
 
   if (flagFirstInvocation)
@@ -1733,23 +1987,36 @@ static SeqAlignPtr ProfileMatchX (Uint1Ptr seq, Int4 seqpos, FloatHi cutoff,
     scoremax = ProfScoreMax (ppp);
   }
 
-  while (*seq != '\0')
+  while (*seq != 0)
   {
     if (*seq >= (Uint1) 'A' && *seq <= (Uint1) 'Z')
-      score += (ppp->score[*seq-'A']);
+      score += (ppp->score[*seq-' ']);
     if (ppp->next != NULL)
     {
       seq++;
       seqpos++;
       sap = ProfileMatchX (seq, seqpos, cutoff, strand, sip, ppp->next,
-                           scoremax, score, prlnmx, topology, FALSE, FALSE);
+                           scoremax, score, prlnmx, topology, FALSE, FALSE,
+                           flagBoundaryCondition);
     }
     else
     {
-      if (score >= cutoff)
-        sap = ProfSeqAlign (seqpos, prlnmx, strand, score, sip, topology);
+      if (flagBoundaryCondition)
+      {
+        lowcut = cutoff * 0.90;
+        highcut = cutoff + (cutoff - (cutoff * 0.90));
+        if (score >= lowcut && score <= highcut)
+          sap = ProfSeqAlign (seqpos, prlnmx, strand, score, sip, topology);
+        else
+          sap = NULL;
+      }
       else
-        sap = NULL;
+      {
+        if (score >= cutoff)
+          sap = ProfSeqAlign (seqpos, prlnmx, strand, score, sip, topology);
+        else
+          sap = NULL;
+      }
     }
     if (flagFirstInvocation)
     {
@@ -1775,16 +2042,97 @@ static SeqAlignPtr ProfileMatchX (Uint1Ptr seq, Int4 seqpos, FloatHi cutoff,
   return saphead;
 }
 
+static SeqAlignPtr ProfileMatchXInt (Uint1Ptr seq, Int4 seqpos,
+                                     Int4 cutoff, Uint1 strand,
+                                     SeqIdPtr sip, ComProfPtr ppp,
+                                     Int4 scoremax, Int4 score,
+                                     Int4 prlnmx, Int2 topology,
+                                     Boolean flagFirstInvocation,
+                                     Boolean flagFindAllMatches,
+                                     Boolean flagBoundaryCondition)
+{
+  SeqAlignPtr  saphead = NULL, sap;
+  FloatHi      lowcut, highcut;
+
+  if (ppp == NULL || *seq == 0)
+    return saphead;
+
+  if (flagFirstInvocation)
+  {
+    score = 0;
+    prlnmx = ProfLenMax (ppp);
+    scoremax = (Int4) ProfScoreMax (ppp);
+  }
+
+  while (*seq != 0)
+  {
+    score += (ppp->iscore[*seq]);
+    if (ppp->next != NULL)
+    {
+      seq++;
+      seqpos++;
+      sap = ProfileMatchXInt (seq, seqpos, cutoff, strand, sip, ppp->next,
+                              scoremax, score, prlnmx, topology,
+                              FALSE, FALSE, flagBoundaryCondition);
+    }
+    else
+    {
+      if (flagBoundaryCondition)
+      {
+        lowcut = cutoff * 0.8;
+        highcut = cutoff + (cutoff - (cutoff * 0.8));
+        if (score >= lowcut && score <= highcut)
+          sap = ProfSeqAlign (seqpos, prlnmx, strand, (FloatHi) score, sip,
+                              topology);
+        else
+          sap = NULL;
+      }
+      else
+      {
+        if (score >= cutoff)
+          sap = ProfSeqAlign (seqpos, prlnmx, strand, (FloatHi) score, sip,
+                              topology);
+        else
+          sap = NULL;
+      }
+    }
+    if (flagFirstInvocation)
+    {
+      score = 0;
+      if (flagFindAllMatches)
+      {
+        SeqAlignLink (&saphead, sap);
+        seq--;
+        seqpos--;
+      }
+      else
+      {
+        return sap;
+      }
+    }
+    else
+    {
+      return sap;
+    }
+    seq++;
+    seqpos++;
+  }
+  return saphead;
+}
+
 extern SeqAlignPtr ProfileMatch (Uint1Ptr seq, Int4 seqpos, FloatHi cutoff,
                                  Uint1 strand, SeqIdPtr sip, ComProfPtr ppp,
-                                 Int2 topology, Boolean flagFindAllMatches)
+                                 Int2 topology, Boolean flagFindAllMatches,
+                                 Boolean flagBoundaryCondition)
 {
   return ProfileMatchX (seq, seqpos, cutoff, strand, sip, ppp,
-                        0.0, 0.0, 0, topology, TRUE, flagFindAllMatches);
+                        0.0, 0.0, 0, topology, TRUE, flagFindAllMatches,
+                        flagBoundaryCondition);
 }
 
 extern SeqAlignPtr ProfileMatchBioseq (BioseqPtr bsp, ComProfPtr ppp,
-                                       FloatHi cutoff)
+                                       ComProfPtr ippp, FloatHi cutoff,
+                                       Boolean flagBoundaryCondition)
 {
   SeqAlignPtr saphead = NULL, sap;
   SeqPortPtr  spp;
@@ -1808,20 +2156,70 @@ extern SeqAlignPtr ProfileMatchBioseq (BioseqPtr bsp, ComProfPtr ppp,
                              seqchunk)) != NULL)
   {
     sap = ProfileMatchX (seq, start, cutoff, Seq_strand_plus, bsp->id,
-                         ppp, 0.0, 0.0, 0, bsp->topology, TRUE, TRUE);
+                         ppp, 0.0, 0.0, 0, bsp->topology, TRUE, TRUE,
+                         flagBoundaryCondition);
     SeqAlignLink (&saphead, sap);
-    if (ISA_na (bsp->mol))
+    if (ippp != NULL)
     {
-      ppp = InvertProfile (ppp);
-      sap = ProfileMatchX (seq, start, cutoff, Seq_strand_plus, bsp->id,
-                           ppp, 0.0, 0.0, 0, bsp->topology, TRUE, TRUE);
+      sap = ProfileMatchX (seq, start, cutoff, Seq_strand_minus, bsp->id,
+                           ippp, 0.0, 0.0, 0, bsp->topology, TRUE, TRUE,
+                           flagBoundaryCondition);
       SeqAlignLink (&saphead, sap);
     }
     MemFree (seq);
     start += (seqchunk - proflenmax + 1);
   }
   SeqPortFree (spp);
-  return sap;
+  return saphead;
+}
+
+extern SeqAlignPtr IntProfileMatchBioseq (BioseqPtr bsp, ComProfPtr ppp,
+                                          ComProfPtr ippp, Int4 cutoff,
+                                          Boolean flagBoundaryCondition)
+{
+  SeqAlignPtr saphead = NULL, sap;
+  SeqPortPtr  spp;
+  Uint1Ptr    seq, seqtmp;
+  Int4        proflenmax;
+  Int4        seqchunk, start;
+
+  if (ISA_na (bsp->mol))
+  {
+    spp = SeqPortNew (bsp, 0, bsp->length-1, 0, Seq_code_iupacna);
+  }
+  else
+  {
+    spp = SeqPortNew (bsp, 0, bsp->length-1, 0, Seq_code_iupacaa);
+  }
+
+  proflenmax = ProfLenMax (ppp);
+  seqchunk = MAXCHUNKSIZE;
+  start = 0;
+  while ((seq = GetSeqChunk (spp, start, bsp->length,
+                             seqchunk)) != NULL)
+  {
+    seqtmp = seq;
+    while (*seqtmp != 0)
+    {
+      *seqtmp -= ' ';    /* OKAY -- shouldn't been blanks in seq */
+      seqtmp++;
+    }
+    sap = ProfileMatchXInt (seq, start, cutoff, Seq_strand_plus, bsp->id,
+                            ppp, 0, 0, 0, bsp->topology, TRUE, TRUE,
+                            flagBoundaryCondition);
+    SeqAlignLink (&saphead, sap);
+    if (ippp != NULL)
+    {
+      sap = ProfileMatchXInt (seq, start, cutoff, Seq_strand_minus, bsp->id,
+                              ippp, 0, 0, 0, bsp->topology, TRUE, TRUE,
+                              flagBoundaryCondition);
+      SeqAlignLink (&saphead, sap);
+    }
+    MemFree (seq);
+    start += (seqchunk - proflenmax + 1);
+  }
+  SeqPortFree (spp);
+  return saphead;
 }
 
 /* Molecular Weight */
@@ -2057,6 +2455,7 @@ extern void EmbedMolecularWeightInfo (SeqAlignPtr saph, BioseqPtr bsp)
     return;
 
   start = 0;
+  stop = 0;
   sap = saph;
   while (sap != NULL)
   {
@@ -2164,7 +2563,7 @@ static int LIBCALLBACK SeqAlignStartCompProc (VoidPtr ptr1, VoidPtr ptr2)
   return 0;
 }
 
-extern void SeqAlignSortByStart (SeqAlignPtr PNTR sap)
+extern void URK_SeqAlignSortByStart (SeqAlignPtr PNTR sap)
 {
   SeqAlignPtr sapf, sapt, PNTR saph;
   Int4 num, i;
@@ -2238,7 +2637,7 @@ static int LIBCALLBACK SeqAlignLenCompProc (VoidPtr ptr1, VoidPtr ptr2)
   return 0;
 }
 
-extern void SeqAlignSortByLength (SeqAlignPtr PNTR sap)
+extern void URK_SeqAlignSortByLength (SeqAlignPtr PNTR sap)
 {
   SeqAlignPtr sapt, PNTR saph;
   Int4 num, i;
@@ -2304,7 +2703,7 @@ static int LIBCALLBACK SeqAlignMWCompProc (VoidPtr ptr1, VoidPtr ptr2)
   return 0;
 }
 
-extern void SeqAlignSortByMolWt (SeqAlignPtr PNTR sap)
+extern void URK_SeqAlignSortByMolWt (SeqAlignPtr PNTR sap)
 {
   SeqAlignPtr sapt, PNTR saph;
   Int4 num, i;
@@ -2339,3 +2738,60 @@ extern void SeqAlignSortByMolWt (SeqAlignPtr PNTR sap)
   }
   return;
 }
+
+static int LIBCALLBACK SeqAlignScoreCompProc (VoidPtr ptr1, VoidPtr ptr2)
+{
+  SeqAlignPtr sap1, sap2;
+
+  if (ptr1 != NULL && ptr2 != NULL)
+  {
+    sap1 = *((SeqAlignPtr PNTR) ptr1);
+    sap2 = *((SeqAlignPtr PNTR) ptr2);
+    if (sap1 != NULL && sap2 != NULL)
+    {
+      if (sap1->score->value.realvalue >
+          sap2->score->value.realvalue)
+        return 1;
+      else if (sap1->score->value.realvalue <
+               sap2->score->value.realvalue)
+        return -1;
+    }
+  }
+  return 0;
+}
+
+extern void URK_SeqAlignSortByScore (SeqAlignPtr PNTR sap)
+{
+  SeqAlignPtr sapt, PNTR saph;
+  Int4 num, i;
+
+  num = 0;
+  sapt = *sap;
+  while (sapt != NULL)
+  {
+    num++;
+    sapt = sapt->next;
+  }
+
+  if (num > 1)
+  {
+    saph = MemNew ((size_t) ((num+1) * sizeof (SeqAlignPtr)));
+    i = 0;
+    sapt = *sap;
+    while (sapt != NULL && i < num)
+    {
+      saph[i] = sapt;
+      sapt = sapt->next;
+      i++;
+    }
+    HeapSort (saph, num, sizeof (SeqAlignPtr), SeqAlignScoreCompProc);
+    for (i = 0; i < num; ++i)
+    {
+      sapt = saph[i];
+      sapt->next = saph[i+1];
+    }
+    *sap = saph[0];
+    MemFree (saph);
+  }
+  return;
+}
