@@ -29,13 +29,32 @@
 *
 * Version Creation Date:   10/23/92
 *
-* $Revision: 6.5 $
+* $Revision: 6.11 $
 *
 * File Description: 
 *
 * Modifications:  
 * --------------------------------------------------------------------------
 * $Log: picture.h,v $
+* Revision 6.11  1999/10/13 17:45:46  kans
+* added entityID, itemID, and itemtype to primitive internal structure, added Get and Set functions
+*
+* Revision 6.10  1999/10/04 19:39:09  kans
+* include vibdefns.h
+*
+* Revision 6.9  1999/10/04 17:16:32  kans
+* include ncbidraw.h instead of vibrant.h, a couple Nlm_ prefixes
+*
+* Revision 6.8  1998/07/01 18:27:43  vakatov
+* Use "const" qualifier somewhere
+*
+* Revision 6.7  1998/06/30 00:32:03  vakatov
+* Nlm_AddAttribute():  pass "color" as constant
+*
+* Revision 6.6  1998/06/01 17:29:20  vakatov
+* Extended rounded rect. options -- let it roundup only two(adjacent)
+* corners(see "flags" arg in Nlm_AddRoundedRectangle())
+*
 * Revision 6.5  1998/04/27 16:05:46  vakatov
 * + AddRoundedRectangle()
 *
@@ -66,8 +85,12 @@
 #ifndef _PICTURE_
 #define _PICTURE_
 
-#ifndef _VIBRANT_
-#include <vibrant.h>
+#ifndef _NCBIDRAW_
+#include <ncbidraw.h>
+#endif
+
+#ifndef _VIBDEFNS_
+#include <vibdefns.h>
 #endif
 
 #ifdef __cplusplus
@@ -315,7 +338,7 @@ extern void          Nlm_SegmentBox     PROTO((Nlm_SegmenT segment,
 Nlm_BoxPtr box));
 
 extern void          Nlm_AddAttribute   PROTO((Nlm_SegmenT parent, 
-Nlm_Uint1 flags, Nlm_Uint1Ptr color, Nlm_Int1 linestyle, Nlm_Int1 shading, 
+Nlm_Uint1 flags, const Nlm_Uint1* color, Nlm_Int1 linestyle, Nlm_Int1 shading, 
 Nlm_Int1 penwidth, Nlm_Int1 mode));
 extern void          Nlm_SetDefArrowSize PROTO((Nlm_Int1 arrowSize));
 extern Nlm_PrimitivE Nlm_AddPrimitive PROTO((Nlm_PrimDefPtr pdp,
@@ -325,9 +348,16 @@ Nlm_Int4 left, Nlm_Int4 top, Nlm_Int4 right, Nlm_Int4 bottom, Nlm_Uint2 primID))
 extern Nlm_PrimitivE Nlm_AddRectangle   PROTO((Nlm_SegmenT parent, 
 Nlm_Int4 left, Nlm_Int4 top, Nlm_Int4 right, Nlm_Int4 bottom, Nlm_Int2 arrow,
 Nlm_Boolean fill, Nlm_Uint2 primID));
+
+#define ROREC_TOP    0x1
+#define ROREC_BOTTOM 0x2
+#define ROREC_LEFT   0x4
+#define ROREC_RIGHT  0x8
 extern Nlm_PrimitivE Nlm_AddRoundedRectangle PROTO((Nlm_SegmenT parent, 
 Nlm_Int4 left, Nlm_Int4 top, Nlm_Int4 right, Nlm_Int4 bottom,
-Nlm_Int4 oval_w, Nlm_Int4 oval_h, Nlm_Boolean fill, Nlm_Uint2 primID));
+Nlm_Int4 oval_w, Nlm_Int4 oval_h, Nlm_Boolean fill, Uint4 flags,
+Nlm_Uint2 primID));
+
 extern Nlm_PrimitivE Nlm_AddSegRect     PROTO((Nlm_SegmenT parent, 
 Nlm_Boolean fill, Nlm_Uint2 primID));
 extern Nlm_PrimitivE Nlm_AddLine        PROTO((Nlm_SegmenT parent, 
@@ -343,10 +373,10 @@ extern Nlm_PrimitivE Nlm_AddMarker      PROTO((Nlm_SegmenT parent,
 Nlm_Int4 pntX, Nlm_Int4 pntY, Nlm_Int2 length, Nlm_Int2 orient, 
 Nlm_Uint2 primID));
 extern Nlm_PrimitivE Nlm_AddLabel       PROTO((Nlm_SegmenT parent, 
-Nlm_Int4 pntX, Nlm_Int4 pntY, Nlm_CharPtr string, Nlm_Int2 size, 
+Nlm_Int4 pntX, Nlm_Int4 pntY, const Nlm_Char* string, Nlm_Int2 size, 
 Nlm_Int2 offset, Nlm_Int2 align, Nlm_Uint2 primID));
 extern Nlm_PrimitivE Nlm_AddTextLabel   PROTO((Nlm_SegmenT parent, 
-Nlm_Int4 pntX, Nlm_Int4 pntY, Nlm_CharPtr string, Nlm_FonT font, 
+Nlm_Int4 pntX, Nlm_Int4 pntY, const Nlm_Char* string, Nlm_FonT font, 
 Nlm_Int2 offset, Nlm_Int2 align, Nlm_Uint2 primID));
 extern Nlm_PrimitivE Nlm_AddPolygon     PROTO((Nlm_SegmenT parent,
 Nlm_Int2 vernum, Nlm_PntPtr points, Nlm_Boolean fill, Nlm_Uint2 primID));
@@ -379,7 +409,9 @@ Nlm_VoidPtr userdata, Nlm_SegmentExploreProc callback));
 
 /*Working with primitive */
 
-extern Nlm_PrimitivE Nlm_GetPrimitive PROTO((Nlm_SegmenT segment, Uint2 primCt));
+extern Nlm_PrimitivE Nlm_GetPrimitive PROTO((Nlm_SegmenT segment, Nlm_Uint2 primCt));
+extern void Nlm_SetPrimitiveIDs PROTO((Nlm_PrimitivE prim, Nlm_Uint2 entityID, Nlm_Uint2 itemID, Nlm_Uint2 itemtype, Nlm_Uint2 primID));
+extern void Nlm_GetPrimitiveIDs PROTO((Nlm_PrimitivE prim, Nlm_Uint2Ptr entityIDPtr, Nlm_Uint2Ptr itemIDPtr, Nlm_Uint2Ptr itemtypePtr, Nlm_Uint2Ptr primIDPtr));
 extern void Nlm_GetPrimDrawAttribute PROTO((Nlm_PrimitivE prim, 
 Nlm_Uint1Ptr color, Nlm_Int1Ptr plinestyle, Nlm_Int1Ptr pshading, 
 Nlm_Int1Ptr ppenwidth, Nlm_Int1Ptr pmode, Nlm_Int1Ptr phighlight));
@@ -478,6 +510,8 @@ extern Nlm_enumPrimAddOrder Nlm_ChangeAddPrimOrder
 #define UnlinkSegment Nlm_UnlinkSegment
 #define ExploreSegment Nlm_ExploreSegment
 #define GetPrimitive Nlm_GetPrimitive
+#define SetPrimitiveIDs Nlm_SetPrimitiveIDs
+#define GetPrimitiveIDs Nlm_GetPrimitiveIDs
 #define GetPrimDrawAttribute Nlm_GetPrimDrawAttribute
 #define ChangePrimAttribute Nlm_ChangePrimAttribute
 #define OffsetPrim Nlm_OffsetPrim
