@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   04/03/95
 *
-* $Revision: 6.1 $
+* $Revision: 6.2 $
 *
 * File Description: 
 *
@@ -38,6 +38,10 @@
 * Date     Name        Description of modification
 * -------  ----------  -----------------------------------------------------
 * $Log: prim3d5.c,v $
+* Revision 6.2  1998/08/12 23:04:05  vakatov
+* [64-bit platforms]  Made "idBuffer" be VoidPtr[] rather than Int4[];
+* Fixed bugs mostly caused by casting between Int4 vars and 8-byte pointers
+*
 * Revision 6.1  1997/11/26 21:30:09  vakatov
 * Fixed errors and warnings issued by C and C++ (GNU and Sun) compilers
 *
@@ -79,7 +83,7 @@ if((Uint2)z>=*((Uint2Ptr)zptr)) \
 
 #define SETPIXELID(zptr,imageptr,z,curID) \
 if((Uint2)z>=*((Uint2Ptr)zptr)) \
-{*((Uint2Ptr)zptr)=(Uint2)z;*((Uint4Ptr)imageptr)=(Uint4)curID;}
+{*((Uint2Ptr)zptr)=(Uint2)z;*((VoidPtr*)imageptr)=curID;}
 
 /*****************************************************************************
 *
@@ -902,38 +906,40 @@ static void Nlm_Poly3DHitT ( Nlm_Poly3DPtr p )
       x1 -= reg1; x2 -= reg1;
       reg1 = (y-Nlm_stCon.ymin)*Nlm_stCon.widthCur;
       reg3 = (long)(Nlm_stCon.zBuffer + (reg1 + x1));
-      reg1 = (long)(Nlm_stCon.idBuffer + reg1 );
-      reg2 = reg1 + (x2<<2);
-      reg1 += (x1<<2);
-      while ( ((Uint4)reg2 - (Uint4)reg1) >= 16 ){
+      reg1 = (long)(Nlm_stCon.idBuffer + reg1);
+      reg2 = reg1 + x2 * sizeof(Nlm_stCon.idBuffer[0]);
+      reg1 += x1 * sizeof(Nlm_stCon.idBuffer[0]);
+      reg2 -= 4 * sizeof(Nlm_stCon.idBuffer[0]);
+      while (reg1 <= reg2) {
         reg4 = curz;
         curz = reg4 + stepZX;
         reg4 >>= 10;
         SETPIXELID(reg3,reg1,reg4,p);
         reg3++; reg3++;
-        reg1 +=4;
+        reg1 += sizeof(Nlm_stCon.idBuffer[0]);
         reg4 = curz;
         curz = reg4 + stepZX;
         reg4 >>= 10;
         SETPIXELID(reg3,reg1,reg4,p);
         reg3++; reg3++;
-        reg1 +=4;
+        reg1 += sizeof(Nlm_stCon.idBuffer[0]);
         reg4 = curz;
         curz = reg4 + stepZX;
         reg4 >>= 10;
         SETPIXELID(reg3,reg1,reg4,p);
         reg3++; reg3++;
-        reg1 += 4;
+        reg1 += sizeof(Nlm_stCon.idBuffer[0]);
         reg4 = curz;
         curz = reg4 + stepZX;
         reg4 >>= 10;
         SETPIXELID(reg3,reg1,reg4,p);
         reg3++; reg3++;
-        reg1 += 4;
+        reg1 += sizeof(Nlm_stCon.idBuffer[0]);
       }
-      reg1 -= 4;
+      reg1 -= sizeof(Nlm_stCon.idBuffer[0]);
+      reg2 += 4 * sizeof(Nlm_stCon.idBuffer[0]);
       do {
-        reg1 += 4;
+        reg1 += sizeof(Nlm_stCon.idBuffer[0]);
         reg4 = curz;
         curz = reg4 + stepZX;
         reg4 >>= 10;
@@ -1053,7 +1059,7 @@ Boolean Nlm_AddVertPoly3D ( Nlm_Picture3D pic, Nlm_Prim3D poly,
   if ( p == NULL ) return FALSE;
   vNum = p->vNum + 1;
   if ( vNum == 1 ){
-    vertPtr = (Int4Ptr) MemNew( sizeof(long)*300 );
+    vertPtr = (Int4Ptr) MemNew( sizeof(Int4)*300 );
     p->vert = vertPtr;
     x1 = p->x1; y1 = p->y1; z1 = p->z1;
     x2 = p->x2; y2 = p->y2; z2 = p->z2;
@@ -1063,8 +1069,8 @@ Boolean Nlm_AddVertPoly3D ( Nlm_Picture3D pic, Nlm_Prim3D poly,
     x2 = *vertPtr++; y2 = *vertPtr++; z2 = *vertPtr++;
     vertPtr += 3;
   } else if ( ((vNum/50)*50) == vNum ){
-    vertPtr = (Int4Ptr) MemNew( (vNum+50)*6*sizeof(long) );
-    MemCopy ( vertPtr, p->vert, (vNum-1)*6*sizeof(long) );
+    vertPtr = (Int4Ptr) MemNew( (vNum+50)*6*sizeof(Int4) );
+    MemCopy ( vertPtr, p->vert, (vNum-1)*6*sizeof(Int4) );
     MemFree ( p->vert );
     p->vert = vertPtr;
     vertPtr += (vNum-3)*6;
