@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/1/91
 *
-* $Revision: 6.5 $
+* $Revision: 6.8 $
 *
 * File Description: 
 *       Vibrant alias definitions
@@ -37,6 +37,15 @@
 * Modifications:  
 * --------------------------------------------------------------------------
 * $Log: vibdefns.h,v $
+* Revision 6.8  2001/09/10 17:34:21  bazhin
+* Added function Nlm_SetTextCursorBlinkRate(Nlm_TexT t, Nlm_Int2 msec).
+*
+* Revision 6.7  1999/04/06 14:23:25  lewisg
+* add opengl replacement for viewer3d
+*
+* Revision 6.6  1998/06/01 17:20:13  vakatov
+* Added code to draw/fill 90-degree arc/pie (quadrants)
+*
 * Revision 6.5  1998/03/22 03:00:57  kans
 * changed names to RegisterServiceProc and RegisterResultProc
 *
@@ -308,6 +317,7 @@ extern "C" {
 #define SetTextEditable Nlm_SetTextEditable
 #define SetTextCursorPos Nlm_SetTextCursorPos
 #define GetTextCursorPos Nlm_GetTextCursorPos
+#define SetTextCursorBlinkRate Nlm_SetTextCursorBlinkRate
 
 #define StaticPrompt Nlm_StaticPrompt
 
@@ -317,6 +327,9 @@ extern "C" {
 #define SimplePanel Nlm_SimplePanel
 #define AutonomousPanel4 Nlm_AutonomousPanel4
 #define AutonomousPanel Nlm_AutonomousPanel
+#ifdef _OPENGL
+#define Autonomous3DPanel Nlm_Autonomous3DPanel
+#endif /* _OPENGL */
 #define SetPanelClick Nlm_SetPanelClick
 #define SetPanelExtra Nlm_SetPanelExtra
 #define GetPanelExtra Nlm_GetPanelExtra
@@ -670,6 +683,11 @@ extern "C" {
 #define PaintArc Nlm_PaintArc
 #define InvertArc Nlm_InvertArc
 
+#define EraseQuadrant Nlm_EraseQuadrant
+#define FrameQuadrant Nlm_FrameQuadrant
+#define PaintQuadrant Nlm_PaintQuadrant
+#define InvertQuadrant Nlm_InvertQuadrant
+
 #define ErasePoly Nlm_ErasePoly
 #define FramePoly Nlm_FramePoly
 #define PaintPoly Nlm_PaintPoly
