@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.1 $
+* $Revision: 6.11 $
 *
 * File Description: 
 *
@@ -40,6 +40,36 @@
 *
 *
 * $Log: vibforms.c,v $
+* Revision 6.11  1999/10/21 16:50:40  kans
+* use SafeSetValue for enum lists and spreadsheets
+*
+* Revision 6.10  1999/10/20 22:09:44  kans
+* AlistDialogData public, has userdata field, convenience functions set userdata, initial value
+*
+* Revision 6.9  1999/10/18 21:47:39  kans
+* MakeEnumFieldAlistFromValNodeList strips paren and changes slash to dash
+*
+* Revision 6.8  1999/10/18 15:33:03  kans
+* added MakeEnumFieldAlistFromValNodeList
+*
+* Revision 6.7  1999/08/20 15:57:07  kans
+* CreateEnumListDialog calculates width if passed in 0
+*
+* Revision 6.6  1999/06/17 18:05:14  kans
+* fixed EnumPopupDialogToUIEnumPtr
+*
+* Revision 6.5  1999/06/16 20:55:02  kans
+* added CreateEnumListDialog using SingleList
+*
+* Revision 6.4  1999/06/16 19:03:25  kans
+* added CreateEnumPopupDialog
+*
+* Revision 6.3  1999/06/16 17:44:53  kans
+* added DuplicateEnumFieldAlist
+*
+* Revision 6.2  1999/03/11 16:10:25  kans
+* StringHasNoText and TrimSpacesAroundString moved to ncbistr
+*
 * Revision 6.1  1997/11/26 21:30:17  vakatov
 * Fixed errors and warnings issued by C and C++ (GNU and Sun) compilers
 *
@@ -219,7 +249,7 @@ CharPtr GetEnumName (UIEnum val, EnumFieldAssocPtr al)
 {
   for (; al->name != NULL; al++)
      if (al->value == val) return al->name;
-  Message(MSG_ERROR, "GetEnumName: %ld", (long)val);
+  Message(MSG_POSTERR, "GetEnumName: %ld", (long)val);
   return NULL;
 }
 
@@ -230,7 +260,7 @@ Boolean InitEnumPopup (PopuP lst, EnumFieldAssocPtr al, UIEnumPtr pdef)
   EnumFieldAssocPtr efap;
   CharPtr PNTR titles;
   if (al == NULL) {
-    Message(MSG_ERROR, "in InitEnumPopup");
+    Message(MSG_POSTERR, "in InitEnumPopup");
     return FALSE;
   }
   /*
@@ -259,7 +289,7 @@ Boolean InitEnumPopup (PopuP lst, EnumFieldAssocPtr al, UIEnumPtr pdef)
     MemFree (titles);
   }
   if (ii > 0) {
-    SetValue (lst, ii);
+    SafeSetValue (lst, ii);
     return TRUE;
   } else
     return FALSE;
@@ -277,8 +307,8 @@ void SetEnumPopup (PopuP lst, EnumFieldAssocPtr al, UIEnum val)
 {
   Int2 i;
   for (i = 1; al->name != NULL; i++, al++)
-     if (al->value == val) { SetValue (lst, i); return; }
-  Message(MSG_ERROR, "SetEnumPopup: %ld", (long)val);
+     if (al->value == val) { SafeSetValue (lst, i); return; }
+  Message(MSG_POSTERR, "SetEnumPopup: %ld", (long)val);
 } 
 
 CharPtr GetEnumPopupByName (PopuP lst, EnumFieldAssocPtr al)
@@ -294,8 +324,8 @@ void SetEnumPopupByName (PopuP lst, EnumFieldAssocPtr al, CharPtr name)
 {
   Int2 i;
   for (i = 1; al->name != NULL; i++, al++)
-     if (StringICmp (al->name, name) == 0) { SetValue (lst, i); return; }
-  Message(MSG_ERROR, "SetEnumPopupByName: %s", name);
+     if (StringICmp (al->name, name) == 0) { SafeSetValue (lst, i); return; }
+  Message(MSG_POSTERR, "SetEnumPopupByName: %s", name);
 }
 
 Boolean WhereInEnumPopup (EnumFieldAssocPtr al, CharPtr name, UIEnumPtr pval)
@@ -341,6 +371,35 @@ void SortEnumFieldAlist (EnumFieldAssocPtr alist)
   HeapSort (alist, count, sizeof (EnumFieldAssoc), CompareAlists);
 }
 
+EnumFieldAssocPtr DuplicateEnumFieldAlist (EnumFieldAssocPtr alist)
+
+{
+  EnumFieldAssocPtr  ap1, ap2;
+  size_t             count;
+  EnumFieldAssocPtr  newap;
+
+  if (alist == NULL) return NULL;
+  ap1 = alist;
+  count = 0;
+  while (ap1->name != NULL) {
+    ap1++;
+    count++;
+  }
+  if (count < 1) return NULL;
+  newap = MemNew (sizeof (EnumFieldAssoc) * (count + 1));
+  if (newap == NULL) return NULL;
+  ap1 = alist;
+  ap2 = newap;
+  while (ap1->name != NULL) {
+    ap2->name = StringSaveNoNull (ap1->name);
+    ap2->value = ap1->value;
+    ap1++;
+    ap2++;
+  }
+  ap2->name = NULL;
+  return newap;
+}
+
 EnumFieldAssocPtr FreeEnumFieldAlist (EnumFieldAssocPtr alist)
 
 {
@@ -355,6 +414,50 @@ EnumFieldAssocPtr FreeEnumFieldAlist (EnumFieldAssocPtr alist)
   return NULL;
 }
 
+EnumFieldAssocPtr MakeEnumFieldAlistFromValNodeList (ValNodePtr vlist)
+
+{
+  EnumFieldAssocPtr  ap, newap;
+  Char               ch;
+  size_t             count, i;
+  CharPtr            ptr;
+  ValNodePtr         vnp;
+
+  if (vlist == NULL) return NULL;
+  count = ValNodeLen (vlist);
+  if (count < 1) return NULL;
+
+  newap = MemNew (sizeof (EnumFieldAssoc) * (count + 1));
+  if (newap == NULL) return NULL;
+
+  for (ap = newap, vnp = vlist, i = 0;
+       vnp != NULL && i < count;
+       vnp = vnp->next, ap++, i++) {
+    ap->name = StringSaveNoNull ((CharPtr) vnp->data.ptrvalue);
+    ptr = StringStr (ap->name, " (");
+    if (ptr != NULL) {
+      *ptr = '\0';
+    }
+    if (ap->name != NULL) {
+      ptr = ap->name;
+      ch = *ptr;
+      while (ch != '\0') {
+        if (ch == '/') {
+          *ptr = '-';
+        }
+        ptr++;
+        ch = *ptr;
+      }
+    }
+    ap->value = (UIEnum) vnp->choice;
+  }
+
+  ap->name = NULL;
+  ap->value = 0;
+
+  return newap;
+}
+
 PopuP CreateEnumPopupListInitVal (GrouP prnt, Boolean macLike, PupActnProc actn,
                                   VoidPtr data, EnumFieldAssocPtr al, UIEnum val)
 
@@ -381,6 +484,124 @@ PopuP CreateEnumPopupListInitName (GrouP prnt, Boolean macLike, PupActnProc actn
   return p;
 }
 
+/* popup list autonomous dialog - copies alist, frees on cleanup */
+
+static void UIEnumPtrToEnumPopupDialog (DialoG d, Pointer data)
+
+{
+  AlistDialogPtr  adp;
+  Int4Ptr         intptr;
+  Int4            val;
+
+  adp = (AlistDialogPtr) GetObjectExtra (d);
+  intptr = (Int4Ptr) data;
+  if (adp != NULL && intptr != NULL) {
+    val = *intptr;
+    SetEnumPopup (adp->pop, adp->alist, (UIEnum) val);
+  }
+}
+
+static Pointer EnumPopupDialogToUIEnumPtr (DialoG d)
+
+{
+  AlistDialogPtr  adp;
+  Int4Ptr         intptr;
+  UIEnum          val;
+
+  intptr = NULL;
+  adp = (AlistDialogPtr) GetObjectExtra (d);
+  if (adp != NULL) {
+    if (GetEnumPopup (adp->pop, adp->alist, &val)) {
+      adp->intvalue = (Int4) val;
+      intptr = (&adp->intvalue);
+    }
+  }
+  return (Pointer) intptr;
+}
+
+static void ClearEnumPopupDialog (GraphiC g, VoidPtr data)
+
+{
+  AlistDialogPtr  adp;
+
+  adp = (AlistDialogPtr) data;
+  if (adp != NULL) {
+    FreeEnumFieldAlist (adp->alist);
+  }
+  MemFree (data);
+}
+
+PopuP CreateEnumPopupDialog (GrouP prnt, Boolean macLike, PupActnProc actn,
+                             EnumFieldAssocPtr al, UIEnum val, Pointer userdata)
+
+{
+  AlistDialogPtr  adp;
+  PopuP           p;
+
+  if (prnt == NULL || al == NULL) return NULL;
+  adp = (AlistDialogPtr) MemNew (sizeof (AlistDialogData));
+  if (adp == NULL) return NULL;
+  p = PopupList (prnt, macLike, actn);
+  if (p == NULL) {
+    MemFree (adp);
+    return NULL;
+  }
+  SetObjectExtra (p, adp, ClearEnumPopupDialog);
+  adp->dialog = (DialoG) p;
+  adp->todialog = UIEnumPtrToEnumPopupDialog;
+  adp->fromdialog = EnumPopupDialogToUIEnumPtr;
+  adp->pop = p;
+  adp->alist = DuplicateEnumFieldAlist (al);
+  adp->userdata = userdata;
+  InitEnumPopup (p, adp->alist, NULL);
+  SetEnumPopup (p, adp->alist, val);
+  return p;
+}
+
+LisT CreateEnumListDialog (GrouP prnt, Int2 width, Int2 height, LstActnProc actn,
+                           EnumFieldAssocPtr al, UIEnum val, Pointer userdata)
+
+{
+  AlistDialogPtr     adp;
+  EnumFieldAssocPtr  ap;
+  Int2               i;
+  Int2               len;
+  LisT               p;
+
+  if (prnt == NULL || al == NULL) return NULL;
+  adp = (AlistDialogPtr) MemNew (sizeof (AlistDialogData));
+  if (adp == NULL) return NULL;
+  if (width == 0) {
+    SelectFont (systemFont);
+    for (ap = al, i = 1; ap->name != NULL; i++, ap++) {
+      len = StringWidth (ap->name);
+      if (len > width) {
+        width = len;
+      }
+    }
+    width += stdCharWidth - 1;
+    width /= stdCharWidth;
+    width += 1;
+  }
+  p = SingleList (prnt, width, height, actn);
+  if (p == NULL) {
+    MemFree (adp);
+    return NULL;
+  }
+  SetObjectExtra (p, adp, ClearEnumPopupDialog);
+  adp->dialog = (DialoG) p;
+  adp->todialog = UIEnumPtrToEnumPopupDialog;
+  adp->fromdialog = EnumPopupDialogToUIEnumPtr;
+  adp->pop = (PopuP) p;
+  adp->alist = DuplicateEnumFieldAlist (al);
+  adp->userdata = userdata;
+  for (ap = adp->alist; ap->name != NULL; ap++) {
+    ListItem (p, ap->name);
+  }
+  SetEnumPopup ((PopuP) p, adp->alist, val);
+  return p;
+}
+
 #ifdef VAR_ARGS
 void CDECL RepeatProcOnHandles (proc, va_alist)
 HandleActnProc proc;
@@ -894,47 +1115,6 @@ extern void SafeSetTitle (Handle a, CharPtr title)
   }
 }
 
-extern CharPtr TrimSpacesAroundString (CharPtr str)
-
-{
-  Uchar    ch;	/* to use 8bit characters in multibyte languages */
-  CharPtr  dst;
-  CharPtr  ptr;
-
-  if (str != NULL && str [0] != '\0') {
-    dst = str;
-    ptr = str;
-    ch = *ptr;
-    while (ch != '\0' && ch <= ' ') {
-      ptr++;
-      ch = *ptr;
-    }
-    while (ch != '\0') {
-      *dst = ch;
-      dst++;
-      ptr++;
-      ch = *ptr;
-    }
-    *dst = '\0';
-    dst = NULL;
-    ptr = str;
-    ch = *ptr;
-    while (ch != '\0') {
-      if (ch != ' ') {
-        dst = NULL;
-      } else if (dst == NULL) {
-        dst = ptr;
-      }
-      ptr++;
-      ch = *ptr;
-    }
-    if (dst != NULL) {
-      *dst = '\0';
-    }
-  }
-  return str;
-}
-
 extern CharPtr SaveStringFromText (TexT t)
 
 {
@@ -986,24 +1166,6 @@ extern ValNodePtr GetVnpFromText (TexT t, ValNodePtr vnp, Boolean last)
   return vnp;
 }
 
-extern Boolean StringHasNoText (CharPtr str)
-
-{
-  Uchar  ch;	/* to use 8bit characters in multibyte languages */
-
-  if (str != NULL) {
-    ch = *str;
-    while (ch != '\0') {
-      if (ch > ' ') {
-        return FALSE;
-      }
-      str++;
-      ch = *str;
-    }
-  }
-  return TRUE;
-}
-
 extern Boolean TextHasNoText (TexT t)
 
 {
@@ -1334,7 +1496,7 @@ static void RedrawTagList (DialoG d)
                 SetEnumPopup ((PopuP) GetTagListControl (tlp, i, j), tlp->alists [j], (UIEnum) 0);
               }
             } else {
-              SetValue (GetTagListControl (tlp, i, j), 0);
+              SafeSetValue (GetTagListControl (tlp, i, j), 0);
             }
             break;
           default :
@@ -1355,7 +1517,7 @@ static void RedrawTagList (DialoG d)
             if (tlp->alists != NULL) {
               SetEnumPopup ((PopuP) GetTagListControl (tlp, i, j), tlp->alists [j], (UIEnum) 0);
             } else {
-              SetValue (GetTagListControl (tlp, i, j), 0);
+              SafeSetValue (GetTagListControl (tlp, i, j), 0);
             }
             break;
           default :
@@ -1557,10 +1719,10 @@ static TexT FindNextText (TagListPtr tlp, Int2 i, Int2 j)
         }
         val = GetValue (tlp->bar);
         if (val < tlp->max) {
-          SetValue (tlp->bar, val + 1);
+          SafeSetValue (tlp->bar, val + 1);
         } else {
           CheckExtendTag (tlp);
-          SetValue (tlp->bar, tlp->max);
+          SafeSetValue (tlp->bar, tlp->max);
         }
       }
     }
@@ -1667,7 +1829,7 @@ static void ResetTagList (DialoG d)
             if (tlp->alists != NULL) {
               SetEnumPopup ((PopuP) GetTagListControl (tlp, i, j), tlp->alists [j], (UIEnum) 0);
             } else {
-              SetValue (GetTagListControl (tlp, i, j), 0);
+              SafeSetValue (GetTagListControl (tlp, i, j), 0);
             }
             break;
           default :
@@ -1820,7 +1982,7 @@ extern DialoG CreateTagListDialogEx (GrouP h, Uint2 rows, Uint2 cols,
               SetObjectExtra (GetTagListControl (tlp, i, j), tlp, NULL);
               al = alists [j];
               if (al == NULL) {
-                Message(MSG_ERROR, "in InitEnumList");
+                Message(MSG_POSTERR, "in InitEnumList");
               } else {
                 lst = (LisT) GetTagListControl (tlp, i, j);
                 for (k = 1; al->name != NULL; k++, al++) {
