@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/1/91
 *
-* $Revision: 6.0 $
+* $Revision: 6.4 $
 *
 * File Description: 
 *       Vibrant inclusion of underlying windowing system toolbox functions,
@@ -43,6 +43,18 @@
 *
 *
 * $Log: vibincld.h,v $
+* Revision 6.4  2000/07/08 20:44:14  vakatov
+* Get all "#include" out of the 'extern "C" { }' scope;  other cleanup...
+*
+* Revision 6.3  1999/06/22 15:14:54  lewisg
+* fix image library so that works on linux with > 8 bits
+*
+* Revision 6.2  1999/04/06 14:23:25  lewisg
+* add opengl replacement for viewer3d
+*
+* Revision 6.1  1998/07/14 16:44:27  vakatov
+* Added VibrantIsGUI() and <internal> Nlm_VibrantSetGUI()
+*
 * Revision 6.0  1997/08/25 18:57:08  madden
 * Revision changed to 6.0
 *
@@ -265,11 +277,14 @@ typedef  struct  Nlm_boxrec {
   Nlm_BoxData     box;
 } Nlm_BoxRec, PNTR Nlm_BoxPtr;
 
+
+
 #ifdef WIN_MAC
 #define Nlm_WindowTool  WindowPtr
 #define Nlm_PortTool    GrafPtr
 #define Nlm_ShellTool   Nlm_Handle
 #define Nlm_MainTool    Nlm_Handle
+#define Nlm_ColorMTool  PaletteHandle
 #endif
 
 #ifdef WIN_MSWIN
@@ -277,6 +292,7 @@ typedef  struct  Nlm_boxrec {
 #define Nlm_PortTool    HDC
 #define Nlm_ShellTool   Nlm_Handle
 #define Nlm_MainTool    Nlm_Handle
+#define Nlm_ColorMTool  HPALETTE
 #endif
 
 #ifdef WIN_MOTIF
@@ -284,6 +300,7 @@ typedef  struct  Nlm_boxrec {
 #define Nlm_PortTool    GC
 #define Nlm_ShellTool   Widget
 #define Nlm_MainTool    Widget
+#define Nlm_ColorMTool  Colormap
 #endif
 
 extern  Nlm_WindoW     Nlm_desktopWindow;
@@ -498,6 +515,8 @@ void        Nlm_RecTToRectTool PROTO((Nlm_RectPtr src, Nlm_RectTool PNTR dst));
 void        Nlm_LocalToGlobal PROTO((Nlm_PointPtr pt));
 void        Nlm_GlobalToLocal PROTO((Nlm_PointPtr pt));
 
+void        Nlm_VibrantSetGUI PROTO((void));
+
 #ifdef WIN_MSWIN
 extern  Nlm_Boolean Nlm_VibrantDisabled PROTO((void));
 extern  HDC         Nlm_GetPicWinHDC    PROTO((void));
@@ -558,6 +577,21 @@ extern Nlm_Boolean Nlm_ProcessKeydown PROTO((Nlm_GraphiC g, WPARAM wParam,
 #ifdef WIN_MOTIF
 #include <Xm/FileSB.h>
 #include <Xm/CutPaste.h>
+
+/*
+ * Test X windows to see what kind of color buffer is available.
+ * Necessary because Linux supports only the type available
+ * from the hardware. Code in vibwndws.c. Returns TRUE if match and visinfo.
+ */
+
+extern Nlm_Boolean Nlm_CheckX(XVisualInfo *visinfo);
+
+
+#ifdef __cplusplus
+}
+#endif
+
+
 #ifdef OS_UNIX
 #include <unistd.h>
 #endif
@@ -565,10 +599,5 @@ extern Nlm_Boolean Nlm_ProcessKeydown PROTO((Nlm_GraphiC g, WPARAM wParam,
 #endif
 #endif
 
-
-#ifdef __cplusplus
-}
-#endif
-
 #endif
 
