@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/1/91
 *
-* $Revision: 6.2 $
+* $Revision: 6.5 $
 *
 * File Description: 
 *       Vibrant list functions
@@ -41,6 +41,15 @@
 *
 *
 * $Log: viblists.c,v $
+* Revision 6.5  2000/02/07 20:17:35  lewisg
+* minor bug fixes, use gui font for win32
+*
+* Revision 6.4  2000/01/07 00:22:46  thiessen
+* fixes for LessTif and OpenGL X visual selection
+*
+* Revision 6.3  1998/07/22 17:35:24  kans
+* reset list turns off drawing mode if invisible
+*
 * Revision 6.2  1997/12/12 21:08:31  kans
 * a number of symbols changed in the latest CodeWarrior release, now using headers from Apple
 *
@@ -616,6 +625,8 @@ static void Nlm_ResetList (Nlm_GraphiC l, Nlm_Boolean savePort)
   Nlm_WindoW    tempPort;
 #ifdef WIN_MAC
   Nlm_Int2      col;
+  Nlm_Boolean   doUpdate;
+  Nlm_Boolean   parentWindowVisible;
   Nlm_BaR       sb;
 #endif
 
@@ -623,9 +634,14 @@ static void Nlm_ResetList (Nlm_GraphiC l, Nlm_Boolean savePort)
   Nlm_GetListData ((Nlm_LisT) l, &ldata);
   c = ldata.handle;
 #ifdef WIN_MAC
+  doUpdate = Nlm_GetEnabled ((Nlm_GraphiC) l);
+  parentWindowVisible = Nlm_GetVisible ((Nlm_GraphiC) l) &&
+                        Nlm_GetAllParentsVisible ((Nlm_GraphiC) l);
+  LSetDrawingMode (parentWindowVisible && doUpdate, c);
   LDelColumn (0, 0, c);
   LDelRow (0, 0, c);
   col = LAddColumn (1, 0, c);
+  LSetDrawingMode (doUpdate, c);
   sb = ldata.scrollBar;
   if (sb != NULL) {
     Nlm_DoReset ((Nlm_GraphiC) sb, FALSE);
@@ -1258,6 +1274,10 @@ static void Nlm_NewList (Nlm_LisT l, Nlm_Int2 width,
   Cardinal        n;
   Arg             wargs [15];
 #endif
+#ifdef WIN_MSWIN
+  Nlm_FntPtr      fntptr;
+#endif
+
 
   Nlm_GetRect ((Nlm_GraphiC) l, &r);
   wptr = Nlm_ParentWindowPtr ((Nlm_GraphiC) l);
@@ -1303,6 +1323,9 @@ static void Nlm_NewList (Nlm_LisT l, Nlm_Int2 width,
     Nlm_Message (MSG_ERROR, "ListProc subclass error");
   }
   SetWindowLong (c, GWL_WNDPROC, (LONG) lpfnNewListProc);
+  fntptr = (Nlm_FntPtr) Nlm_HandLock (Nlm_systemFont);
+  SetWindowFont(c, fntptr->handle, FALSE);
+  Nlm_HandUnlock(Nlm_systemFont);
 #endif
 #ifdef WIN_MOTIF
   sb = NULL;
@@ -1359,7 +1382,12 @@ static void Nlm_NewList (Nlm_LisT l, Nlm_Int2 width,
     Widget hsb;
     XtVaGetValues(XtParent(c), XmNhorizontalScrollBar, &hsb, NULL);
     XtVaSetValues(hsb, XmNheight,            (Dimension)1,   NULL);
+#ifdef LESSTIF_VERSION
+    /* widget of width 0 causes LessTif to complain - why do this? */
+    XtVaSetValues(hsb, XmNwidth,             (Dimension)1,   NULL);
+#else
     XtVaSetValues(hsb, XmNwidth,             (Dimension)0,   NULL);
+#endif
     XtVaSetValues(hsb, XmNborderWidth,       (Dimension)0,   NULL);
     XtVaSetValues(hsb, XmNmappedWhenManaged, (Boolean)FALSE, NULL);
   }}
@@ -1540,7 +1568,7 @@ extern void Nlm_ListItem (Nlm_LisT l, Nlm_CharPtr title)
     Nlm_SetListNumItems (l, num);
     len = (Nlm_Int2) Nlm_StringLen (title);
     LSetCell (title, len, cell, c);
-    LSetDrawingMode (doUpdate, c);
+    LSetDrawingMode (TRUE, c);
     vis = Nlm_GetListVisLines (l);
     visDelta =  num - vis;
     if (visDelta > 0) {
